/************************************************************************/
/* chara.h								*/
/*                                                                      */
/* The standard CHARA header file					*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar et al.                                  */
/* Date   :                                                             */
/************************************************************************/

#ifndef __CHARA__
#define __CHARA__

/************************************************************************/
/* The stuff for User Interfaces and Modules		                */
/************************************************************************/

/* Here we define the various telescope names */

#define NUM_DELAY_LINES 6
#define NUM_SCOPES      6
#define S1      0
#define S2      1
#define E1      2
#define E2      3
#define W1      4
#define W2      5

/* Now we define the various beams */

#define BEAM_NONE	-1
#define NUM_BEAMS	6
#define BEAM_1		0
#define BEAM_2		1
#define BEAM_3		2
#define BEAM_4		3
#define BEAM_5		4
#define BEAM_6		5

/* And the PoPs */

#define NUM_POPS	5
#define POP_NONE	-1
#define POP_1		0
#define POP_2		1
#define POP_3		2
#define POP_4		3
#define POP_5		4
#define END_MIRROR	POP_5

/* Now the filter bands */

#define K_BAND 		0
#define H_BAND 		1
#define CONT_K		2
#define BRAC_GAM	3
#define MAX_BAND	BRAC_GAM

/************************************************************************/
/* The stuff for User Interfaces   		                        */
/************************************************************************/
#ifndef MODULE

#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

#ifndef CHARA_TIME
#define CHARA_TIME int
#endif

#ifndef NOERROR
#ifndef MESSAGE
#define MESSAGE 1 /* Use error system for putting up a message */
#endif
#define NOERROR 0 /* No error has occured */
#define WARNING (-1) /* A warning, nothing too dangerous has happened */
#define ERROR   (-2) /* Some kind of major error.Prog does not have to reboot */
#define FATAL   (-3) /* A fatal error has occured. The programme must exit(1)*/
#endif

#include <sys/types.h>
#include <sys/timeb.h>
#include <time.h>
#include <stdio.h>

/* errors for open_output() (stored in open_output_error) */
#define EOO_BADTYPE     1       /* incorrect file extension */
#define EOO_BADRNFILE   3       /* cant open run number secret file */
#define EOO_BADRNO      4       /* cant read the run number */
#define EOO_BADFILE     5       /* cant open the output file */
#define EOO_BADDIR      6       /* cant access/make a required directory */
#define EOO_WEXIST      7       /* write attempted on existing file */
#define EOO_BADTIME	8	/* invalid time -  does not exist */
#define EOO_BADNAME	9	/* invalid name -  does not exist */

/* warnings */
#define WOO_BADSTAR     0x10    /* cant access star number info */
#define WOO_NOOBS       0x20    /* no oberserv name stored */
#define WOO_NOBASE      0x40    /* no baseline information stored */
#define WOO_NOZD        0x80    /* no zen dist info stored */
#define WOO_NOHA        0x100   /* no hr angle info stored */
#define WOO_NOENV       0x200   /* no environment info stored */
#define WOO_NORNO       0x400   /* no run number secret file found */

/* bad input to string_to_time() and string_to_angle() */
#define BADSTR          1

/* other defines */

#define LNBUFFSIZE      64      /* max standard info line length */
#define OUTPUT_BASE     "/chara/chara/etc/"
#define APPEND TRUE


/* control flag values */
#define NOBLK   0x4     /* dont block on lock */
#define LCKBLK  0x8     /* wait for locks to clear */
#define WTRECV  0x10    /* wait for a receiver */
#define FLBLK   0x20    /* wait for file to exist */
#define PRUDSK  0x40    /* Unix disk protocol */

/* error returns */
#define EWBLK   10      /* would block */
#define ENRECV  11      /* no receiver */
#define EBADFL  12      /* no file */
#define EBADPRO 13      /* bad protocol */

/* general info */
#define MBUFFSIZE 1024  /* message buffer size */
#define HBUFFSIZE 128   /* header ditto */
#define TIMEOUT 20      /* time out time (s) */

/*
 * Constant definitions
 */

#ifndef M_PI
#define M_PI              3.14159265358979323846
#endif
#define M_PI_2            1.57079632679489661923

#ifndef PI
#define PI      (M_PI)
#endif
#define HALF_PI (M_PI_2)
#define TWO_PI  (2.0*M_PI)

/* Stuff for IR filter selection */

#define NUM_IR_FILTERS	4
#define K_BAND_CEN      (2.1329e-6)
#define K_BAND_WIDTH    (0.3489e-6)
#define H_BAND_CEN      (1.6731e-6)
#define H_BAND_WIDTH    (0.2854e-6)
#define CONT_K_CEN	(2.0894e-6)
#define CONT_K_WIDTH	(0.0235e-6)
#define BRAC_GAM_CEN	(2.1638e-6)
#define BRAC_GAM_WIDTH	(0.0216e-6)

extern float ir_filter_center[NUM_IR_FILTERS];
extern float ir_filter_width[NUM_IR_FILTERS];
extern char  *ir_filter_names[NUM_IR_FILTERS+1];

extern char *channel_types[3];
extern char *scope_types[NUM_SCOPES+1];
extern char *ople_types[NUM_DELAY_LINES+1];
extern char *pop_types[NUM_POPS+1];
extern char *beam_types[NUM_BEAMS+1];

/*
 *      Define some convenient epochs (all in days)
 *
 *      Fundamental epoch: 4713 BC January 1.5
 *      Modified epoch:    1858 November 17.0
 *      UNIX epoch:        1970 January 1.0
 *      SUSI clock epoch:  1990 January 1.0
 *      J2000 epoch:       2000 January 1.5
 */

#define JD_OF_EPOCH     2447891.50000000        /* days         */
#define MJD_OF_EPOCH      47891.00000000

#define JD_2000         2451545.00000000
#define MJD_2000          51544.50000000

#define UNIX_EPOCH      2440587.50000000
#define MJD_UNIX          40587.00000000

/*
 *      Default values.
 *      These are used by the various data input routines.  If no
 *      data is available, the relevant parameter is set to a default
 *      value.  Defaults may result in silly numbers being generated;
 *      however programs using them should not crash because of
 *      undefined or out of range values.  Defaults have the qualifier
 *      DFT added.
 */
                                                                                
/*
 *              Defaults for the telescope stations
 *              All default angles are in radians.
 */

#define REL_HEIGHT_DFT  0.00
                                                                                
/*
 *              Default environmental parameters
 */
#define TEMP_DFT        294.11  /* Temperature in kelvin                */
#define PRESSURE_DFT    986.26  /* Pressure in millibar (at 220 m asl)  */
#define RH_DFT          0.50    /* Relative humidity 0 < RH < 1.0       */
#define CO2_DFT         400     /* CO2 content in ppm                   */
#define WIND_DFT        0.00    /* wind speed (km/hr)                   */
#define LAPSE_DFT       0.0065  /* adiabatic lapse rate (K/m)           */
                                                                                
/*
 *              Default system parameters
 */

#define OBS_WAVE_DFT    2.300   /* Observing wavelength (microns)       */
#define MET_WAVE_DFT    1.319   /* Metrology wavelength (microns)       */
#define POLEX_DFT       0.00    /* Polar motion x (arcsec)              */
#define POLEY_DFT       0.00    /* Polar motion y (arcsec)              */
#define DUT_DFT         0.00    /* DUT (seconds of time)                */
#define HEIGHT_DFT      220.0   /* Height asl (meters)                  */
                                                                                
/*              Defaults for the star parameters                */

#define NAME_DFT        "NO NAME"       /* Star name                    */
#define PM_DFT          0.00            /* pm (same for RA & dec),rads  */
#define PX_DFT          0.00            /* parallax (arcsec)            */
#define RV_DFT          0.00            /* radial velocity (km)         */
#define VMAG_DFT        0.00            /* visual magnitude (magnitude) */
#define KMAG_DFT        0.00            /* K magnitude                  */
#define SP_DFT          "UNKNOWN"       /* spectral type                */
#define CAT_DFT         "UNKNOWN"       /* source catalog information   */

/*
 *      SOME CONVERSION FACTORS
 */

#define SECS_IN_DAY     86400.0
#define S2R             (TWO_PI/SECS_IN_DAY)
#define AS2R    (4.848136811095360E-06) /* arc seconds to radians */
#define D2R     (PI / 180.0)    /* degrees to radians     */
#define M2R     (D2R / 60)      /* minutes to radians     */
#define FT2M	((12.0*2.54)/100.0) /* Feet to metres */


/*
 *	Astromod's tick time.  This is the number of seconds between
 *	each interation.
 */
#define ASTROMOD_TICK_TIME       (10)

/*
 *	Maximum number of telescopes in the array
 */
#define Max_Tel		(6)

/*
 *      Where the data is kept.  Mostly used by astromod but is accessible
 *	to other programs if they need to read astromod's output.
 */

        /*      
         * The following files have to exist prior
         * to running astromod.  They contain necessary data.
         */
#define BASE_DIRECTORY          "/ctrscrut/chara/"
#define ETC_DIRECTORY           BASE_DIRECTORY "etc/"
#define PICT_DIRECTORY          BASE_DIRECTORY "pictures/"
#define TELESCOPE_DATA_FILE     ETC_DIRECTORY "telescopes.chara"
#define TELESCOPE_ORIGIN_LOG_FILE     ETC_DIRECTORY "telescope_origin.log"
#define ENVIRON_FILE            ETC_DIRECTORY "environ"
#define STAR_DATA_FILE          "/chara/observe/etc/starlist"
#define ABRIG_STAR_DATA_FILE    ETC_DIRECTORY "starlist_abrig"
#define SYSTEM_DATA_FILE        ETC_DIRECTORY "system.chara"

/* Gere's where socket manger keeps it's stuff and where it runs  */

#define SOCKET_MANAGER_FILE     ETC_DIRECTORY "socket_manager.dat"
#define SOCKET_MANAGER_MACHINE  "ctrscrut.chara-array.org"
#define SOCKET_MANAGER_PORT     2200

        /*
         * These files are created by astromod so they should
         * not exist.  If they do, they will be overwritten.
         */
#define DATA_DIRECTORY          BASE_DIRECTORY "data/"
#define DATA_FILE               "astromod.dat"
#define THE_STAR_FILE           "the_star.dat"  
#define THE_CONF_FILE           "the_conf.dat"
#define BASELINE_FILE           "baselines.dat"
#define ASTRO_PID_FILE          ETC_DIRECTORY "astromod.pid"

/* Some files used by various other utilities */

#define WOB_DISP_PID_FILE          ETC_DIRECTORY "wobdisp.pid"
#define REF_DISP_PID_FILE          ETC_DIRECTORY "refdisp.pid"
#define IR_DISP_PID_FILE          ETC_DIRECTORY "irdisp.pid"
#define NIRO_DISP_PID_FILE          ETC_DIRECTORY "nirodisp.pid"
#define TEL_DISP_PID_FILE          ETC_DIRECTORY "telescope_%s.pid"
#define TIPTILT_DISP_PID_FILE          ETC_DIRECTORY "ttdisp.pid"

/* Stuff defining te default length of integrations for data collection */

#define DATA_SCANS              200
#define SHUTTER_SCANS_A         (DATA_SCANS/10)
#define SHUTTER_SCANS_B         (DATA_SCANS/3)

/* How many NIRO pixels are there per ARCSEC? */

#define NIRO_ARCSEC_PER_PIX     1.01
#define NIRO_PIX_PER_ARCSEC     (1.0/NIRO_ARCSEC_PER_PIX)

/* Default minimum wieght for envelope tracking method */

#define DEFAULT_MINWEIGHT	1.0

/* How many telescopes? */

#define MAX_NUMBER_TELESCOPES   (6)

/*      A structure for the date and time of an epoch   */

typedef struct  s_time {
                double  mjd;    /* modified Julian date (in days)       */
                double  ut;     /* time (in seconds)                    */
                } TIME;

/*      A structure for the date and time of an epoch (integer format)  */

typedef struct  s_longtime {
                int mjd;       /* modified Julian date (in days)       */
                int ut;        /* time  (in seconds)                   */
                } LONGTIME;

/* standard time types */

/*
 * This structure is here because of 64/32 bit trouble.
 * The last two members had been long int but they are here
 * forced to be ints so they are 4 bytes long.
 */

struct chara_tm
{
  int tm_sec;                   /* Seconds.     [0-60] (1 leap second) */
  int tm_min;                   /* Minutes.     [0-59] */
  int tm_hour;                  /* Hours.       [0-23] */
  int tm_mday;                  /* Day.         [1-31] */
  int tm_mon;                   /* Month.       [0-11] */
  int tm_year;                  /* Year - 1900.  */
  int tm_wday;                  /* Day of week. [0-6] */
  int tm_yday;                  /* Days in year.[0-365] */
  int tm_isdst;                 /* DST.         [-1/0/1]*/

# ifdef __USE_BSD
  int tm_gmtoff;           /* Seconds east of UTC.  */
  int tm_zone_pointer_as_int;        /* Timezone abbreviation.  */
# else
  int __tm_gmtoff;         /* Seconds east of UTC.  */
  int __tm_zone_pointer_as_int;      /* Timezone abbreviation.  */
# endif
};

/* Structure that holds header information from a CLASSIC Fits file. */

typedef struct s_classic_header {
	char	hd[80];
	int	chara_number;
	char	object[80];
	char	ut_date[80];
	int	doy;
	char	ut_time[80];
	char	scope1[80];
	char	scope2[80];
	float	az;
	float	el;
	char	hour_angle[80];
	float	u;
	float	v;
	int	samples_per_scan;
	int	number_of_scans;
	int	shutter_a;
	int	shutter_b;
	int	lowsnr;
	int	nskip;
	int	number_data_scans;
	float	samples_per_second;
	int	pixel_size;
	int	pixel_row;
	int	pixel_col1;
	int	pixel_col2;
	int	number_of_reads;
	int	number_of_loops;
	int	reset_boundary;
	char	readout_mode[80];
	char	dither_state[80];
	float	lambda;
	float	dither_range;
	float	fringe_frequency;
		} CLASSIC_HEADER;

/*
 *      A generic angle type
 */

typedef struct s_angle {double az, el;} ANGLE;

/*
 *      A data type for polar motion coordinates
 */
typedef struct s_polar {double x, y;}   POLAR;

/*
 *      Generic vector type
 */

typedef struct s_vector {double x,y,z;} VECTOR;

/*
 *      Baseline structure
 */

typedef struct  s_baseline {
		char	tel1[81];	/* First telescope in baseline    */
		char	tel2[81];	/* Second telescope in baseline   */
                VECTOR  baseline;       /* The baseline vector in m       */
                double  length;         /* The modulus of the vector in m */
                } BASELINE;

/*
 *      This next structure holds all the star data, converted
 *      to standard form.
 */

#ifndef __CHARA_MESSAGES__
struct  s_star {
                char name[33];     /*  star ID                          */
                int chara_number;  /*  CHARA catalogue number.          */
		char irc[33];	   /*  IRC number of star		*/
		char hr[33];	   /*  HR number of star		*/
		char hd[33];	   /*  HD number of star		*/
		char sao[33];	   /*  SAO number of star		*/
                double ra;         /*  right ascension (rad)            */
                char rastr[33];    /*  ra display string                */
                double dec;        /*  declination (rad)                */
                char decstr[33];   /*  dec display string               */
                double pm_ra;      /*  ra proper motion (rad/year)      */
                double pm_dec;     /*  dec proper motion (rad/yr)       */
                double parallax;   /*  parallax (arcseconds)            */
                double radvel;     /*  radial velocity (km/s)           */
                double vmag;       /*  V magnitude                      */
                double kmag;       /*  K magnitude		        */
                char spectype[21]; /*  spectral type                    */
                };
#endif

typedef struct s_star STAR_DATA;

/*
 *      The SYSTEM data structure consists of parameters which are
 *      valid for the entire CHARA system.
 */
typedef struct  s_system {
                double longitude;  /* CHARA reference longitude         */
                double latitude;   /* CHARA reference latitude          */
                char fixed_cart[5];    /* OPLE cart which is stationary     */
                double height;     /* height asl in meters of ref plane */
                double obs_wavelength; /* observing wavelength */
                double met_wavelength; /* metrology wavelength */
                POLAR pole;        /* polar motion coordinates          */
                double dut;        /* UT1 - UTC                         */
		int num_telescopes;/* number of telescopes in the array */
		int num_baselines; /* number of baselines used		*/
		char scopes_used[20]; /* telescopes used		*/
                }       SYSTEM;

/*
 *	Data structure for astromod
 */
typedef struct	s_astromod_data {
		int record_length;
		int mjd;
		int ut;
		double n_air_obs;
		double n_air_met;
		ANGLE position_ref;
		ANGLE position_noref;
		double cart_pos[Max_Tel];
#ifdef INCLUDE_ROTATION
		float **quad_rotation_matrix[Max_Tel];
		float **acq_rotation_matrix[Max_Tel];
#endif
		}	ASTROMOD_DATA;

/*
 *      This structure holds data relevant to a particular telescope
 *      Refer to Iphase doco for explanation of some of these terms
 */
typedef struct s_telescope {
                char    arm;            /* which arm (S,E,or W) */
                int     station;        /* station number       */
                int     default_beam;   /* default beam number  */
                int     beam;           /* beam number          */
                int     default_pop;    /* Default pop position */
                int     pop;            /* Pop in use           */
                double  longitude;      /* station longitude    */
                double  latitude;       /* station latitude     */
                VECTOR  baseline;       /* baseline vector      */
                double  ec;             /* cos(EL) effects      */
                double  sas;            /* Swash, sin component */
                double  sac;            /* Swash, cos component */
                double  hsa2;           /* Harmonic sin(2*Az)   */
                double  wlf;            /* white light fringe
                                           position             */
                double  airpath;        /* Air path in m        */
                double  internal;       /* For internal fringes */
                double  beampath[NUM_BEAMS]; /* Extra path due to beam choice */
		double  lightpipe;      /* Path in light pipe   */
                double  poppath[NUM_POPS];      /* Path due to Pop selection */
                                                                                
                ANGLE   star;           /* the star azimuth
                                           and elevation        */
                VECTOR  to_star;        /* unit vector pointing
                                           to star*/
                double  hour_angle;     /* hour angle of
                                           the star             */
                                                                                
                } TELESCOPE;
                                                                                
/*
 *      Next we have a structure which holds all the relevant
 *      stuff for the path compensator.
 */
                                                                                
typedef struct  s_path_data {
                ANGLE  star;            /* the star az and el           */
                VECTOR to_star;         /* The apparent star
                                           position vector              */
                double hour_angle;      /* hour angle of the star       */
                int fringe_type;        /* SKY, AUTOCOL or INTERNAL     */
                } PATH_DATA;
                                                                                
/*
 *      The ENVIRON type contains the current temperature, pressure,
 *      humidity and windspeed.
 */
typedef struct  s_environ {
                double temp;            /*temperature (kelvin)          */
                double pressure;        /*pressure (mB or hPa)          */
                double rel_humid;       /*relative humidity (0 -1)      */
                double CO2;             /* amount of CO2 (ppm)          */
                double wind;            /* windspeed (km/hour)          */
                double lapse;           /* adiabatic lapse rate (K/m)   */
                }       ENVIRON;

/* These are mean values of the telescopes */

#define CHARA_LONGITUDE	string_to_angle("-118 3 25.6858")
#define CHARA_LATITUDE	string_to_angle("34 13 33.3065")
#define CHARA_HEIGHT	1725.21
#define LONG_DFT        CHARA_LONGITUDE
#define LAT_DFT         CHARA_LATITUDE
#define UT      1
#define GMT     UT
#define PST    2       /* without daylight savings */

/*
 * A macro to get the current time
 */

#define UNIX_EPOCH_DIFF (7304.0*24.0*3600.0)/*Diff between UNIX and SUSI epoch*/
#define time_now() (((double)time((time_t *)0)-(double)UNIX_EPOCH_DIFF)*S2R)

/* Defines required for John and Ettore's fringe tracking routine. */

/* calculate square of number */

#ifndef sqr
#define sqr(A) (A * A)
#endif

/* Function definitions using cproto */

/* convert.c */

double  string_to_angle(char *s);
char    *angle_to_string(double angle, char *display);
double  string_to_time(char *s);
char    *time_to_string(double time, char *display);
void	chara_tm_to_tm(struct chara_tm *from, struct tm *to);
void	tm_to_chara_tm(struct tm *from, struct chara_tm *to);
void	get_ut_date(int *year, int *month, int *day, int *doy);

/* files.c */

int fgetline(char *buffer, int buf_len, FILE *file);
int GetDataLine(char buffer[], int buflen, FILE *fp);
char *ToUpper(char *s);
int read_data_file(char *filename, struct chara_tm *gmt_now,
        CHARA_TIME      **now,
        float           **dither,
        int        	**h_1,
        int        	**h_2,
        int        	**k_1,
        int        	**k_2,
        float           *mean_time);
int read_data_file_irimage(char *filename, struct chara_tm *gmt_now,
        CHARA_TIME      **now,
        float           **dither,
        int        	**h_1,
        int        	**h_2,
        int        	**k_1,
        int        	**k_2,
        float           *mean_time);
int read_data_file_classic(char *filename, struct chara_tm *gmt_now,
        CHARA_TIME      **now,
        float           **dither,
        int        	**h_1,
        int        	**h_2,
        int        	**k_1,
        int        	**k_2,
        float           *mean_time);
int	read_classic_header(char *filename, struct s_classic_header *header);
int     find_dither_length(float *dither, int n);
char    *find_token_line(char *basename, char *extname,
                        char *token, char *result);
char read_server_ports(char *name, char *machine,
                       int *com_port, int *data_port, int *server_port);
int read_breakir_sample(char text, char reverse_byte, FILE *file,
                CHARA_TIME *now, float *dither, int *d1, int *d2);
int get_breakir_data_format(char *basename, char *text, char *reverse_byte);
int read_breakir_file(int text, int reverse_bytes, char *filename,
        CHARA_TIME **now, float **dith, int **d1, int **d2);

/* read_astromod.c */

int open_astromod (char *directory, char *filename);
int close_astromod (void);
int get_star_data (void);
int get_baseline_data (void);
int get_config_data (void);
int get_astromod_data (int time);
int get_cart_pos(int time, char *name, double *position);
#ifdef INCLUDE_ROTATION
int get_tiptilt_matrix(int time, char *name, float **rot_matrix);
int get_acquisition_matrix(int time, char *name, float **rot_matrix);
#endif
int get_star_name(char *name);
int get_star_hd(char *hd);
int get_star_chara_number(void);
int get_fixed_cart(char *fixed_cart);
int get_met_wavelength(double *met_wavelength);
int get_n_air_obs(int time, double *n_air_obs);
int get_n_air_met(int time, double *n_air_met);
int GetTelescopeNumber (char *tel_name, char *tels_used);
int check_unique(char *pidfile, char *machine);
int get_current_astromod_data(int time, ASTROMOD_DATA **data);
int get_current_star_data(int time, STAR_DATA **data);
int get_current_baseline_data(int time, BASELINE **data);
int get_current_chara_data(int time, SYSTEM **data);

/* times.c */

double GetTime(TIME *time_ptr);
struct tm *get_std_time(int type);
struct tm *get_std_time(int type);

/* astrocalc.c */

void calc_uv(double dec, double ha, double Bx, double By, double Bz, 
                double lambda, double *u, double *v);

/* stardata.c */

int read_star_data_main(STAR_DATA *star);
int read_star_data_abrig(STAR_DATA *star);
void close_starfile(STAR_DATA *star);
int update_abrig(STAR_DATA *star);
int get_star_by_number(int number, STAR_DATA *star);
int get_star_by_hr(int number, STAR_DATA *star);
int get_star_by_hd(int number, STAR_DATA *star);
int get_star_by_sao(int number, STAR_DATA *star);
int get_star_by_irc(int ircnum, STAR_DATA *star);

/* parse_sp_type.c */

int parse_sp_type(STAR_DATA *star,float *arr_sp_type);

/* extract_system_data.c */

int extract_system_data(SYSTEM *chara, TELESCOPE *Chara_scopes);
int ExtractTelescopeData(SYSTEM *chara, TELESCOPE *Chara_scopes);
int ExtractSystemData(SYSTEM *data);

/* fringe_track.c */

float	*fft_fringe(float *fringe, int n, int fft_n, char subtract_mean);
float	*invfft_fringe(float *fft, int n, int fft_n);
float *fft_freqs(int fft_n, float sample_time);
float	*make_bandpass_filter(float *freq, int fft_n, float center, float fwhm);
float   *make_lowpass_filter(float *freq, int fft_n, float edge, float fwhm);
float   *make_highpass_filter(float *freq, int fft_n, float edge, float fwhm);
void	use_filter(float *fft, float *filter, int fft_n);
float	*ps_fringe(float *fft, int fft_n);
float	*envelope_fringe(float *fft, int n, int fft_n, int *frg_pos);
int  gdt_fringe_track(float *fft, int n, int fft_n, float sample_time, 
		float frg_freq, float frg_width, float *weight, int *frg_pos);
float fringe_weight(float *ps, float *freq, int fft_n,
                float frg_freq, float frg_width);
float find_fringes_avg(float *data, int n);
float find_fringes_rms(float *data, int n, float avg);
float find_fringes_rms_parts(float *data, int n, int p);
int find_fringes_outliers(float *data, int n, float rms_noise, 
			  float threshold, int min_number_fringes);

/************************************************************************/
/* The stuff for Modules						*/
/************************************************************************/
#else /* MODULE */

#endif
#endif
