/************************************************************************/
/* fringe_track.c                                                       */
/*                                                                      */
/* ROutines to analyse fringe packets, look for fringes etc.		*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar	                                        */
/* Date   : April 2005							*/
/************************************************************************/

#include <stdlib.h>
#include <string.h>
#include "../nrc/nrc.h"
#include <unistd.h>
#include <math.h>
#include "chara.h"

/************************************************************************/
/* fft_fringe()								*/
/*									*/
/* Routine to calculate the FFT of a fringe packet. It does zero	*/
/* padding and places it in an array of the size specified by the caller*/
/* Returns the FFT of the fringe.					*/
/* Returns the FFT, as defined by the realft routine in Numerical 	*/
/* Recipies, or a NULL of it fails. Memory is allocated here.		*/
/************************************************************************/

float	*fft_fringe(float *fringe,	/* [1..n] The data of the fringe */
		    int n,		/* How many data points in fringe? */
		    int fft_n,		/* How many points in the FFT? */
		    char subtract_mean) /* Do we subtract the mean first? */
{
	float	*fft;	/* Here's where the data goes */
	float	mean;
	int	start;
	float	y, c, m, mult;
	int	apodize_length;
	int	i;

	/* Is this big enough? */

	if (fft_n < n) return NULL;
	
	/* Where do we put it in the FFT array? */

	start = (fft_n -n)/2;
	if (start < 4) return NULL;

	/* Allocate the memory */

	fft = vector(1, fft_n);

	/* Subtract mean if we need to */

	if (subtract_mean)
	{
		/* If we are subtracting the mean we zero pad. */

		mean = 0.0;
		for(i=1; i<=n; i++) mean += fringe[i];
		mean /= (float)n;
		for(i=1; i<=n; i++) fringe[i] -= mean;
	}

	/* Now fill out the padded array */

	for(i=1; i<=start; i++) fft[i] = fringe[1];
	for(; i < start+n; i++) fft[i] = fringe[i - start + 1];
	for(; i <= fft_n; i++)  fft[i] = fringe[n];

	/* And apply apodization. See log book 4 page 159. */

	apodize_length = start/2;

	m = 3.0/(apodize_length - 1);
	c = -1.0 * m * apodize_length;
	if (fringe[1] != 0.0)
	{
		mult = (fringe[1] + fringe[n])/(2.0*fringe[1]);
		fft[fft_n] = fft[1] = mult * fringe[1];
		for(i = 2; i <= apodize_length; i++)
		{
			y = m * i + c;
			fft[i] *= (mult+(1.0-mult)*exp(-1.0*y*y));
		}
	}
	
	c = m*(apodize_length - fft_n - 1.0);
	if (fringe[n] != 0.0)
	{
		mult = (fringe[1] + fringe[n])/(2.0*fringe[n]);
		for(i = fft_n-apodize_length+1; i < fft_n; i++)
		{
			y = m * i + c;
			fft[i] *= (mult+(1.0-mult)*exp(-1.0*y*y));
		}
	}

	/* Do the FFT */

	realft(fft, fft_n, 1);

	/* That should be all */

	return fft;

} /* fft_fringe() */

/************************************************************************/
/* invfft_fringe()							*/
/*									*/
/* Routine to calculate the inverse FFT of a fringe packet. 		*/
/* Returns the fringe, as extracted from the inverse FFT.		*/
/* Memory allocated in this function. Assumes input like that created 	*/
/* by fft_fringe. 							*/
/* Returns NULL it fails. Memory is allocated here.			*/
/* Uses local copy so FFT is not changed.				*/
/************************************************************************/

float	*invfft_fringe(float *fft,	/* [1..fft_n] The data of the FFT */
		    int n,		/* How many data points in fringe? */
		    int fft_n)		/* How many points in the FFT? */
{
	float	*fringe;
	int	start;
	int	i;
	float	norm;
	float	*copy;

	/* Is this big enough? */

	if (fft_n < n) return NULL;

	/* Where did we put it in the FFT array? */

	start = (fft_n - n)/2;

	/* Allocate the memory */

	fringe = vector(1, n);
	copy = vector(1, fft_n);

	/* copy */

	for(i=1; i<= fft_n; i++) copy[i] = fft[i];

	/* What is the normalization factor? */

	norm = 2.0/(float)fft_n;

	/* Do the inverse transform */

	realft(copy, fft_n, -1);

	/* Now fill in the right part back into the fringe */

	for(i = 1; i <=n ; i++) fringe[i] = norm*copy[i + start - 1];

	/* That should be all */

	free_vector(copy, 1, fft_n);

	return fringe;

} /* invfft_fringe() */

/************************************************************************/
/* fft_freqs()								*/
/*									*/
/* Calculates and returns a vector of the frequencies in the FFT.	*/
/* Returns NULL on error.						*/
/* Returns freqs[0..fft_n/2].						*/
/* These frequencies should correspond one to one with the output of	*/
/* ps_fringe(), but will not quite match those of fft_fringe.		*/
/* Memory allocated here, but you ened to free it up later.		*/
/************************************************************************/

float *fft_freqs(int fft_n, float sample_time)
{
	float	*freq;
	float	first_freq;
	int	i;

	/* Allocate memory */

	freq = vector(0, fft_n/2);

	/* WHat is the lowest frequency? */

	first_freq = 1.0/((float)fft_n * sample_time);

	/* OK, fill out the frequencies */

	for(i=0; i<=fft_n/2; i++) freq[i] = (float)i * first_freq;

	/* That is all */

	return freq;

} /* fft_freqs() */

/************************************************************************/
/* make_bandpass_filter()						*/
/*									*/
/* Based on a frequency array, a mid frequency and a FWHM, calculates	*/
/* a filter function, which is returned, based on a Gausian function.	*/
/* Memory allocated in this function. Returns NULL on error.		*/
/* Array returned is bp[0..fft_n/2].					*/
/************************************************************************/

float	*make_bandpass_filter(float *freq, int fft_n, float center, float fwhm)
{
	float	*bp;
#ifdef USE_GAUSSIAN
	int	i;
	float	sd;
	float	t1, t2;

	/* Convert FWHM to standard deviation */

	sd = fwhm / 2.354820045;

	/* Allocate memory */

	bp = vector(0, fft_n/2);

	/* Do the calculation */

	t2 = -2.0 * sd * sd;	
	for(i=0; i<= fft_n/2; i++)
	{
		t1 = freq[i]-center;
		bp[i] = exp((t1*t1)/t2);
	}
#else
	float	*lp, *hp;
	int	i;

	/* Create low pass and high pass filters */

	lp = make_lowpass_filter( freq, fft_n, center+fwhm/2.0, fwhm/4.0);
	hp = make_highpass_filter(freq, fft_n, center-fwhm/2.0, fwhm/4.0);

	/* Combine them into a bandpass */

	bp = vector(0, fft_n/2);
	for(i=0; i<= fft_n/2; i++) bp[i] = lp[i] * hp[i];

	/* Free memory */

	free_vector(lp, 0, fft_n/2);
	free_vector(hp, 0, fft_n/2);
#endif

	/* That is all */

	return bp;

} /* make_bandpass_filter() */

/************************************************************************/
/* make_lowpass_filter()                                               */
/*                                                                      */
/* Based on a frequency array, a mid frequency and a FWHM, calculates   */
/* a filter function, which is returned, based on a Gausian function.   */
/* Memory allocated in this function. Returns NULL on error.            */
/* Array returned is lp[0..fft_n/2].                                    */
/************************************************************************/
                                                                                
float   *make_lowpass_filter(float *freq, int fft_n, float edge, float fwhm)
{
        int     i;
        float   sd;
        float   *lp;
        float   t1, t2;
                                                                                
        /* Convert FWHM to standard deviation */
                                                                                
        sd = fwhm / 2.354820045;
                                                                                
        /* Allocate memory */
                                                                                
        lp = vector(0, fft_n/2);
                                                                                
        /* Do the calculation */
                                                                                
        t2 = -2.0 * sd * sd;
        for(i=0; i<= fft_n/2; i++)
        {
                if (freq[i] < edge)
                {
                        lp[i] = 1.0;
                }
                else
                {
                        t1 = freq[i]-edge;
                        lp[i] = exp((t1*t1)/t2);
                }
        }
                                                                                
        /* That is all */
                                                                                
        return lp;
                                                                                
} /* make_lowpass_filter() */

/************************************************************************/
/* make_highpass_filter()                                                */
/*                                                                      */
/* Based on a frequency array, a mid frequency and a FWHM, calculates   */
/* a filter function, which is returned, based on a Gausian function.   */
/* Memory allocated in this function. Returns NULL on error.            */
/* Array returned is hp[0..fft_n/2].                                    */
/************************************************************************/
                                                                                
float   *make_highpass_filter(float *freq, int fft_n, float edge, float fwhm)
{
        int     i;
        float   sd;
        float   *hp;
        float   t1, t2;
                                                                                
        /* Convert FWHM to standard deviation */
                                                                                
        sd = fwhm / 2.354820045;
                                                                                
        /* Allocate memory */
                                                                                
        hp = vector(0, fft_n/2);
                                                                                
        /* Do the calculation */
                                                                                
        t2 = -2.0 * sd * sd;
        for(i=0; i<= fft_n/2; i++)
        {
		if (freq[i] > edge)
		{
			hp[i] = 1.0;
		}
		else
		{
			t1 = freq[i]-edge;
			hp[i] = exp((t1*t1)/t2);
		}
        }
                                                                                
        /* That is all */
                                                                                
        return hp;
                                                                                
} /* make_highpass_filter() */

/************************************************************************/
/* use_filter()								*/
/*									*/
/* Given a filter and an FFT this will impose the filter on the FFT.	*/
/* FFT should have been created bu fft_fringe().			*/
/************************************************************************/

void	use_filter(float *fft, float *filter, int fft_n)
{
	int	n,i,j;

	n = fft_n/2;

	/* DC part first */

	fft[1] *= filter[0];

	/* Nyquist part */

	fft[2] *= filter[1];

	/* All the rest */

	for (i=2; i<= fft_n/2; i++)
	{
		j = (i-1)*2+1;

		fft[j] *= filter[i];
		fft[j+1] *= filter[i];
	}

} /* use_filter() */

/************************************************************************/
/* ps_fringe()								*/
/*									*/
/* Given the output of fft_fringe(), this routine returns the power	*/
/* spectrum, or NULL if there's an error. Memory is allocated here.	*/
/* The returned vector is the ps[0..fft_n/2].				*/
/************************************************************************/

float	*ps_fringe(float *fft,		/* [1..fft_n] The fft the fringe */
		    int fft_n)		/* How many data points in the fft? */
{
	float	*ps;
	int	n,i,j;

	/* Allocate the memory */

	n = fft_n/2;
	ps = vector(0, n);

	/* Do the DC and highest frequency calculation */

	ps[0] = (fft[1]*fft[1]);
	ps[n] = (fft[2]*fft[2]);

	/* Now the remaining frequencies. */

	for(i=1, j = 3; i<n; i++, j+=2)
	{
		ps[i] = (fft[j]*fft[j] +
		       fft[j+1]*fft[j+1]);
	}

	/* That should be all */

	return ps;

} /* ps_fringe() */

/************************************************************************/
/* envelope_fringe()							*/
/*									*/
/* Given the output of fft_fringe(), this routine returns the enevelope	*/
/* of the fringe, or NULL if there's an error. Memory is allocated here.*/
/* The returned vector is the env[1..fft_n].				*/
/* Note that this routine does not work well unless you have removed	*/
/* the DC part of the data. Using the subtract_mean part of the		*/
/* fft_fringe() routine will help here.					*/
/************************************************************************/

float	*envelope_fringe(float *fft,	/* [1..fft_n] The fft the fringe */
		    int n,		/* Number of points in data */
		    int fft_n,		/* How many data points in the fft? */
		    int *frg_pos)	/* Where the fringes might be. */
{
	float	*envelope_fft;
	float	*envelope;
	float	norm;
	int	i,j,k;
	int	start;
	float	max = 0.0;

	/* Allocate memory */

	k = fft_n << 1;
	envelope_fft = vector(1, k);
	envelope = vector(1, n);

	/* This will not work unless we have removed the DC part. */

	envelope_fft[1] = envelope_fft[2] = 0.0;

	/* 
	 * Now fill out only the positive parts of the FFT, 
	 * All negative frequencies are zero.
	 */

	for(i=3; i <= fft_n; i++) envelope_fft[i] = fft[i];
	for(; i <= k; i++) envelope_fft[i] = 0.0;
	envelope_fft[3] = envelope_fft[4] = 0.0;

	/* Do the inverse transorm */

	four1(envelope_fft, fft_n, -1);

	/* Now find the amplitude of the result with normalization */

	norm = (float)fft_n/2.0;
 
        /* Where did we put it in the FFT array? */
 
        start = (fft_n - n)/2;
        *frg_pos = 1;
 
        for(i=1; i<=n; i++)
        {
            j = ((i+start-1)<<1) - 1;
            envelope[i]= sqrt(envelope_fft[j]*envelope_fft[j] +
                envelope_fft[j+1]*envelope_fft[j+1])/norm;
            if (envelope[i] > max)
            {
                *frg_pos = i;
                max = envelope[i];
            }
        }
 
        /* Free memory */
 
        free_vector(envelope_fft, 1, k);
 
        /* That should be all */
 
        return envelope;
 
} /* envelope_fringe() */

/************************************************************************/
/* gdt_fringe_track()                                                   */
/*									*/
/* This code from JM for finding the fringe packet.			*/
/* Fringe packet Tracker based on pedretti et al. 2004. 		*/
/* input: fft data, npoint, fringe peak/half-width (in nyquist units) 	*/
/* returns: fringepos, weight 						*/
/* This has been slightly modified by Theo.				*/
/************************************************************************/

int  gdt_fringe_track(float *fft,         /* output from fft_fringe() */
	     int n,		 /* Samples in original fringe */
	     int fft_n,          /*   number of samples in scan (input) */
	     float sample_time,	 /* Sample period each sample (S) */
	     float frg_freq,	 /* Expected fringe frequency (Hz) */
	     float frg_width,	 /* How wide an area do we search? (Hz) */
	     float *weight,      /*   fringes weighting       */
	     int *frg_pos   	 /*   fringe pos in fringe array */
	   )
{
	short int j,k;
	int maxn; /* maxn to deal with artificially high Nyquist frequency */
	float noise_mean, power_mean, rawPhase, abs_rawPhase, amplitude ;
	float dPhase,re,im,a,b,c,d;
	float *crossSpec;
	fcomplex *xft;
	int pos = 0;                  /*   position of fringe peak */
	float	fringepos;
	int	start,stop;
	int filtWidth;      /*   width of fringe peak    */

	/* Allocate memory */

	crossSpec = vector(0,fft_n/2);
	xft = fcvector(0, fft_n/2);
 
	/* Work out some starting points */

	pos = (int)(frg_freq*((float)fft_n * sample_time) + 0.5);
	filtWidth = (int)(0.5 * pos * frg_width/frg_freq + 0.5);
	maxn = 2*pos;
	if (maxn > fft_n/2 ) maxn=fft_n/2; /* fft_n is power of 2 */

	/*   complex product between fft and fft shifted one sample */
	/*   also calculates pwr. spec., and X pwr. spec. */
	    
	for (k = 1; k < (maxn-1); ++k) 
	{
	  j = k<<1;
	  a = *(fft + j );
	  b = *(fft + j + 1);
	  c = *(fft + j + 2);
	  d = *(fft + j + 3);
	  re = a*c + b*d;
	  im = -a*d + b*c;
	  (xft+k)->r = re;
	  (xft+k)->i = im;
	  *(crossSpec + k) = sqr(re) + sqr(im);

	}

	/* 
	 * Calculate noise in all pspec but the filter.
	 * Customized for CHARA version since because of 
	 * funny data-repeating in scan resulting in false Nyquist frequency.
	 * Will assume true Nyquist is twice the fringe frequency, as 
	 * expected for 4 pix/fringe.
	 */

	if ((start = pos - filtWidth) < 1) start = 1;
	if ((stop = pos + filtWidth) > maxn) start = maxn;

	noise_mean = 0.0;
	power_mean = 0.0;

	for(k = start; k <= stop; k++) power_mean += *(crossSpec + k);
	for(; k<= maxn; k++) noise_mean += *(crossSpec + k);

	noise_mean = noise_mean / (maxn - pos - filtWidth);
	power_mean = power_mean / (filtWidth + filtWidth);

	if (fft_n % 2 == 0) 
	{ 
	  /* fft_n is even */ 

	  *(crossSpec + maxn ) = 
		sqr((xft + maxn)->r);  /* estimated true Nyquist freq. */    
	}

	/*   are fringes there?   */
	
	*weight = (power_mean - noise_mean) /  noise_mean;

	if (*weight <= 0.0 || isnan(*weight)) *weight = 1e-9;

	/*  No thresholding here -- assume fringes are there */

	re = 0.0;
	im = 0.0;

	/*   integrate re and im part of fringe peak in x spectrum */

	for (k = start; k <= stop; k++) 
	{
	    /* we weight the real and imaginary part by the power spectrum */

	    amplitude = sqrt(*(crossSpec + k));  /* weighting ... */

	    re += amplitude * (xft + k)->r;
	    im += amplitude * (xft + k)->i;
	}

	/*  calculate diff. phase */

	rawPhase=atan2(im, re);	              
	abs_rawPhase=fabs(rawPhase);
	dPhase=(-abs_rawPhase/rawPhase)*(abs_rawPhase - PI);
	fringepos = (dPhase * fft_n/(2*PI))+ fft_n/2; 
	*frg_pos = n/2 - fft_n/2 + fringepos + 0.5;

	/* Free memory */

	free_vector(crossSpec, 0,fft_n/2);
	free_fcvector(xft, 0, fft_n/2);

	/* That is all */

	return (1L); /* successfully finishd calculation */

} /* gdt_fringe_track() */

/************************************************************************/
/* fringe_weight()							*/
/*									*/
/* Works out fringe weight, a ratio of power on and off fringes.	*/
/************************************************************************/

float fringe_weight(float *ps, float *freq, int fft_n, 
		float frg_freq, float frg_width)
{
	float	off_fringe, on_fringe;
	int	on_start, on_stop, weight_dist;
	int	off_start, off_stop;
	int 	i;

	/* Where are we integtrating power? */

	on_start = (int)((frg_freq - frg_width/2.0)/freq[1]+0.5);
	on_stop  = (int)((frg_freq + frg_width/2.0)/freq[1]+0.5);
	weight_dist   = on_stop-on_start+1;

	off_start = on_start+weight_dist;
	off_stop = on_stop+weight_dist;

	if (on_start < 1) on_start = 1;
	if (on_stop > fft_n/2) on_stop = fft_n/2;
	if (off_start < 1) off_start = 1;
	if (off_stop > fft_n/2) off_start = fft_n/2;

	/* First, work out the power near the fringe. */

	on_fringe = 0.0;
	for(i=on_start; i<= on_stop; i++) on_fringe += ps[i];

	/* 
	 * Now the power off the fringe. 
	 * This assumes that the fringes are more than Nyquist sampled.
	 */

	off_fringe = 0.0;
	for(i=off_start; i<= off_stop; i++) off_fringe += ps[i];

	/* Fractional fringe power is what we want */

	return (on_fringe - off_fringe)/off_fringe;

} /* fringe_weight() */

/************************************************************************/
/* find_fringes_avg()                                                   */
/*   Computes and returns the average of a 1D vector                    */
/*   DATA of size (1..N)                                                */
/*                                                                      */
/*  author: Antoine Merand 2006/12/05                                   */
/************************************************************************/

float find_fringes_avg(float *data, int n)
{
	int i; 
	float avg;
	
	if (n <= 0) return 0.0;

	avg = 0.0;
	
	for (i=1;i<n;i++)
		avg += data[i];
	avg = avg/n;
	
	return avg;

} /* find_fringes_avg() */

/************************************************************************/
/* find_fringes_rms()                                                   */
/* Compute and returns the root mean square of a 1D vector              */
/* DATA of size (1..N)                                                  */
/*                                                                      */
/* author: Antoine Merand 2006/12/05                                    */
/************************************************************************/
float find_fringes_rms(float *data, int n, float avg)
{
	int i;
	float rms;
	
	if (n <= 0) return 0.0;

	rms = 0.0;
	
	for (i=1;i<n;i++)
		rms += (data[i]-avg)*(data[i]-avg);
	rms = rms/n;
	
	return rms;

} /* find_fringes_rms() */


/************************************************************************/
/* find_fringes_rms_parts                                               */
/*   Compute the root mean square of P equal parts of a 1D vector       */
/*   DATA (size 1..N) return the min of these RMS's                     */
/*                                                                      */
/*   author: Antoine Merand 2006/12/05                                  */
/************************************************************************/
float find_fringes_rms_parts(float *data, int n, int p)
{
	int i,j;
	float min_rms;
	float avg;
	float rms;
	float *tmp;
	
	if ((tmp = malloc((n/p)*sizeof(float))) == NULL )
		printf("error\n");
	
	min_rms = 1.0e32;
	
	for (j=0;j<p;j++)
	{
		for (i=0;i<n/p;i++)
			tmp[i] = data[j*n/p+i];
		avg = find_fringes_avg(tmp, n/p);
		rms = find_fringes_rms(tmp, n/p, avg);
		if (rms<min_rms)
			min_rms = rms;
	}
	free(tmp);

	return min_rms;

} /* find_fringes_rms_parts() */


/************************************************************************/
/* find_fringes_outliers()						*/
/*   Takes a 1D scan DATA of size 1..N and detects fringe packet.       */
/*   !Assume that there is only one fringe packet in DATA scan!        	*/
/*   - if fringes are not found, returns -1				*/
/*   - if fringes are found, returns the position of the fringe		*/
/*     packet inside the scan (within 0..N-1)				*/
/*									*/   
/*   requires:								*/
/*   - RMS_NOISE: root mean square of a scan without fringes (noise)	*/
/*   - THRESHOLD: number of sigmas above the noise for wich		*/
/*     one considers a fringe is detected                               */
/*   - MIN_NUMBER_FRINGES               				*/
/*									*/   
/*   the function will detect all fringes and return the median		*/
/*     index of these detections.					*/
/*									*/
/*   author: Antoine Merand 2006/12/05					*/
/************************************************************************/

int find_fringes_outliers(float *data, int n, float rms_noise, 
			  float threshold, int min_number_fringes)
{
	float  avg;
	int    i, n_outliers, med_ind;
	int    *outliers;
	
	med_ind=-1; /* default: fringes not found */
	
	if ((outliers = malloc(n*sizeof(int)))== NULL )
		printf("error\n");
	
	avg = find_fringes_avg(data,n);
	
	n_outliers = 1; 
	
	/* flag outliers by their index */

	for (i=1;i<n;i++)
	{
		if (((data[i]-avg)*(data[i]-avg))>threshold*threshold*rms_noise)
		{
			outliers[i] = n_outliers;
			n_outliers++;
		}
		else
			outliers[i] = 0;
	}
	
	
	if (n_outliers>=min_number_fringes) /* if any outliers */
	{
		/* find median index */

		for (i=1;i<n;i++)
		{
			if (outliers[i]>n_outliers/2)
			{
				med_ind = i; 
				i = n; 
			}
		}
	}
	
	free(outliers);
	
	return med_ind;

} /* find_fringes_outliers() */
