/************************************************************************/
/* com.c 								*/
/*                                                                      */
/* Small utility to connect to a socket and send a command to it.	*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                          		*/
/* Date   : March 1999							*/
/************************************************************************/

#include <stdio.h>
#include <netdb.h>
#include <resolv.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <string.h>
#include <sys/time.h>
#include "charaui.h"
#include "../../charalib/chara.h"
#include "../../../cliserv/socket_manager/server/socket_manager.h"

int	char_waiting(int fd);
#define BUFFER_SIZE 1024

int main(int argc, char **argv)
{
        struct sockaddr_in server; /* Server address */
	struct hostent *hp;	/* Pointer to host data */
	int	socket_p;	/* Socket number */
	int	buf_size;	/* Size of buffers */
	int n;
	char	command[BUFFER_SIZE];
        int     error_level;
        time_t  start_time;
	char	c,*p;
	char	machine[256];
	int	com_port, data_port, server_port;
	struct s_process_sockets process;

	/* Test command line */

	if (argc < 3)
	{
		fprintf(stderr,"usage: %s name command\n",argv[0]);
		exit(-1);
	}

        /* First try and get the info from the socket manager */

	init_jobs();
        if (socket_manager_get_process(argv[1], &process) == 0)
        {
                strcpy(machine, process.machine);
                com_port = process.commands;
                data_port = process.data;
                server_port = process.messages;
        } else if (!read_server_ports(argv[1], machine, 
                &com_port, &data_port, &server_port))
        {
            fprintf(stderr, "Process %s is not running or does not exist.\n", 
                argv[1]);
            fprintf(stderr,"Don't forget to use S1 and not s1 for scopes.\n");
            fprintf(stderr,"You could try checking with te sockman program.\n");
            exit(-2);
        }
        if (com_port <= 0)
        {
            fprintf(stderr,"Process %s seems to not have a command port.\n",
                argv[1]);
            exit(-3);
        }

	/* First see if we can identify the host. */

	if((hp = gethostbyname(machine)) == 0)
	{
		fprintf(stderr,"Unknown host.\n");
		exit(-2);
	}

	/* Create a socket */

	if ((socket_p = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
		fprintf(stderr,"Failed to create socket\n");
		exit(-3);
	}

	/* Force buffer sizes to BUFFER_SIZE */

	buf_size = BUFFER_SIZE;

	if (setsockopt(socket_p, SOL_SOCKET, SO_SNDBUF, 
		(char *)&buf_size, sizeof(buf_size)) < 0)
	{
		close(socket_p);
		fprintf(stderr,"Failed to change send buffer size to %d\n",
			buf_size);
		exit(-4);
	}

	if (setsockopt(socket_p, SOL_SOCKET, SO_RCVBUF, 
		(char *)&buf_size, sizeof(buf_size)) < 0)
	{
		close(socket_p);
		fprintf(stderr,"Failed to change receive buffer size to %d\n",
			buf_size);
		exit(-5);
	}

	/* Create server address */

	server.sin_family = AF_INET;
	strncpy((char *)&server.sin_addr, (char *)hp->h_addr, hp->h_length);
	server.sin_port = htons(com_port);

	/* Let's try and connect now.... */

	if (connect(socket_p, (struct sockaddr *)&server, sizeof(server)) < 0)
	{
		close(socket_p);
		fprintf(stderr,"Failed to connect.\n");
		perror("Error ");
		exit(-6);
	}
	
	/* Assemble the command */

	strcpy(command,argv[2]);
	for(n=3; n<argc;n++)
	{
		strcat(command," ");
		strcat(command,argv[n]);
	}

	/* Start off by sending  the command */

	write(socket_p, command, sizeof(command));

	/* Now wait for the reply */

	start_time = time(NULL);
	while(1)
	{
		/* Is there something to read ? */

		if (char_waiting(socket_p))
		{
			/* Get the string */

			for(p = command, c = 0; 
			    c != '\n' && c != '\r' && p-command < BUFFER_SIZE; 
			    p++)
			{
				if (read(socket_p, &c, 1) != 1)
				{
					close(socket_p);
					fprintf(stderr,
						"Error reading from socket.\n");
					exit(0);
				}
				*p = c;
			}
			*p = 0;

			/* Display the string */

			printf("%s",command);
			fflush(stdout);

			/* Is this the final message ? */

			if (sscanf(command,SOCKET_COMMAND_STR,&error_level)==1)
			{
				break;
			}

		}
		
		/* Has the user typed anything? */

		if (char_waiting(0))
		{
			fgets(command, BUFFER_SIZE, stdin);
			write(socket_p,command,sizeof(command));
		}

		/* Have we timed out ? */

                if (start_time+1000 < time(NULL))
                {
			close(socket_p);
                        fprintf(stderr,
                        "Timed out waiting for command to complete.\n");
			exit(-4);
                }
	}

	close(socket_p);

	/* Print message about what happened */

	printf("Command finished with ");
	switch(error_level)
	{
		case MESSAGE: printf("MESSAGE.\n");
			      break;

		case NOERROR: printf("NOERROR.\n");
			      break;

		case WARNING: printf("WARNING.\n");
			      break;

		case ERROR:   printf("ERROR.\n");
			      break;

		case FATAL:   printf("FATAL.\n");
			      break;

		default:      printf("UNKNOWN.\n");
			      break;
	}

	/* Exit, returning the error level */

	exit(error_level);
}

/************************************************************************/
/* char_waiting()							*/
/*									*/
/* Unix version of kbhit like function. Returns 1 if a character 	*/
/* waiting at the port, 0 otherwise.					*/
/* Uses the select(2) routine to do it's work. 				*/
/* From background.c and stripped .					*/
/************************************************************************/

int	char_waiting(int fd)
{
	/* Locals */

	fd_set rfds;		/* File discriptor set */
        struct timeval tv;	/* Time to wait */

	/* Set tings up */

	FD_ZERO(&rfds);		 /* Zero the file discriptor set */
        FD_SET(fd, &rfds);	 /* Add fd to the set */
	tv.tv_sec = 0;		 /* Wait no more than 1 usec */
	tv.tv_usec = 1;

	if (select(FD_SETSIZE, &rfds, NULL, NULL, &tv) < 0) 
	{
		/* Something is wrong. */

		perror("char_waiting ");
		return 0;
	}

	if (FD_ISSET(fd, &rfds)) /* Was there something? */
	{
		return 1;
	}

	/* Must have been nothing there */

	return 0;

} /* char_waiting() */
