/************************************************************************/
/* script.c								*/
/*                                                                      */
/*	This is the main code for scripting.				*/
/*                                                                      */
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/************************************************************************/

/*
 * Include files
 */

#include "charaui.h"

/* Globals */

SCRIPTS  first_script;
SCRIPTS  last_script;

/************************************************************************/
/* init_script() 							*/
/*									*/
/* Initialize the scripts. Set's first_script and last_script pointing 	*/
/* at each other.  							*/
/************************************************************************/

void init_scripts(void)
{
	first_script.last = NULL;
	first_script.next = &last_script;
	last_script.last = &first_script;
	last_script.next = NULL;

	strcpy(last_script.name,"LAST");
	strcpy(first_script.name,"FIRST");

} /* init_scripts() */

/************************************************************************/
/* delete_scripts() 							*/
/*									*/
/* Delete the scripts. Set's first_script and last_script pointing 	*/
/* at each other. Frees memory for all the others.			*/
/************************************************************************/

void delete_scripts(void)
{
	struct sscripts *next;
	struct sscripts *last;

	for (next = last_script.last; next != &first_script; )
	{
		last = next;
		next = last->last;
		free(last);
	}

} /* delete_script() */

/************************************************************************/
/* new_script()								*/
/* 									*/
/* Set's up new memory for a script structure at the end of the linked 	*/
/* list of scripts. 							*/
/* Returns a pointer to the new entry or NULL on failure.		*/
/************************************************************************/

struct sscripts *new_script(struct sscripts data)
{
	int	i;
	struct sscripts *new;
	unsigned char *pdata,*pnew;

	/* Allocate the memory */

	if ((new = malloc(sizeof(*new))) == NULL) return NULL;

	/* Copy in the data */

	pnew = (unsigned char *)new;
        pdata = (unsigned char *)&data; 
	for(i=0; i<sizeof(data); i++) *pnew++ = *pdata++;

	/* Now make sure all the pointers are done */

	new->last = last_script.last;
	new->next = &last_script;
	last_script.last = new;
	(new->last)->next = new;
	
	/* That is all */

	return new;

} /* new_script() */

/************************************************************************/
/* delete_script() 							*/
/* 									*/
/* Removes a script from the linked list of scripts. It refuses to 	*/
/* remove one that does not exist, and it refuses to remove either 	*/
/* the first or last script.						*/
/* Returns TRUE if all went well and FALSE otherwise.			*/
/************************************************************************/

int	delete_script(struct sscripts *old)
{
	struct sscripts *this;

	/* Ensure it isn't first last or not on the list */

	for(this = &first_script; 
		this != old && this != &last_script; 
		this = this->next);
	if (this == &last_script || old == &first_script) return FALSE;

	/* OK, set pointers for one before and after it */

	(old->last)->next = old->next;
	(old->next)->last = old->last;

	/* Remove the memory */

	free(old);

	/* That's the lot */

	return TRUE;

} /* delete_script() */

/************************************************************************/
/* parse_script()							*/
/*									*/
/* Opens a file and tries to turn it into a script. 			*/
/*									*/
/* 	- Work out the name from the filename (1st argument).		*/
/* 	- Check that it isn't the same as a function or a menu.		*/
/*	- Check that it is a unique name for a script.			*/
/*	- Create space for it and set it's name.			*/
/* 	- Add it to the linked list.					*/
/************************************************************************/

int parse_script(int argc, char **argv)
{
	char	filename[LENGTH_CL+1];
	int	i;
	char	temp[LENGTH_CL+1];
	char	*c;
	struct 	sscripts new,*pnew;
	struct	slabel {	
			char name[81];
			int  pos;
		      } label[MAXSCRIPT];
	int	number_labels = 0;
	int	line_pos = 0;
	FILE	*input;
	char	line[256];
	char	*firstword,*rest;

	/* Check the command line */

	if (argc > 1)
	{
		strcpy(filename,argv[1]);
	}
	else
        {
                strcpy(filename,"script.scr");
                clean_command_line();
                message(system_window,"Enter the script filename.");
                if (line_edit(command_window,0,2,filename,LENGTH_CL,
                        STRING,return_true,TRUE) == KEY_ESC) return NOERROR;
        }

	/* Find the last '.' in the name */

	strcpy(temp, filename);
	for(c = temp; *c != 0; c++); /* Go to end of string */
	for(; c != temp && *c != '.'; c--); /* Go back to the last '.' */
	if (c == temp)
	{
		/* We found no '.', better add what we need */

		for(c = temp; *c != 0; c++);
		strcat(temp,".scr");
		strcat(filename,".scr");
	}

	/* Check the extension is correct */

	if (strcmpi(c+1,"scr"))
	{
		return error(ERROR,"Extension must be \'.scr\'.");
	}

	/* Work out what the name is */

	for(*c = 0; *c != '/' && c != temp; c--); /* Go back to last '/' */
	if (*c == '/') ++c;
	strcpy(new.name, c);

	/* Make sure this is not the same as a function */

	if (find_function(new.name) != (int)NULL)
	{
		return error(ERROR,"A function by the name %s already exists.",
				new.name);
	}

	/* Make sure this is not the same as a menu */

	if (find_menu(new.name) != (struct smenu *)NULL)
	{
		return error(ERROR,"A menu by the name %s already exists.",
				new.name);
	}

	/* Does a script by this name already exist ? */

	if ((pnew = find_script(new.name)) != (struct sscripts *)NULL)
	{
		if (!ask_yes_no("A script by this name already exists.",
				"Do you wish to overwrite it?")) return NOERROR;
		if (!delete_script(pnew)) 
			return error(ERROR,"Failed to delete script %s.",
			pnew->name);
	}

	/* Set defaults */

	for(i=0; i<MAXSCRIPT; i++)
	{
		strcpy(new.lines[i].function,"");
		strcpy(new.lines[i].arguments,"");
		new.lines[i].on_noerror = -1;
		new.lines[i].on_message = -1;
		new.lines[i].on_warning = -1;
		new.lines[i].on_error = -1;
		new.lines[i].on_fatal = -1;
		new.lines[i].on_goto = -1;
	}

	/* Open the file */

	if ((input = fopen(filename,"r")) == NULL)
	{
		return error(ERROR,"File %s does not exist.\n",filename);
	}

	/* Go through the file and compile the list of labels */

	while(getline(line, 256, input) != (char *)EOF)
	{
		/* Does this line contain the character ':' */

		for(c=line; *c != 0 && *c != ':'; c++);

		if (*c == ':') /* We found a label */
		{
			if (number_labels >= MAXSCRIPT)
			{
				fclose(input);
				return error(ERROR,"Too many labels.");
			}

			/* Set the first non white space character to null */

			for(c = line; *c != ':' && *c != ' ' && *c != '\t';c++);
			*c = 0;

			/* Copy it into the array of structures */

			strcpy(label[number_labels].name,line);
			label[number_labels].pos = line_pos;
			++number_labels;
		}
		else
		{
			if (strcmpi("onmessage",(firstword = 
					split_string(line,&rest))) != 0 &&
			    strcmpi("onyes",firstword) != 0 &&
			    strcmpi("onnoerror",firstword) != 0 &&
			    strcmpi("onwarning",firstword) != 0 &&
			    strcmpi("onerror",firstword) != 0 &&
			    strcmpi("onfatal",firstword) != 0) line_pos++;
		}
	}
	rewind(input);

	/* We now go through the file again and look at each line */

	line_pos = 0;
	while(getline(line, 256, input) != (char *)EOF)
	{
		/* Ignore the labels */

		for(c=line; *c != 0 && *c != ':'; c++);
		if (*c == ':') continue;

		/* Is this an onsomething or goto command ? */

                if (strcmpi("onmessage",(firstword = split_string(line,&rest)))
                                                        == 0 ||
			    strcmpi("onyes",firstword) == 0 ||
			    strcmpi("onnoerror",firstword) == 0 ||
			    strcmpi("onwarning",firstword) == 0 ||
			    strcmpi("onerror",firstword) == 0 ||
			    strcmpi("onfatal",firstword) == 0 ||
			    strcmpi("goto",firstword) == 0)
                {
			/* Yes it is, what is the label to go to ? */

			for(i=0; i<MAXSCRIPT;i++)
			{
				if (strcmpi(rest,label[i].name) == 0) break;
			}

			/* Does the label exist? */

			if (i == MAXSCRIPT)
			{
				fclose(input);
				return error(ERROR,
					"Undefined label %s.", rest );
			}
			
			/* OK set the correct field */

			if (strcmpi("onmessage",firstword) == 0 ||
			    strcmpi("onyes",firstword) == 0)
			{
				/* YES == MESSAGE!!! */

				new.lines[line_pos].on_message = label[i].pos;
			}
			else if (strcmpi("onnoerror",firstword) == 0)
			{
				new.lines[line_pos].on_noerror = label[i].pos;
			}
			else if (strcmpi("onwarning",firstword) == 0)
			{
				new.lines[line_pos].on_warning = label[i].pos;
			}
			else if (strcmpi("onerror",firstword) == 0)
			{
				new.lines[line_pos].on_error = label[i].pos;
			}
			else if (strcmpi("onfatal",firstword) == 0)
			{
				new.lines[line_pos].on_fatal = label[i].pos;
			}
			else /* Must be a goto */
			{
				strcpy(new.lines[line_pos].function,"goto");
				new.lines[line_pos].on_goto = label[i].pos;
				if (++line_pos >= MAXSCRIPT)
				{
				    fclose(input);
				    return error(ERROR,"Too many commands.");
				}
			}

			/* That is all for this line */

			continue;
		}

		/* Is this an existing function or script ? */

		if (find_function(firstword)==(int)NULL
				&& find_script(firstword) == NULL)
		{
			fclose(input);
			return error(ERROR,
				"%s is not a function or a script name.",
				firstword);
		}

		/* Copy the function and argument over */

		strcpy(new.lines[line_pos].function,firstword);
		strcpy(new.lines[line_pos].arguments,rest);

		/* Copy the current set of error targets */

		if (line_pos <MAXSCRIPT-1)
		{
			new.lines[line_pos+1].on_message = 
				new.lines[line_pos].on_message;
			new.lines[line_pos+1].on_noerror = 
				new.lines[line_pos].on_noerror;
			new.lines[line_pos+1].on_warning = 
				new.lines[line_pos].on_warning;
			new.lines[line_pos+1].on_error = 
				new.lines[line_pos].on_error;
			new.lines[line_pos+1].on_fatal = 
				new.lines[line_pos].on_fatal;
		}

		/* Incriment the line_pos counter */

		if (++line_pos >= MAXSCRIPT)
		{
		    fclose(input);
		    return error(ERROR,"Too many commands.");
		}
	}
	fclose(input);

	/* Add this new script to the list */

	if ((pnew = new_script(new)) == (struct sscripts *)NULL)
	{
		return error(ERROR,
			"Failed to add new script %s.\n",new.name);
	}

#ifdef TEST
	/* Spit out what we done */

	strcat(filename,".output");

	if ((input = fopen(filename,"w")) == NULL)
	{
		return error(ERROR,"Failed to open file %s.",filename);
	}

	fprintf(input,"SCRIPT: %s\n",pnew->name);
	fprintf(input,"Last: %s\n",pnew->last->name);
	fprintf(input,"Next: %s\n",pnew->next->name);

	for(i=0;i<MAXSCRIPT;i++)
	{
		if (strlen(pnew->lines[i].function) == 0) break;
		fprintf(input,"Line %d: %s %s\n",i,pnew->lines[i].function,
					  pnew->lines[i].arguments);
		fprintf(input,
			"mes: %d noerr: %d warn:%d err:%d fatal: %d goto: %d\n",
			pnew->lines[i].on_message,
			pnew->lines[i].on_noerror,
			pnew->lines[i].on_warning,
			pnew->lines[i].on_error,
			pnew->lines[i].on_fatal,
			pnew->lines[i].on_goto);
	}
	fclose(input);
#endif
	
	/* That is all */

	message(system_window,"Added script %s",new.name);


	return NOERROR;

} /* parse_script() */

/************************************************************************/
/* call_script()							*/
/*									*/
/* Given a pointer to a script structure this function tries to run	*/
/* the script through. It calls each function in turn and checks the	*/
/* returned error level. The error level is compared to the various	*/
/* pointers in the structure for what to do on which error level.	*/
/* for example script_pt->lines[i]->on_message tells it where to go	*/
/* on a message. If no such pointer is set the action depends on the	*/
/* error level. On NOERROR and MESSAGE, it continues on to the next	*/
/* function in the list. On WARNING, it askes the user if it should	*/
/* continue. On ERROR or FATAL it stops. The return value is always 	*/
/* the error level returned by the last function.			*/
/* Error messages are turned off during a script process.  		*/
/************************************************************************/

int	call_script(struct sscripts *script_pt)
{	
	int	pos;
	int	funct_pt;
	int	error_level;
	char	s[81];
	struct sscripts *call_script_pt;

	/* Stop error messages comming through */

	error_messages_off(0,NULL);

	/* Loop through the script */

	error_level = NOERROR;
        for(pos=0;pos<MAXSCRIPT;pos++)
	{
		/* Are we done yet ? */

                if (strlen(script_pt->lines[pos].function) == 0) break;

		/* Is this a GOTO? */

		if (strcmpi(script_pt->lines[pos].function,"goto")==0)
		{
			pos = script_pt->lines[pos].on_goto - 1;
			continue;
		}

		/* Attempt to find the function */

		if ((funct_pt = find_function(script_pt->lines[pos].function))==
			(int)NULL)
		{
			/* Well it ain't a function, is it a script? */

                        if ((call_script_pt =
				find_script(script_pt->lines[pos].function)) !=
				(struct sscripts *)NULL)
                        {
				/* Yup, call the script */

                                error_level = call_script(call_script_pt);
			}
			else
			{
				error_level = error(ERROR,
			    "Script %s tried to call non-existant command %s.",
			    script_pt->name, script_pt->lines[pos].function);
				break;
			}
		}
		else
		{

			/* It is a function, so run it */

			strncpy(s,script_pt->lines[pos].arguments,81);
			error_level = call_function(funct_pt, s);
		}
		
		/* React to the returned error level */

		if (error_level == MESSAGE)
		{
			if (script_pt->lines[pos].on_message != -1)
			{
				pos = script_pt->lines[pos].on_message - 1;
				continue;
			}
		}
		else if (error_level == NOERROR)
		{
			if (script_pt->lines[pos].on_noerror != -1)
			{
				pos = script_pt->lines[pos].on_noerror - 1;
				continue;
			}
		}
		else if (error_level == WARNING)
		{
			if (script_pt->lines[pos].on_warning != -1)
			{
				pos = script_pt->lines[pos].on_warning - 1;
				continue;
			}
			else
			{
				sprintf(s,
					"The function %s returned as WARNGING.",
					script_pt->lines[pos].function);
					
				if (!ask_yes_no(s,"Should we continue?"))
				{
					break;
				}
				continue;
			}
		}
		else if (error_level == ERROR)
		{
			if (script_pt->lines[pos].on_error != -1)
			{
				pos = script_pt->lines[pos].on_error - 1;
				continue;
			}
			else
			{
				break;
			}
		}
		else if (error_level == FATAL)
		{
			if (script_pt->lines[pos].on_fatal != -1)
			{
				pos = script_pt->lines[pos].on_fatal - 1;
				continue;
			}
			else
			{
				break;
			}
		}
		else
		{
			error_messages_on(0,NULL);
			error_level = error(ERROR,
				"Unknown error level returned by function %s.",
				script_pt->lines[pos].function);
			break;
		}
	}

	/* Let user know what happened */

	sprintf(s,"Script %s exits with error level ",script_pt->name);
	switch(error_level)
	{
		case MESSAGE: strcat(s,"MESSAGE.");
			      break;
		case NOERROR: strcat(s,"NOERROR.");
			      break;
		case WARNING: strcat(s,"WARNING.");
			      break;
		case ERROR:   strcat(s,"ERROR.");
			      break;
		case FATAL:   strcat(s,"FATAL.");
			      break;
		default:      strcat(s,"UNKNOWN.");
	}
	if (active_socket == -1)
	{
		mvwaddstr(system_window,1,0,s);
		wrefresh(system_window);
	} 
	else
	{
		socket_print(active_socket,"%s\n",s);
	}

	/* Turn on error message again */

	error_messages_on(0,NULL);

	/* That is all */

	return error_level;

} /* call_script() */

/************************************************************************/
/* user_message()							*/
/* 									*/
/* User callable function to put a message in the system window.	*/
/* Concatinates all arguments together into a single string and then	*/
/* prints the text in the system window using the function message().	*/
/* Always returns NOERROR.						*/
/************************************************************************/

int	user_message(int argc, char **argv)
{
	char	s[81];
	int	i;

	*s = 0;

	for(i=1; i<argc; i++)
	{
		if (strlen(s)+strlen(argv[i])+1 > 80)
		{
			error(WARNING,"User string too long for message.");
			break;
		}
		if (strlen(s) != 0) strcat(s," ");
		strcat(s,argv[i]);
	}

	message(system_window,s);

	return NOERROR;

} /* user_message() */

/************************************************************************/
/* user_ask_yes_no() 							*/
/* 									*/
/* User callable version of ask_yes_no. Uses arguments as the first 	*/
/* and sets the second string to NULL. Retunrs either YES or NO. Note	*/
/* that YES is the same as MESSAGE and NO is the same as NOERROR, so 	*/
/* it should be possible to use this within scripts and get sensible 	*/
/* behaviour.								*/
/************************************************************************/

int user_ask_yes_no(int argc, char **argv)
{
	char	s[81];
	int	i;

	*s = 0;

	for(i=1; i<argc; i++)
	{
		if (strlen(s)+strlen(argv[i])+2 > 80)
		{
			error(WARNING,"User string too long for message.");
			break;
		}
		if (strlen(s) != 0) strcat(s," ");
		strcat(s,argv[i]);
	}
	strcat(s,"?");

	return ask_yes_no(s,"");

} /* user_ask_yes_no() */

/************************************************************************/
/* do_nothing() 							*/
/*									*/
/* User callable function that does nothing. Returns NOERROR.		*/
/* Often usefull as place holders in scripts, also good for final	*/
/* item in any script.							*/
/************************************************************************/

int do_nothing(int argc, char **argv)
{
	return NOERROR;

} /* do_nothing() */
