/************************************************************************/
/* server.c								*/
/*                                                                      */
/* Routines for using a socket to accept outside commands.  		*/
/* In this one the outside commands are in the form of binary messages	*/
/* and this should look exactly like the cliserv server model.		*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY SERVER LIB				*/
/*		   Based on the CHARA User Interface			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar 		                        */
/* Date   : August 2002							*/
/************************************************************************/

/*
 * Include files
 */

#include "charaui.h"
#include "../../../cliserv/include/chara_messages.h"
#include <netdb.h>
#include <resolv.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <signal.h>

/* Define this for no networked machines */

/* #define STANDALONE */

/* 
 * Local globals
 */

static int	message_socket = -1;	/* socket used for outside messages */
static struct {
		int  fd;		/* File descriptor */     
		char *machine; 	/* Name of connecting machine */
			} client_sockets[MAX_CONNECTIONS]; /* Active channels */
static char	server_computer_name[81];	/* This machine's name */
static int	server_port;		/* The port we are listening too */
static int	curr_client = 0;		/* Current socket to check */


static int (*(message_jobs[MAX_JOBS]))(struct smessage *message);

static int current_socket = -1;

#define MAX_SIGNAL 256

static unsigned char client_signals[MAX_SIGNAL];

static bool sent_signal_message = FALSE;

/* Globals */

int client_socket = -1;                 /*
                                         * The current client socket.
                                         * Set to -1 if no client socket.
                                         */

char *client_machine;                   /*
                                         * The current server machine.
                                         * Set to 0 if no server machine.
                                         */

char	*server_name;			/* What we call ourselves */

/************************************************************************/
/* open_message_socket()						*/
/* 									*/
/* Opens a message socket that can be used for accepting and sending	*/
/* outside messages. 							*/
/* Tries to find a port that works.					*/
/*									*/
/* Returns error level.							*/
/*									*/
/* This routine is set up so that it will accept a connection from 	*/
/* anyone in our domain, so be very carefull about this.		*/
/************************************************************************/

int open_message_socket(void)
{
        struct sockaddr_in server; /* Server address */
	int	i;
        struct hostent *hp;     	/* Pointer to host data */
	int	buffer_size;
	char	name[81];		
	int	port;

	/* Has this already been done? */

	if (message_socket != -1)
	{
		message(system_window,"Command socket already exists.");
		return NOERROR;
	}

	/* Setup a default name */

	strcpy(server_computer_name,"Unknown");

	/* Clear the current sockets */

	client_socket = -1;
	client_machine = NULL;
	for(i=0; i< MAX_CONNECTIONS; i++) client_sockets[i].fd = -1;

	/* Loop through until we find a socket that works */

	for(port = DEFAULT_SERVER_PORT; port < DEFAULT_SERVER_PORT+100; port++)
	{
		/* Create a socket */

		if ((message_socket = socket(AF_INET, SOCK_STREAM, 0)) < 0)
		{
			message_socket = -1;
			continue;
		}

		/* Force buffer sizes to buffer_size */

		buffer_size = SERVER_SOCKET_BUFFER;
		if (setsockopt(message_socket, SOL_SOCKET, SO_SNDBUF, 
			(char *)&buffer_size, sizeof(buffer_size)) < 0)
		{
			close(message_socket);
			message_socket = -1;
			continue;
		}

		if (setsockopt(message_socket, SOL_SOCKET, SO_RCVBUF, 
			(char *)&buffer_size, sizeof(buffer_size)) < 0)
		{
			close(message_socket);
			message_socket = -1;
			continue;
		}

		/* Set up server address */

		server.sin_family = AF_INET;
		server.sin_addr.s_addr = htonl(INADDR_ANY);
		server.sin_port = htons(port);

		/* Now bind the socket to this address */

		if (bind(message_socket,
			(struct sockaddr *)&server,sizeof(server)) < 0)
		{
			close(message_socket);
			message_socket = -1;
			continue;
		}

		/* Create a queue for client connection requests */

		if (listen(message_socket, 0) < 0)
		{
			close(message_socket);
			message_socket = -1;
			continue;
		}

		/* If we get here it worked */

		break;

	}

	if (port >= DEFAULT_SERVER_PORT+100)
		error(ERROR, "Failed to open server port.\n");

	/* Find out our own name... */

        if(gethostname(name,81) < 0 || (hp = gethostbyname(name)) == 0)
        {
                close(message_socket);
		message_socket = -1;
		message(system_window,"Can't identify ourselves!.");
                return ERROR;
        }
	strcpy(server_computer_name,hp->h_name);
	server_port = port;

	/* Reset current socket pointer. */

	curr_client = 0;

	/* Clear all the signals */

	for(i=0; i<MAX_SIGNAL; i++) client_signals[i] = FALSE;

	return NOERROR;

} /* open_message_socket() */

/************************************************************************/
/* close_message_socket()						*/
/* 									*/
/* Close the message socket if it's open. Remember this could cause     */
/* trouble if you do this while the channel is server. If the message	*/
/* socket is not open this function will do nothing.			*/
/*									*/
/************************************************************************/

void close_message_socket(void)
{
	int i;

	/* Do nothing if it isn't open */

	if (message_socket == -1) return;

	/* Clear the current sockets */

	client_socket = -1;
	client_machine = NULL;
	for(i=0; i< MAX_CONNECTIONS; i++) 
	{
		if (client_sockets[i].fd != -1) 
		{
			close(client_sockets[i].fd);
			client_sockets[i].fd = -1;
			free(client_sockets[i].machine);
		}
	}

	/* Close it */

	close(message_socket);

	/* Reset things */

	message_socket = -1;
	server_port = -1;
	signal(SIGPIPE, SIG_DFL);

} /* close_message_socket() */

/************************************************************************/
/* process_message_socket() 						*/
/*									*/
/* Checks through the array of sockets and see's if any have become	*/
/* domant. If any have it releases them. Checks the message_socket	*/
/* to see if there are any new connects. Adds them to the array of	*/
/* sockets if there are. Checks through the sockets array to see	*/
/* if any new messages have arrived. If so, set's client_socket to	*/
/* that socket, runs the message and returns a message to the socket	*/
/* concerning success or failure.					*/
/*									*/
/* Returns the error level.						*/
/************************************************************************/

int	process_message_socket(void)
{
	int	i;			/* Looping */
        struct sockaddr_in connector; 	/* Connector's address */
	unsigned int	len;			/* Lenght of address */
#ifndef STANDALONE
        struct hostent *hp;     	/* Pointer to host data */
	char	*domain;		/* Pointer to domain in name */
#endif
	message_struct  themessage;	/* The message */

	/* 
	 * If there is no message_socket do nothing.
	 */

	if (message_socket == -1) return NOERROR;

	/* 
	 * OK, let's loop look at the current socket 
	 * and see if any are dormant or have any data in them to read.
	 */

	/* Let's find the next socket to process */

	for(i = curr_client+1; i < MAX_CONNECTIONS && 
		client_sockets[i].fd == -1; i++);
	if (i >= MAX_CONNECTIONS)
	{
		for(i = 0; i < curr_client && client_sockets[i].fd == -1; i++);
	}
	curr_client = i;

	/* If there is a message waiting do something about it */

	if (client_sockets[i].fd != -1 && char_waiting(client_sockets[i].fd))
	{
		/* Read in message type */

		len = ui_read_socket(client_sockets[i].fd,
			(char *)&themessage.type,
			sizeof(themessage.type));
		if (len <= 0)
		{
			/* Read failed, connection has died */

			client_sockets[i].fd = -1;
			message(system_window,
				"%s just logged off.",
				client_sockets[i].machine);
			free(client_sockets[i].machine);
			return MESSAGE;
		}
		if (len != sizeof(themessage.type))
		{
			message(system_window,"Failed to read message type.");
			return ERROR;
		}

		/* read in data size */

		len=ui_read_socket(client_sockets[i].fd,
			(char *)&themessage.length,
			sizeof(themessage.length));
		if (len <= 0)
		{
			/* Read failed, connection has died */

			client_sockets[i].fd = -1;
			message(system_window,
				"%s just logged off.",
				client_sockets[i].machine);
			free(client_sockets[i].machine);
			return MESSAGE;
		}
		else if (len != sizeof(themessage.length))
		{
		    message(system_window,
			"Failed to read message length for message %d",
			themessage.type);
		    return ERROR;
		}

		/* Allocate memory for message data */

		if (themessage.length == 0)
		{
			themessage.data = NULL;
		}
		else
		{
		    if ((themessage.data = malloc(themessage.length)) == NULL)
		    {
			error(FATAL,"Ran out of memory processing message %d.",
				themessage.type);
		    }

		    /* Read in the data */

		    len = ui_read_socket(client_sockets[i].fd,
					(char *)themessage.data,
				themessage.length);
		    if (len <= 0)
		    {
			/* Read failed, connection has died */

			client_sockets[i].fd = -1;
			free(client_sockets[i].machine);
			return MESSAGE;
		    }
		    else if (len != themessage.length)
		    {
			  message(system_window,
				"Got only %d bytes out of %d for message %d",
			    len, themessage.length,
			    themessage.type);
		   	 return ERROR;
		    }
		}

		/*
		 * Now, there is a chance we have been called
		 * from within a process being run by a client.
		 * In this case we only allow UI_SIGNAL messages
		 * through, all others are blocked.
		 */

		if (client_socket != -1)
		{
#ifdef SEND_SIGNAL_MESSAGE
			if (themessage.type != UI_SIGNAL)
			{
			    if (!sent_signal_message)
			    {
				ui_print_message(client_sockets[i].fd,
					"Server busy, only signals allowed.");
			    	sent_signal_message = TRUE;
			    }
			}
			else
			{
				ui_run_message_job(&themessage);
			}
#else
			if (themessage.type == UI_SIGNAL)
				ui_run_message_job(&themessage);
#endif
		}
		else
		{
			/* This becomes the server socket */

			client_socket = client_sockets[i].fd;
			client_machine = client_sockets[i].machine;

			/* Process the message type */

			ui_run_message_job(&themessage);

			/* OK, we're finished with this socket for now */
	
			client_socket = -1;
			client_machine = NULL;
			sent_signal_message = FALSE;
		}

		/* Free memory for message data */

		if (themessage.data != NULL) free(themessage.data);
	}

	/* See if someone wants a new connection */

	if (char_waiting(message_socket))
	{
		/* Find an empty slot for this new socket */

		for(i=0; i<MAX_CONNECTIONS && client_sockets[i].fd != -1; i++);

		if (i < MAX_CONNECTIONS)  /* We found one */
		{
			len = sizeof(connector);
                        if ((client_sockets[i].fd = accept(message_socket,
				(struct sockaddr *)&connector,&len)) < 0)
                        {
                                message(system_window,
				      "Failed to accept socket connection.");
                        }

#ifndef STANDALONE
			/* OK, extract the name */

			if ((hp = gethostbyaddr((char *)&connector.sin_addr,
                                        len,connector.sin_family)) == NULL)
			{
				message(system_window,
				"Failed to identify host. Blocking socket.");
				close(client_sockets[i].fd);
				client_sockets[i].fd = -1;
			}
			else
			{
				/* Put the name of the machine in structure */

				if ((client_sockets[i].machine = 
					malloc(strlen(hp->h_name)+1)) == NULL)
				{
					error(FATAL,
				"Ran out of memory during a socket connection");
				}
				strcpy(client_sockets[i].machine,hp->h_name);

				/* Separrate out it's domain part. */

				for(domain=hp->h_name; *domain != 0 &&
					*domain != '.'; domain++);
				if (*domain == '.') ++domain;

				/* Is this OK ? */

				if (strcmp(server_computer_name,hp->h_name) && 
				    strcmp(domain,ALLOWED_DOMAIN))
				{
					close(client_sockets[i].fd);
					client_sockets[i].fd = -1;
					free(client_sockets[i].machine);
					message(system_window,
			"Host %s not in correct domain. Blocking socket.", 
					hp->h_name);
				}
				else 
				{
				    message(system_window,"%s just connected.",
					hp->h_name);
				    ui_print_log(
					"%s accepted connection from %s.",
					TITLE, hp->h_name);
				}
			}
#else
			if ((client_sockets[i].machine = 
				malloc(strlen("localhost")+1)) == NULL)
			{
				error(FATAL,
				"Ran out of memory during a socket connection");
			}
			strcpy(client_sockets[i].machine,"localhost");

			message(system_window, "%s just connected.",
					"localhost");
			ui_print_log("%s accepted connection from %s.",
				TITLE, "localhost");
#endif
		}
	}

	return NOERROR;

} /* process_message_socket() */

/************************************************************************/
/* get_server_name()							*/
/*									*/
/* What is the name of our server? 					*/
/************************************************************************/

char *get_server_name(char *name)
{

	strcpy(name, server_name);
	return name;

} /* get_server_name() */

/************************************************************************/
/* get_server_computer_name()						*/
/*									*/
/* Get the name of this machine. Expects the string sent to have memory */
/* already allocated. Returns the string sent as an argument.		*/
/* Note that if the socket message channel is not open yet you'll	*/
/* get garbage in the string.						*/
/************************************************************************/

char *get_server_computer_name(char *name)
{

	strcpy(name, server_computer_name);
	return name;

} /* get_server_computer_name() */

/************************************************************************/
/* flush_client_socket()						*/
/*									*/
/* Flush a single socket. Will not close it if there's nothing there.	*/
/************************************************************************/

void flush_client_socket(int sock)
{
	int	count,closed;
	char	c;

	closed = 0;
	count=0;

	if (sock == -1) return;

	while (char_waiting(sock)) 
	{
		if (read(sock, &c, 1) == 1) 
			count++;
		else
			break;
	}

	if (count > 0)
	{
		message(system_window,"Flushed %d bytes.", count);
	}

} /* flush_client_socket() */

/************************************************************************/
/* flush_client_sockets()						*/
/*									*/
/* Goes through all message sockets and empties them of any text.	*/
/* Will also close them out if required. This function should be called */
/* whenever you stop listening to the sockets.				*/
/************************************************************************/

void flush_client_sockets(void)
{
	int	i;
	int	count,closed;
	char	c;

	for(closed = 0, count=0, i=0; i<MAX_CONNECTIONS; i++)
	{
		if (client_sockets[i].fd == -1) continue;

		while (char_waiting(client_sockets[i].fd))
		{
			if (read(client_sockets[i].fd, &c, 1) <= 0)
			{
				/* Read failed, connection has died */

				closed++;
				client_sockets[i].fd = -1;
				free(client_sockets[i].machine);
				break;
			}
			count++;
		}
	}

	if (count + closed > 0)
	{
		message(system_window,"Flushed %d bytes and closed %d sockets.",
			count, closed);
	}

} /* flush_client_sockets() */

/************************************************************************/
/* ui_send_message()							*/
/*									*/
/* Sends a message to a given socket. Returns error level.		*/
/* Same as send_message in server library.				*/
/************************************************************************/

int ui_send_message(int sock, struct smessage *mess)
{
	int	i;

	/* 
         * There is a small chance that this socket has gone away.
         * For this reason we now check and do nothing if it has.
         */

        if (!write_ready(sock)) 
        {
                message(system_window,
			"Attempted to write to non-existant socket.");

                /* Write failed, remove connection if it's listed */

		for(i=0; i < MAX_CONNECTIONS; i++)
		{
			if (client_sockets[i].fd == -1) continue;

			if (client_sockets[i].fd == sock)
			{
				client_sockets[i].fd = -1;
				message(system_window,
					"%s just logged off.",
					client_sockets[i].machine);
				free(client_sockets[i].machine);
			}
		}

		return ERROR;
        }

	/* Set the current socket to be the one we're dealing with */

	current_socket = sock;

	/* Send the data */

	i = sizeof(mess->type);
	if (write(sock, &(mess->type), i) != i)
	{
		message(system_window,"Failed to write message type (%d).",
				mess->type);
		return ERROR;
	}

	i = sizeof(mess->length);
	if (write(sock, &(mess->length), i) != i)
	{
		message(system_window,"Failed to write message length (%d).",
				mess->length);
		return ERROR;
	}

	if (mess->length > 0 &&
		write(sock, mess->data, mess->length) != mess->length)
	{
		message(system_window, "Failed to write message (%d) data.",
				mess->type);
		return ERROR;
	}

	/* Set current socket back to -1 */

	current_socket = -1;

	/* That is all */

	return NOERROR;

} /* ui_send_message() */

/************************************************************************/
/* send_message_all()							*/
/*									*/
/* Sends the same message to all connected sockets.			*/
/* Returns error level.							*/
/* Same as send_message_all in server library.				*/
/************************************************************************/

int ui_send_message_all(struct smessage *message)
{
	int 	i;
	int	error_level;

	for(i=0; i<MAX_CONNECTIONS; i++)
	{
		if (client_sockets[i].fd != -1) 
		{
		    if ((error_level = 
			ui_send_message(client_sockets[i].fd, message)) != 
				NOERROR)
		    {
			return error_level;
		    }
		}
	}

	return NOERROR;

} /* ui_send_message_all(); */

/************************************************************************/
/* sig_pipe()								*/
/*									*/
/* For catching the SIGPIPE signal. Will see if it was caused by one	*/
/* of our clients and remove that client it was.			*/
/************************************************************************/

void    sig_pipe(int sig)
{
	int	i;

	/* is the current socket one of our clients? */

	if (current_socket == -1)
	{
		sig_pipe_text_sockets(sig);
		return;
	}

	for(i = 0; i < MAX_CONNECTIONS; i++)
	{
		if (client_sockets[i].fd == -1) continue;

		if (client_sockets[i].fd == current_socket)
		{
			client_sockets[i].fd = -1;
			message(system_window,
				"%s caused SIGPIPE. Logged off.",
				client_sockets[i].machine);
			free(client_sockets[i].machine);
		}
	}

} /* sig_pipe() */

/************************************************************************/
/* ui_read_socket()                                                     */
/*                                                                      */
/* Tries to read a given number of bytes into a given location from     */
/* a socket. Will keep trying to read until it get's all the bytes      */
/* it expects or it times out. Returns number of bytes recieved.        */
/* Alomst the same as read_socket() in server library.			*/
/************************************************************************/

int ui_read_socket(int fd, char *buf, size_t count)
{
        time_t  start;
        int     n,len;

        start = time(NULL);
        len = 0;
        while(count > 0)
        {
                if (time(NULL) > start + TIME_OUT) break;

#warning This used to be just < and it caused big delays
                if((n = read(fd, buf, count)) <= 0) return n;

                len += n;
                buf += n;
                count -= n;
        }

        return len;

} /* ui_read_socket() */


/************************************************************************/
/* standard_jobs()							*/
/*									*/
/* Set's up all standard jobs.						*/
/************************************************************************/

void standard_jobs(void)
{
	int i;

	for (i=0; i<MAX_JOBS; i++) message_jobs[i] = NULL;

	/* Setup the standard jobs */

	ui_add_message_job(NOOP, message_noop);
	ui_add_message_job(MESSAGE, message_message);
	ui_add_message_job(PING, message_ping);
	ui_add_message_job(ALL_NOOP, message_all_noop);
	ui_add_message_job(ALL_PING, message_all_ping);
	ui_add_message_job(UI_STATUS, message_status);
	ui_add_message_job(UI_SIGNAL, message_signal);
	ui_add_message_job(SEND_MESSAGE_ALL_CLIENTS, 
		message_send_message_all_clients);

} /* standard_jobs() */

/************************************************************************/
/* ui_add_message_job()							*/
/*									*/
/* Adds a new message job to the setup. FATAL error if job already	*/
/* exists.								*/
/* Almost the same as add_message_job in the server library.		*/
/************************************************************************/

void ui_add_message_job(unsigned int type,int (*job)(struct smessage *message))
{
	/* Is this message place already taken ? */

	if (message_jobs[type] != NULL)
	{
		error(FATAL,"Tried to create two message jobs for message %d.",
				type);
	}

	/* Is it a reasonable message type */

	if (type >= MAX_JOBS)
	{
		error(FATAL,"Tried to add job beyond message %d\n",MAX_JOBS);
	}

	/* Otherwise put it in place */

	message_jobs[type] = job;

} /* ui_add_message_job() */

/************************************************************************/
/* ui_run_message_job()							*/
/*									*/
/* Attempts to run message job.						*/
/* Copy of run_message_job in server library.				*/
/************************************************************************/

void ui_run_message_job(struct smessage *mess)
{
	/* Is it safe to run this message? */

	if (mess->type >= MAX_JOBS)
	{
		error(ERROR,"Message job outside of range (%d).", mess->type);
		return;
	}
	else if (message_jobs[mess->type] == NULL)
	{
		error(ERROR,"Message job with no function (%d).", mess->type);
		return;
	}

	/* Yes it is, go ahead */

	(*(message_jobs[mess->type]))(mess);

} /* ui_run_message_job() */

/************************************************************************/
/* set_server_name()							*/
/*									*/
/* Set server name to a particular string.				*/
/************************************************************************/

void set_server_name(char *name)
{
	server_name = name;

} /* set_server_name() */

/************************************************************************/
/* message_noop()							*/
/*									*/
/* A do nothing message handler.					*/
/* Returns error level.							*/
/************************************************************************/

int message_noop(struct smessage *themessage)
{
	if (themessage->length != 0)
	{
		message(system_window,"Received NOOP with data.");
	}

	message(system_window,"NOOP from %s",client_machine);
	return NOERROR;

} /* message_noop() */

/************************************************************************/
/* message_message()							*/
/*									*/
/* Should simply spit out the string in the data.			*/
/* Returns error level.							*/
/************************************************************************/

int message_message(struct smessage *mess)
{
	char *s,*p1,*p2;
	int	i;

	if (mess->length == 0)
	{
		message(system_window,
			"%s: No message string.", client_machine);
	}
	else
	{
		if ((s = malloc(mess->length+1)) == NULL)
		{
			error(FATAL,"Ran out of memory geting message string.");
		}

		for(i=0, p1=s, p2=(char *)mess->data;
		    i < mess->length; i++, p1++, p2++) *p1 = *p2;
		*p1 = 0;

		message(system_window,"%s: %s",client_machine,s);

		free(s);
	}

	return NOERROR;

} /* message_message() */

/************************************************************************/
/* message_ping()							*/
/*									*/
/* Sends back a message of type MESSAGE claiming we are alive.		*/
/* Returns error level.							*/
/************************************************************************/

int message_ping(struct smessage *themessage)
{
	char	name[256];

	if (themessage->length != 0)
	{
		message(system_window,"Recieved PING with data.");
	}

	message(system_window,"Recieved PING from %s.",client_machine);
	return ui_print_message(client_socket,"%s on %s is alive.",
			server_name,get_server_computer_name(name));

} /* message_ping() */

/************************************************************************/
/* message_all_noop()							*/
/* 									*/
/* Send a NOOP mesage to all connected machines.			*/
/* Returns error level.							*/
/************************************************************************/

int message_all_noop(struct smessage *message)
{
	message_struct mess;

	mess.type = NOOP;
	mess.length = 0;
	mess.data = NULL;

	/* Send the message */

	return ui_send_message_all(&mess);

} /* message_all_noop() */

/************************************************************************/
/* message_all_ping()							*/
/* 									*/
/* Send a PING mesage to all connected machines.			*/
/* Returns error level.							*/
/************************************************************************/

int message_all_ping(struct smessage *message)
{
	message_struct mess;

	mess.type = PING;
	mess.length = 0;
	mess.data = NULL;

	/* Send the message */

	return ui_send_message_all(&mess);

} /* message_all_ping() */

/************************************************************************/
/* message_status()							*/
/*									*/
/* Will send a copy of the current text in the status window to the	*/
/* the client asking for it.						*/
/************************************************************************/

int message_status(struct smessage *message)
{
	int    i,j;
	struct s_status status;
	message_struct mess;

	/* suck the line into memory */

	for (j=0; j<STATUS_LINES ; j++)
	{
		for (i=0; i<STATUS_COLS ; i++)
		{
			status.line[j][i] = mvwinch(status_window,j,i);
		}
		status.line[j][STATUS_COLS] = 0;
	}

	/* And send the message back to the client */

	mess.type = UI_STATUS;
	mess.length = sizeof(status);
	mess.data = (unsigned char *) &status;

	/* Send the message */

	return ui_send_message(client_socket, &mess);

} /* message_status() */

/************************************************************************/
/* message_signal()							*/
/*									*/
/* For when we get a UI_SIGNAL message. Set's signal to TRUE.		*/
/************************************************************************/

int message_signal(struct smessage *message)
{
	unsigned char *signal;

	if (message->length != sizeof(*signal))
	{
		ui_print_message(client_socket, "SIGNAL of wrong data size.");
		return ERROR;
	}

	signal = (unsigned char *) message->data;

	client_signals[*signal] = TRUE;

	return NOERROR;

} /* message_signal() */

/************************************************************************/
/* message_send_message_all_clients()					*/
/*									*/
/* For when we get a SEND_MESSAGE_ALL_CLIENTS message. 			*/
/************************************************************************/

int message_send_message_all_clients(struct smessage *message)
{
	struct smessage pmess;
	int	*type;

	if (message->length != sizeof(*type))
	{
		ui_print_message(client_socket, 
			"SEND_MESSAGE_ALL_CLIENTS with bad data.");
		return ERROR;
	}

	type = (int *)message->data;
	pmess.type = *type;
	pmess.length = 0;
	pmess.data = NULL;

	return ui_send_message_all(&pmess);

} /* message_send_message_all_clients() */

/************************************************************************/
/* any_client_signal_set()						*/
/*									*/
/* Returns TRUE if any signals are set.					*/
/************************************************************************/

bool any_client_signal_set(void)
{
	int i;

	for(i=0; i<MAX_SIGNAL; i++) 
	{
		if (client_signals[i]) return TRUE;
	}

	return FALSE;

} /* any_client_signal_set() */

/************************************************************************/
/* client_signal_set()							*/
/*									*/
/* Tells you if a particular client signal has been set. Returns TRUE 	*/
/* or FALSE.								*/
/************************************************************************/

bool client_signal_set(unsigned char signal)
{
	return client_signals[signal];

} /* client_signal_set() */

/************************************************************************/
/* clear_client_signal()						*/
/*									*/
/* Turns off given signal.						*/
/************************************************************************/

void clear_client_signal(unsigned char signal)
{

	client_signals[signal] = FALSE;

} /* clear_client_signal() */

/************************************************************************/
/* ui_print_message()							*/
/* 									*/
/* Works like a printf only sends the string as a message of type	*/
/* MESSAGE to the given socket.						*/
/* Returns error level.							*/
/************************************************************************/

int ui_print_message(int sock, char *fmt, ...)
{
	message_struct message;
        char    string[9999];
        va_list args;

        /* Create the string */

        va_start(args, fmt);
        vsprintf(string, fmt, args);

	/* Create the message */

	message.type = MESSAGE;
	message.length = strlen(string);
	message.data = (unsigned char *)string;

	/* Send the message */

	return ui_send_message(sock, &message);

} /* ui_print_message() */

/************************************************************************/
/* ui_print_message_all()						*/
/* 									*/
/* Works like a printf only sends the string as a message of type	*/
/* MESSAGE to the all sockets.						*/
/* Returns error level.							*/
/************************************************************************/

int ui_print_message_all(char *fmt, ...)
{
	message_struct message;
        char    string[9999];
        va_list args;

        /* Create the string */

        va_start(args, fmt);
        vsprintf(string, fmt, args);

	/* Create the message */

	message.type = MESSAGE;
	message.length = strlen(string);
	message.data = (unsigned char *)string;

	/* Send the message */

	return ui_send_message_all(&message);

} /* ui_print_message_all() */

/************************************************************************/
/* send_server_stopping()                                               */
/*                                                                      */
/* Send a SERVER_STOPPING mesage to all connected machines.             */
/* Returns error level.                                                 */
/************************************************************************/

int send_server_stopping(void)
{
        message_struct mess;

        mess.type = SERVER_STOPPING;
        mess.length = 0;
        mess.data = NULL;

        /* Send the message */

        return ui_send_message_all(&mess);

} /* send_server_stopping() */

/************************************************************************/
/* ui_print_log()							*/
/* 									*/
/* Works like a printf only sends the string as a message of type	*/
/* CENTRAL_SCRUTINIZER_LOG to the all sockets.				*/
/* It is likely only the central scutinizer itself will do anything	*/
/* with the result.							*/
/* Returns error level.							*/
/************************************************************************/

int ui_print_log(char *fmt, ...)
{
	message_struct message;
        char    string[9999];
        va_list args;

        /* Create the string */

        va_start(args, fmt);
        vsprintf(string, fmt, args);

	/* Create the message */

	message.type = CENTRAL_SCRUTINIZER_LOG;
	message.length = strlen(string);
	message.data = (unsigned char *)string;

	/* Send the message */

	return ui_send_message_all(&message);

} /* ui_print_message_all() */

/************************************************************************/
/* get_server_port()                                                    */
/*                                                                      */
/* Returns the server port.                                             */
/************************************************************************/

int get_server_port()
{
        return server_port;

} /* get_server_port() */
