/************************************************************************/
/* startui.c								*/
/*                                                                      */
/* Starts User Interafce.						*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Tony Johnson & Theo ten Brummelaar                          */
/* Date   : Original Version 1990 - ported to Linux 1998                */
/************************************************************************/

/*
 * Include files
 */

#include "charaui.h"
#include "../../charalib/chara.h"
#include "../../../cliserv/socket_manager/server/socket_manager.h"
#include "../../astromod/astromod.h"

static struct smenu *mainmenu;	/* Pointer to main window */
static void (*user_open_function)(void) = NULL;
char	sockman_name[256];

/************************************************************************/
/* start_ui()								*/
/*									*/
/* Sets up and starts user interface.					*/
/*									*/
/************************************************************************/

void start_ui(void)
{
	signal( SIGINT, sig_end);	/* Catch external kill signal */

        /* Make sure we catch and SIGPIPE signals */

        if (signal(SIGPIPE, sig_pipe) == SIG_ERR)
        {
                error(ERROR,"Failed to grab SIGPIPE signal");
        }

        /* Is there a user defined starting function call ? */

        if ( user_open_function != NULL) (*user_open_function)();

	menu(mainmenu, (struct smenu *)NULL, 0);	/* Start menu system */

	/* Note: Exit of the programme is via ui_end() */

} /* start_ui() */

/************************************************************************/
/* initialise_ui()							*/
/*									*/
/* This function will initialise everything that needs to be 		*/
/* initialised prior to putting up the main menu.			*/
/* Consult design and source for more information.			*/
/* The parameter menufile is the file name (full path) of the menu	*/
/* description file, helpfile is the same for help.			*/
/* The port is the port used for the command socket.  			*/
/* NOTE: This calls init_jobs() so it should never be called again,	*/
/* and this function should be caleld before opening any other server	*/
/* connections as a client.						*/
/*									*/
/************************************************************************/

void 	initialise_ui(char *menufile, char *helpfile, char *server_name)
{
	int	com_port, data_port, server_port;
	char	machine[256];
	char	filename[256];
	char	hostname[256];
	char	s[512];
	FILE	*output;
	struct  s_process_sockets process;

	*sockman_name = 0; /* We have no socket name yet. */

	init_curses();  /* Initialise the curses system */

	working();	/* Message to user */

	/* Now we check with the process manager */

	init_jobs();
        switch (socket_manager_get_process(server_name, &process))
        {
            case 0:

                /* Yes, the process already exists, ping it */

                switch(socket_manager_ping_process(process))
                {
                    case 0: error(FATAL,
                           "Live process %s already running on %s:%d.\n",
                            process.name, process.machine,
                            process.messages);
                            break;

                    default: error(FATAL,
"Socket manager reports dead process %s on %s.\nTry removing this entry using sockman client.\n",
                            process.name, process.machine);
                            break;
                }
                break;

                /* Other error condiftions */

            case 1: error(FATAL,"Socket Manager dead!\n");
                break;

            case 2: error(FATAL, 
                        "Failed to speak with socket manager.\n");
                break;

            case 4: error(FATAL, 
                        "Names do not match with socket manager.\n");
                break;

                /* Any other response means all is OK to continue */

            default: break;
        }

	/* 
	 * Does this program already seem to be running? 
	 * This is here for backards compat with older clients.
	 */

	sprintf(filename, "%s%s.port", ETC_DIRECTORY, server_name);
	if (read_server_ports(server_name, machine, &com_port,
				&data_port, &server_port))
	{
		/* There's a file there..... is there someone listening? */

		if (send_socket_command(machine, com_port, "ping") == NOERROR)
		{
			error(FATAL,
			    "%s already running on %s.\n",server_name,machine);
		}

		sprintf(s,
			"%s already running on %s.\n",server_name,machine);
		strcat(s,"It is not responding to PING.\n");
		sprintf(machine,
			"Please remove file %s if all is OK.\n", filename);
		strcat(s, machine);
		error(FATAL,s);
	}

	init_auto();	/* Initialise the auto function caller */

	init_display();	/* Initialise the display structure */

	if (open_command_socket() != NOERROR)
	{
		error(WARNING,"No socket command functionality.");
	}

	if (open_data_socket() != NOERROR)
	{
		error(WARNING,"No data socket functionality.");
	}

        /* Set up the client port */

        if (open_message_socket() != NOERROR)
        {
                error(ERROR,
                    "Failed to open client port - no client functionality.");
        }
        set_server_name(server_name);
        standard_jobs();

	init_help(helpfile);	/* after init_curses !!!! */

	init_scripts();

	mainmenu = init_menu_structure(menufile); /* Parse data in menus file */

	message(system_window,
		"Command port = %d Data port = %d Message port = %d", 
		get_command_port(), get_data_port(), get_server_port());

	/* 
	 * Write the ports file 
	 * This is for backwards compatibility for those older
	 * clients who do not know about the socket manager.
	 */

	if ((output = fopen(filename,"w")) == NULL)
	{
		error(FATAL,"Failed to open ports file.\n");
	}
	fprintf(output, "%s %d %d %d\n", get_machine_name(hostname), 
		get_command_port(), get_data_port(), get_server_port());
	fclose(output);

	/* Now register with the real socket manager */

	strcpy(process.name, server_name);
	strcpy(process.machine, get_machine_name(hostname));
	process.pid = getpid();
	process.commands = get_command_port();
	process.data = get_data_port();
	process.messages = get_server_port();
	*process.restart_command = 0;

	init_jobs();
	switch(socket_manager_set_process(process))
	{
		
                case 1: unlink(filename);
			error(FATAL,"Failed to talk to socket manager.");
                        break;

                case 2: unlink(filename);
			error(FATAL,
                                "Failed to send message to socket manager.");
                        break;

                case 3: unlink(filename);
			error(FATAL, "Process %s already exists!.", 
				server_name);
                        break;

                case 4: unlink(filename);
			error(FATAL,
                                "Failed to match names with socket manager.");
                        break;

                case 5: unlink(filename);
			error(FATAL,
                                "Machine name %s not allowed.",process.machine);
                        break;

                case 6: unlink(filename);
			error(FATAL, "Restart command more than one line.");
                        break;

                case 7: unlink(filename);
			error(FATAL,"Name, machine or PID is missin.");
                        break;

                default: break;

	}
	strcpy(sockman_name, server_name);

	not_working();	/* Message to user */

} /* initialise_ui() */

/************************************************************************/
/* set_user_open_function()                                             */
/*                                                                      */
/* Setup a function for the system to call when started. Good for       */
/* proper setup of things.                                              */
/************************************************************************/

void set_user_open_function(void (*function)(void))
{
        user_open_function = function;

} /* set_user_open_function() */

