/************************************************************************/
/* status_window.c 	                                                */
/*                                                                      */
/* Displays the UI status in a text window.				*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE                        */
/*                 Based on the SUSI User Interface                     */
/*              In turn based on the CHIP User interface                */
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA                 */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu                                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/* Author : Theo ten Brummelaar                                         */
/* Date   : April 2002                                                  */
/************************************************************************/
#define USE_GTK
#include "../include/cliserv.h"
#include "../../../cliserv/include/chara_messages.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <chara.h>
#include <slalib.h>

/* Size of label for a single character */

#define LABEL_Y	14
#define LABEL_X	9

/* Globals */

static GtkWidget *status_window;
static GtkWidget *status_labels[STATUS_LINES][STATUS_COLS];
static char status_window_up = FALSE;
static guint status_background_id;
static struct s_status *status;

/************************************************************************/
/* create_status_window()						*/
/*									*/
/* Routine for creating the clock control window. This should be self	*/
/* contained and give you all you need to control the basic clock 	*/
/* functionality. Argumentis the window name.				*/
/* The data argument is a name to put on the window.			*/
/************************************************************************/

GtkWidget* create_status_window (GtkButton *button, gpointer *data)
{
	GtkWidget *ok_button;
	GtkWidget *fixed;
	int	i,j;
	struct s_status_window *info;

	/* Do nothing if we exist already */

	if (status_window_up) return status_window;

	/* Get the information */

	info = (struct s_status_window *)data;

	/* Create a new window */

	status_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (status_window), info->name);
	gtk_container_set_border_width (GTK_CONTAINER (status_window), 1);

	/* Create a fixed area to put things in */

	fixed = gtk_fixed_new ();
	gtk_widget_ref (fixed);
	gtk_object_set_data_full (GTK_OBJECT (status_window), "fixed", fixed,
                            (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (fixed);
	gtk_container_add (GTK_CONTAINER (status_window), fixed);

        /* Add labels for text */

	for(j=0; j < STATUS_LINES; j++)
	for(i=0; i < STATUS_COLS; i++)
	{
        	status_labels[j][i] = gtk_label_new(" ");
		gtk_widget_ref(status_labels[j][i]);
		gtk_object_set_data_full (GTK_OBJECT (status_window), "X", 
			status_labels[j][i], (GtkDestroyNotify) 
			gtk_widget_unref);
		gtk_widget_show (status_labels[j][i]);
		gtk_fixed_put (GTK_FIXED (fixed), status_labels[j][i], 
			i*LABEL_X, j*LABEL_Y);
		gtk_widget_set_uposition (status_labels[j][i],
			i*LABEL_X, j*LABEL_Y);
		gtk_widget_set_usize (status_labels[j][i], 
			LABEL_X, LABEL_Y);
	}

	/* OK button */

	ok_button = gtk_button_new_with_label ("OK");
	gtk_signal_connect(GTK_OBJECT (ok_button), "clicked",
	                    GTK_SIGNAL_FUNC (status_ok_callback), NULL);
	gtk_widget_ref(ok_button);
	gtk_widget_show (ok_button);
	gtk_fixed_put (GTK_FIXED (fixed), ok_button, 0, j*LABEL_Y);
	gtk_widget_set_uposition (ok_button, 0, j*LABEL_Y);
	gtk_widget_set_usize (ok_button, STATUS_COLS*LABEL_X, 1.5*LABEL_Y);

	/* Now, we can show this window if we like */

	gtk_widget_show(status_window);

	/* Make sure we only get one version of this window */

	status_window_up = TRUE;

	/* Start routine for background processing */

	status_background_id = gtk_timeout_add(1000,
				status_background, data);

	/* Run for the first time */

	status_background(data);

	/* That should be all */

	return status_window;

} /* create_status_window() */

/************************************************************************/
/* status_ok_callback()							*/
/*									*/
/* Callback function for clock control OK button. Stop background	*/
/* processing and destroy clock control window.				*/
/************************************************************************/

void status_ok_callback(GtkButton *button, gpointer user_data)
{
	if (!status_window_up) return;

	gtk_widget_destroy(status_window);
	gtk_timeout_remove(status_background_id);
	status_window_up = FALSE;

} /* status_ok_callback() */

/************************************************************************/
/* status_background()							*/
/*									*/
/* Here's the background job for looking at clock outputs and so on.	*/
/************************************************************************/

gint status_background(gpointer data)
{
        struct smessage mess;
	struct s_status_window *info;

	/* Get the information */

	info = (struct s_status_window *)data;

	/* Send message requesting status information */

	mess.type = UI_STATUS;
	mess.length = 0;
	mess.data = NULL;

	if (!send_message(info->server, &mess))
	{
                create_message_window("Failed to send STATUS message.");
		status_ok_callback(NULL, NULL);
	}

	return TRUE;

} /* status_background() */

/************************************************************************/
/* message_status()							*/
/*									*/
/* Reacts to a STATUS message comming from the server. Does nothing	*/
/* if we're not in status mode. Display text if we are.			*/
/************************************************************************/

int message_status(int server, struct smessage *mess)
{
	int 	i,j;
	char	s[2];

	/* If the winodw isn't open do nothing */

	if (!status_window_up) return NOERROR;
	
	/* Otherwise check that all is OK with the data */

	if (mess->length != sizeof(*status))
	{
		create_message_window(
			"Received STATUS message with wrong data.");
		status_ok_callback(NULL, NULL);
		return NOERROR;
	}

	status = (struct s_status *) mess->data;

	/* Put the text in place */

	s[1] = 0;
        for (j=0; j<STATUS_LINES ; j++)
        for (i=0; i<STATUS_COLS ; i++)
        {
		s[0] = status->line[j][i];
		gtk_label_set_text((GtkLabel *)status_labels[j][i], s);
        }

	return NOERROR;

} /* message_status() */
