/************************************************************************/
/* socket.c								*/
/*                                                                      */
/* Routines for using a socket to accept outside commands.  		*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY SERVER LIB				*/
/*		   Based on the CHARA User Interface			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar 		                        */
/* Date   : October 2001						*/
/************************************************************************/

/* This is part of the server */

/* First we include everything as a client */

#include "../include/cliserv.h"
#undef __CLISERV__

/* Now as a SERVER */

#define SERVER

/*
 * Include files
 */

#include "../include/cliserv.h"
#include "../../../cliserv/include/chara_messages.h"
#include "../../../cliserv/socket_manager/server/socket_manager.h"
#include <netdb.h>
#include <resolv.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <signal.h>

/* Define this for no networked machines */

/* #define STANDALONE */

/* 
 * Local globals
 */

static int	message_socket = -1;	/* socket used for outside messages */
static struct {
		int  fd;		/* File descriptor */     
		char *machine; 	/* Name of connecting machine */
			} sockets[MAX_CONNECTIONS]; /* Active channels */
static int	message_type_backlog[MAX_CONNECTIONS];
static char	myname[81];		/* This machine's name */
static int	myport;			/* The port we are listening too */
static int	curr_sock = 0;		/* Current socket to check */
static int current_socket = -1;
static struct s_process_sockets process;

/* Globals */

int active_socket = -1;                 /*
                                         * The current active socket.
                                         * Set to -1 if no active socket.
                                         */

char *active_machine = "";              /*
                                         * The current active machine.
                                         * Set to 0 if no active machine.
                                         */

bool use_type_backlog = FALSE;
#warning THERE IS A PROBLEM WITH THE BACKLOG MECHANISM IT ASSUMES ONLY ONE

/************************************************************************/
/* auto_open_message_socket()						*/
/* 									*/
/* For trying to open a socket and automatically choose, and register	*/
/* with socket_manager, all the information conserning this device.	*/
/* Returns error level.							*/
/************************************************************************/

int auto_open_message_socket(char *name, char *restart_command)
{
	int	port;

	/* Make sure we have a name */

	if (strlen(name) == 0 || name == NULL) 
	{
		return error(FATAL,"Process has no name.\n");
	}

	/* First we need to set ourselves up as a client for a while */

	init_jobs();

	/* See if there is already something by this name? */

	switch (socket_manager_get_process(name, &process))
	{
	    case 0:

		/* Yes, the process already exists, ping it */

		switch(socket_manager_ping_process(process))
		{
		    case 0: return error(ERROR,
			   "Live process %s already running on %s:%d PID %d.\n",
			    process.name, process.machine,
			    process.messages, process.pid);
		    	    break;

		    default: error(ERROR,
		   "Socket manager reports dead process %s on %s:%d PID %d.\n",
			    process.name, process.machine,
			    process.messages, process.pid);
			    return error(ERROR, 
			    "Try removing this entry using sockman client.\n");
			    break;
		}
		break;

		/* Other error condiftions */

	    case 1: return error(ERROR,"Socket Manager dead!\n");
		break;

	    case 2: return error(ERROR, 
			"Failed to speak with socket manager.\n");
		break;

	    case 4: return error(ERROR, 
			"Names do not match with socket manager.\n");
		break;

		/* Any other response means all is OK to continue */

	    default: break;
	}

	/* OK, try and find an open port */

	for(port = DEFAULT_SERVER_PORT;
	    port < DEFAULT_SERVER_PORT + 1000 &&
	    open_message_socket(port) != NOERROR;
	    port++);

	/* Did we get one? */

	if (port >= DEFAULT_SERVER_PORT + 1000)
	{
		return error(ERROR,"Failed to find port to use.\n");
	}

	/* All is well, now fill out the structure */

	strcpy(process.name, name);
	if (gethostname(process.machine, 80) < 0)
        {
                return error(ERROR,"Failed to get host name.\n");
        }
	process.pid = getpid();
	process.commands = -1;
	process.data = -1;
	process.messages = port;
	strcpy(process.restart_command, restart_command);

	/* And register it with the socket manager */

	switch(socket_manager_set_process(process))
	{
		case 1: return error(ERROR,
				"Failed to talk to socket manager.");
			break;

		case 2: return error(ERROR,
				"Failed to send message to socket manager.");
			break;

		case 3: return error(ERROR,
				"Process %s already exists!.", name);
			break;

		case 4: return error(ERROR,
				"Failed to match names with socket manager.");
			break;

		case 5: return error(ERROR,
				"Machine name %s not allowed.",process.machine);
			break;

		case 6: return error(ERROR,
				"Restart command more than one line.");
			break;

		case 7: return error(ERROR,
				"Name, machine or PID is missing.");
			break;

		default: break;
	}

	/* All seems to be well */

	return NOERROR;

} /* auto_open_message_socket() */

/************************************************************************/
/* open_message_socket()						*/
/* 									*/
/* Opens a message socket that can be used for accepting and sending	*/
/* outside messages. Accepts the port number to be used and buffer 	*/
/* sizes as an arguments. 						*/
/*									*/
/* Returns error level.							*/
/*									*/
/* This routine is set up so that it will accept a connection from 	*/
/* anyone in our domain, so be very carefull about this.		*/
/************************************************************************/

int open_message_socket(int port)
{
        struct sockaddr_in server; /* Server address */
	int	i;
        struct hostent *hp;     	/* Pointer to host data */
	int	buffer_size;
	char	name[81];		

	/* Has this already been done? */

	if (message_socket != -1)
	{
		return error(ERROR,"Command socket already exists.");
	}

	/* Setup a default name */

	strcpy(myname,"Unknown");

	/* Clear the current sockets */

	active_socket = -1;
	active_machine = NULL;
	for(i=0; i< MAX_CONNECTIONS; i++)
	{
		sockets[i].fd = -1;
		message_type_backlog[i] = -1;
	}

        /* Create a socket */

        if ((message_socket = socket(AF_INET, SOCK_STREAM, 0)) < 0)
        {
		message_socket = -1;
                return error(ERROR,"Failed to open message socket.");
        }

        /* Force buffer sizes to buffer_size */

	buffer_size = SERVER_SOCKET_BUFFER;
        if (setsockopt(message_socket, SOL_SOCKET, SO_SNDBUF, 
                (char *)&buffer_size, sizeof(buffer_size)) < 0)
        {
                close(message_socket);
		message_socket = -1;
                return error(ERROR,
			"Failed to change message socket write buffer size.");
        }

        if (setsockopt(message_socket, SOL_SOCKET, SO_RCVBUF, 
                (char *)&buffer_size, sizeof(buffer_size)) < 0)
        {
                close(message_socket);
		message_socket = -1;
                return error(ERROR,
			"Failed to change message socket read buffer size.");
        }

        /* Set up server address */

        server.sin_family = AF_INET;
        server.sin_addr.s_addr = htonl(INADDR_ANY);
        server.sin_port = htons(port);

        /* Now bind the socket to this address */

        if (bind(message_socket,(struct sockaddr *)&server,sizeof(server)) < 0)
        {
                close(message_socket);
		message_socket = -1;
                return error(ERROR,"Failed to bind socket.");
        }

	/* Create a queue for client connection requests */

        if (listen(message_socket, 0) < 0)
        {
                close(message_socket);
		message_socket = -1;
                return error(ERROR,"Failed to listen to socket");
        }

	/* Find out our own name... */

        if(gethostname(name,81) < 0 || (hp = gethostbyname(name)) == 0)
        {
                close(message_socket);
		message_socket = -1;
                return error(ERROR,"Can't identify ourselves!.");
        }
	strcpy(myname,hp->h_name);
	myport = port;

	/* Reset current socket pointer. */

	curr_sock = 0;

        /* Make sure we catch and SIGPIPE signals */

        if (signal(SIGPIPE, sig_pipe) == SIG_ERR)
        {
                error(ERROR,"Failed to grab SIGPIPE signal");
        }

	/* Set up the signals */

	setup_signals();

#ifdef STANDALONE
        error(MESSAGE,"Started stand alone message sockets.");
#else
	error(MESSAGE,"Started message sockets on port %d.", myport);
#endif

	/* Fill out a dumby process */

	*process.name = 
	*process.machine = 
	*process.restart_command = 0;
	process.pid = process.commands =
	process.data = process.messages = -1;
	
	/* Let people know if we're using the type backlog */

	if (use_type_backlog)
		error(MESSAGE,"Using TYPE BACKLOG.");
	else
		error(MESSAGE,"Not using TYPE BACKLOG.");

	/* It all seems to be working */

	return NOERROR;

} /* open_message_socket() */

/************************************************************************/
/* set_process_name()							*/
/* 									*/
/* Let's you set the process name, but only if it isn't already sey.	*/
/************************************************************************/

void set_process_name(char *name)
{
	if (*process.name == 0) strcpy(process.name, name);

} /* set_process_name() */

/************************************************************************/
/* close_message_socket()						*/
/* 									*/
/* Close the message socket if it's open. Remember this could cause     */
/* trouble if you do this while the channel is active. If the message	*/
/* socket is not open this function will do nothing.			*/
/*									*/
/************************************************************************/

void close_message_socket(void)
{
	int i;

	/* Do nothing if it isn't open */

	if (message_socket == -1) return;

	/* Clear the current sockets */

	active_socket = -1;
	active_machine = NULL;
	for(i=0; i< MAX_CONNECTIONS; i++) 
	{
		if (sockets[i].fd != -1) 
		{
			close(sockets[i].fd);
			sockets[i].fd = -1;
			free(sockets[i].machine);
		}
	}

	/* Close it */

	close(message_socket);

	/* Reset things */

	message_socket = -1;
	myport = -1;
        signal(SIGPIPE, SIG_DFL);

	/* Tell the socket manager we have gone now */

	if (*process.name != 0)
	{
		switch(socket_manager_remove_process(process.name))
		{
			case 1: error(ERROR, 
				"Failed to speak to socket_manager.");
				break;

			case 2: error(ERROR, 
				"Failed to send message to socket_manager.");
				break;

			case 3: error(ERROR, 
				"Socket_manager did not know about %s.",
					process.name);
				break;

			case 4: error(ERROR, 
				"Failed to match name with socket_manager.");
				break;

			default: break;
		}
	}

	/* Blank out globals */

	*process.name = 
	*process.machine = 
	*process.restart_command = 0;
	process.pid = process.commands =
	process.data = process.messages = -1;
	
	/* That should be all */

} /* close_message_socket() */

/************************************************************************/
/* process_message_socket() 						*/
/*									*/
/* Checks through the array of sockets and see's if any have become	*/
/* domant. If any have it releases them. Checks the message_socket	*/
/* to see if there are any new connects. Adds them to the array of	*/
/* sockets if there are. Checks through the sockets array to see	*/
/* if any new messages have arrived. If so, set's active_socket to	*/
/* that socket, runs the message and returns a message to the socket	*/
/* concerning success or failure.					*/
/*									*/
/* Returns the error level.						*/
/************************************************************************/

int	process_message_socket(void)
{
	int	i;			/* Looping */
        struct sockaddr_in connector; 	/* Connector's address */
	unsigned int	len;			/* Lenght of address */
#ifndef STANDALONE
        struct hostent *hp;     	/* Pointer to host data */
	char	*domain;		/* Pointer to domain in name */
#endif
	message_struct  message;	/* The message */

	/* 
	 * If there is no message_socket or if there is an active_socket
	 * do nothing.
	 */

	if (message_socket == -1 || active_socket != -1) return NOERROR;

	/* 
	 * OK, let's loop look at the current socket 
	 * and see if any are dormant or have any data in them to read.
	 */

	/* Let's find the next socket to process */

	for(i = curr_sock+1; i < MAX_CONNECTIONS && sockets[i].fd == -1; i++);
	if (i >= MAX_CONNECTIONS)
	{
		for(i = 0; i < curr_sock && sockets[i].fd == -1; i++);
	}
	curr_sock = i;

	/* If there is a message waiting do something about it */

	if (sockets[i].fd != -1 && char_waiting(sockets[i].fd))
	{
		if (use_type_backlog && message_type_backlog[i] != -1)
		{
			message.type = message_type_backlog[i];
			message_type_backlog[i] = -1;
		}
		else
		{
			/* Read in message type */

			len = read_socket(sockets[i].fd,(char *)&message.type,
				sizeof(message.type));

			if (len <= 0)
			{
			    /* Read failed, connection has died */

			    sockets[i].fd = -1;
			    error(MESSAGE, "%s just logged off.",
					sockets[i].machine);
			    free(sockets[i].machine);
			    return MESSAGE;
			}
			if (len != sizeof(message.type))
			{
			    return error(ERROR,"Failed to read message type.");
			}
		}

		/* read in data size */

		len=read_socket(sockets[i].fd,(char *)&message.length,
			sizeof(message.length));
		if (len <= 0)
		{
			/* Read failed, connection has died */

			sockets[i].fd = -1;
			error(MESSAGE,
				"%s just logged off.\n",
				sockets[i].machine);
			free(sockets[i].machine);
			return MESSAGE;
		}
		else if (len != sizeof(message.length))
		{
		    return error(ERROR,
			"Failed to read message length for message %d",
			message.type);
		}

		/* Allocate memory for message data */

		if (message.length == 0)
		{
			message.data = NULL;
		}
		else
		{
		    if ((message.data = malloc(message.length)) == NULL)
		    {
			error(FATAL,"Ran out of memory processing message %d.",
				message.type);
		    }

		    /* Read in the data */

		    len = read_socket(sockets[i].fd, (char *)message.data, 
				message.length);
		    if (len <= 0)
		    {
			/* Read failed, connection has died */

			sockets[i].fd = -1;
			error(MESSAGE,
				"%s just logged off.\n",
				sockets[i].machine);
			free(sockets[i].machine);
			return MESSAGE;
		    }
		    else if (len != message.length)
		    {
		   	 return error(ERROR,
			    "Got only %d bytes out of %d for message %d",
			    len, message.length,
			    message.type);
		    }
		}

		/* This becomes the active socket */

		active_socket = sockets[i].fd;
		active_machine = sockets[i].machine;

		/* Process the message type */

		server_run_message_job(&message);

		/* OK, we're finished with this socket for now */
	
		active_socket = -1;
		active_machine = NULL;

		/* Free memory for message data */

		if (message.data != NULL) free(message.data);
	}

	/* See if someone wants a new connection */

	if (char_waiting(message_socket))
	{
		/* Find an empty slot for this new socket */

		for(i=0; i<MAX_CONNECTIONS && sockets[i].fd != -1; i++);

		if (i < MAX_CONNECTIONS)  /* We found one */
		{
			len = sizeof(connector);
                        if ((sockets[i].fd = accept(message_socket,
				(struct sockaddr *)&connector,&len)) < 0)
                        {
                                error(ERROR,
				      "Failed to accept socket connection.\n");
                        }

			/* OK, extract the name */

#ifndef STANDALONE
			if ((hp = gethostbyaddr((char *)&connector.sin_addr,
                                        len,connector.sin_family)) == NULL)
			{
				error(ERROR,
				"Failed to identify host. Blocking socket.");
				switch(h_errno)
				{
				    case HOST_NOT_FOUND:
					error(ERROR,"ERROR: Host not found.");
					break;

				    case NO_ADDRESS:
					error(ERROR,"ERROR: No address.");
					break;

				    case NO_RECOVERY:
					error(ERROR,"ERROR: No recovery.");
					break;

				    case TRY_AGAIN:
					error(ERROR,"ERROR: Try again.");
					break;

				    default:
					error(ERROR,"ERROR: Unknown.");
					break;
				}
				close(sockets[i].fd);
				sockets[i].fd = -1;
			}
			else
			{
				/* Put the name of the machine in structure */

				if ((sockets[i].machine = 
					malloc(strlen(hp->h_name)+1)) == NULL)
				{
					error(FATAL,
				"Ran out of memory during a socket connection");
				}
				strcpy(sockets[i].machine,hp->h_name);

				/* Separrate out it's domain part. */

				for(domain=hp->h_name; *domain != 0 &&
					*domain != '.'; domain++);
				if (*domain == '.') ++domain;

				/* Is this OK ? */

				if (strcmp(myname,hp->h_name) && 
				    strcmp(domain,ALLOWED_DOMAIN))
				{
					close(sockets[i].fd);
					sockets[i].fd = -1;
					free(sockets[i].machine);
					error(ERROR,
			"Host %s not in correct domain. Blocking socket.", 
					hp->h_name);
				}
				else 
				{
				    error(MESSAGE,
				        "%s just connected.",
					hp->h_name);
/* This may be the problem with sockman. It sometimes forces it to
				try and talk to dead sockets.
				    print_log(
                                        "%s accepted connection from %s.",
                                        myname, hp->h_name);
*/
                                }

			}
#else
                        if ((sockets[i].machine =
                                malloc(strlen("localhost")+1)) == NULL)
                        {
                                error(FATAL,
                                "Ran out of memory during a socket connection");
                        }
                        strcpy(sockets[i].machine,"localhost");

                        error(MESSAGE, "%s just connected.", "localhost");
/* This may be the problem with sockman. It sometimes forces it to
				try and talk to dead sockets.
			print_log("%s accepted connection from %s.",
                                myname, "localhost");
*/
#endif

		}
	}

	return NOERROR;

} /* process_message_socket() */

/************************************************************************/
/* get_server_name()							*/
/*									*/
/* Get the name of this machine. Expects the string sent to have memory */
/* already allocated. Returns the string sent as an argument.		*/
/* Note that if the socket message channel is not open yet you'll	*/
/* get garbage in the string.						*/
/************************************************************************/

char *get_server_name(char *name)
{

	strcpy(name, myname);
	return name;

} /* get_server_name() */

/************************************************************************/
/* flush_socket()							*/
/*									*/
/* Flush a single socket. Will not close it if there's nothing there.	*/
/************************************************************************/

void flush_socket(int sock)
{
	int	count,closed;
	char	c;

	closed = 0;
	count=0;

	if (sock == -1) return;

	while (char_waiting(sock)) 
	{
		if (read(sock, &c, 1) == 1) 
			count++;
		else
			break;
	}

	if (count > 0)
	{
		error(MESSAGE,"Flushed %d bytes.", count);
	}

} /* flush_socket() */

/************************************************************************/
/* flush_sockets()							*/
/*									*/
/* Goes through all message sockets and empties them of any text.	*/
/* Will also close them out if required. This function should be called */
/* whenever you stop listening to the sockets.				*/
/************************************************************************/

void flush_sockets(void)
{
	int	i;
	int	count,closed;
	char	c;

	for(closed = 0, count=0, i=0; i<MAX_CONNECTIONS; i++)
	{
		if (sockets[i].fd == -1) continue;

		while (char_waiting(sockets[i].fd))
		{
			if (read(sockets[i].fd, &c, 1) <= 0)
			{
				/* Read failed, connection has died */

				closed++;
				sockets[i].fd = -1;
				free(sockets[i].machine);
				break;
			}
			count++;
		}
	}

	if (count + closed > 0)
	{
		error(MESSAGE,"Flushed %d bytes and closed %d sockets.",
			count, closed);
	}

} /* flush_sockets() */

/************************************************************************/
/* server_send_message()						*/
/*									*/
/* Sends a message to a given socket. Returns error level.		*/
/************************************************************************/

int server_send_message(int sock, struct smessage *message)
{
	int	i;
	int	socket_index;
	int	len;
	unsigned short int	type;

	/* Never try to talk to a non-existant socket */

	if (sock <= 0) return NOERROR;

	/* Which socket index is this? */

	for(i=0; i< MAX_CONNECTIONS && sockets[i].fd != sock; i++);
	if (i < MAX_CONNECTIONS)
		socket_index = i;
	else
		socket_index = -1;

	/* 
         * There is a small chance that this socket has gone away.
         * For this reason we now check and do nothing if it has.
         */

        if (!write_ready(sock) || exception(sock)) 
        {
		/* We must remove this socket from the list of connections */

                sockets[socket_index].fd = -1;
                free(sockets[socket_index].machine);
                return error(MESSAGE,
			"Attempted to write to non-existant socket.");
        }

        /* Set the current socket to be the one we're dealing with */

        current_socket = sock;

	/* Has the socket client gone away? */

	if (use_type_backlog && sock != -1 && char_waiting(sock))
        {
                /* Read in message type */

                len = read_socket(sock,(char *)&type, sizeof(type));

                if (len <= 0)
                {
                        /* Read failed, connection has died */

                        sockets[socket_index].fd = -1;
                        free(sockets[socket_index].machine);
                        return error(MESSAGE,
			"A client died just as I was writing the type to it.");
                }
		else if (len == 2)
		{
			message_type_backlog[socket_index] = type;
		}
		else
			error(MESSAGE,"Strange data in socket.");
	}

        /* Send the type */

	i = sizeof(message->type);
	if (write(sock, &(message->type), i) != i)
	{
		for(i=0; i< MAX_CONNECTIONS && sockets[i].fd != sock; i++);
		if (i < MAX_CONNECTIONS)
		{
                        sockets[i].fd = -1;
                        free(sockets[i].machine);
		}
		return error(ERROR,
			"Failed to write message type (%d).",
				message->type);
	}

	/* Has the socket client gone away? */

	if (use_type_backlog && sock != -1 && char_waiting(sock))
        {
                /* Read in message type */

                len = read_socket(sock,(char *)&type, sizeof(type));

                if (len <= 0)
                {
                        /* Read failed, connection has died */

                        sockets[socket_index].fd = -1;
                        free(sockets[socket_index].machine);
                        return error(MESSAGE,
			"A client died just as I was writing the length to it.");
                }
		else if (len == 2)
		{
			message_type_backlog[socket_index] = type;
		}
		else
			error(MESSAGE,"Strange data in socket.");
	}

	/* Send the length */

	i = sizeof(message->length);
	if (write(sock, &(message->length), i) != i)
	{
		for(i=0; i< MAX_CONNECTIONS && sockets[i].fd != sock; i++);
		if (i < MAX_CONNECTIONS)
		{
                        sockets[i].fd = -1;
                        free(sockets[i].machine);
		}
		return error(ERROR,
			"Failed to write message length (%d).",
				message->length);
	}

	/* Has the socket client gone away? */

	if (use_type_backlog && sock != -1 && char_waiting(sock))
        {
                /* Read in message type */

                len = read_socket(sock,(char *)&type, sizeof(type));

                if (len <= 0)
                {
                        /* Read failed, connection has died */

                        sockets[socket_index].fd = -1;
                        free(sockets[socket_index].machine);
                        return error(MESSAGE,
			"A client died just as I was writing the data to it.");
                }
		else if (len == 2)
		{
			message_type_backlog[socket_index] = type;
		}
		else
			error(MESSAGE,"Strange data in socket.");
	}

	/* Send the data if there is any */

	if (message->length > 0 &&
		write(sock, message->data, message->length) != message->length)
	{
		for(i=0; i< MAX_CONNECTIONS && sockets[i].fd != sock; i++);
		if (i < MAX_CONNECTIONS)
		{
                        sockets[i].fd = -1;
                        free(sockets[i].machine);
		}
		return error(ERROR, "Failed to write message (%d) data.",
				message->type);
	}

        /* Set current socket back to -1 */

        current_socket = -1;

	return NOERROR;

} /* server_send_message() */

/************************************************************************/
/* server_send_message_all()						*/
/*									*/
/* Sends the same message to all connected sockets.			*/
/* Returns error level.							*/
/************************************************************************/

int server_send_message_all(struct smessage *message)
{
	int 	i;
	int	failed;

	for(failed = FALSE, i=0; !failed && i<MAX_CONNECTIONS; i++)
	{
	    if (sockets[i].fd != -1) 
	    {
		if (server_send_message(sockets[i].fd, message) != NOERROR)
		{
			failed = TRUE;
		}
	    }
	}

	if (failed) 
		return ERROR;
	else
		return NOERROR;

} /* server_send_message_all(); */

/************************************************************************/
/* sig_pipe()                                                           */
/*                                                                      */
/* For catching the SIGPIPE signal. Will see if it was caused by one    */
/* of our clients and remove that client it was.                        */
/************************************************************************/

void    sig_pipe(int sig)
{
        int     i;

        /* is the current socket one of our clients? */

        if (current_socket == -1)
        {
                error(FATAL,"Caught SIGPIPE with no current client socket.");
        }

        for(i = 0; i < MAX_CONNECTIONS; i++)
        {
                if (sockets[i].fd == -1) continue;

                if (sockets[i].fd == current_socket)
                {
                        sockets[i].fd = -1;
                        error(MESSAGE,
                                "%s caused SIGPIPE. Logged off.",
                                sockets[i].machine);
                        free(sockets[i].machine);
                }
        }

} /* sig_pipe() */

/************************************************************************/
/* read_socket()                                                        */
/*                                                                      */
/* Tries to read a given number of bytes into a given location from     */
/* a socket. Will keep trying to read until it get's all the bytes      */
/* it expects or it times out. Returns number of bytes recieved.        */
/************************************************************************/

int read_socket(int fd, char *buf, size_t count)
{
        time_t  start;
        int     n,len;

        start = time(NULL);
        len = 0;
        while(count > 0)
        {
                if (time(NULL) > start + TIME_OUT) break;

#warning This used to be just < and it caused big delays
                if((n = read(fd, buf, count)) <= 0) return n;

                len += n;
                buf += n;
                count -= n;
        }

        return len;

} /* read_socket() */
