/************************************************************************/
/* uiclient_standard.c							*/
/*                                                                      */
/* Routines to respond to standard messages from server.		*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar			                        */
/* Date   : Wed Oct 10 03:42:20 PDT 2001				*/
/************************************************************************/

#include <charaui.h>
#include "../include/cliserv.h"
#include "../../../cliserv/include/chara_messages.h"
#include "uiclient.h"

/************************************************************************/
/* set_up_standard_jobs()						*/
/*									*/
/* Puts functions to deal with standard messages into message que.	*/
/* Returns TRUE if all goes well, FALSE otherwise.			*/
/*									*/
/************************************************************************/

int set_up_standard_jobs(int server)
{

	if (server == -1) return FALSE;

	if (!add_message_job(server, NOOP, ui_message_noop)) return FALSE;
	if (!add_message_job(server, MESSAGE, ui_message_message)) return FALSE;
	if (!add_message_job(server, PING, ui_message_ping)) return FALSE;
	if (!add_message_job(server, ALL_NOOP, ui_message_noop)) return FALSE;
	if (!add_message_job(server, ALL_PING, ui_message_ping)) return FALSE;
	if (!add_message_job(server, UI_STATUS, ui_message_noop)) return FALSE;
	if (!add_message_job(server, UI_SIGNAL, ui_message_noop)) return FALSE;
	if (!add_message_job(server, CENTRAL_SCRUTINIZER_LOG, 
		ui_message_noop)) return FALSE;
	if (!add_message_job(server, SERVER_STOPPING, 
		ui_message_server_stopping)) return FALSE;

	return TRUE;

} /* set_up_standard_jobs() */

/************************************************************************/
/* message_noop()							*/
/*									*/
/* A do nothing message handler.					*/
/* Returns error level.							*/
/************************************************************************/

int ui_message_noop(int server, struct smessage *mess)
{
	return NOERROR;

} /* ui_message_noop() */

/************************************************************************/
/* ui_message_message()							*/
/*									*/
/* Should simply spit out the string in the data.			*/
/* Returns error level.							*/
/************************************************************************/

int ui_message_message(int server, struct smessage *mess)
{
	char *s,*p1,*p2;
	int	i;

	if (mess->length == 0)
	{
		error(MESSAGE,"Message with no message string.");
	}
	else
	{
		if ((s = malloc(mess->length+1)) == NULL)
		{
			error(FATAL,"Ran out of memory geting message string.");
		}

		for(i=0, p1=s, p2=(char *)mess->data;
		    i < mess->length; i++, p1++, p2++) *p1 = *p2;
		*p1 = 0;

		error(MESSAGE,s);

		free(s);
	}

	return NOERROR;

} /* ui_message_message() */

/************************************************************************/
/* ui_message_ping()							*/
/*									*/
/* Sends back a message of type MESSAGE claiming we are alive.		*/
/* Returns error level.							*/
/************************************************************************/

int ui_message_ping(int server, struct smessage *mess)
{
	char	name[256];

	if (mess->length != 0)
	{
		error(ERROR,"Recieved PING with data.");
	}

	message(system_window,"I've been pinged by %s.",servers[server].name);
	print_message(server, "%s on %s is alive.",
			TITLE,get_machine_name(name));

} /* ui_message_ping() */

/************************************************************************/
/* send_noop()								*/
/*									*/
/* Sends a NOOP message to the server.					*/
/************************************************************************/

int send_noop(int server)
{
	struct smessage message;

	if (!server_socket_open(server)) 
	{
		return error(ERROR,"Server socket not open.");
	}

	message.type = NOOP;
	message.length = 0;
	message.data = NULL;

	if (!send_message(server, &message))
		return error(ERROR,"Failed to send NOOP.");

	return NOERROR;

} /* int send_noop() */

/************************************************************************/
/* send_ping()								*/
/*									*/
/* Sends a PING message to the server.					*/
/************************************************************************/

int send_ping(int server)
{
	struct smessage message;

	if (!server_socket_open(server)) 
	{
		return error(ERROR,"Server socket not open.");
	}

	message.type = PING;
	message.length = 0;
	message.data = NULL;

	if (!send_message(server, &message))
		return error(ERROR,"Failed to send PING.");

	return NOERROR;

} /* int send_ping() */

/************************************************************************/
/* send_message_message()						*/
/*									*/
/* Sends a MESSAGE message to the server.				*/
/************************************************************************/

int send_message_message(int server, char *s)
{
	if (!server_socket_open(server)) 
	{
		return error(ERROR,"Server socket not open.");
	}

	return print_message(server, s);

} /* int send_message_message() */

/************************************************************************/
/* send_all_noop()							*/
/*									*/
/* Sends a ALL_NOOP message to the server.				*/
/************************************************************************/

int send_all_noop(int server)
{
	struct smessage message;

	if (!server_socket_open(server)) 
	{
		return error(ERROR,"Server socket not open.");
	}

	message.type = ALL_NOOP;
	message.length = 0;
	message.data = NULL;

	if (!send_message(server, &message))
		return error(ERROR,"Failed to send ALL_NOOP.");

	return NOERROR;

} /* int send_all_noop() */

/************************************************************************/
/* send_all_ping()							*/
/*									*/
/* Sends a ALL_PING message to the server.				*/
/************************************************************************/

int send_all_ping(int server)
{
	struct smessage message;

	if (!server_socket_open(server)) 
	{
		return error(ERROR,"Server socket not open.");
	}

	message.type = ALL_PING;
	message.length = 0;
	message.data = NULL;

	if (!send_message(server, &message))
		return error(ERROR,"Failed to send ALL_PING.");

	return NOERROR;

} /* int send_all_ping() */

/************************************************************************/
/* background_process_server_socket()					*/
/* 									*/
/* Process server messages.						*/
/************************************************************************/

int background_process_server_socket(int server)
{
	if (!server_socket_open(server)) return NOERROR;

	switch(process_server_socket(server))
	{
		case 0: break;

		case 1: message(system_window,
				"Server socket has died.");
			close_server_socket(server);
			break;

		case 2: message(system_window,
				"Failed to read enough data frm server.");
			break;

		case 3: message(system_window,
				"Failed to allocate message memory.");
			break;

		case 4: message(system_window,
				"Failed to run message job.");
			break;

		default: message(system_window,
				"Unknown error in message process.");
			break;
	}

	return NOERROR;

} /* background_process_server_socket() */

/************************************************************************/
/* ui_message_server_stopping()						*/
/*									*/
/* When a server exits it will send this message, so we close the port. */
/************************************************************************/

int ui_message_server_stopping(int server, struct smessage *mess)
{
	if (mess->length != 0)
	{
		error(ERROR,"Recieved SERVER_STOPPING with data.");
	}

	error(WARNING, "Server %s stopped, closing port.",servers[server].name);
	close_server_socket(server);

	return NOERROR;


} /* ui_message_server_stopping() */
