/************************************************************************/
/* plot.c								*/
/*									*/
/* Routine to plot an array in a window.				*/
/* Part of the simpleX library.						*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : July 1999   						*/
/*									*/
/************************************************************************/

#include <stdlib.h>
#include <math.h>
#include <values.h>
#include "../nrc/nrc.h"
#include "nsimpleX.h"

unsigned int plot_color = 0;

/************************************************************************/
/* plot_funct()	 							*/
/*									*/
/* Plots a float array in a window. It is quite simple minded -- it	*/
/* forces the range to completely fill the window.			*/
/* The array is data[1..num_data]					*/
/* Min and max can either be set to zero, in which case the routine	*/
/* will work out min and max for itself. If they are set to some values */
/* these values will be used by the routine.				*/
/* Returns a picture ready for display or NULL on error.		*/
/************************************************************************/

char *plot_funct(int width, int height, float *data, int num_data,
		float *min, float *max)
{
	int	last_y,next_y,next_x;
	int	low_y,high_y;
	float	xmultiplier;
	float	ymultiplier;
	char	*picture;
	char	*p8;
	unsigned short int *p16;
	unsigned int *p32;
	int	i,j;
	int	last_x;

	/* Find the maximum and minimum of the array if required */

	if (*min == 0 && *max == 0)
	{
		*max = -MAXFLOAT;
		*min = MAXFLOAT;
		for(i=1; i<=num_data; i++)
		{
			if (data[i] < *min) *min = data[i];
			if (data[i] > *max) *max = data[i];
		}
	}

	/* There needs to be a range to do this. */

	if (*max == *min) 
	{
		max = max + 1;
		min = min - 1;
	}

	/* Calculate the translation parameters */

	xmultiplier = ((float)num_data) / ((float)width);

	ymultiplier = ((float)height - 1) / (*max - *min);

	last_y = -1;
	last_x = -1;

	/* Create the picture array */

	if ((picture = malloc(width * height * theBitmapBytes)) == NULL) 
		return NULL;

	/* OK go through the array */

	switch(theDepthBytes)
	{

	    case 1: 
		/* Make the whole thing black */

		p8 = picture;
		for(i=0; i<width * height; i++) *p8++ = BLACK;

		for(i=1; i<=num_data; i++)
		{
			/* Which picture pixel in X direction is this? */

			next_x = (i-1) / xmultiplier;
			if (next_x < 0) 
				next_x = 0;
			else if (next_x > width-1) 
				next_x = width-1;

			/* and which pixel in Y does this represent?  */

			next_y = (data[i]-*min) * ymultiplier;
			if (next_y < 0) 
				next_y = 0;
			else if (next_y >= height) 
				next_y = height - 1;

			/* We may have to draw a horizontal line */

			if (last_x != -1)
			{
				for(j = last_x; j<= next_x; j++)
				{
					p8 = picture+width*(height-last_y-1)+j;
					*p8 = plot_color;
				}
			}

			/* Now we may need a vertical line */

			if (last_y == -1) 
			{
				low_y = high_y = next_y;
			}
			else if (last_y > next_y)
			{
				low_y = next_y;
				high_y = last_y;
			}
			else
			{
				low_y = last_y;
				high_y = next_y;
			}

			for(j = low_y; j<= high_y; j++)
			{
				p8 = picture+width*(height-j-1)+next_x;
				*p8 = plot_color;
			}

			/* Keep track of last position */

			last_y = next_y;
			last_x = next_x;

		}

		/* We may need a final horizontal line at the end */

		for(j = last_x; j<= width; j++)
		{
			p8 = picture+width*(height-last_y-1)+j;
			*p8 = plot_color;
		}
		break;

	    case 2: 
		/* Make the whole thing black */

		p16 = (unsigned short int *)picture;
		for(i=0; i<width * height; i++) *p16++ = BLACK;

		for(i=1; i<=num_data; i++)
		{
			/* Which picture pixel in X direction is this? */

			next_x = (i-1) / xmultiplier;
			if (next_x < 0) 
				next_x = 0;
			else if (next_x > width-1) 
				next_x = width-1;

			/* and which pixel in Y does this represent?  */

			next_y = (data[i]-*min) * ymultiplier;
			if (next_y < 0) 
				next_y = 0;
			else if (next_y >= height) 
				next_y = height - 1;

			/* We may have to draw a horizontal line */

			if (last_x != -1)
			{
				for(j = last_x; j<= next_x; j++)
				{
					p16 = (unsigned short int*)picture +
						width*(height-last_y-1) + j;
					*p16 = plot_color;
				}
			}

			/* Now we may need a vertical line */

			if (last_y == -1) 
			{
				low_y = high_y = next_y;
			}
			else if (last_y > next_y)
			{
				low_y = next_y;
				high_y = last_y;
			}
			else
			{
				low_y = last_y;
				high_y = next_y;
			}

			for(j = low_y; j<= high_y; j++)
			{
				p16 = (unsigned short int*)picture +
					width*(height-j-1) + next_x;
				*p16 = plot_color;
			}

			/* Keep track of last position */

			last_y = next_y;
			last_x = next_x;

		}

		/* We may need a final horizontal line at the end */

		for(j = last_x; j<= width; j++)
		{
			p16 = (unsigned short int *)picture +
				width*(height-last_y-1) + j;
			*p16 = plot_color;
		}
		break;


	    default: 
		/* Make the whole thing black */

		p32 = (unsigned int *)picture;
		for(i=0; i<width * height; i++) *p32++ = BLACK;

		for(i=1; i<=num_data; i++)
		{
			/* Which picture pixel in X direction is this? */

			next_x = (i-1) / xmultiplier;
			if (next_x < 0) 
				next_x = 0;
			else if (next_x > width-1) 
				next_x = width-1;

			/* and which pixel in Y does this represent?  */

			next_y = (data[i]-*min) * ymultiplier;
			if (next_y < 0) 
				next_y = 0;
			else if (next_y >= height) 
				next_y = height - 1;

			/* We may have to draw a horizontal line */

			if (last_x != -1)
			{
				for(j = last_x; j<= next_x; j++)
				{
					p32 = (unsigned int *)picture +
						width*(height-last_y-1) + j;
					*p32 = plot_color;
				}
			}

			/* Now we may need a vertical line */

			if (last_y == -1) 
			{
				low_y = high_y = next_y;
			}
			else if (last_y > next_y)
			{
				low_y = next_y;
				high_y = last_y;
			}
			else
			{
				low_y = last_y;
				high_y = next_y;
			}

			for(j = low_y; j<= high_y; j++)
			{
				p32 = (unsigned int *)picture +
					width*(height-j-1) + next_x;
				*p32 = plot_color;
			}

			/* Keep track of last position */

			last_y = next_y;
			last_x = next_x;

		}

		for(j = last_x; j<= width; j++)
		{
			p32 = (unsigned int *)picture +
				width*(height-last_y-1) + j;
			*p32 = plot_color;
		}
		break;
		break;
	}

	/* That is all */

	return picture;

} /* plot_funct() */

/************************************************************************/
/* merge_plots()							*/
/*									*/
/* Merges the data in one plot with another. Merged plots are placed	*/
/* in the same memory area as original first plot, which is also 	*/
/* returned. Plots must have the same size.				*/
/************************************************************************/

char *merge_plots(char *plot1, char *plot2, int size_x, int size_y)
{
	int	i,j;
	unsigned char	*p1_8,*p2_8;
	unsigned short int *p1_16, *p2_16;
	unsigned int *p1_32, *p2_32;

	switch(theDepthBytes)
	{
	    case 1:
		for(p1_8 = (unsigned char *)plot1, 
			p2_8 = (unsigned char *)plot2, i=0; i < size_x; i++)
		for(j=0; j < size_y; j++)
		{
			if (*p2_8 != BLACK) *p1_8 = *p2_8;
			p1_8++;
			p2_8++;
		}
		break;

	    case 2:
		for(p1_16 = (unsigned short int *)plot1, 
			p2_16 = (unsigned short int *)plot2, i=0; 
			i < size_x; i++)
		for(j=0; j < size_y; j++)
		{
			if (*p2_16 != BLACK) *p1_16 = *p2_16;
			p1_16++;
			p2_16++;
		}
		break;

	    default:
		for(p1_32 = (unsigned int *)plot1, 
			p2_32 = (unsigned int *)plot2, i=0; 
			i < size_x; i++)
		for(j=0; j < size_y; j++)
		{
			if (*p2_32 != BLACK) *p1_32 = *p2_32;
			p1_32++;
			p2_32++;
		}
		break;
	}

	return plot1;

} /* merge_plots() */

/************************************************************************/
/* set_plot_color()							*/
/*									*/
/* Sets the global plot_color to a value represented by the RGB values.	*/
/* RGB values are from 0 to 65536.					*/
/* Returns TRUE if all goes well, FALSE if it fails.			*/
/************************************************************************/

int set_plot_color(int red, int green, int blue)
{
	XColor  color;

	color.red = red;
	color.green = green;
	color.blue = blue;
	color.flags = DoRed | DoGreen | DoBlue;
	if (!XAllocColor(theDisplay,theCmap,&color))
	{
		plot_color = WHITE;
		return FALSE;
	}
										
	plot_color = color.pixel;
	return TRUE;

} /* set_plot_color() */
