/************************************************************************/
/* picom_comms.c							*/
/*                                                                      */
/* Low-level routines to talk to PICOMOTOR Multi-Axis Driver box.	*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Nils Turner & Stuart Taylor					*/
/* Date   : 22 February 2001						*/
/************************************************************************/

#include <charaui.h>
#include "picom.h"

static struct sserial pico_port;
static bool port_open = FALSE;

#define TIME_OUT 2

/************************************************************************/
/* pico_port_open()							*/
/*									*/
/* Opens the serial port to the New Focus 8732 Picomotor Multi-Axis	*/
/* Driver.								*/
/************************************************************************/

int pico_port_open(int argc, char **argv)
{
	char		s[81];	        /* Edit string */
	int	return_value;

	if (port_open) return error(ERROR,"Port already open");

	socket_test_args(1,"TTYport");

	/* Check command line */

	if (argc < 2) 	/* No port given */
	{
		sprintf(s,"/dev/cua1");
		if (quick_edit("NF 8732 serial port",s, s,NULL,STRING) == 
			KEY_ESC) 
		{
			return NOERROR;
		}

		/* Try to open the pico_port */

		pico_port = open_serial_port(s,FALSE);
	}
	else
	{
		pico_port = open_serial_port(argv[1],FALSE);
	}

	/* Did it work ? */

	if (pico_port.fd == -1) 
	{
		if (argc > 1)
			error(WARNING,"Sorry, you cannot talk to %s",argv[1]);
		else
			error(WARNING,"Sorry, you cannot talk to %s",s);

		return WARNING;
	}

	/* Set the baud rate */

	if ((return_value = set_serial_baud_rate(&pico_port, B19200)) !=
		NOERROR)
	{		
		error(WARNING,"Could not set baud rate for port %s.",
			pico_port.name);
		return return_value;
	}

	/* Set it to No parity */

	if ((return_value = set_serial_parity(&pico_port, OFF)) != NOERROR)
	{		
		error(WARNING,"Couldn't set parity for port %s.",
			pico_port.name);
		return return_value;
	}

	/* Set it to 8 bits */

	if ((return_value = set_serial_bitlength(&pico_port, 8)) != NOERROR)
	{		
		error(WARNING,"Couldn't set bit length for port %s.",
			pico_port.name);
		return return_value;
	}

	/* Set it to 1 stop bit */

	if ((return_value = set_serial_stopbits(&pico_port, 1)) != NOERROR)
	{		
		error(WARNING,"Couldn't set stop bits for port %s.",
			pico_port.name);
		return return_value;
	}

	/* No handshaking */

        if ((return_value = set_serial_xonxoff(&pico_port, OFF)) != NOERROR)
	{
		close_serial_port(pico_port);
		pico_port.fd = -1;
		return return_value;
	}

        if ((return_value = set_serial_hard_handshake(&pico_port, OFF)) !=
		NOERROR)
	{
		close_serial_port(pico_port);
		pico_port.fd = -1;
		return return_value;
	}

	port_open = TRUE;

	return NOERROR;

} /* pico_port_open() */  

/************************************************************************/
/* pico_port_close()							*/
/*									*/
/* Closes the serial port to the New Focus 8732 Picomotor Multi-Axis	*/
/* Driver.								*/
/************************************************************************/

int pico_port_close(int argc, char **argv)
{

	if (!port_open)
	{
		return NOERROR;
	}

	close_serial_port(pico_port);

	port_open = FALSE;

	return NOERROR;

} /* pico_port_close() */

/************************************************************************/
/* pico_get_string()                                                    */
/*                                                                      */
/* listens to the PICO serial port and waits for a response.            */
/* which is returned. Strips off trailing ^J.                           */
/* Returns string, which will be empty if something went wrong.         */
/* String is assumed to be long enough to hold result and a terminating */
/* NULL is added to the end.                                            */
/************************************************************************/

char *pico_get_string(char *s)
{
	time_t	start_time;
	char	*c;
	int	kounter;        

	/* If the port is not open, send warning and return. */

	if (!port_open)
	{
		*s = 0;
		return s;
	}

	start_time = time(NULL);

	c = s;

	kounter = 0;

	do
	{
		background();
		while ((read(pico_port.fd,c,1) != 1) && 
			(time(NULL) < start_time+TIME_OUT))
		{
			if (time(NULL) >= start_time+TIME_OUT)
			{
				*s = 0;

				return s;
			}
		}
		c++;
	}
	while (*(c-1) != '\r' && *(c-1) != '\n' &&
		(time(NULL) < start_time+TIME_OUT));

	*c = 0;

	return s;

} /* pico_get_string() */

/************************************************************************/
/* pico_idn()								*/
/*                                                                      */
/* Send the identification command to the NF 8732 box, copy its reply	*/
/* to the input string, and return 1 on success, 0 on failure.		*/
/************************************************************************/

int pico_idn(char *s)
{
	if (!port_open)
	{
		error(ERROR,"Serial port for picomotors not open.");
		return 0;
	}

	while (!pico_op_complete());

	serial_print(pico_port,"@idn?\r");
	pico_get_string(s);

	return 1;

} /* pico_idn() */

/************************************************************************/
/* pico_reset()								*/
/*                                                                      */
/* Send the reset command to the NF 8732 box.  Return 1 if successful,	*/
/* 0 if unsuccesful.							*/
/************************************************************************/

int pico_reset(void)
{
       	char	s[256];

        if (!port_open)
        {
		error(ERROR,"Serial port for picomotors not open.");
                return 0;
        }

	while (!pico_op_complete());

        serial_print(pico_port,"@rst\r");
	pico_get_string(s);

	if (!strncmp(s,"OK",2))
	{
		return 1;
	}
	else
	{
		return 0;
	}
} /* call_pico_reset() */

/************************************************************************/
/* pico_go_to_local()							*/
/*                                                                      */
/* Send the go to local command to the NF 8732 box.  Return 1 if	*/
/* successful, 0 if unsuccessful.					*/
/************************************************************************/

int pico_go_to_local(void)
{
       	char	s[256];

        if (!port_open)
        {
		error(ERROR,"Serial port for picomotors not open.");
		return 0;
        }

	while (!pico_op_complete());

	serial_print(pico_port,"@gtl\r");
	pico_get_string(s);

	if (!strncmp(s,"OK",2))
	{
		return 1;
	}
	else
	{
		return 0;
	}
} /* pico_go_to_local() */

/************************************************************************/
/* pico_set_channel()							*/
/*                                                                      */
/* Set the channel on the NF 8732 box.	Return the channel number if	*/
/* successful, 0 if the port is not open, and -1 if the channel number	*/
/* is out of range.  The channel is a three digit number:  digit 1 is	*/
/* the slot number, 2 the connector number, and 3 the axis number.	*/
/************************************************************************/

int pico_set_channel(int channel)
{
       	char	s[256];

        if (!port_open)
        {
		error(ERROR,"Serial port for picomotors not open.");
		return 0;
        }

	while (!pico_op_complete());

	serial_print(pico_port,"@inst:nsel%d\r",channel);
	pico_get_string(s);

	if (!strncmp(s,"OK",2))
	{
		return channel;
	}
	else
	{
		return -1;
	}
} /* pico_set_channel() */

/************************************************************************/
/* pico_get_channel()							*/
/*                                                                      */
/* Get the channel from the NF 8732 box.  Return the channel number if	*/
/* successful, 0 if unsuccessful.  The channel is a three digit number:	*/
/* digit 1 is the slot number, 2 the connector number, and 3 the axis	*/
/* number.								*/
/************************************************************************/

int pico_get_channel(void)
{
	char	s[256];

	if (!port_open)
	{
		error(ERROR,"Serial port for picomotors not open.");
		return 0;
	}

	while (!pico_op_complete());

	serial_print(pico_port,"@inst:nsel?\r");
	pico_get_string(s);

	return atoi(s);

} /* pico_get_channel() */

/************************************************************************/
/* pico_set_frequency()							*/
/*                                                                      */
/* Set the frequency of the current channel on the NF 8732 box.	 Return	*/
/* 1 if succesful, 0 if the port is not open, and -1 if the frequency	*/
/* out of range.							*/
/************************************************************************/

int pico_set_frequency(double frequency)
{
       	char	s[256];

        if (!port_open)
        {
		error(ERROR,"Serial port for picomotors not open.");
		return 0;
        }

	while (!pico_op_complete());

	serial_print(pico_port,"@sour:puls:freq%f\r",frequency);
	pico_get_string(s);

	if (!strncmp(s,"OK",2))
	{
		return 1;
	}
	else
	{
		return -1;
	}
} /* pico_set_frequency() */

/************************************************************************/
/* pico_init_frequency()						*/
/*                                                                      */
/* Sets the frequency of all the channels on the NF 8732 box.  Return	*/
/* 1 if succesful, 0 if the port is not open, and -1 if the frequency	*/
/* out of range.							*/
/************************************************************************/

int pico_init_frequency(double frequency)
{
	int	slot;
	int	connector;
	int	axis;
	int	channel;
       	char	s[256];

        if (!port_open)
        {
		error(ERROR,"Serial port for picomotors not open.");
		return 0;
        }

	while (!pico_op_complete());

	for (slot=1;slot<=5;slot++)
	{
		for (connector=1;connector<=4;connector++)
		{
			for (axis=1;axis<=3;axis++)
			{
				channel = slot*100 + connector*10 + axis;
				pico_set_channel(channel);

				serial_print(pico_port,"@sour:puls:freq%f\r",
						frequency);
				pico_get_string(s);

				if (strncmp(s,"OK",2))
				{
					return -1;
				}
			}
		}
	}

	return 1;

} /* pico_init_frequency() */

/************************************************************************/
/* pico_get_frequency()							*/
/*                                                                      */
/* Get the frequency of the current channel on the NF 8732 box.  Return	*/
/* 1 if successful, and 0 if unsuccesful.				*/
/************************************************************************/

int pico_get_frequency(char *s)
{
	if (!port_open)
	{
		error(ERROR,"Serial port for picomotors not open.");
		return 0;
	}

	while (!pico_op_complete());

	serial_print(pico_port,"@sour:puls:freq?\r");
	pico_get_string(s);

	return 1;

} /* pico_get_frequency() */

/************************************************************************/
/* pico_op_complete()							*/
/*                                                                      */
/* Query the NF 8732 box as to whether it is done with the last sent	*/
/* command.  Return 1 if it is done, 0 if operations are still pending.	*/
/************************************************************************/

int pico_op_complete(void)
{
	char	s[256];

	if (!port_open)
	{
		error(ERROR,"Serial port for picomotors not open.");
		return 0;
	}

	message(system_window,"Waiting for operation to complete.");
	serial_print(pico_port,"@opc?\r");
	pico_get_string(s);
	werase(system_window);
	wrefresh(system_window);

	if (strlen(s) == 0)
	{
		error(ERROR,"Timed out waiting for string.");
		return 1;
	}

	if(!strncmp(s,"0",1))
	{
		return 0;
	}
	else
	{
		return 1;
	}
} /* pico_op_complete() */

/************************************************************************/
/* pico_run_one_axis()							*/
/*                                                                      */
/* Moves the PseudoConn A channel the input number of counts.  Returns	*/
/* 1 if the move was successful, 0 if not successful.			*/
/************************************************************************/

int pico_run_one_axis(int counts,PseudoConn *conn)
{
	int	channel;
	char	s[256];

	if (!port_open)
	{
		error(ERROR,"Serial port for picomotors not open.");
		return 0;
	}

	if (!pico_check_pseudo_conn(conn))
	{
		return 0;
	}

	while (!pico_op_complete());

	if (conn->dirc_a !=0)
	{
		channel = conn->slot_a*100 + conn->conn_a*10 + conn->axis_a;

		pico_set_channel(channel);

		if (conn->dirc_a == FORWARD)
		{
			serial_print(pico_port,"@sour:dir:%s\r","cw");
			pico_get_string(s);
		}
		else
		{
			serial_print(pico_port,"@sour:dir:%s\r","ccw");
			pico_get_string(s);
		}

		serial_print(pico_port,"@sour:puls:coun%d\r",counts);
		pico_get_string(s);
	}

	return 1;

} /* pico_run_one_axis() */

/************************************************************************/
/* pico_run_two_axis()							*/
/*                                                                      */
/* Moves both PseudoConn channels the input number of counts.  Returns	*/
/* 1 if the move was successful, 0 if not successful.			*/
/************************************************************************/

int pico_run_two_axis(int counts,PseudoConn *conn)
{
	int	channel;
	char	s[256];
	int	counts_b;

	if (!port_open)
	{
		error(ERROR,"Serial port for picomotors not open.");
		return 0;
	}

	if (!pico_check_pseudo_conn(conn))
	{
		return 0;
	}

	counts_b = counts/conn->count_ratio;

	while (!pico_op_complete());

	if (conn->dirc_a != 0)
	{
		channel = conn->slot_a*100 + conn->conn_a*10 + conn->axis_a;

		pico_set_channel(channel);

		if (conn->dirc_a == FORWARD)
		{
			serial_print(pico_port,"@sour:dir:%s\r","cw");
			pico_get_string(s);
		}
		else
		{
			serial_print(pico_port,"@sour:dir:%s\r","ccw");
			pico_get_string(s);
		}

		serial_print(pico_port,"@sour:puls:coun%d\r",counts);
		pico_get_string(s);
	}

	if ((conn->slot_a != conn->slot_b) || (conn->conn_a != conn->conn_b))
	{
		while (!pico_op_complete());
	}

	if (conn->dirc_b != 0)
	{
		channel = conn->slot_b*100 + conn->conn_b*10 + conn->axis_b;

		pico_set_channel(channel);

		if (conn->dirc_b == FORWARD)
		{
			serial_print(pico_port,"@sour:dir:%s\r","cw");
			pico_get_string(s);
		}
		else
		{
			serial_print(pico_port,"@sour:dir:%s\r","ccw");
			pico_get_string(s);
		}

		serial_print(pico_port,"@sour:puls:coun%d\r",counts_b);
		pico_get_string(s);
	}

	return 1;

} /* pico_run_two_axis() */

/************************************************************************/
/* pico_check_pseudo_conn()						*/
/*                                                                      */
/* Checks whether all the channel parameters in the PseudoConn		*/
/* structure are within range.  Returns 1 if all is well, and 0 if	*/
/* something is out of range.						*/
/************************************************************************/

int pico_check_pseudo_conn(PseudoConn *conn)
{
	if (	(conn->slot_a > 5) ||
		(conn->slot_a < 1) ||
		(conn->slot_b > 5) ||
		(conn->slot_b < 1) ||
		(conn->conn_a > 4) ||
		(conn->conn_a < 1) ||
		(conn->conn_b > 4) ||
		(conn->conn_b < 1) ||
		(conn->axis_a > 3) ||
		(conn->axis_a < 1) ||
		(conn->axis_b > 3) ||
		(conn->axis_b < 1))
	{
		return 0;
	}
	else if (	(conn->dirc_a > 2) ||
			(conn->dirc_a < 0) ||
			(conn->dirc_b > 2) ||
			(conn->dirc_b < 0))
	{
		return 0;
	}

	return 1;

} /* pico_check_pseudo_conn() */

/************************************************************************/
/* pico_init_pseudo_conn()						*/
/*                                                                      */
/* Initializes the pseudo connector to slot_a 1, conn_a 1, axis_a 1,	*/
/* dirc_a 1, ratio 1.0, slot_b 1, conn_b 1, axis_b 2, and dirc_b 1.	*/
/* Returns 1.								*/
/************************************************************************/

int pico_init_pseudo_conn(PseudoConn *conn)
{
	conn->slot_a = 1;
	conn->conn_a = 1;
	conn->axis_a = 1;
	conn->dirc_a = 1;
	conn->count_ratio = 1.0;
	conn->slot_b = 1;
	conn->conn_b = 1;
	conn->axis_b = 1;
	conn->dirc_b = 1;

	return 1;

} /* pico_init_pseudo_conn() */
