/*
 * spooler.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#define _GNU_SOURCE

#include <sys/time.h>
#include <sys/types.h>
#include <pthread.h>
#include <semaphore.h> 
#include <unistd.h>
#include <sys/stat.h>
#include <signal.h>
#include <string.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <sched.h>
#include <errno.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h> 
#include <rtai/pipe.h>

#include "spooler.h"
#include <fitsio.h>
#include "cliserv.h"
#include "chara_messages.h"

/* Common variables */

boolean end = FALSE;
int fd, sockfd[SOCKNUM];               /* File descriptors for pipe and sockets */
char buffer[SOCKNUM * BUFLEN];         /* Array  containing data from sockets */
pthread_attr_t attr;
sem_t sem_fits;

/*****************************************************/
/* signal-handler, to ensure clean exit on Ctrl-C    */
/*****************************************************/
void cleanExit(int dummy)
{
  end = TRUE;
  usleep(2000000);
  exit(0);
}/*  end cleanExit */


/*****************************************************/
/* Print out cfitsio error messages and exit program */
/*****************************************************/
void printerror( int status)
{
    if (status)
    {
       fits_report_error(stderr, status); /* print error report */
       exit( status );    /* terminate the program, returning error status */
    }
    return;
}

void error(char *msg)
{
    perror(msg);
    exit(0);
}

/*****************************************************/
/* Open a server for querying status                 */
/*****************************************************/
int openServer(host_struct host)
{
  int sockfd, portno, n, i;
  struct sockaddr_in serv_addr;
  struct hostent *server;

  sockfd = socket(AF_INET, SOCK_STREAM, 0);
  if (sockfd < 0) 
    error("ERROR opening socket");
  server = gethostbyname(host.hostname);
  if (server == NULL) {
    fprintf(stderr,"ERROR, no such host\n");
    exit(0);
  }
  bzero((char *) &serv_addr, sizeof(serv_addr));
  serv_addr.sin_family = AF_INET;
  bcopy((char *)server->h_addr, 
	(char *)&serv_addr.sin_addr.s_addr,
	server->h_length);
  for(i=0;i<6;i++){
    portno=host.port+i;
    serv_addr.sin_port = htons(portno);
    if (connect(sockfd,&serv_addr,sizeof(serv_addr)) < 0){
      printf("ERROR connecting to port %d of %s\n",portno, host.hostname);
      sockfd = -1;
    } else {
      printf("Connecting to port %d of %s\n",portno, host.hostname);
      break;
    }
  }
  return(sockfd);
} /* end openServer */

/*****************************************************/
/* Query server for status                           */
/*****************************************************/
int queryServer(int sockfd, char *msg)
{
  int n;

  n = write(sockfd,msg,strlen(msg));
  if (n < 0) {
    error("ERROR writing to socket");
    return -1;
  } else {
    return 0;
  }
} /* end queryServer */

/*****************************************************/
/* Read server for status                           */
/*****************************************************/
int readServer(int sockfd, char *buffer)
{
  int n;
  int length=0;

  bzero(buffer,BUFLEN);
  while((length < BUFLEN)){
    n = read(sockfd,(buffer+length),10);
    if (n < 0) 
      error("ERROR reading from socket");
    length=length+n;
  }
  return 0;
} /* end readServer */


/*****************************************************/
/* Check if can read from server using select        */
/*****************************************************/
/* end checkServer */

/*****************************************************/
/* Recover time string from GPS status message       */
/*****************************************************/
char *returnTime(char *buffer,char *keyword, char *outbuf)
{ 
  memcpy(outbuf,strstr(strstr(buffer,keyword),":")+3,8);
  *(outbuf+9)=NULL;
  return(outbuf);
} /* end returnTime */


/*****************************************************/
/* Tread reads socket and writes to a buffer         */
/*****************************************************/
void *socketThread(void *threadarg)
{
  int  threadId;
  threadData *tdata;
  tdata = (threadData *) threadarg;
  size_t mystacksize;
 
  pthread_attr_getstacksize (&attr, &mystacksize);

  threadId = tdata->thread_id;

  while (!end) {

     /* Query server for info*/
    if(sockfd[tdata->socknum] < 0) {
      bzero(buffer+tdata->socknum*BUFLEN,BUFLEN);
      printf("%d: Server id down!!! Restart the server and the spooler \n", threadId);
    } else { 
      if (0 == queryServer(sockfd[tdata->socknum], tdata->message)) {
	printf("%d: Ask server for status...\n", threadId);
	readServer(sockfd[tdata->socknum], buffer+tdata->socknum*BUFLEN);
      }
    } 
    sem_wait(&sem_fits); 
    //usleep(SLEEPTIME);
  }
   pthread_exit((void *) 0);
} /* end socketThread */

/*****************************************************/
/* Tread reads pipe and writes to disk               */
/*****************************************************/
void *spoolThread(void *threadarg)
{

  struct smessage message;      /* Messages passed from astropci_usr.c */
  header_struct header;         /* Structure passed from astropci_usr.c */
  FILE *tmpFile;                /* Temporary file to ckeck existence of fits files */
  int pixelCount;               /* Length of message from astropci_usr.c  */
  boolean fileOpen = FALSE;     /* Is fits file open? */
  int length = 0;               /* Total message space to malloc */
  double start1, start2;        /* Vars for timing thread execution */
  int fileNum = 0;              /* last open fits file */
  char fileName[50];            /* Fits filename */
  char gdate[10];               /* System date */
  char fitsdate[19];            /* System UT date and time */  
  char dir1[50], dir2[50];      /* Dirs names to create file structure */
  char tmpbuf[LLENGTH];         /* Temporary buffet for time in header */
  char target[12];              /* Target Name */
  time_t rawTime;               /* Structures used to get System time and */
  struct tm *gmt;               /* UT time */
  double outbuf[18];            /* Delay lines position  */

/* Fits definitions in Header  */
/* Copied from cfitsio example */
  long naxis=NAXIS_LEN, naxes[NAXIS_LEN], fpixel = 1, firstpix[NAXIS_LEN] = {1,1,1};
  fitsfile *fptr;               /* FITS file pointer, defined in fitsio.h */
  int status = 0;               /* CFITSIO status value MUST be initialized to zero!  */

  int  threadId;
  threadData *tdata;
  tdata = (threadData *) threadarg;
  size_t mystacksize;
  
  pthread_attr_getstacksize (&attr, &mystacksize);

  threadId = tdata->thread_id;
  
  while (!end) {
   
    if (read(sockfd[SPOOLFD],& message.type, sizeof(message.type)) == -1) {
/*       perror("Command read error\n"); */
/*       printf("errno = %d\n",errno); */
	
    } else {
      
      switch (message.type) {

      case HEADER:

	firstpix[2]=1;
	start1 = (double)clock();
	//printf("%d:Time elapsed: %f \n", threadId, (start1 - start2) / CLOCKS_PER_SEC);
	start2 = start1;

	if (read(sockfd[SPOOLFD],&message.length, sizeof(message.length)) == -1)
	  printf("%d:Header length read error\n", threadId);
	
	if (read(sockfd[SPOOLFD], (unsigned char *) &header, message.length) == -1)
	  printf("%dHeader read error\n", threadId);

	pixelCount = header.rows * header.cols * header.nreads;
	 
	length = pixelCount * sizeof(unsigned short);
	if((message.data = (unsigned char *) malloc(length)) == NULL) {
	  printf("%d:Data malloc error\n", threadId);
	  exit -1;
	}

	if(!fileOpen && header.do_image_save){

	  /* Get UT time */
	  time ( &rawTime );
	  gmt = gmtime ( &rawTime );
	  
	  /* Parse delay line status message and return offsets */

	  sscanf(buffer+OPLEFD*BUFLEN,OPLEFMT,target, outbuf, outbuf+1, outbuf+2, outbuf+3,
		 outbuf+4, outbuf+5, outbuf+6, outbuf+7, outbuf+8,
		 outbuf+9, outbuf+10, outbuf+11, outbuf+12, outbuf+13,
		 outbuf+14, outbuf+15, outbuf+16, outbuf+17);
 

	  sprintf (fitsdate, "%04d-%02d-%02dT%02d:%02d:%02d", gmt->tm_year + 1900, gmt->tm_mon + 1, gmt->tm_mday, gmt->tm_hour, gmt->tm_min, gmt->tm_sec);

	  sprintf (gdate, "%02d/%02d/%04d", gmt->tm_mon + 1, gmt->tm_mday, gmt->tm_year + 1900);
	  sprintf(dir1,"%s%02d%s/", header.image_directory, gmt->tm_year + 1900, mon[gmt->tm_mon]);
	  sprintf(dir2,"%s%02d%s%02d", dir1, gmt->tm_year + 1900, mon[gmt->tm_mon], gmt->tm_mday);

	  if(chdir(dir1)){
	    fileNum = 0;
	    printf("%d:Create directory %s\n", threadId, dir1);	  
	    /* Open dir with directory name */
	    mkdir(dir1,0755);
	  }
	  if(chdir(dir2)){
	    fileNum = 0;
	    printf("%d:Create directory %s\n", threadId, dir2);	  
	    /* Open dir with directory name */
	    mkdir(dir2,0755);
	  }
	
	  /* -----------------------------------
	     Create the FITS file.
	     ----------------------------------- */

	  do{
	    /* Check existence of  fits file */
	    sprintf(fileName,"%s/mirc%04d.fits", dir2, fileNum);
	    tmpFile = fopen(fileName, "r");
	    if(tmpFile == NULL) {
	      //printf("%d:Using %s\n", threadId, fileName);	      
	      fileNum ++;
	      break;
	    }
	    fclose(tmpFile);
	    printf("%d:Skipping %s\n", threadId,fileName);
	    fileNum ++;
	  } while(fileNum < 9999);
	  
	  status=0;

	  printf("%d:Create fits file %s\n", threadId,fileName);
      	  fits_create_file(&fptr,fileName,&status);

	  naxes[1] = header.rows;
	  naxes[0] = header.cols * header.nreads;
	  naxes[2] = header.numberOfFrames;	  

	  if ( fits_create_img(fptr,  USHORT_IMG, naxis, naxes, &status) )
	    printerror( status );

	  /* Write keywords; must pass the ADDRESS of the value */
	  fits_update_key(fptr, TSTRING, "DATE", &fitsdate, "File creation date (YYYY-MM-DDThh:mm:ss UTC)", &status);
	  fits_update_key(fptr, TSTRING, "TELESCOP", "CHARA array 330m max baseline, 6dishes", "Telescope", &status);
	  fits_update_key(fptr, TSTRING, "INSTURME", "MIRC spectro/combiner", "The data acquisition instrument", &status);
	  fits_update_key(fptr, TSTRING, "ORIGIN", "Mount Wilson Institute", "Origin of the Observation", &status);
	  fits_update_key(fptr, TSTRING, "SITELAT","34.13", "Latitude (Geodetic, VLBI, to be verified)", &status);
	  fits_update_key(fptr, TSTRING, "SITELONG","118.03", "Longitude (Geodetic, VLBI, to be verified)", &status);
	  fits_update_key(fptr, TSTRING, "SITEELEV","1742.00", "Altitude above MSL, to be verified", &status);
	  fits_update_key(fptr, TSTRING, "HISTORY", "Multi-Dish FITS data", "File modification history", &status);
/* 	  fits_update_key(fptr, TSTRING, "COMMENT", 'comment', "Any additional information", &status); */
	  fits_update_key(fptr, TSTRING, "OBJECT", target, "Target name", &status);
/* 	  fits_update_key(fptr, TSTRING, "OBSERVER", header.observer, "Observer name", &status); */
	  fits_update_key(fptr, TSTRING, "DATE-OBS", gdate, "UT date (YYYY-MM-DD)", &status);
	  fits_update_key(fptr, TSTRING, "UTC-OBS", returnTime(buffer+GPSFD*BUFLEN,"GPS", tmpbuf), "Universal Time hh:mm:ss", &status);
	  fits_update_key(fptr, TSTRING, "LST-OBS", returnTime(buffer+GPSFD*BUFLEN,"Sid", tmpbuf), "Local Sidereal Time hh:mm:ss", &status);
	  fits_update_key(fptr, TSTRING, "CHARA-TM",returnTime(buffer+GPSFD*BUFLEN,"CHARA", tmpbuf), "CHARA time  hh:mm:ss", &status);
	  fits_update_key(fptr, TSTRING, "LOST-TKS",returnTime(buffer+GPSFD*BUFLEN,"Tks", tmpbuf), "CHARA lost Ticks in RT Clock t", &status);
	  fits_update_key(fptr, TSTRING, "LOST-SEC",returnTime(buffer+GPSFD*BUFLEN,"Sec", tmpbuf), "CHARA lost seconds in rt clock s", &status);


	  fits_update_key(fptr, TDOUBLE, "S1-TARGE", outbuf+S1_TARGET, "Delay line S1 target metrology", &status);
	  fits_update_key(fptr, TDOUBLE, "S2-TARGE", outbuf+S2_TARGET, "Delay line S2 target metrology", &status);
	  fits_update_key(fptr, TDOUBLE, "E1-TARGE", outbuf+E1_TARGET, "Delay line E1 target metrology", &status);
	  fits_update_key(fptr, TDOUBLE, "E2-TARGE", outbuf+E2_TARGET, "Delay line E2 target metrology", &status);
	  fits_update_key(fptr, TDOUBLE, "W1-TARGE", outbuf+W1_TARGET, "Delay line W1 target metrology", &status);
	  fits_update_key(fptr, TDOUBLE, "W2-TARGE", outbuf+W2_TARGET, "Delay line W2 target metrology", &status);


	  fits_update_key(fptr, TFLOAT, "WAVELEN", &header.lambda, "Central wavelength", &status);
	  fits_update_key(fptr, TFLOAT, "BANDWID", &header.bandwidth, "Bandwidth of spectrum", &status);
	  fits_update_key(fptr, TFLOAT, "EXPOSURE", &header.tint, "Effective integration time in ms", &status);
	  fits_update_key(fptr, TSHORT, "ROWOFFS", &header.rowOffset, "Sub-image Y offset prom pixel 0", &status);
	  fits_update_key(fptr, TSHORT, "COLOFFS", &header.colOffset, "Sub-image X offset prom pixel 0", &status);
	  fits_update_key(fptr, TSHORT, "NREADS", &header.nreads, "Number of multiple reads for pixel", &status);
	  fits_update_key(fptr, TSHORT, "FRMPRST", &header.framesPerReset, "Number of frames per reset", &status);
	  fits_update_key(fptr, TFLOAT, "VOFFSET", &header.voffset, "PICNIC offset voltage", &status);
 	  fits_update_key(fptr, TFLOAT, "VD", &header.vd, "PICNIC drain bias", &status);
 	  fits_update_key(fptr, TFLOAT, "ICTL", &header.ictl, "PICNIC warm OA offset voltage", &status);
	  //printf("%d:Filled in header info\n", threadId);
	  fileOpen = TRUE;
	}
	sem_post (&sem_fits);
	sem_post (&sem_fits);
	sem_post (&sem_fits);

	break;

      case DATA:

	if (read(sockfd[SPOOLFD],&message.length, sizeof(message.length)) == -1) 
	  printf("%d:Data length read error\n", threadId);
	
	// allocate data buffer
	if (length == message.length) 
	  if((message.data = (unsigned char *) realloc(message.data, message.length)) == NULL)
	    printf("%d:Data realloc error\n", threadId);
	
	length = message.length;
	if (read(sockfd[SPOOLFD], (unsigned char *) message.data, message.length) == -1) {
	  printf("%d:Data read error\n", threadId); 
	} else if(fileOpen) {

/* 	  totalBytes += message.length; */
	  /* Write the image data. */

	  if (fits_write_pix(fptr, TUSHORT, firstpix, naxes[0]*naxes[1],
			     message.data, &status))
	    printerror( status ); 

	}
	/* loop over all planes of the cube (2D images have 1 plane) */
	firstpix[2] ++;
	break;

      case END:
	free(message.data);

	if(fileOpen) {
	  printf("\n ************************** %d:Closing the current FITS file %s\n ", threadId,fileName);
	  fits_close_file(fptr,&status);
	  fileOpen = FALSE;
	}

	break;

      case KILL:
	printf("%d:Kill spooler process\n", threadId);
	end = TRUE;	break;
            
      default:
	  
	break;
      }
    }
  }
  sem_post (&sem_fits);
  sem_post (&sem_fits);
  sem_post (&sem_fits);
  pthread_exit((void *) 0);
} /* end spoolThread */

int main (int argc, char *argv[])
{
  FILE *tmpFile;                   /* Temporary file to ckeck open pipe */
  unsigned long mask = 1;          /* Set affinity to processors 1 */
  int devNum=0;                    /* Open pipe number */
  char devName[50];                /* Pipe's device name */
  int i;                           /* used in for closing sockets fds */

/* Assign threads to array of functions */
  PTF fd_tread[NUM_THREADS] = {&socketThread, &socketThread, &socketThread, &spoolThread};

/* Define thread control vars */
  pthread_t threads[NUM_THREADS];
  int rc, t, status;
  size_t stacksize;
  threadData thread_data_array[NUM_THREADS];

  /* Init semaphore */
  sem_init(&sem_fits, 0, 1);


/* Set affinity to CPU 1 (0 for current process) */
  if (sched_setaffinity(0, sizeof(mask), (unsigned long *) &mask) < 0) { 
    perror("sched_setaffinity");
  }	  

/* install signal handler */
  signal(SIGTERM, cleanExit);	
  signal(SIGINT, cleanExit);

/* Open sockets */
  sockfd[GPSFD]=openServer(gps);
  sockfd[OPLEFD]=openServer(ople);
  sockfd[TIPTILTFD]=openServer(tiptilt);

  devNum=0;
  do{
    /* Check existence of pipe */
    sprintf(devName,"/proc/rtai/registry/pipes/pipe%d", devNum);
    tmpFile = fopen(devName, "r");
    if(tmpFile == NULL) {
      sprintf(devName,"/proc/rtai/registry/pipes/pipe%d", devNum-1);
      printf("Opening %s\n",devName);	      
      break;
    }
    fclose(tmpFile);
    printf("Skipping %s\n",devName);
    devNum ++;
  } while(devNum < 23);
  
  /* Open pipe */
  sockfd[SPOOLFD] = open(devName, O_RDONLY);

  /* Initialize and set thread detached attribute */
  pthread_attr_init(&attr);
  pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
  pthread_attr_getstacksize (&attr, &stacksize);

  printf("Default stack size = %d\n", stacksize);
  stacksize = sizeof(double)*N*N+MEGEXTRA;
  printf("Amount of stack needed per thread = %d\n",stacksize);
  pthread_attr_setstacksize (&attr, stacksize);
  printf("Creating threads with stack size = %d bytes\n",stacksize);

  for(t=0;t<NUM_THREADS;t++){
    printf("Creating thread %d\n", t);

    thread_data_array[t].thread_id = t;
    thread_data_array[t].socknum = t;
    thread_data_array[t].message = STATUSMSG;

    rc = pthread_create(&threads[t], &attr, fd_tread[t], 
			(void *) &thread_data_array[t]);
    if (rc){
      printf("ERROR; return code from pthread_create() is %d\n", rc);
      exit(-1);
    }
  }

  /* Free attribute and wait for the other threads */
  pthread_attr_destroy(&attr);
  for(t=0; t<NUM_THREADS; t++) {
    rc = pthread_join(threads[t], (void **)&status);
    if (rc)
      {
	printf("ERROR; return code from pthread_join() is %d\n", rc);
	exit(-1);
      }
    printf("Completed join with thread %d status= %d\n",t, status);
  }

  for(i=0;i<SOCKNUM;i++) close(sockfd[i]);
  pthread_exit(NULL);
  sem_destroy(&sem_fits); 
}
