/************************************************************************/
/* astrogtk_control_window.c                                            */
/*                                                                      */
/* Routine for controlling the basic CHARA astrogtk functions.		*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE                        */
/*                 Based on the SUSI User Interface                     */
/*              In turn based on the CHIP User interface                */
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA                 */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu                                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/* Author : Theo ten Brummelaar                                         */
/* Date   : May   2006                                                  */
/************************************************************************/

#include "astrogtk.h"
#include "../../cliserv/include/chara_messages.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include "../charalib/chara.h"
#include "../slalib/slalib.h"

/* Globals */

static GtkWidget *astrogtk_control_window;
static char astrogtk_control_window_up = FALSE;
struct s_star_message { int server; int chara_number; int hd;};
static struct s_star_message nostar;
static struct s_star_message zenith;
static struct s_star_message internal;
static struct s_star_message autocoll;
static struct s_star_message chara;
static GtkWidget *chara_entry;
static GtkWidget *hd_entry;

#define ASTRO_CONTROL_WIDTH	200

/************************************************************************/
/* create_astrogtk_control_window()					*/
/*									*/
/* ROutine for creating the astrogtk control window. This should be self*/
/* contained and give you all you need to control the basic astrogtk 	*/
/* functionality. Argumentis the window name.				*/
/************************************************************************/

GtkWidget* create_astrogtk_control_window (GtkButton *thebutton, gpointer *data)
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *button;
	GtkWidget *label;
	char	s[81];
	struct astro_control_window_data *the_data;
	int	hd;
	struct s_star star;

	/* Do nothing if we exist already */

	if (astrogtk_control_window_up) return astrogtk_control_window;

	/* Get the data */

	the_data = (struct astro_control_window_data *)data;

	/* Setup message data */

	nostar.server = the_data->server;
	nostar.chara_number = NOSTAR;
	nostar.hd = 0;
	zenith.server = the_data->server;
	zenith.chara_number = ZENITH;
	zenith.hd = 0;
	internal.server = the_data->server;
	internal.chara_number = INTERNAL;
	internal.hd = 0;
	autocoll.server = the_data->server;
	autocoll.chara_number = AUTOCOLL;
	autocoll.hd = 0;
	chara.server = the_data->server;
	chara.chara_number = chara_number[the_data->server];
	chara.hd = 0;

	/* Find out which star we are looking at */

	if (!astrogtk_get_current_star(the_data->server))
	    create_message_window("Failed to get current star.");

	/* Get the HD number for this star */

	if (chara_number[the_data->server] > 0)
	{
		if (!get_star_by_number(chara_number[the_data->server],&star))
		{
		    create_message_window("Failed to find CHARA_%d.",
			chara_number[the_data->server]);
		    hd = 0;
		}
		else
		{
			sscanf(star.hd,"HD_%d",&hd);
		}
	}
	else
		hd = 0;

	/* Create the window itself */

	astrogtk_control_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title (GTK_WINDOW (astrogtk_control_window),
		the_data->name);
	gtk_container_set_border_width (GTK_CONTAINER 
		(astrogtk_control_window), 1);

	/* Box to put things in */

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_add (GTK_CONTAINER (astrogtk_control_window), vbox);
	gtk_widget_show(vbox);

        /* Make an hbox to put things in */

        hbox = gtk_hbox_new(FALSE, 0);
        gtk_container_add (GTK_CONTAINER (vbox), hbox);
        gtk_widget_show(hbox);

	/* NOSTAR button */

	button = gtk_button_new_with_label ("NOSTAR");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC (astrogtk_setstar_callback), (gpointer)&nostar);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_widget_set_usize(button, ASTRO_CONTROL_WIDTH/2, 24);
	gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_show(button);

	/* ZENITH button */

	button = gtk_button_new_with_label ("ZENITH");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC (astrogtk_setstar_callback), (gpointer)&zenith);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_widget_set_usize(button, ASTRO_CONTROL_WIDTH/2, 24);
	gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_show(button);

        /* Make an hbox to put things in */

        hbox = gtk_hbox_new(FALSE, 0);
        gtk_container_add (GTK_CONTAINER (vbox), hbox);
        gtk_widget_show(hbox);

	/* INTERNAL button */

	button = gtk_button_new_with_label ("INTERNAL");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC (astrogtk_setstar_callback),
		(gpointer)&internal);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_widget_set_usize(button, ASTRO_CONTROL_WIDTH/2, 24);
	gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_show(button);

	/* AUTOCOLL button */

	button = gtk_button_new_with_label ("AUTOCOLL");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
		GTK_SIGNAL_FUNC (astrogtk_setstar_callback),
		(gpointer)&autocoll);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_widget_set_usize(button, ASTRO_CONTROL_WIDTH/2, 24);
	gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_show(button);

        /* Make an hbox to put things in */

        hbox = gtk_hbox_new(FALSE, 0);
        gtk_container_add (GTK_CONTAINER (vbox), hbox);
        gtk_widget_show(hbox);

        /* Now put the label and the entry area for CHARA number */

        label = gtk_label_new("CHARA Number");
        gtk_widget_set_usize (label, ASTRO_CONTROL_WIDTH/2, 24);
        gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
        gtk_widget_show(label);

        chara_entry = gtk_entry_new ();
        sprintf(s,"%d", chara_number[the_data->server]);
        gtk_entry_set_text (GTK_ENTRY (chara_entry),s);
        gtk_widget_set_usize (chara_entry, ASTRO_CONTROL_WIDTH/2, 24);
        gtk_box_pack_start(GTK_BOX(hbox), chara_entry,TRUE,TRUE, 0);
        gtk_signal_connect (GTK_OBJECT (chara_entry), "activate",
                        GTK_SIGNAL_FUNC (astrogtk_chara_callback), NULL);
        gtk_widget_show(chara_entry);

        /* Make an hbox to put things in */

        hbox = gtk_hbox_new(FALSE, 0);
        gtk_container_add (GTK_CONTAINER (vbox), hbox);
        gtk_widget_show(hbox);

        /* Now put the label and the entry area for HD number */

        label = gtk_label_new("HD Number");
        gtk_widget_set_usize (label, ASTRO_CONTROL_WIDTH/2, 24);
        gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
        gtk_widget_show(label);

        hd_entry = gtk_entry_new ();
        sprintf(s,"%d", hd);
        gtk_entry_set_text (GTK_ENTRY (hd_entry),s);
        gtk_widget_set_usize (hd_entry, ASTRO_CONTROL_WIDTH/2, 24);
        gtk_box_pack_start(GTK_BOX(hbox), hd_entry,TRUE,TRUE, 0);
        gtk_signal_connect (GTK_OBJECT (hd_entry), "activate",
                        GTK_SIGNAL_FUNC (astrogtk_hd_callback), NULL);
        gtk_widget_show(hd_entry);

        /* Make an hbox to put things in */

        hbox = gtk_hbox_new(FALSE, 0);
        gtk_container_add (GTK_CONTAINER (vbox), hbox);
        gtk_widget_show(hbox);

	/* CANCEL button */

	button = gtk_button_new_with_label ("CANCEL");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
	    GTK_SIGNAL_FUNC (astrogtk_control_ok_callback), (gpointer)P_FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_widget_set_usize(button, ASTRO_CONTROL_WIDTH/2, 24);
	gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_show(button);

	/* OK button */

	button = gtk_button_new_with_label ("OK");
	gtk_signal_connect (GTK_OBJECT (button), "clicked",
	    GTK_SIGNAL_FUNC (astrogtk_control_ok_callback), (gpointer)P_TRUE);
	gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
        gtk_widget_set_usize(button, ASTRO_CONTROL_WIDTH/2, 24);
	gtk_container_set_border_width (GTK_CONTAINER(button),1);
	gtk_widget_show(button);

	/* Now, we can show this window if we like */

	gtk_widget_show(astrogtk_control_window);

	/* Make sure we only get one version of this window */

	astrogtk_control_window_up = TRUE;

	/* That should be all */

	return astrogtk_control_window;

} /* create_astrogtk_control_window() */

/************************************************************************/
/* astrogtk_control_ok_callback()					*/
/*									*/
/* Callback function for astrogtk control OK button. Stop background	*/
/* processing and destroy astrogtk control window.			*/
/************************************************************************/

void astrogtk_control_ok_callback(GtkButton *button, gpointer doit)
{
	if (!astrogtk_control_window_up) return;

	if (*((int *)doit))
	{
		astrogtk_chara_callback(button, NULL);
	}

	gtk_widget_destroy(astrogtk_control_window);
	astrogtk_control_window_up = FALSE;

} /* astrogtk_control_ok_callback() */

/************************************************************************/
/* astrogtk_setstar_callback()						*/
/*									*/
/************************************************************************/

void astrogtk_setstar_callback(GtkButton *button, gpointer user_data)
{
	struct	s_star_message *star;
	struct	smessage mess;
	char	s[81];

	star = (struct s_star_message *)user_data;

	mess.type = ASTROMOD_SET_STAR;
	mess.data = (unsigned char *)&(star->chara_number);
	mess.length = sizeof(int);

	if (!send_message(star->server, &mess))
	{
	    create_message_window("Failed to send ASTROMOD_SET_STAR message.");
	    return;
	}

	chara_number[star->server] = star->chara_number;

        sprintf(s,"%d", star->chara_number);
        gtk_entry_set_text (GTK_ENTRY (chara_entry),s);

        sprintf(s,"%d", star->hd);
        gtk_entry_set_text (GTK_ENTRY (hd_entry),s);
	

} /* astrogtk_setstar_callback() */

/************************************************************************/
/* astrogtk_chara_callback()						*/
/*									*/
/************************************************************************/

void astrogtk_chara_callback(GtkButton *button, gpointer user_data)
{
	char	*entry;
	struct s_star star;
	int	chara_number;
	int	hd;
	
	entry = gtk_entry_get_text(GTK_ENTRY(chara_entry));
	sscanf(entry,"%d", &chara_number);
	if (chara_number <= 0) return;
	if (!get_star_by_number(chara_number,&star))
	{
	    create_message_window("Failed to find CHARA_%d.",chara_number);
	    return;
	}
	else
	{
	    sscanf(star.hd,"HD_%d",&hd);
	}
	chara.chara_number = chara_number;
	chara.hd = hd;

	astrogtk_setstar_callback(button, (gpointer)&chara);

} /* astrogtk_chara_callback() */

/************************************************************************/
/* astrogtk_hd_callback()						*/
/*									*/
/************************************************************************/

void astrogtk_hd_callback(GtkButton *button, gpointer user_data)
{
	char	*entry;
	struct s_star star;
	int	hd;
	
	entry = gtk_entry_get_text(GTK_ENTRY(hd_entry));
	sscanf(entry,"%d", &hd);
	if (!get_star_by_hd(hd,&star))
	{
	    create_message_window("Failed to find HD_%d.",hd);
	    return;
	}
	chara.chara_number = star.chara_number;
	chara.hd = hd;

	astrogtk_setstar_callback(button, (gpointer)&chara);

} /* astrogtk_hd_callback() */

/************************************************************************/
/* astrogtk_get_current_star()						*/
/*									*/
/* Try and find out what the current star is right now.			*/
/* Returns TRUE if things go well.					*/
/************************************************************************/

bool astrogtk_get_current_star(int server)
{
        struct smessage mess;
        time_t  current_time;
	int	old_chara_number;

        /* The current targets are not valid */

	old_chara_number = chara_number[server];
        chara_number[server] = -100;

        /* Send message requesting targets */

        mess.type = ASTROMOD_WHICH_STAR;
        mess.length = 0;
        mess.data = NULL;

        if (!send_message(server, &mess))
        {
                create_message_window(
                        "Failed to send ASTROMOD_WHICH_STAR message.");
		chara_number[server] = old_chara_number;
		return FALSE;
        }

        /* Now wait for a while and see if they are updated */

        current_time = time(0);
        while(chara_number[server] == -100)
        {
                process_server_socket(server);
                if (time(0) > current_time + 2)
		{
			chara_number[server] = old_chara_number;
			return FALSE;
		}
        }

        /* That is all */

        return TRUE;

} /* astrogtk_get_current_star() */
