/************************************************************************/
/* astromod_calc.c                                                      */
/*                                                                      */
/* Main routines for caculation of all things for astrometric modeling. */
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                  	*/
/* Date   : April 2006                                                  */
/************************************************************************/

#include "../charalib/chara.h"
#include "../../cliserv/include/chara_messages.h"
#include "astromod.h"

//#define DEBUG

/* Globals */

TELESCOPE scopes[MAX_NUMBER_TELESCOPES];
SYSTEM    chara;
static char astromod_initialized = FALSE;

/************************************************************************/
/* astromod_init()							*/
/*									*/
/* Forces system to re-read system information files.			*/
/* Returns 0 if all went well, -1 if it fails to get the data.		*/
/* Returns -2 if the beams are bad, or -3 if pops are bad.		*/
/************************************************************************/

int astromod_init(int *beams, int *pops)
{
	int	i;

	/* Get system data */

	if (extract_system_data(&chara, scopes) != 0) return -1;
	
#ifdef DEBUG
	if (beams != NULL && pops != NULL)
	{
	  for(i=0; i<NUM_SCOPES; i++)
		printf("%d %d %d\n", i, beams[i], pops[i]);
	}
#endif

	/* Set up the beams */

	if (beams == NULL)
	  for(i=0;i<NUM_SCOPES;i++) scopes[i].beam = scopes[i].default_beam;
	else
	{
	  for(i=0;i<NUM_SCOPES;i++)
	  {
	    if (beams[i] < 0 || beams[i] > NUM_BEAMS) return -2;
	    scopes[i].beam = beams[i];
	  }
	}

	/* Setup the pops */

	if (pops == NULL)
	  for(i=0;i<NUM_SCOPES;i++) scopes[i].pop = scopes[i].default_pop;
	else
	{
	  for(i=0;i<NUM_SCOPES;i++)
	  {
	    if (pops[i] < 0 || pops[i] > NUM_POPS) return -3;
	    scopes[i].pop = pops[i];
	  }
	}

	astromod_initialized = TRUE;

	return 0;

} /* astromod_init() */

/************************************************************************/
/* astromod_mjd()							*/
/*									*/
/* Works out the modified Julian Date based on a date and a time.	*/
/* Returns result, or -1 if not initialized, -2 on error.		*/
/************************************************************************/

double astromod_mjd(int year,             /* Which year */
		    int month,            /* 1..12, the month */
		    int day,              /* 1..31 the day of month */
		    double ut)            /* UT time, 2*M_PI = 24hrs */ 
{
	double	mjd;
	int	j;

	if (!astromod_initialized) return -1.0;

	/* Calculate the epoch */

	slaCldj(year, month, day, &mjd, &j);

	/* Did this work? */

	if (j >= 1 && j <=3) return -2.0;

	/* Now add the timne of data */

	mjd += (ut/(2.0*M_PI));

#ifdef DEBUG
	fprintf(stderr,"mjd = %.9lf\n", mjd);
#endif

	return mjd;

} /* astromod_mjd() */

/************************************************************************/
/* astromod_atomic_time()						*/
/*									*/
/* Works out the international atomic time from modifiede julian date.	*/
/* Returns time or -1 if uninitialized.					*/
/************************************************************************/

double astromod_atomic_time(double mjd)
{
	double	tai,tt;

	if (!astromod_initialized) return -1.0;

	/* International atomic time calculation */

	tai = mjd + slaDat(mjd)/SECS_IN_DAY;
	tt = tai + 32.184/SECS_IN_DAY;

#ifdef DEBUG
	fprintf(stderr,"tai = %.9lf\n", tai);
	fprintf(stderr,"tt  = %.9lf\n", tt);
#endif

	return tt;

} /* astromod_atomic_time() */

/************************************************************************/
/* astromod_azel_date()							*/
/*									*/
/* All results in radians.						*/
/* Return 0 if all goes well, -1 on initialization error.		*/
/* Returns -2 on atomic time error, -3 on MJD error.			*/
/************************************************************************/

int astromod_azel_date(struct s_star star, 	/* Information on the star */
		       int    year, 	/* Which year */
		       int    month,	/* 1..12, the month */
		       int    day, 	/* 1..31 the day of month */
		       double ut, 	/* UT time, 2PI=24hrs */ 
		       double *az, 	/* Star position in radians */
		       double *el,	/* Star position in radians */
		       double *ref_el,	/* Refracted star position in radians*/
		       double *hour_angle)	/* Hour angle */
{
	double	mjd;

	if (!astromod_initialized) return -1;

	/* Get modified Julian Date */

	mjd = astromod_mjd(year, month, day, ut);
	if (mjd < 0.0) return -3;

	return astromod_azel_mjd(star, mjd, az, el, ref_el, hour_angle);

} /* astromod_azel_date() */

/************************************************************************/
/* astromod_azel_mjd()							*/
/*									*/
/* All results in radians.						*/
/* Return 0 if all goes well, -1 on initialization error.		*/
/* Returns -2 on atomic time error.					*/
/************************************************************************/

int astromod_azel_mjd(struct s_star star, 	/* Information on the star */
		      double mjd,		/* Modified Julian Date */
		      double *az, 		/* Star position in radians */
		      double *el,		/* Star position in radians */
		      double *ref_el,	/* Refracted star position in radians*/
		      double *hour_angle)	/* Hour angle */
{
	double  temp, pressure, rel_humid, lapse;
	double	tt;
	double	ara, adec;
	double 	aob, zob, hob, dob, rob;
	int	i;
	VECTOR	to_star;

	if (!astromod_initialized) return -1;

	/* Are we pointing at anything useful? */

	if (star.chara_number <= 0)
	{
		*az = 0.0;
		*el = M_PI/2.0;
		*ref_el = M_PI/2.0;
		*hour_angle = 0.0;
		return 0;
	}

	/* Get the atomic time, done manually here for speed */

	if ((tt = astromod_atomic_time(mjd)) < 0.0) return -2;

	/* Find apparant position of star on the sky */

	slaMap(star.ra, star.dec, star.pm_ra, star.pm_dec,
			star.parallax, star.radvel, 2000.0, tt, &ara, &adec);

	/* Get refracted star position. */

#warning We should have REAL weather data here.

	temp = TEMP_DFT;
	pressure = PRESSURE_DFT;
	rel_humid = RH_DFT;
	lapse = LAPSE_DFT;

	slaAop(ara, adec, mjd, chara.dut,
			chara.longitude, chara.latitude,
			chara.height,
			chara.pole.x, chara.pole.y,
			temp, pressure, rel_humid, chara.obs_wavelength, lapse,
			&aob, &zob, &hob, &dob, &rob);

	*ref_el = M_PI/2.0 - zob;

	/* Get real star position */

	temp = 300.0;
	pressure = 0.0;
	rel_humid = 0.0;
	lapse = 0.0;
		
	slaAop(ara, adec, mjd, chara.dut,
			chara.longitude, chara.latitude,
			chara.height,
			chara.pole.x, chara.pole.y,
			temp, pressure, rel_humid, chara.obs_wavelength, lapse,
			&aob, &zob, &hob, &dob, &rob);

	*az = slaDranrm(aob);
	*el = M_PI/2.0 - zob;
	while (hob < M_PI) hob += (2.0*M_PI);
	while (hob > M_PI) hob -= (2.0*M_PI);
	*hour_angle = hob;
	

#ifdef DEBUG
	fprintf(stderr,"    Az = %.4f\n",*az/D2R);
	fprintf(stderr,"    El = %.4f\n",*el/D2R); 
	fprintf(stderr,"Ref El = %.4f\n",*ref_el/D2R); 
#endif

	/* We assume all scopes are pointing at this star */

	to_star = astromod_star_vector(*az, *el);

	for(i=0; i<NUM_SCOPES; i++)
	{
		scopes[i].star.az = *az;
		scopes[i].star.el = *el;
		scopes[i].to_star.x = to_star.x;
		scopes[i].to_star.y = to_star.y;
		scopes[i].to_star.z = to_star.z;
		scopes[i].hour_angle = hob;
	}

	/* That should be all */

	return 0;

} /* astromod_azel_mjd() */

/************************************************************************/
/* astromod_star_vector()						*/
/*									*/
/* Calculates vector pointing to the star.				*/
/************************************************************************/

VECTOR astromod_star_vector(double az, double el)
{
	VECTOR to_star;

	to_star.x = cos(el)*sin(az);
	to_star.y = cos(el)*cos(az);
	to_star.z = sin(el);

	return to_star;

} /* astromod_star_vector() */

/************************************************************************/
/* astromod_cart_position()						*/
/*									*/
/* Calculate the required position for a cart.				*/
/* Result is the position of the cart in meters. Double this to get	*/
/* optical delay.							*/
/* A rediculously large number indicates and error.			*/
/************************************************************************/

double astromod_cart_position(int cart, int ref_cart, double n_air, 
			      double az, double el, int mode)
{
	/*
	 * Internals
	 */
	
	double	DeltaStarPath, DeltaVacuumPath, DeltaAirPath, DeltaCartPos;
	double  model_corrections;
	double	sin_az;
	double	cos_az;
	double	sin_el;
	double	cos_el;
	int 	i,j;
	VECTOR  to_star;
	double  A;
	
	if (!astromod_initialized) return 1e32;

#ifdef DEBUG
	printf("astromod_cart_pos: %d %d %lf %lf %lf\n", cart, ref_cart, 
		n_air, az/D2R, el/D2R);
#endif

	/* Are we the reference star ? */

	if (cart == ref_cart || mode == NOSTAR) return 0.0;

	/*
	 * Set i and j (a more compact looking variables)
	 * to be the moving and reference cart
	 */
	i = cart;
	j = ref_cart;

	
	/* For AUTOCOLL and INTERNAL modes there is no sky contribution */

	if (mode == AUTOCOLL || mode == INTERNAL)
	{
		DeltaStarPath = 0.0;
                model_corrections = 0.0;
	}
	else
	{

		/* Where is the star? */

		to_star = astromod_star_vector(az, el);

/* 
 * According to Iphase the 'j' telescope or reference scope is
 * the number '1' scope and the other 'i' scope is the number '2' scope.
 * Since we define 1 as the reference all solutions should only use T2 values.
 * These should be calculated wrt to the reference scope, ie S1 (normally).
 * The reference scope should have all zeros in it's model file.
 * The OPD is defined as the difference in cart positions:
 * 
 * OPD = cart(2) - cart(1)
 * 
 * So, a positive OPD means increasing the cart distance for chanel 2 and
 * visa versa.
 *
 * The Iphase corrections ADD to the supplied OPDs to give the value
 * you would have got if the corresponding effects had not been present.
 * That means you really need to SUBTRACT the Iphase corrections
 * from cart(2) or add them to cart(1).
 *
 * Let's begin with the X coordinate. The sign convention for Iphase is 
 * different. In Iphase We have Azimuth as zero in the south so
 * Here A is what Iphase uses and Az is what we use, similarly with E and El, so
 * 
 * dOPD = (T1X - T2X) * cos(E) * sin(A)
 *      = (T1X - T2X) * cos(El) * sin(PI - Az)
 *      = (T1X - T2X) * cos(El) * sin(Az)
 *      = (T1X - T2X) * star.x
 *
 * However there is also a sign convention difference, X is +ve in the west in
 * Iphase and +ve in the East in our system so we should use
 *
 * dOPD = X_2 - X_1
 *
 */

	DeltaStarPath = (scopes[i].baseline.x - scopes[j].baseline.x)
		     * to_star.x

/* 
 * As for the Y axis we start with
 *
 * dOPD = (T2Y - T1Y) * cos(E) * cos(A)
 *      = (T2Y - T1Y) * cos(El) * cos(PI - Az)
 *	= (T1Y - T2Y) * cos(El) * cos(Az)
 *      = (T1Y - T2Y) * star.y
 *	
 * However once again we have a sign flip between Iphase and us so we use
 *
 * dOPD = Y_2 - Y_1
 * 
 */
		 + (scopes[i].baseline.y - scopes[j].baseline.y) 
		     * to_star.y

/* 
 * Now for the Z axis we have
 * 
 * dOPD = (T2Z - T1Z) * sin(El) 
 *	= (T2Z - T1Z) * star.z
 * 
 * and no change in sign convention so we get
 */
		 + (scopes[i].baseline.z - scopes[j].baseline.z) 
		     * to_star.z;

/* 
 * Now we consider the other terms like swash and so on.
 * We now have to explicitly take into account the different
 * definitions of A and Az (Az = PI - A).
 * See the Iphase doco for this.
 */

		if ((A = M_PI - az) < 0.0) A += (2.0*M_PI);
		sin_el = sin(el);
		cos_el = cos(el);
		sin_az = sin(A);
		cos_az = cos(A);

#ifdef DEBUG
		printf("El = %lf A = %lf\n", el/D2R, A/D2R);
#endif

/* 
 * Now we can calculate the model corrections as 
 * per the Iphase doco.
 */

	model_corrections =   

/* 
 * First, the Cos(El) effects 
 * These , in Iphase, are listed as
 * 
 * dOPD = (T1EC - T2EC)*cos(E)
 *
 * but the correction needs to be subtracted
 * from the cart(2) position so
 */

		  (scopes[i].ec - scopes[j].ec) * cos_el

/* 
 * Then the Swash sin component listed as
 * 
 * dOPD = (T1SAS - T2SAS) * sin(A) * (1-sin(E))
 *		
 * again, this needs to be subtracted from the 
 * cart(2) position
 */
		
		+ (scopes[i].sas - scopes[j].sas) * sin_az * (1.0 - sin_el)


/* 
 * The swash cos component 
 *
 * dOPD = (T1SAC - T2SAC) * cos(A) * (1-sin(E))
 * 
 * and the same sign flip
 */

		+ (scopes[i].sac - scopes[j].sac) * cos_az * (1.0 - sin_el)

/*
 * The sin(2*Az) Harmonic  
 * with the same sign flip.
 */

		+ (scopes[i].hsa2 - scopes[j].hsa2) * sin(2.0*A);
	}

	/* If this is INTERNAL mode there are no light pipes etc */

	if (mode == INTERNAL)
	{

		DeltaVacuumPath = 0.0;
                DeltaAirPath = scopes[j].internal-scopes[i].internal;
	}
	else
	{

/*
 * The total differential path length in a vacuum is due to 
 * the differential star light path length, the differential
 * light pipes length from the telescopes to the BCL,
 * and the differential POP lengths.
 * We must also make sure we get the sign convention the same
 * as that used in Iphase. In theory we know the vacuum path
 * and Iphase tells us the airpath. In anycase Iphase uses
 * 
 * dOPD = A1L - A2L
 *	
 * to give the position of cart(2).
 */

		DeltaVacuumPath = DeltaStarPath  
			 + (scopes[j].lightpipe - scopes[i].lightpipe)
			 + (scopes[j].poppath[scopes[j].pop] 
			 - scopes[i].poppath[scopes[i].pop]);

/*
 * Then add in the differential path length
 * within the telescopes and otherwise outside the lightpipes.
 * Again the sign convention comes from Iphase:
 *
 * dOPD = A1L - A2L
 */

		DeltaAirPath = scopes[j].airpath - scopes[i].airpath
			       + scopes[j].beampath[scopes[j].beam]
			       - scopes[i].beampath[scopes[i].beam];
	}

/*
 * Mix it all together -  
 * The factor of 0.5 out front comes from the fact
 * that the science light does a double pass.  So for any given
 * cart position, the imposed path delay is doubled. 
 * Note, we are correcting the vacuum path for the refractive index
 * of air, so this is a physical position for the cart.
 */

#warning WE ARE OVER-RIDING VACUUM STUFF!! PERHAPS THIS IS BAD.
	n_air = 1.0;
	DeltaCartPos = 0.5 * (DeltaAirPath + (DeltaVacuumPath / n_air) + 
			model_corrections);

#ifdef DEBUG
	printf("dS = %13.9lf dA = %13.9lf dV = %13.9lf dM = %13.9lf\n", 
	DeltaStarPath, DeltaAirPath, DeltaVacuumPath, model_corrections);
#endif

	/* That should be all */

	return DeltaCartPos;
}

/************************************************************************/
/* astromod_uv()                                                        */
/*                                                                      */
/* Attempts to calculate the UV points given Dec, Hour angle            */
/* in radians and the Bx, By, and Bz of the baseline in meters.         */
/* Wavelength specified in meters also. Result in cycles/arcsec.        */
/* Formulae taken from Wes Traub's summer school notes.			*/
/* Returns 0 if all OK, -2 on wavelength error, -1 if not initialized.	*/
/************************************************************************/

int astromod_uv(double dec, double ha, double Bx, double By, double Bz,
                double lambda, double *u, double *v)
{
	if (!astromod_initialized) return -1;

	if (lambda <= 0.0 || lambda >= 20.0) return -2;

        *u = (Bx * cos(ha) -
                By * sin(chara.latitude) * sin(ha) +
                Bz * cos(chara.latitude) * sin(ha)) / (206265.0 * lambda);

        *v = (Bx * sin(dec) * sin(ha) +
                By * (sin(chara.latitude) * sin(dec) * cos(ha) +
                        cos(chara.latitude) * cos(dec)) -
                Bz * (cos(chara.latitude) * sin(dec) * cos(ha) -
                        sin(chara.latitude) * cos(dec))) / (206265.0 * lambda);

	return 0;

} /* astromod_uv() */
