/************************************************************************/
/* @(#)files.c	1.3 4/16/94						*/
/*									*/
/* Standard file handling routines on UNIX				*/
/*									*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Bill Tango		                                        */
/* Date   : Mar 1993							*/
/************************************************************************/

/* include files */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <curses.h>
#include <errno.h>
#include "../charaui/src/charaui.h"
#include "../rwfits/rwfits.h"
#include "../cfitsio/fitsio.h"
#include "chara.h"
#include "../../modules/irimage/irimage.h"
#include "../../modules/classic/classic.h"
#define MAX_N   100000000

#define index_cube(i,j,k) *(data_cube+i*NUM_DATA_TYPES*data_size+j*data_size+k)

/****************************************************************/
/*	fgetline()						*/
/*	This function reads the next NON-BLANK line into 	*/
/*	buffer. The terminating newline character is retained	*/
/*	in the buffer.  					*/
/*	Returns:  -1 if EOF encountered.			*/
/*		  length of the line (including newline char)	*/
/*		  otherwise.					*/
/*	If fgetline() encounters a blank line (only whitespace) */
/*	it will continue to read lines until it finds a non-	*/
/*	blank line or the end of file.				*/
/****************************************************************/
 
int fgetline(char *buffer, int buf_len, FILE *file)

{
	int i, count;
	
	/* 
	 *   Use fgets to get next line.  If it returns
 	 *   NULL we are at end-of-file, so return -1 and quit.
         */
	if (fgets(buffer, buf_len, file) == NULL)
		return (-1);
	else	{
		/*
		 * Search for non-whitespace character. If
		 * one is found return the value of count and
		 * and quit.
		 */
		count = strlen(buffer);
		for ( i = 0 ; i < count ; ++i)
			if ( !isspace(buffer[i]) ) return (count);
		/*
		 * If we are here line consisted entirely of
 		 * whitespace. Call fgetline() recursively to
	         * get next non-blank line.  
		 */
		return fgetline(buffer, buf_len, file);
		}

}

/****************************************************************/
/*	GetDataLine()						*/
/*								*/
/*	This is a specialized version of fgetline which		*/
/*	ignores comment lines (first character of line # or ;)	*/
/*								*/
/****************************************************************/

int GetDataLine(char buffer[], int buflen, FILE *fp)
{
	/*	Internals	*/
	int ret_val;

	/*	
	 *	Loop through input file.  We return either when
	 *	the end of file is encountered or a non-comment 
	 *	line is found.
	 */
	while ((ret_val = fgetline(buffer, buflen, fp)) > 0)
		{
		if (!( (buffer[0] == '#') || (buffer[0] == ';')) )
			return ret_val;
		}
	/*	If we are here the EOF has been reached	 */
	return ret_val;
}

/****************************************************************/
/*	ToUpper(s)						*/
/*	This function converts a string to upper case.  It is	*/
/*	safer to use than the standard toupper() function.	*/
/****************************************************************/

char *ToUpper(char *s)
{
	/* Internals */
	int i;
	char c;

	for (i = 0; (c = s[i]) != '\0'; ++i)
	{
		/*  Check that c is a lower case 
		 *  character, and convert it if it is.
		 *  Note that toupper() may return garbage if
		 *  the argument is not a lower case letter.
		 */

		/* This assumes a certain order to the character set. */

		if (c >= 'a' && c <= 'z') s[i] = c - 'a' + 'A';

		/* 
		 * This fails on RedHat 9.0
		 *
		 * if ( islower(c) ) s[i] = (char)toupper( c );
		 */
	}

	return s;
}

/************************************************************************/
/* read_data_file()							*/
/*									*/
/* This routine rreads an irimage or classic data file. It ignores	*/
/* Return value is the number of items actually read in.		*/
/* All memory is allocated within this function, and there's a lot!	*/
/* It calls either read_data_file_irimage() or read_data_file_classic() */
/* depending on if it has a .dat or .fit extension.			*/
/************************************************************************/

int read_data_file(char *filename, struct chara_tm *gmt_now,
	CHARA_TIME 	**now,
	float		**dither,
	int		**h_1,
	int		**h_2,
	int		**k_1,
	int		**k_2,
	float		*mean_time)
{
	char	*p, extname[512];

	/* What is the file extension part? */

        for(p = filename; *p != 0; p++);
        for(; *p != '.' && p != filename; p--);
        if (p != filename && *p == '.' && *p != 0)
                strcpy(extname, ++p);
        else
                *extname = 0;

        /* Is this a fits file or what? */

        if (strcmp(extname,"fit") == 0 || strcmp(extname,"fits") == 0)
		return read_data_file_classic(filename, gmt_now,
        		now, dither, h_1, h_2, k_1, k_2, mean_time);
        else if (strcmp(extname,"dat") == 0)
		return read_data_file_irimage(filename, gmt_now,
        		now, dither, h_1, h_2, k_1, k_2, mean_time);
	else
		return 0;

} /* read_data_file() */

/************************************************************************/
/* read_data_file_irimage()						*/
/*									*/
/* All memory is allocated within this function, and there's a lot!	*/
/* read_data_file()							*/
/* This routine rreads an irimage data file. It ignores the processed	*/
/* data columns (sums and diffs) and returns only the raw data.		*/
/* The bias is removed from the data, and a warning issued if there	*/
/* is a bias.								*/
/* Return value is the number of items actually read in.		*/
/* All memory is allocated within this function, and there's a lot!	*/
/* This routine is for the data created by irimage.			*/
/************************************************************************/

int read_data_file_irimage(char *filename, struct chara_tm *gmt_now,
	CHARA_TIME 	**now,
	float		**dither,
	int		**h_1,
	int		**h_2,
	int		**k_1,
	int		**k_2,
	float		*mean_time)
{
	int	fp;
	irimage_save data;
	int bias_h_1, bias_h_2;
        int bias_k_1, bias_k_2;
	int	remove_bias;
	int	n;
	CHARA_TIME 	*pCHARA_TIME;
	float		*pfloat;
	int		*pint;
	int		i;

	/* Allocate the memory */

	if ((*now = malloc(MAX_N*sizeof(**now))) == NULL)
	{
		fprintf(stderr,"Failed to get memory for now array.\n");
		exit(-1);
	}
	if ((*dither = malloc(MAX_N*sizeof(**dither))) == NULL)
	{
		fprintf(stderr,"Failed to get memory for dither array.\n");
		exit(-1);
	}
	if ((*h_1 = malloc(MAX_N*sizeof(**h_1))) == NULL)
	{
		fprintf(stderr,"Failed to get memory for h_1 array.\n");
		exit(-1);
	}
	if ((*h_2 = malloc(MAX_N*sizeof(**h_2))) == NULL)
	{
		fprintf(stderr,"Failed to get memory for h_2 array.\n");
		exit(-1);
	}
	if ((*k_1 = malloc(MAX_N*sizeof(**k_1))) == NULL)
	{
		fprintf(stderr,"Failed to get memory for k_1 array.\n");
		exit(-1);
	}
	if ((*k_2 = malloc(MAX_N*sizeof(**k_2))) == NULL)
	{
		fprintf(stderr,"Failed to get memory for k_2 array.\n");
		exit(-1);
	}

	/* Open the file */

	if ((fp = open(filename,O_RDONLY)) == -1)
	{
		fprintf(stderr,"Failed to open file %s:\n\n%s",
			filename,strerror(errno));
		exit(-1);
	}

	/* Read in the gmt */

	if (read(fp, gmt_now, sizeof(*gmt_now)) != sizeof(*gmt_now)) 
	{
		fprintf(stderr,"Error reading GMT from file %s.\n",filename);
		exit(-2);
	}
	
	/* Read in the bias data */

        if (read(fp, &bias_h_1, sizeof(bias_h_1)) != sizeof(bias_h_1) ||
            read(fp, &bias_h_2, sizeof(bias_h_2)) != sizeof(bias_h_2) ||
            read(fp, &bias_k_1, sizeof(bias_k_1)) != sizeof(bias_k_1) ||
            read(fp, &bias_k_2, sizeof(bias_k_2)) != sizeof(bias_k_2))
	{
		fprintf(stderr,"Failed to read bias data.\n");
		exit(-3);
	}

	if ((remove_bias = (bias_h_1 + bias_h_2 + bias_k_1 + bias_k_1 != 0)))
	{
		fprintf(stderr,"Warning: BIAS is being removed from data.\n");
	}

	/* Note... not really Y2K cool.... */

	gmt_now->tm_year = gmt_now->tm_year + 1900;
	if (gmt_now->tm_year < 1950) gmt_now->tm_year+=100;

	*mean_time = 0.0;
	for (n = 0; read(fp, &data, sizeof(data)) == sizeof(data); n++)
	{
		(*now)[n] = data.now;
		*mean_time += (float) data.now;
		(*dither)[n] = data.dither;
		(*h_1)[n] = data.h_1;
		(*h_2)[n] = data.h_2;
		(*k_1)[n] = data.k_1;
		(*k_2)[n] = data.k_2;
#ifdef DEBUG
		if (n%10000 == 0) printf("Reading file %s n=%d\r",filename,n);
#endif
	}
	*mean_time /= (float) n;
	close(fp);

	/* Now re-arrange to save memory */
			
	if ((pCHARA_TIME = malloc(n * sizeof(*pCHARA_TIME))) == NULL)
	{
	    fprintf(stderr,"Ran out of memory in final now allocation.\n");
	    exit(-1);
	}
	for(i=0; i<n; i++) pCHARA_TIME[i] = (*now)[i];
	free(*now);
	*now = pCHARA_TIME;

	if ((pfloat = malloc(n * sizeof(*pfloat))) == NULL)
	{
	    fprintf(stderr,"Ran out of memory in final dither allocation.\n");
	    exit(-1);
	}
	for(i=0; i<n; i++) pfloat[i] = (*dither)[i];
	free(*dither);
	*dither = pfloat;

	if ((pint = malloc(n * sizeof(*pint))) == NULL)
	{
	    fprintf(stderr,"Ran out of memory in final h_1 allocation.\n");
	    exit(-1);
	}
	for(i=0; i<n; i++) pint[i] = (*h_1)[i];
	free(*h_1);
	*h_1 = pint;

	if ((pint = malloc(n * sizeof(*pint))) == NULL)
	{
	    fprintf(stderr,"Ran out of memory in final h_2 allocation.\n");
	    exit(-1);
	}
	for(i=0; i<n; i++) pint[i] = (*h_2)[i];
	free(*h_2);
	*h_2 = pint;

	if ((pint = malloc(n * sizeof(*pint))) == NULL)
	{
	    fprintf(stderr,"Ran out of memory in final k_1 allocation.\n");
	    exit(-1);
	}
	for(i=0; i<n; i++) pint[i] = (*k_1)[i];
	free(*k_1);
	*k_1 = pint;

	if ((pint = malloc(n * sizeof(*pint))) == NULL)
	{
	    fprintf(stderr,"Ran out of memory in final k_2 allocation.\n");
	    exit(-1);
	}
	for(i=0; i<n; i++) pint[i] = (*k_2)[i];
	free(*k_2);
	*k_2 = pint;

	/* That is all */

	return n;

} /* read_data_file_irimage() */

/************************************************************************/
/* read_data_file_classic()						*/
/*									*/
/* All memory is allocated within this function, and there's a lot!	*/
/* This routine rreads an classic data file. 				*/
/* Return value is the number of items actually read in.		*/
/* All memory is allocated within this function, and there's a lot!	*/
/* This routine is for the data created by classic.			*/
/* In this case h_1 and k_1 will contain the same data.			*/
/************************************************************************/

int read_data_file_classic(char *filename, struct chara_tm *gmt_now,
	CHARA_TIME 	**now,
	float		**dither,
	int		**h_1,
	int		**h_2,
	int		**k_1,
	int		**k_2,
	float		*mean_time)

{
	fitsfile *fptr;
	int     status, nfound, anynulls;
	long int naxis[3], fpixel, n;
	float	*data_cube;
	float   nullval;
	int	year,month,day,doy;
	int	data_size, num_scans;
	char	s[512];
	int	i,j,k;

	/* Open the file and read in the data */

        status = 0;
        if (fits_open_file(&fptr, filename, READONLY, &status))
        {
		fits_report_error(stderr, status);
		return 0;
        }

	status = 0;
        if (fits_read_keys_lng(fptr, "NAXIS", 1, 3, naxis, &nfound, &status))
        {
		status = 0;
		fits_report_error(stderr, status);
		if (fits_close_file(fptr, &status) )
		{
			fits_report_error(stderr, status);
		}
		return 0;
        }

        n = naxis[0] * naxis[2];
        fpixel = 1;
        nullval = 0;
        data_cube = calloc((size_t)naxis[0]*naxis[1]*naxis[2], sizeof(float));
        data_size = naxis[0];
        num_scans = naxis[2];

	status = 0;
        if (fits_read_img(fptr, TFLOAT, fpixel, naxis[0]*naxis[1]*naxis[2], 
		&nullval, data_cube, &anynulls, &status))
        {
		fits_report_error(stderr, status);
		if (fits_close_file(fptr, &status) )
		{
			fits_report_error(stderr, status);
		}
		return 0;
        }

	/* Get the UT date */

	status = 0;
	if (fits_read_key_str(fptr, "CCUTDATE", s,NULL,&status))
			fits_report_error(stderr, status);

        if (sscanf(s,"%d-%d-%d", &year, &month, &day) != 3)
                year = month = day = -1;

	status = 0;
        if (fits_read_key(fptr, TINT, "CCDAYOYR", &doy,NULL,&status))
                doy = -1;

	/* Close the file */

	if (fits_close_file(fptr, &status) )
	{
		fits_report_error(stderr, status);
		return 0;
	}

	/* OK, allocate some memory for this */

	if ((*now = malloc(n * sizeof(*now))) == NULL)
	{
	    fprintf(stderr,"Ran out of memory in final now allocation.\n");
	    return 0;
	}
	if ((*dither = malloc(n * sizeof(*dither))) == NULL)
	{
	    fprintf(stderr,"Ran out of memory in final now allocation.\n");
	    return 0;
	}
	if ((*k_1 = malloc(n * sizeof(*k_1))) == NULL)
	{
	    fprintf(stderr,"Ran out of memory in final now allocation.\n");
	    return 0;
	}
	if ((*k_2 = malloc(n * sizeof(*k_2))) == NULL)
	{
	    fprintf(stderr,"Ran out of memory in final now allocation.\n");
	    return 0;
	}
	if ((*h_1 = malloc(n * sizeof(*h_1))) == NULL)
	{
	    fprintf(stderr,"Ran out of memory in final now allocation.\n");
	    return 0;
	}
	if ((*h_2 = malloc(n * sizeof(*h_2))) == NULL)
	{
	    fprintf(stderr,"Ran out of memory in final now allocation.\n");
	    return 0;
	}

	/* Now stuff the data in the appropriate place. */

	k = 0;
	*mean_time = 0.0;
	for(i=0;i<num_scans;i++)
        for(j=0;j<data_size;j++)
        {
		*mean_time += ((*now)[k] = index_cube(i,DATA_TIME,j));
		(*dither)[k] = index_cube(i,DATA_DITHER,j);
		(*h_1)[k] = (*k_1)[k] = index_cube(i,DATA_SIG1,j);
		(*h_2)[k] = (*k_2)[k] = index_cube(i,DATA_SIG2,j);
		k++;
	}
	*mean_time /= (float)n;
	*mean_time *= TICKS_PER_SECOND;

	/* Fill out the time structure */

	i = (int)(*mean_time)/1000;
	gmt_now->tm_isdst = 0;
	gmt_now->tm_yday = doy;
	gmt_now->tm_wday = -1;
	gmt_now->tm_year = year;
	gmt_now->tm_mon = month-1;
	gmt_now->tm_mday = day;
	gmt_now->tm_hour = i/3600;
	i -= (gmt_now->tm_hour * 3600);
	gmt_now->tm_min = i/60 ;
	gmt_now->tm_sec = i - gmt_now->tm_min * 60;

	/* That should be all */

	return n;
		
} /* read_data_file_classic() */

/************************************************************************/
/* read_classic_header()						*/
/*									*/
/* Try and extract what we can from a CLASSIC data fits file header.	*/
/* Returns 0 if all goes well.						*/
/* -1 if it fails to open the file.					*/
/************************************************************************/

int read_classic_header(char *filename, struct s_classic_header *header)
{
	fitsfile *fptr;
	int     status;

	/* Open the file and read in the data */

        status = 0;
        if (fits_open_file(&fptr, filename, READONLY, &status))
        {
		fits_report_error(stderr, status);
		return -1;
        }

	status = 0;
	if (fits_read_key(fptr, TSTRING, "CCSTARHD", header->hd,
		NULL,&status)) strcpy(header->hd,"Unknown");

	status = 0;
	if (fits_read_key(fptr, TINT, "CCCHARA", &(header->chara_number),
		NULL,&status)) header->chara_number = -1;

	status = 0;
	if (fits_read_key(fptr, TSTRING, "CCOBJTYP", header->object,
		NULL,&status)) strcpy(header->object,"Unknown");

	status = 0;
	if (fits_read_key(fptr, TSTRING, "CCOBJTYP", header->object,
		NULL,&status)) strcpy(header->object,"Unknown");

	status = 0;
	if (fits_read_key(fptr, TSTRING, "CCUTDATE", header->ut_date,
		NULL,&status)) strcpy(header->ut_date,"Unknown");

	status = 0;
	if (fits_read_key(fptr, TINT, "CCDAYOYR", &(header->doy),
		NULL,&status)) header->doy = -1;

	status = 0;
	if (fits_read_key(fptr, TSTRING, "CCUTTIME", header->ut_time,
		NULL,&status)) strcpy(header->ut_time,"Unknown");

	status = 0;
	if (fits_read_key(fptr, TSTRING, "CC_BEAM5", header->scope1,
		NULL,&status)) strcpy(header->scope1,"Unknown");

	status = 0;
	if (fits_read_key(fptr, TSTRING, "CC_BEAM6", header->scope2,
		NULL,&status)) strcpy(header->scope2,"Unknown");

	status = 0;
	if (fits_read_key(fptr, TFLOAT, "CC_AZ", &(header->az),
		NULL,&status)) header->az = -1.0;

	status = 0;
	if (fits_read_key(fptr, TFLOAT, "CC_EL", &(header->el),
		NULL,&status)) header->el = -1.0;

	status = 0;
	if (fits_read_key(fptr, TSTRING, "CC_HA", header->hour_angle,
		NULL,&status)) strcpy(header->hour_angle,"Unknown");

	status = 0;
	if (fits_read_key(fptr, TFLOAT, "CC_U", &(header->u),
		NULL,&status)) header->u = 0.0;

	status = 0;
	if (fits_read_key(fptr, TFLOAT, "CC_V", &(header->v),
		NULL,&status)) header->v = 0.0;

	status = 0;
	if (fits_read_key(fptr, TINT, "CCNDATA", &(header->samples_per_scan),
		NULL,&status)) header->samples_per_scan = 0;

	status = 0;
	if (fits_read_key(fptr, TINT, "CCNSCANS", &(header->number_of_scans),
		NULL,&status)) header->number_of_scans = 0;

	status = 0;
	if (fits_read_key(fptr, TINT, "CCSHUTA", &(header->shutter_a),
		NULL,&status)) header->shutter_a = 0;

	status = 0;
	if (fits_read_key(fptr, TINT, "CCSHUTB", &(header->shutter_b),
		NULL,&status)) header->shutter_b = 0;

	status = 0;
	if (fits_read_key(fptr, TINT, "CCLOWSNR", &(header->lowsnr),
		NULL,&status)) header->lowsnr = 0;

	status = 0;
	if (fits_read_key(fptr, TINT, "CCNSKIP", &(header->nskip),
		NULL,&status)) header->nskip = 0;

	status = 0;
	if (fits_read_key(fptr, TINT, "CCDATSCN", &(header->number_data_scans),
		NULL,&status)) header->number_data_scans = 0;

	status = 0;
	if (fits_read_key(fptr, TFLOAT, "CCSAMPLE", 
		&(header->samples_per_second), NULL,&status))
			header->samples_per_second = 0;

	status = 0;
	if (fits_read_key(fptr, TINT, "CCPIXSZE", &(header->pixel_size),
		NULL,&status)) header->pixel_size = 0;

	status = 0;
	if (fits_read_key(fptr, TINT, "CCPIXROW", &(header->pixel_row),
		NULL,&status)) header->pixel_row = 0;

	status = 0;
	if (fits_read_key(fptr, TINT, "CCPIXCL1", &(header->pixel_col1),
		NULL,&status)) header->pixel_col1 = 0;

	status = 0;
	if (fits_read_key(fptr, TINT, "CCPIXCL2", &(header->pixel_col2),
		NULL,&status)) header->pixel_col2 = 0;

	status = 0;
	if (fits_read_key(fptr, TINT, "CCREADS", &(header->number_of_reads),
		NULL,&status)) header->number_of_reads = 0;

	status = 0;
	if (fits_read_key(fptr, TINT, "CCLOOPS", &(header->number_of_loops),
		NULL,&status)) header->number_of_loops = 0;

	status = 0;
	if (fits_read_key(fptr, TINT, "CCBOUND", &(header->reset_boundary),
		NULL,&status)) header->reset_boundary = 0;

	status = 0;
	if (fits_read_key(fptr, TSTRING, "CCNMODE", header->readout_mode,
		NULL,&status)) strcpy(header->readout_mode,"Unknown");

	status = 0;
	if (fits_read_key(fptr, TSTRING, "CCDITHER", header->dither_state,
		NULL,&status)) strcpy(header->dither_state,"Unknown");
	
	status = 0;
	if (fits_read_key(fptr, TFLOAT, "CCLAMBDA", 
		&(header->lambda), NULL,&status)) header->lambda = 0;

	status = 0;
	if (fits_read_key(fptr, TFLOAT, "CCDRANGE", 
		&(header->dither_range), NULL,&status)) header->dither_range=0;

	status = 0;
	if (fits_read_key(fptr, TFLOAT, "CCDFREQ", 
		&(header->fringe_frequency), NULL,&status))
			header->fringe_frequency = 0;

	status = 0;
	if (fits_close_file(fptr, &status) )
	{
		fits_report_error(stderr, status);
		return -1;
	}

	return 0;

} /* read_classic_header() */

/************************************************************************/
/* find_dither_length()							*/
/*									*/
/* Given the dither data tries to work out how long each dither pattern */
/* is. Returns number representing the full forward and backward	*/
/* motion. Assume half of each is one or the other and that		*/
/* start of file is always start of one dither.				*/
/* Returns result or 0 if it fails for some reason.			*/
/************************************************************************/

int	find_dither_length(float *dither, int n)
{
	int 	i;

	/* Is there a dither motion at all */

	if (dither[0] == 0.0 && dither[1] == 0.0 && dither[2] == 0.0)
	{
		fprintf(stderr,"No dither motion recorded for this file.\n");
		return 0;
	}

	/* Find starting points */

	for(i=3; i < n && 
	    !(dither[i] == dither[0] && 
	      dither[i+1] == dither[1] && 
	      dither[i+2] == dither[2]);
		i++);

	/* Did we go too far ? */

	if (i == n)
	{
		fprintf(stderr,
			"Reached end of file before finding second dither.\n");
		return 0;
	}


	/* We must have found it */

	return i;

} /* find_dither_length() */

/************************************************************************/
/* find_token_line()							*/
/* 									*/
/* Tries to search through a file for a given token and extracts  	*/
/* the first non white space character location form that. The remainder*/
/* of the string is coppied into result, which is also returned.	*/
/* Returns NULL if the token is not found.				*/
/* If more than one token of the same name is found it is the last one	*/
/* that will be returned.						*/
/* Strips line feed at the end of the string.				*/
/************************************************************************/

char    *find_token_line(char *basename, char *extname,
			char *token, char *result)
{
	char	filename[256];
	FILE	*in;
	int	num_space;
	char	s[256];
	char	*p;
	int	i;
	char	found;

        /* Try and open an info file */

	if (*extname != 0)
		sprintf(filename,"%s.%s",basename,extname);
	else
		strcpy(filename, basename);

        if ((in = fopen(filename,"r")) == NULL) return NULL;

	/* How many spaces are there in the token? */

	for(num_space = 0, p = token; *p != 0; p++)
	{
		if (*p == ' ') num_space++;
	}

	/* OK, we go through the file and search for the token */

	*result = 0;
	found = 0;
	while (fgets(s, 256, in) != (char *)NULL)
	{
		/* If it's a comment move on */

		if (*s == ';' || *s == '#') continue;
		
		/* Move through and skip passed the right number of spaces */

		for(p = s, i=0; i<num_space; i++, p++)
		{
			while(*p != ' ' && *p != 0) p++;
			while(*p == ' ' && *p != 0) p++;
			if (*p == 0) continue;
		}

		/* Now move to the first white space character */

		while(*p != ' ' && *p != '\t' && *p != 0) p++;
		if (*p == 0) continue;

		/* Force that to be 0 */

		*p = 0;

		/* Is this the token we're looking for? */

		if (strcmp(s, token) == 0)
		{
			/* Find the first non white space character */

			p++;
			while((*p == ' ' || *p == '\t') && *p != 0) p++;
	
			/* Copy what's left into the result */

			strcpy(result,p);

			/* Remove any linefeed from the end */

			for(p=result; *p != 0; p++);
			
			for(p--;
			    (*p=='\n' || *p=='\r' || *p==' ' || *p=='\t') 
			    && p != result; p--);
			*++p = 0;

			/* This means we've found one, naybe more. */

			found++;
		}
	}

	/* Close the file and go */

	fclose(in);
	if (found)
		return result;
	else
		return NULL;

} /* find_token_line() */

/************************************************************************/
/* read_server_ports()							*/
/*									*/
/* Tries to read the command, data and message ports of a texted	*/
/* based server.							*/
/* Returns TRUE if all goes well, FALSE if not.				*/
/************************************************************************/

char read_server_ports(char *name, char *machine,
		       int *com_port, int *data_port, int *server_port)
{
	char	filename[256];
	FILE	*input;

	/* Assemble the filename */

	sprintf(filename, "%s%s.port", ETC_DIRECTORY, name);

	/* Try and see if one exists */

	if ((input = fopen(filename,"r")) == NULL) return FALSE;

	/* Try and read in the data we need */

	if (fscanf(input,"%s %d %d %d", machine, com_port, 
			data_port, server_port) != 4) return FALSE;

	return TRUE;

} /* read_server_ports() */

/************************************************************************/
/* read_breakir_sample()						*/
/*									*/
/* For reading output files created by breakir.				*/
/* Routine that reads in one sample.					*/
/* Returns 0 if all went well, non-zero otherwise.			*/
/************************************************************************/
 
int read_breakir_sample(char text, char reverse_byte, FILE *file,
		CHARA_TIME *now, float *dither, int *d1, int *d2)
{
	if (text)
	{
		if (fscanf(file,"%d %f %d %d", now, dither, d1, d2) != 4)
		{
			return 1;
		}
	}
	else
	{
		if (fread(now, sizeof(*now), 1, file) == 0) return 1;

		if (fread(dither, sizeof(*dither), 1, file) == 0) return 1;

		if (fread(d1, sizeof(*d1), 1, file) == 0) return 1;

		if (fread(d2, sizeof(*d2), 1, file) == 0) return 1;


		if (reverse_byte)
		{
			reverse_byte_order(now,sizeof(*now));
			reverse_byte_order(dither,
				sizeof(*dither));
			reverse_byte_order(d1,sizeof(*d1));
			reverse_byte_order(d2,sizeof(*d2));
		}
	}

	return 0;

} /* read_breakir_sample() */

/************************************************************************/
/* get_breakir_data_format()						*/
/* 									*/
/* For looking at output files created by breakir.			*/
/* Tries to read the info file and find out what the data format is	*/
/* and how long dither samples are, which is returned.			*/
/* Returns 0 if it failed to find DITHER_SAMPLES or DITHER_FULL_LENGTH.	*/
/************************************************************************/

int get_breakir_data_format(char *basename, char *text, char *reverse_byte)
{
	char	s[256];
	int	i;

        if (find_token_line(basename, "info", "DATA_FORMAT", s) == NULL)
        {
                fprintf(stderr,
			"Failed to get data mode form info file, using IBM.\n");
		*text = FALSE;
		*reverse_byte = FALSE;
        }
	else
	{
		if (strncmp(s,"IBM",3) == 0)
		{
			*text = FALSE;
			*reverse_byte = FALSE;
		}
		else if (strncmp(s,"MAC",3) == 0)
		{
			*text = FALSE;
			*reverse_byte = TRUE;
		}
		else
		{
			*text = TRUE;
			*reverse_byte = FALSE;
		}
	}

        if ((find_token_line(basename,"info","DITHER_SAMPLES",s)==NULL) ||
                (sscanf(s,"%d", &i) != 1))
        {
                fprintf(stderr,
"Failed to get DITHER_SAMPLES from info file, using DITHER_FULL_LENGTH.\n");
		if ((find_token_line(basename, "info", "DITHER_FULL_LENGTH", s) ==
			NULL) || (sscanf(s,"%d", &i) != 1))
		{
			fprintf(stderr,
			"Failed to get DITHER_FULL_LENGTH form info file.\n");
			return 0;
		}
	}

	return i;

} /* get_data_format() */

/************************************************************************/
/* read_breakir_file()							*/
/*									*/
/* Reads data in from a file created by breakir file.			*/
/* Returns number of data points found, or 0 if nothing found.		*/
/* Negative return value indicates a problem.				*/
/* -1 Memory allocation.						*/
/* -2 Opening data file.						*/
/************************************************************************/

int read_breakir_file(int text, int reverse_bytes, char *filename, 
	CHARA_TIME **now, float **dith, int **d1, int **d2)
{
	FILE	*in;
        CHARA_TIME      *pCHARA_TIME;
        float           *pfloat;
        int        *pint;
	int	i;
	int		n;

	/* Allocate memory */

        if ((*now = malloc(MAX_N*sizeof(*now))) == NULL) return -1;

        if ((*dith = malloc(MAX_N*sizeof(*dith))) == NULL) return -1;

        if ((*d1 = malloc(MAX_N*sizeof(*d1))) == NULL) return -1;

        if ((*d2 = malloc(MAX_N*sizeof(*d2))) == NULL) return -1;

	/* Open the data file */

	if ((in = fopen(filename, "r")) == NULL) return -2;

	/* Read in the data */

	for(n=0; n < MAX_N && 
	  read_breakir_sample(text,reverse_bytes,in,
		(*now)+n,(*dith)+n,(*d1)+n,(*d2)+n) == 0; n++);
	fclose(in);

	/* Now create memory the right length */

        if ((pCHARA_TIME = malloc(n * sizeof(*pCHARA_TIME))) == NULL) return -1;
	for(i=0; i<n; i++) pCHARA_TIME[i] = (*now)[i];
	free(*now);
	*now = pCHARA_TIME;

	if ((pfloat = malloc(n * sizeof(*pfloat))) == NULL) return -1;
	for(i=0; i<n; i++) pfloat[i] = (*dith)[i];
	free(*dith);
	*dith = pfloat;

	if ((pint = malloc(n * sizeof(*pint))) == NULL) return -1;
	for(i=0; i<n; i++) pint[i] = (*d1)[i];
	free(*d1);
	*d1 = pint;

	if ((pint = malloc(n * sizeof(*pint))) == NULL) return -1;
	for(i=0; i<n; i++) pint[i] = (*d2)[i];
	free(*d2);
	*d2 = pint;

	return n;

} /* read_file() */
