/************************************************************************/
/* speak.c 								*/
/*                                                                      */
/* Small utility to connect to a socket and allow text comms to it.	*/
/* Used mostly to test out the socket command stuff in the user		*/
/* interface.								*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                          		*/
/* Date   : July 1998							*/
/************************************************************************/

#include <stdio.h>
#include <netdb.h>
#include <resolv.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <string.h>
#include <sys/time.h>
#include "../../charalib/chara.h"
#include "../../charalib/chara.h"
#include "../../../cliserv/socket_manager/server/socket_manager.h"

int	char_waiting(int fd);
#define BUFFER_SIZE 1024

int main(int argc, char **argv)
{
        struct sockaddr_in server; /* Server address */
	struct hostent *hp;	/* Pointer to host data */
	int	socket_p;	/* Socket number */
	int	buf_size;	/* Size of buffers */
	char	line[BUFFER_SIZE];
	int n;
	int	com_port;
	int	data_port;
	int	server_port;
	char	machine[256];
	struct s_process_sockets process;

	/* Test command line */

	if (argc != 2 && argc != 3)
	{
		fprintf(stderr,"usage: %s name\n",argv[0]);
		fprintf(stderr,"or\n");
		fprintf(stderr,"usage: %s machine port\n",argv[0]);
		exit(-1);
	}

	if (argc == 2)
	{
		/* First try and get the info from the socket manager */

		init_jobs();
		if (socket_manager_get_process(argv[1], &process) == 0)
		{
			strcpy(machine, process.machine);
			com_port = process.commands;
			data_port = process.data;
			server_port = process.messages;
		} else if (!read_server_ports(argv[1], machine, 
			&com_port, &data_port, &server_port))
		{
		    fprintf(stderr, 
			"Process %s is not running or does not exist.\n", 
			argv[1]);
		    fprintf(stderr,
			"Don't forget to use S1 and not s1 for scopes.\n");
		    fprintf(stderr,
			"You could try checking with te sockman program.\n");
		    exit(-2);
		}
	}
	else
	{
		/* OK we've been given this stuff manually */

		strcpy(machine, argv[1]);
		sscanf(argv[2],"%d", &com_port);
		data_port = server_port = -1;
	}

	if (com_port <= 0)
	{
	    fprintf(stderr,"Process %s seems to not have a command port.\n",
		argv[1]);
	    exit(-3);
	}

	/* First see if we can identify the host. */

	if((hp = gethostbyname(machine)) == 0)
	{
		fprintf(stderr,"Unknown host.\n");
		exit(-3);
	}

	/* Create a socket */

	if ((socket_p = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
		fprintf(stderr,"Failed to create socket\n");
		exit(-3);
	}

	/* Force buffer sizes to BUFFER_SIZE */

	buf_size = BUFFER_SIZE;

	if (setsockopt(socket_p, SOL_SOCKET, SO_SNDBUF, 
		(char *)&buf_size, sizeof(buf_size)) < 0)
	{
		close(socket_p);
		fprintf(stderr,"Failed to change send buffer size to %d\n",
			buf_size);
		exit(-4);
	}

	if (setsockopt(socket_p, SOL_SOCKET, SO_RCVBUF, 
		(char *)&buf_size, sizeof(buf_size)) < 0)
	{
		close(socket_p);
		fprintf(stderr,"Failed to change receive buffer size to %d\n",
			buf_size);
		exit(-5);
	}

	/* Create server address */

	server.sin_family = AF_INET;
	strncpy((char *)&server.sin_addr, (char *)hp->h_addr, hp->h_length);
	server.sin_port = htons(com_port);

	/* Let's try and connect now.... */

	if (connect(socket_p, (struct sockaddr *)&server, sizeof(server)) < 0)
	{
		close(socket_p);
		fprintf(stderr,"Failed to connect.\n");
		perror("Error ");
		exit(-6);
	}
	
	/* Start off by pinging the device */

	write(socket_p, "ping\n", 5);

	/* Now try and send stuff */

	while(1)
	{
		if (char_waiting(0))
		{
			fgets(line, BUFFER_SIZE, stdin);
			line[strlen(line)-1] = 0;

			/* Did the user type quit? */

			if (strcmp(line,"quit") == 0) break;

			/* Note, we wish to send the terminating NULL */

			if (write(socket_p,line,strlen(line))!=strlen(line))
			{
				fprintf(stderr,"Error writing string.\n");
				perror("Error ");
				break;
			}
		}

		if (char_waiting(socket_p))
		{
			if ((n = read(socket_p, line, BUFFER_SIZE)) <= 0)
			{
				break;
			}
			line[n] = 0;

			/* Did the server type quit or EOF? */

			if (strcmp(line,"sockQuit") == 0 || *line == EOF) 
			{
				printf("Server is closing.\n");
				break;
			}

			printf("%s",line);
		}

	}
		
	printf("Connection closed.\n");

	close(socket_p);
	exit(0);
}

/************************************************************************/
/* char_waiting()							*/
/*									*/
/* Unix version of kbhit like function. Returns 1 if a character 	*/
/* waiting at the port, 0 otherwise.					*/
/* Uses the select(2) routine to do it's work. 				*/
/* From background.c and stripped .					*/
/************************************************************************/

int	char_waiting(int fd)
{
	/* Locals */

	fd_set rfds;		/* File discriptor set */
        struct timeval tv;	/* Time to wait */

	/* Set tings up */

	FD_ZERO(&rfds);		 /* Zero the file discriptor set */
        FD_SET(fd, &rfds);	 /* Add fd to the set */
	tv.tv_sec = 0;		 /* Wait no more than 1 usec */
	tv.tv_usec = 1;

	if (select(FD_SETSIZE, &rfds, NULL, NULL, &tv) < 0) 
	{
		/* Something is wrong. */

		perror("char_waiting ");
		return 0;
	}

	if (FD_ISSET(fd, &rfds)) /* Was there something? */
	{
		return 1;
	}

	/* Must have been nothing there */

	return 0;

} /* char_waiting() */
