/************************************************************************/
/* socket.c								*/
/*                                                                      */
/* Routines for client socket stuff.					*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY CLIENT LIBRARY			*/
/*                 Based on the CHARA User Interface			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar 		                        */
/* Date   : Oct 2001							*/
/************************************************************************/

/*
 * Include files
 */

#include "../include/cliserv.h"
#include "../../../cliserv/include/chara_messages.h"
#include "../../../cliserv/socket_manager/server/socket_manager.h"
#include <netdb.h>
#include <resolv.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>

/************************************************************************/
/* server_socket_open()							*/
/*									*/
/* Returns TRUE if server socket is open, FALSE otherwise.		*/
/************************************************************************/

int server_socket_open(int server)
{
	if (server < 0 || server >= MAX_SERVERS || servers[server].fd == -1) 
		return FALSE;
	else
		return TRUE;

} /* server_socket_open() */

/************************************************************************/
/* open_server_by_name()						*/
/*									*/
/* Tries to talk to socket manager to find out about a given process	*/
/* and then tries to open the server. Returns server number if it works */
/* and -1 otherwise.							*/
/************************************************************************/

int open_server_by_name(char *name)
{
	struct s_process_sockets process;

	/* First we try and find the details of this machine */

	if (socket_manager_get_process(name, &process) != 0) return -1;

	/* Make sure we have what we need */

	if (strlen(process.machine) == 0 || process.messages <= 0) return -1;

	/* Now try and connect to it */

	return open_server_socket(process.machine, process.messages, name);

}

/************************************************************************/
/* open_server_socket()							*/
/*									*/
/* Attempts to open up a socket port for serverion. Requires a 		*/
/* machine name and a port number. Returns the number of the server	*/
/* if successful or a -1 if it fails.					*/
/************************************************************************/

int open_server_socket(char *machine, int port, char *name)
{
        struct sockaddr_in server; 	/* Server address */
        struct hostent *hp;     	/* Pointer to host data */
	int	buf_size;		/* Size of buffer */
	int	n;			/* The server number being used */

	/* Find first available slot for the server */

	for(n=0; n<MAX_SERVERS && servers[n].fd != -1; n++);
	if (n >= MAX_SERVERS) 
	{
		return -1;
	}

        /* First see if we can identify the host. */

        if((hp = gethostbyname(machine)) == 0)
        {
		servers[n].fd = -1;
                return -1;
        }

        /* Create a socket */

        if ((servers[n].fd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
        {
		servers[n].fd = -1;
                return -1;
        }

        /* Force buffer sizes to SOCKET_BUFFER */

        buf_size = SERVER_SOCKET_BUFFER;

        if (setsockopt(servers[n].fd, SOL_SOCKET, SO_SNDBUF,
                (char *)&buf_size, sizeof(buf_size)) < 0)
        {
                close(servers[n].fd);
		servers[n].fd = -1;
                return -1;
        }

        if (setsockopt(servers[n].fd, SOL_SOCKET, SO_RCVBUF,
                (char *)&buf_size, sizeof(buf_size)) < 0)
        {
                close(servers[n].fd);
		servers[n].fd = -1;
                return -1;
        }

        /* Create server address */

        server.sin_family = AF_INET;
        strncpy((char *)&server.sin_addr, (char *)hp->h_addr, hp->h_length);
        server.sin_port = htons(port);

        /* Let's try and connect now.... */

        if (connect(servers[n].fd, 
		(struct sockaddr *)&server, 
		sizeof(server)) < 0)
        {
                close(servers[n].fd);
		servers[n].fd = -1;
                return -1;
        }

	/* Setup the name */

	strcpy(servers[n].name, name);

	/* And the port */

	servers[n].port = port;

	/* And the machine */

	strcpy(servers[n].machine, machine);

	/* All is well it seems */

	return n;

} /* open_server_socket() */

/************************************************************************/
/* close_server_socket()						*/
/*									*/
/* Attempts to close socket port for serverion. 			*/
/************************************************************************/

void close_server_socket(int server)
{
	int	i;

        if (server < 0 || server >= MAX_SERVERS) return;
	if (servers[server].fd == -1) return;

	close(servers[server].fd);
	servers[server].fd = -1;
	servers[server].port = -1;
	servers[server].name[0] = 0;
	servers[server].machine[0] = 0;
	for (i=0; i<MAX_JOBS; i++) servers[server].message_jobs[i] = NULL;

} /* close_server_socket() */

/************************************************************************/
/* process_server_socket()						*/
/*									*/
/* Will look and see if anything is waiting in the socket.		*/
/* If there is will try and read in message and run appropriate		*/
/* message handler.							*/
/* The client should call this regular like.				*/
/* Returns 0 if things go well.						*/
/* 1 if socket has died.						*/
/* 2 if not enough data was found.					*/
/* 3 if ran out of data.						*/
/* 4 if no appriate jobs was found.					*/
/* False normally means socket hasbeen closed by server.		*/
/************************************************************************/

int process_server_socket(int server)
{
	message_struct message;
	int	len,rlen;

	/* If the socket is not open do nothing */

        if (server < 0 || server >= MAX_SERVERS) return 0;
	if (servers[server].fd == -1) return 0;

	/* Is anything there? */

	if (client_char_waiting(servers[server].fd))
	{
		/* Try and read in a message type and length */

		len = sizeof(message.type);
		if ((rlen = client_read_socket(servers[server].fd, 
			(char *)&(message.type), len)) != len)
		{
			if (rlen <= 0)
			{
				close_server_socket(server);
				return 1;
			}
			else
			{
				return 2;
			}
		}

		len = sizeof(message.length);
		if ((rlen = client_read_socket(servers[server].fd, 
			(char *)&(message.length) , len)) != len)
		{
			if (rlen <= 0)
			{
				close_server_socket(server);
				return 1;
			}
			else
			{
				return 2;
			}
		}

		/* Allocate the memory for the data */

		if (message.length == 0)
		{
			message.data = NULL;
		}
		else
		{
			if ((message.data = malloc((size_t)message.length)) == NULL)
			{
				flush_server_socket(server);
				return 3;
			}

			/* Go get the data */

			if ((rlen = client_read_socket(servers[server].fd, 
				(char *)message.data, 
				message.length)) != message.length)
			{
				if (rlen <= 0)
				{
					close_server_socket(server);
					return 1;
				}
				else
				{
					return 2;
				}
			}
		}

		/* Run the job if there is one */

		if (!run_message_job(server, &message))
		{
			if (message.data != NULL) free(message.data);
			return 4;
		}

		if (message.data != NULL) free(message.data);
	}

	return 0;

} /* process_server_socket() */

/************************************************************************/
/* client_read_socket()							*/
/* 									*/
/* Tries to read a given number of bytes into a given location from	*/
/* a socket. Will keep trying to read until it get's all the bytes	*/
/* it expects or it times out. Returns number of bytes recieved.	*/
/************************************************************************/

int client_read_socket(int fd, char *buf, size_t count)
{
	time_t	start;
	int	n,len;

	start = time(NULL);
	len = 0;
	while(count > 0)
	{
		if (time(NULL) > start + TIME_OUT) break;

		if((n = read(fd, buf, count)) < 0) return n;

		len += n;
		buf += n;
		count -= n;
	}

	return len;

} /* client_read_socket() */

/************************************************************************/
/* flush_server_socket()						*/
/*									*/
/* Removes any data from the server socket.				*/
/************************************************************************/

void flush_server_socket(int server)
{
	char c;

        if (server < 0 || server >= MAX_SERVERS) return;

	while(client_char_waiting(servers[server].fd))
	{
		if (read(servers[server].fd, &c, 1) <= 0)
		{
			close_server_socket(server);
			break;
		}
	}

} /* flush_server_socket() */

/************************************************************************/
/* send_message()							*/
/*									*/
/* Returns TRUE of FALSE depending on how things go.			*/
/************************************************************************/

int send_message(int server, struct smessage *message)
{
	int	i;

        if (server < 0 || server >= MAX_SERVERS) return FALSE;
	if (servers[server].fd == -1) return FALSE;

	/* 
         * There is a small chance that this socket has gone away.
         * For this reason we now check and do nothing if it has.
         */

        if (!client_write_ready(servers[server].fd)) return FALSE;

	/* Otherwise let's just send the data */

	i = sizeof(message->type);
	if (write(servers[server].fd, &(message->type), i) != i) return FALSE;
	i = sizeof(message->length);
	if (write(servers[server].fd, &(message->length), i) != i) return FALSE;

	if (message->length > 0 && 
		write(servers[server].fd, message->data, message->length) != 
		message->length)
	{
		return FALSE;
	}

	return TRUE;

} /* send_message() */

/************************************************************************/
/* print_message()							*/
/* 									*/
/* Works like a printf only sends the string as a message of type	*/
/* MESSAGE to the given socket.						*/
/* Returns TRUE if all OK.						*/
/************************************************************************/

int print_message(int server, char *fmt, ...)
{
	message_struct mess;

        /* Internal Variables */

        char    string[9999];
        va_list args;

        /* Create the string */

        va_start(args, fmt);
        vsprintf(string, fmt, args);

	/* Create the message */

	mess.type = MESSAGE;
	mess.length = strlen(string);
	mess.data = (unsigned char *)string;

	/* Send the message */

	return send_message(server, &mess);

} /* print_message() */
