/************************************************************************/
/* char_waiting.c							*/
/*                                                                      */
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY SERVER LIB				*/
/*                 Based on the CHARA User Interface			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                          		*/
/* Date   : Oct 2001							*/
/************************************************************************/

/* This is part of the SERVER code */

#define SERVER

/*
 * Include files
 */

#include "../include/cliserv.h"
#include <errno.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>

/************************************************************************/
/* char_waiting()							*/
/*									*/
/* Unix version of kbhit like function. Returns TRUE if a character 	*/
/* waiting at the port, FALSE otherwise.				*/
/* Note that this function can cause problems, so for real keyboard	*/
/* use I go back to a non-block keyboard I/O.				*/
/************************************************************************/

int	char_waiting(int fd)
{
	/* Locals */

	fd_set rfds;		/* File discriptor set */
        struct timeval tv;	/* Time to wait */

	if (fd <0) return FALSE;

	/* Set tings up */

	FD_ZERO(&rfds);		 /* Zero the file discriptor set */
        FD_SET(fd, &rfds);	 /* Add stdin to the set */
	tv.tv_sec = 0;		 /* Wait no more than 1 usec */
	tv.tv_usec = 1;

	if (select(FD_SETSIZE, &rfds, NULL, NULL, &tv) < 0) 
	{
		/* Something is wrong. */

		if (errno != EINTR)
		error(WARNING,
			"char_waiting(): Select() Function returned errno %d.",
			errno);
		return FALSE;
	}

	if (FD_ISSET(fd, &rfds)) /* Was there something? */
	{
		return TRUE;
	}

	/* Must have been nothing there */

	return FALSE;

} /* char_waiting() */

/************************************************************************/
/* write_ready()							*/
/*									*/
/* Returns TRUE if a port is ready for writing,	FALSE otherwise.	*/
/* Uses the select(2) routine to do it's work. 				*/
/************************************************************************/

int	write_ready(int fd)
{
	/* Locals */

	fd_set wfds;		/* File discriptor set */
        struct timeval tv;	/* Time to wait */

	if (fd <0) return FALSE;

	/* Set tings up */

	FD_ZERO(&wfds);		 /* Zero the file discriptor set */
        FD_SET(fd, &wfds);	 /* Add fd to the set */
	tv.tv_sec = 0;		 /* Wait no more than 1 usec */
	tv.tv_usec = 1000;

	if (select(FD_SETSIZE, NULL, &wfds, NULL, &tv) < 0) 
	{
		/* Something is wrong. */

		if (errno != EINTR)
		error(WARNING,
			"write_ready(): Select() Function returned errno %d.",
			errno);
		return FALSE;
	}

	if (FD_ISSET(fd, &wfds)) /* Was there something? */
	{
		return TRUE;
	}

	/* Must have been nothing there */

	return FALSE;

} /* write_ready() */

/************************************************************************/
/* exception()								*/
/*									*/
/* Returns TRUE if a port has an exception, FALSE otherwise.		*/
/* Uses the select(2) routine to do it's work. 				*/
/* From charaui!!  							*/
/************************************************************************/

int	exception(int fd)
{
	/* Locals */

	fd_set efds;		/* File discriptor set */
        struct timeval tv;	/* Time to wait */

	if (fd <0) return FALSE;

	/* Set tings up */

	FD_ZERO(&efds);		 /* Zero the file discriptor set */
        FD_SET(fd, &efds);	 /* Add fd to the set */
	tv.tv_sec = 0;		 /* Wait no more than 1 usec */
	tv.tv_usec = 1;

	if (select(FD_SETSIZE, NULL, NULL, &efds, &tv) < 0) 
	{
		/* Something is wrong. */

		if (errno != EINTR)
		error(WARNING,
			"exception(): Select() Function returned errno %d.",
			errno);
		return FALSE;
	}

	if (FD_ISSET(fd, &efds)) /* Was there something? */
	{
		return TRUE;
	}

	/* Must have been nothing there */

	return FALSE;

} /* exception() */

/************************************************************************/
/* getline()								*/
/* 									*/
/* Reads a line from a given stream as text. Leading white space is 	*/
/* ignored. A line whose first character is '#' is considered		*/
/* a comment and is ignored. The new line character at the end of the	*/
/* string (if present) is replaced by a NULL. Getline returns a pointer */
/* to the string or EOF if the end of the file has been reached.	*/
/* Getline will read up to n-1 characters and then terminate.		*/
/* If a '#' is in the string the rest of the string is considered	*/
/* a comment and is ignored.						*/
/* If the line contains the text "NULL", it will return a NULL string.	*/
/* 									*/
/************************************************************************/

char 	*getline(char *string, int n, FILE *stream)
{
	/* Internal variables */

	int	c;		/* Character */
	int	loop;		/* Boolian loop control */
	char	*retval;	/* Return value of function */
	char	junk[81];	/* Junk string for comments */

	/* Set return value to string begining */

	retval = string;

	loop = TRUE;

	/* Find start of string */

	while(loop)
	{
		switch(c = getc(stream))
		{
			/* If character is white space ignore it */

			case ' '  :
			case '\t' : 
			case '\n' :
			case '\r' : continue;

			/* If character indicates a comment junk the comment */

			case '#'  : if (fgets(junk, n, stream) == NULL)
				            return (char *) EOF;

				    break;

			/* If reach EOF return EOF */

			case EOF  : return (char *) EOF;

			/* Otherwise we have found the strart of the string */

			default   : ungetc(c, stream);
				    loop = FALSE;
				    break;
		}
	}

	/* Read in the string */

	while ( --n > 0 && (c = getc(stream)) != EOF && c != '\n' && c != '\r'
			&& c != '#') *string++ = c;

	/* Look at how we exited the previous loop */

	switch( c )
	{
		/* If it was by EOF return value must be EOF */

		case EOF : retval = (char *) EOF;
			   break;

		/* If it was by a comment junk the comment */

		case '#' : if (fgets(junk, n, stream) == NULL)
			           retval = (char *) EOF;

			   break;

		/* Otherwise do nothing */

		default  : break;
	}

	/* Strip any tabs or spaces from the end of the string */

	while ( *--string == '\t' || *string == ' ')
		;

	/* Put terminating NULL on end of string */

	*++string = (char)NULL;

	/* see if the string is "NULL" text */

	if (strcmp(retval,"NULL") == 0)
		*retval = (char)NULL;

	/* Return  pointer to the string */

	return retval;

} /* getline() */
