/************************************************************************/
/* job_control.c							*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY SERVER LIB   				*/
/*                 Based on the CHARA User Interface			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar 		                        */
/* Date   : Oct 2001							*/
/************************************************************************/

#define SERVER

/*
 * Include files
 */

#include "../include/cliserv.h"
#include "../../../cliserv/include/chara_messages.h"

/*
 * Some local globals
 */

static int (*(top_jobs[MAX_JOBS]))(void);
static int (*(message_jobs[MAX_JOBS]))(struct smessage *message);
static int (*(background_jobs[MAX_JOBS]))(void);

static int current_background_job;

static void (*close_down)(void);

char	*server_name;
static char server_has_been_initialized = FALSE;

/************************************************************************/
/* server_init_jobs()							*/
/*									*/
/* Set's up top jobs and message jobs.					*/
/************************************************************************/

void server_init_jobs(void)
{
	int	i;

	if (server_has_been_initialized) return;

	/* First set all jobs to NULL */

	for (i=0; i<MAX_JOBS; i++)
	{
		top_jobs[i] = NULL;
		message_jobs[i] = NULL;
		background_jobs[i] = NULL;
	}
	close_down = NULL;

	/* Now we setup the standard jobs */

	server_add_message_job(NOOP, message_noop);
	server_add_message_job(MESSAGE, message_message);
	server_add_message_job(PING, message_ping);
	server_add_message_job(ALL_NOOP, message_all_noop);
	server_add_message_job(ALL_PING, message_all_ping);
	server_add_message_job(UI_STATUS, message_uistatus);
        server_add_message_job(UI_SIGNAL, message_signal);

	server_has_been_initialized = TRUE;

} /* server_init_jobs() */

/************************************************************************/
/* server_add_message_job()						*/
/*									*/
/* Adds a new message job to the setup. FATAL error if job already	*/
/* exists.								*/
/************************************************************************/

void server_add_message_job(unsigned int type, 
		int (*job)(struct smessage *message))
{
	/* Is this message place already taken ? */

	if (message_jobs[type] != NULL)
	{
		error(FATAL,"Two message jobs for message %d.", type);
	}

	/* Is it a reasonable message type */

	if (type >= MAX_JOBS)
	{
		error(FATAL,"Job beyond message %d\n",MAX_JOBS);
	}

	/* Otherwise put it in place */

	message_jobs[type] = job;

	error(MESSAGE,"Added message type %d.",type);

} /* server_add_message_job() */

/************************************************************************/
/* add_top_job()							*/
/* 									*/
/* Adds a job to the top job list.					*/
/************************************************************************/

void add_top_job(int (*job)(void))
{
	int	i;

	/* Find a new position for the job */

	for(i=0; i<MAX_JOBS && top_jobs[i] != NULL; i++);

	if (i >= MAX_JOBS)
	{
		error(FATAL,"Tried to install too many top jobs.");
	}

	/* Install it there */

	top_jobs[i] = job;

	error(MESSAGE,"Added top job %d.",i);

} /* add_top_job() */

/************************************************************************/
/* add_background_job()							*/
/* 									*/
/* Adds a job to the background job list.				*/
/************************************************************************/

void add_background_job(int (*job)(void))
{
	int	i;

	/* Find a new position for the job */

	for(i=0; i<MAX_JOBS && background_jobs[i] != NULL; i++);

	if (i >= MAX_JOBS)
	{
		error(FATAL,"Tried to install too many background jobs.");
	}

	/* Install it there */

	background_jobs[i] = job;

	error(MESSAGE,"Added background job %d.",i);

} /* add_background_job() */

/************************************************************************/
/* process_job()							*/
/*									*/
/* Processes jobs and messages once.					*/
/************************************************************************/

void process_job(void)
{
	int	i;

	/* Always run all top jobs */

	for(i=0; i<MAX_JOBS; i++)
	{
		if (top_jobs[i] != NULL) 
		{
			(*(top_jobs[i]))();
		}
	}

	/* See if there are any messages to process */

	process_message_socket();

	/* Now find the next background job */

	for(i = current_background_job+1; 
	    i < MAX_JOBS && background_jobs[i] == NULL; 
	    i++);
	if (i >= MAX_JOBS)
	{
		for(i = 0; 
		    i < current_background_job && 
			background_jobs[i] == NULL; 
		    i++);
	}
	current_background_job = i;

	/* And run it */

	if (background_jobs[current_background_job] != NULL)
		(*(background_jobs[current_background_job]))();

} /* process_job() */

/************************************************************************/
/* process_jobs()							*/
/*									*/
/* Here is a routine you may never return from without receiveing a	*/
/* control C. Loops around and processes jobs and messages.		*/
/* Uses process_job() to do this.					*/
/************************************************************************/

void process_jobs(void)
{
	/* First make sure we setup the closing script. */

	signal(SIGINT, close_server);
	signal(SIGKILL, close_server);
	signal(SIGQUIT, close_server);
	signal(SIGTERM, close_server);
#warning THIS DOESNT SEEM TO WORK!
	//signal(SIGPIPE, close_server);

	/* Now, we loop forever */

	while(TRUE) process_job();

} /* process_jobs() */

/************************************************************************/
/* setup_close_down()							*/
/*									*/
/* Sets up a close function sent by outside world.			*/
/************************************************************************/

void setup_close_down(void (*close_down_routine)(void))
{
		close_down = close_down_routine;

} /* setup_close_down() */

/************************************************************************/
/* close_server()							*/
/*									*/
/* Routine to close things down in a graceful way... I hope.		*/
/************************************************************************/

void close_server(int sig)
{
        system("stty sane");

	/* Run close_down function */

	if (close_down != NULL) (*close_down)();

	/* Tell clients we are leaving */

	send_server_stopping();

	/* Close all sockets */

	close_message_socket();

	/* That should be all */

	error(MESSAGE,"Closing down server.");

	exit(0);

} /* close_server() */

/************************************************************************/
/* server_run_message_job()						*/
/*									*/
/* Attempts to run message job.						*/
/************************************************************************/

void server_run_message_job(struct smessage *message)
{
	/* Is it safe to runthis message? */

	if (message->type >= MAX_JOBS)
	{
		error(ERROR,"Message job outside of range (%d).",message->type);
		return;
	}
	else if (message_jobs[message->type] == NULL)
	{
		error(ERROR,"Message job with no function (%d).",message->type);
		return;
	}

	/* Yes it is, go ahead */

	(*(message_jobs[message->type]))(message);

} /* server_run_message_job() */

/************************************************************************/
/* set_server_name()							*/
/*									*/
/* Set server name to a particular string.				*/
/************************************************************************/

void set_server_name(char *name)
{
	server_name = name;

} /* set_server_name() */
