/************************************************************************/
/* standard_messages.c							*/
/*                                                                      */
/* Jobs for standard messages.						*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY SERVER LIB				*/
/*		   Based on the CHARA User Interface			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar 		                        */
/* Date   : October 2001						*/
/************************************************************************/

/* This is part of the server */

#define SERVER

/*
 * Include files
 */

#include "../include/cliserv.h"
#include "../../../cliserv/include/chara_messages.h"

/* GLobal for signal control */

#define MAX_SIGNAL 256

static unsigned char client_signals[MAX_SIGNAL];

/************************************************************************/
/* setup_signals()							*/
/*									*/
/* Set's all signals to zero.						*/
/************************************************************************/

void setup_signals(void)
{
	int	i;

	for(i=0; i<MAX_SIGNAL; i++) client_signals[i] = FALSE;

} /* setup_signals() */

/************************************************************************/
/* message_noop()							*/
/*									*/
/* A do nothing message handler.					*/
/* Returns error level.							*/
/************************************************************************/

int message_noop(struct smessage *message)
{
	if (message->length != 0)
	{
		return error(ERROR,"Received NOOP with data.");
	}

	error(MESSAGE,"NOOP from %s",active_machine);
	return NOERROR;

} /* message_noop() */

/************************************************************************/
/* message_message()							*/
/*									*/
/* Should simply spit out the string in the data.			*/
/* Returns error level.							*/
/************************************************************************/

int message_message(struct smessage *message)
{
	char *s,*p1,*p2;
	int	i;

	if (message->length == 0)
	{
		error(MESSAGE,"%s: No message string.", active_machine);
	}
	else
	{
		if ((s = malloc(message->length+1)) == NULL)
		{
			error(FATAL,"Ran out of memory geting message string.");
		}

		for(i=0, p1=s, p2=(char *)message->data;
		    i < message->length; i++, p1++, p2++) *p1 = *p2;
		*p1 = 0;

		error(MESSAGE,"%s: %s",active_machine,s);

		free(s);
	}

	return NOERROR;

} /* message_message() */

/************************************************************************/
/* message_ping()							*/
/*									*/
/* Sends back a message of type MESSAGE claiming we are alive.		*/
/* Returns error level.							*/
/************************************************************************/

int message_ping(struct smessage *message)
{
	char	name[256];

	if (message->length != 0)
	{
		error(ERROR,"Recieved PING with data.");
	}

	error(MESSAGE,"Recieved PING from %s.",active_machine);
	return server_print_message(active_socket,"%s on %s is alive.",
			server_name,get_server_name(name));

} /* message_ping() */

/************************************************************************/
/* message_all_noop()							*/
/* 									*/
/* Send a NOOP mesage to all connected machines.			*/
/* Returns error level.							*/
/************************************************************************/

int message_all_noop(struct smessage *message)
{
	message_struct mess;

	mess.type = NOOP;
	mess.length = 0;
	mess.data = NULL;

	/* Send the message */

	return server_send_message_all(&mess);

} /* message_all_noop() */

/************************************************************************/
/* message_all_ping()							*/
/* 									*/
/* Send a PING mesage to all connected machines.			*/
/* Returns error level.							*/
/************************************************************************/

int message_all_ping(struct smessage *message)
{
	message_struct mess;

	mess.type = PING;
	mess.length = 0;
	mess.data = NULL;

	/* Send the message */

	return server_send_message_all(&mess);

} /* message_all_ping() */

/************************************************************************/
/* message_uistatus()							*/
/*									*/
/* Place holder for UI based message.					*/
/************************************************************************/

int message_uistatus(struct smessage *message)
{

	error(MESSAGE,"Recieved UISTATUS from %s.",active_machine);

	return NOERROR;

} /* message_uistatus() */
 
/************************************************************************/
/* message_signal()                                                     */
/*                                                                      */
/* For when we get a UI_SIGNAL message. Set's signal to TRUE.           */
/************************************************************************/

int message_signal(struct smessage *message)
{
        unsigned char   *signal;

        if (message->length != sizeof(*signal))
        {
                server_print_message(active_socket, 
			"SIGNAL of wrong data size.");
                return ERROR;
        }

        signal = (unsigned char *) message->data;

        client_signals[*signal] = TRUE;

        return NOERROR;

} /* message_signal() */

/************************************************************************/
/* any_client_signal_set()                                              */
/*                                                                      */
/* Returns TRUE if any signals are set.                                 */
/************************************************************************/

bool any_client_signal_set(void)
{
        int i;

        for(i=0; i<MAX_SIGNAL; i++) 
        {
                if (client_signals[i]) return TRUE;
        }

        return FALSE;

} /* any_client_signal_set() */

/************************************************************************/
/* client_signal_set()                                                  */
/*                                                                      */
/* Tells you if a particular client signal has been set. Returns TRUE   */
/* or FALSE.                                                            */
/************************************************************************/

bool client_signal_set(unsigned char signal)
{

        return client_signals[signal];

} /* client_signal_set() */

/************************************************************************/
/* clear_client_signal()                                                */
/*                                                                      */
/* Turns off given signal.                                              */
/************************************************************************/

void clear_client_signal(unsigned char signal)
{

        client_signals[signal] = FALSE;

} /* clear_client_signal() */

/************************************************************************/
/* server_print_message()						*/
/* 									*/
/* Works like a printf only sends the string as a message of type	*/
/* MESSAGE to the given socket.						*/
/* Returns error level.							*/
/************************************************************************/

int server_print_message(int sock, char *fmt, ...)
{
	message_struct message;
        char    string[9999];
        va_list args;

        /* Create the string */

        va_start(args, fmt);
        vsprintf(string, fmt, args);

	/* Create the message */

	message.type = MESSAGE;
	message.length = strlen(string);
	message.data = (unsigned char *)string;

	/* Send the message */

	return server_send_message(sock, &message);

} /* server_print_message() */

/************************************************************************/
/* server_print_message_all()						*/
/* 									*/
/* Works like a printf only sends the string as a message of type	*/
/* MESSAGE to the all sockets.						*/
/* Returns error level.							*/
/************************************************************************/

int server_print_message_all(char *fmt, ...)
{
	message_struct message;
        char    string[9999];
        va_list args;

        /* Create the string */

        va_start(args, fmt);
        vsprintf(string, fmt, args);

	/* Create the message */

	message.type = MESSAGE;
	message.length = strlen(string);
	message.data = (unsigned char *)string;

	/* Send the message */

	return server_send_message_all(&message);

} /* server_print_message_all() */

/************************************************************************/
/* print_log()								*/
/* 									*/
/* Works like a printf only sends the string as a message of type       */
/* CENTRAL_SCRUTINIZER_LOG to the all sockets.                          */
/* It is likely only the central scutinizer itself will do anything     */
/* with the result.                                                     */
/* Returns error level.                                                 */
/************************************************************************/

int print_log(char *fmt, ...)
{
	message_struct message;
        char    string[9999];
        va_list args;

        /* Create the string */

        va_start(args, fmt);
        vsprintf(string, fmt, args);

	/* Create the message */

	message.type = CENTRAL_SCRUTINIZER_LOG;
	message.length = strlen(string);
	message.data = (unsigned char *)string;

	/* Send the message */

	return server_send_message_all(&message);

} /* print_log() */

/************************************************************************/
/* send_server_stopping()						*/
/* 									*/
/* Send a SERVER_STOPPING mesage to all connected machines.		*/
/* Returns error level.							*/
/************************************************************************/

int send_server_stopping(void)
{
	message_struct mess;

	mess.type = SERVER_STOPPING;
	mess.length = 0;
	mess.data = NULL;

	/* Send the message */

	return server_send_message_all(&mess);

} /* send_server_stopping() */
