/************************************************************************/
/* vis2_snr.c								*/
/*									*/
/* Routines for SNR calculations.					*/
/*									*/
/************************************************************************/
/*                                        	                      	*/
/* (C) This source code and its associated executable           	*/
/* program(s) are copyright.                                    	*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : April 1995							*/
/************************************************************************/

#include <math.h>
#include <nrc.h>
#include "interferometry.h"
#include <values.h>

/************************************************************************/
/* vis2_snr_susi_no_noise()						*/
/*									*/
/* Calculates the SNR of the visibility squared estimator 		*/
/* It uses the SUSI method of visibility measurement.			*/
/* Assumes no detector noise.						*/
/* n represents expected count in single spectral channel.		*/
/************************************************************************/

float	vis2_snr_susi_no_noise(float v, float n)
{
	float 	denom;
	float	n2,n3,n4;
	float   v2,v4;

	/* There are two pixels in this scheme */

	n /= 2.0;

	n2 = n * n;
	n3 = n2 * n;
	n4 = n2 * n2;

	v2 = v * v;
	v4 = v2 * v2;
	
	denom = 0.5*n4*v4 + 4.0*n3*v2 + 2*n2;

	denom = sqrt(denom);

	return (n2*v2/denom);

} /* vis2_snr_susi_no_noise() */

/************************************************************************/
/* vis2_snr_susi_noise()						*/
/*									*/
/* Calculates the SNR of the visibility  squared estimator.		*/
/* It uses the modified SUSI method of visibility measurement.	 	*/
/* It includes detector noise and assumes nothing about it.		*/
/* n represents expected count in single spectral channel.		*/
/************************************************************************/

float   vis2_snr_susi_noise(float v, float n, float s)
{
	float 	denom;
	float	n2,n3,n4;
	float   v2,v4;
	float	s2,s4;

	/* There are two pixels in this scheme */

	n /= 2.0;

	n2 = n * n;
	n3 = n2 * n;
	n4 = n2 * n2;

	v2 = v * v;
	v4 = v2 * v2;
	
	s2 = s * s;
	s4 = s2 * s2;

	denom = 0.5 * n4 * v4 + 
		4.0 * (2.0*n3 + n2)*(1 - 0.5*v2)+ 
		8.0 * s2 * (n2 * (1 + 0.5*v2) + n) +
		4.0 * s4;

	denom = sqrt(denom);

	return (n2*v2/denom);

} /* vis2_snr_susi_noise() */

/************************************************************************/
/* vis2_snr_susi_known_noise()						*/
/*									*/
/* Calculates the SNR of the visibility  squared estimator.		*/
/* It uses the modified SUSI method of visibility measurement.	 	*/
/* It includes detector noise and assumes it is well known.		*/
/* n represents expected count in single spectral channel.		*/
/************************************************************************/

float   vis2_snr_susi_known_noise(float v, float n, float s)
{
	float 	denom;
	float	n2,n3,n4;
	float   v2,v4;
	float	s2,s4;

	/* There are two pixels in this scheme */

	n /= 2.0;

	n2 = n * n;
	n3 = n2 * n;
	n4 = n2 * n2;

	v2 = v * v;
	v4 = v2 * v2;
	
	s2 = s * s;
	s4 = s2 * s2;

	denom = 0.5*n4*v4 + 4.0*n3*v2 + 2*n2 +
		s2*(4.0*n2*v2 + 4*n + 0.5) + 2.0*s4;

	denom = sqrt(denom);

	return (n2*v2/denom);

} /* vis2_snr_susi_known_noise() */

/************************************************************************/
/* vis2_snr_TT_no_noise()						*/
/*									*/
/* Calculates the SNR of the visibility squared estimator 		*/
/* It uses the Tango/Twiss method of visibility measurement.		*/
/* Assumes no detector noise.						*/
/* n represents expected count in single spectral channel.		*/
/************************************************************************/

float	vis2_snr_TT_no_noise(float v, float n)
{
	float 	denom;
	float	n2,n3;
	float   v2;

	/* There are four pixels in this scheme */

	n /= 4.0;

	n2 = n * n;
	n3 = n2 * n;

	v2 = v * v;
	
	denom = 2.0*n3*v2 + n2;

	denom = sqrt(denom);

	return (n2*v2/denom);

} /* vis2_snr_TT_no_noise() */

/************************************************************************/
/* vis2_snr_TT_noise()							*/
/*									*/
/* Calculates the SNR of the visibility  squared estimator.		*/
/* It uses the modified Tango/Twiss method of visibility measurement. 	*/
/* It includes detector noise and assumes nothing about it.		*/
/* n represents expected count in single spectral channel.		*/
/************************************************************************/

float   vis2_snr_TT_noise(float v, float n, float s)
{
	float 	denom;
	float	n2,n3,n4;
	float   v2,v4;
	float	s2,s4;

	/* There are four pixels in this scheme */

	n /= 4.0;

	n2 = n * n;
	n3 = n2 * n;
	n4 = n2 * n2;

	v2 = v * v;
	v4 = v2 * v2;
	
	s2 = s * s;
	s4 = s2 * s2;

	denom = n3*(16.0 - 6.0*v2) +
		n2*(4.0 - v2) +
		s2*(n2*(16.0+2.0*v2) + 6.0*n) + 4.0*s4;

	denom = sqrt(denom);

	return (n2*v2/denom);

} /* vis2_snr_TT_noise() */

/************************************************************************/
/* vis2_snr_TT_known_noise()						*/
/*									*/
/* Calculates the SNR of the visibility  squared estimator.		*/
/* It uses the modified TAngo/Twiss method of visibility measurement. 	*/
/* It includes detector noise and assumes it is well known.		*/
/* n represents expected count in single spectral channel.		*/
/************************************************************************/

float   vis2_snr_TT_known_noise(float v, float n, float s)
{
	float 	denom;
	float	n2,n3,n4;
	float   v2,v4;
	float	s2,s4;

	/* There are four pixels in this scheme */

	n /= 4.0;

	n2 = n * n;
	n3 = n2 * n;
	n4 = n2 * n2;

	v2 = v * v;
	v4 = v2 * v2;
	
	s2 = s * s;
	s4 = s2 * s2;

	denom = 2.0*n3*v2 + n2 + s2*(2.0*n2*v2+2.0*n+0.25) + s4;

	denom = sqrt(denom);

	return (n2*v2/denom);

} /* vis2_snr_TT_known_noise() */

/************************************************************************/
/* vis2_snr_beletic()							*/
/*									*/
/* Calculates the SNR of the visibility  squared estimator for the ccd  */
/* case. 								*/
/* Uses the Beletic formulation as shown in Appendix P of NSF report.	*/
/* n is the total expected counts reaching the whole detector.		*/
/************************************************************************/

float	vis2_snr_beletic(float v, float n, float s, float npix)
{
	float 	denom;
	float	n2,n3,n4;
	float   v2;
	float	s2,s4;

	n2 = n * n;
	n3 = n2 * n;
	n4 = n2 * n2;

	v2 = v * v;
	
	s2 = s * s;
	s4 = s2 * s2;

	denom = n2 + 0.5*n3*v2 + 2.0*npix*s2*(0.25*n2*v2 + n) + npix*npix*s4;

        denom = sqrt(denom);

        return (0.25*n2*v2/denom);

} /* vis2_snr_beletic() */
