/************************************************************************/
/* matrox_signal.c							*/
/*									*/
/* Routines to work with the meteor matrox frame grabber.		*/
/* These files use the signaled mode, that is, the framge grabber sends */
/* USRSIG2 when an image is ready. It is the fastest mode, but seems	*/
/* to conflict with select(2) calls and other keyboard activity.	*/
/************************************************************************/
/*                                        	                      	*/
/*     Center for High Angular Resolution Astronomy			*/
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.		*/
/*									*/
/*									*/
/* Telephone: 1-404-651-1882						*/
/* Fax      : 1-404-651-1389						*/
/* email    : theo@chara.gsu.edu					*/
/* WWW	    : http://www.chara.gsu.edu/~theo/theo.html			*/
/*									*/
/* (C) This source code and its associated executable           	*/
/* program(s) are copyright.                                    	*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : April 1999   						*/
/************************************************************************/

#include "matrox_meteor.h"
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/fcntl.h>
#include <ioctl_meteor.h>
#include <stdlib.h>
#include <signal.h>

int meteor_fd = -1;
static unsigned char	*mmbuf = NULL;
static void (*frame_handler)(unsigned char *frame, int cols, int rows);
static int size8 = 0;
static int size16 = 0;
static struct meteor_frame_offset	frame_offset;
static int number_of_frames = 0;
static int columns = 0;
static int rows = 0;
static struct meteor_mem *mm = NULL;
static int frame_ptr=0;

/************************************************************************/
/* open_meteor_signal()							*/
/*									*/
/* Tries to open the matrox meteor board. If a filename is given it	*/
/* tries to use it, otherwise it uses the default name. Returns 0	*/
/* if it works or the following errors:					*/
/* 1 - Device already open or failed to open it.			*/
/* 2 - ioctl call for FORMAT failed.					*/
/* 3 - ioctl call for geom failed.					*/
/* 4 - ioctl call for contrast failed.					*/
/* 5 - ioctl call for brightness failed.				*/
/* 6 - ioctl call for FPS failed.					*/
/* 7 - ioctl for frame offset failed					*/
/* 8 - memory allocation failed						*/
/* 9 - signal allocation failed						*/
/* 10 - signal setting in driver failed					*/
/* 11- Failed to tell driver to start getting frames.			*/
/*									*/
/* If there is an error the errorno global will be set appropriately.   */
/************************************************************************/

int open_meteor_signal(char *devname, 
	int col, 
	int row, 
	void (*handler)(unsigned char *, int cols, int rows))
{
	char	*s;
	struct meteor_geomet geo;
	int		c;
	struct meteor_capframe frame;
	long size;

	/* Is it already open? */

	if (meteor_fd != -1) return 1;

	/* Otherwise. let's open it */
	
	if (devname == NULL)
		s = DEFAULT_DEVICE;
	else
		s = devname;

	if ((meteor_fd = open(s,O_RDONLY)) < 0) return 1;

	/* It will always be NTCS as far as we are concerned */

        c = METEOR_FMT_NTSC;

        if (ioctl(meteor_fd, METEORSFMT, &c) < 0) 
	{
		close(meteor_fd);
		meteor_fd = -1;
		return 2;
	}

	if (col < 1) col = DEFAULT_COLS;
	if (row < 1) row = DEFAULT_ROWS;

	geo.rows = row;
	geo.columns = col;
	geo.frames = NOFRAMES;
	geo.oformat = METEOR_GEO_YUV_PLANER; /* Always B&W */

        if (ioctl(meteor_fd, METEORSETGEO, &geo) < 0) 
	{
		close(meteor_fd);
		meteor_fd = -1;
		return 3;
	}

	/* Set defaut Contrast and Brightness */

	if (last_brightness < 0)
	{
		if (set_meteor_brightness(DEFAULT_BRIGHTNESS)) 
		{
			close(meteor_fd);
			meteor_fd = -1;
			return 4;
		}
	}
	else
	{
		if (set_meteor_brightness(last_brightness)) 
		{
			close(meteor_fd);
			meteor_fd = -1;
			return 4;
		}
	}

	if (last_contrast < 0)
	{
		if (set_meteor_contrast(DEFAULT_CONTRAST)) 
		{
			close(meteor_fd);
			meteor_fd = -1;
			return 5;
		}
	}
	else
	{
		if (set_meteor_contrast(last_contrast)) 
		{
			close(meteor_fd);
			meteor_fd = -1;
			return 5;
		}
	}

	/* Set maximum frames per second */

	c = FPS;
	if (ioctl(meteor_fd,METEORSFPS,&c) < 0) 
	{
		close(meteor_fd);
		meteor_fd = -1;
		return 6;
	}

	/* Setup the frame_handler */

	frame_handler = handler;

	/* Get the frame offset */

	if (ioctl(meteor_fd,METEORGFROFF,&frame_offset) < 0) 
	{
		close(meteor_fd);
		meteor_fd = -1;
		return 7;
	}

	/* Setup the sizes and so on */

	size8 = col * row;
	size16 = size8*2;
	columns = col;
	rows = row;
	size = ((col*row*NOFRAMES*2+4095)/4096)*4096;

	/* Allocate the memory */

	if ((mmbuf=(unsigned char *) mmap((caddr_t)0, 
				          size + 4096,
					  PROT_READ | PROT_WRITE, 
				 	  MAP_FILE|MAP_PRIVATE, 
					  meteor_fd, 
					  (off_t)0)) == (unsigned char *)(-1)) 
	{
		close(meteor_fd);
		meteor_fd = -1;
		return 8;
	}

	mm = (struct meteor_mem *)(mmbuf+size);

	/* Make sure we have our signal catching ready */

	if (signal(SIGUSR2, signal_handler) == SIG_ERR)
	{
		close(meteor_fd);
		meteor_fd = -1;
		return 9;
	}

	/* Tell the driver to use this signal */

	c = SIGUSR2;
   	if (ioctl(meteor_fd, METEORSSIGNAL, &c) < 0)
	{
		close(meteor_fd);
		meteor_fd = -1;
		return 10;
	}

	/* Now instruct the driver to start getting frames */

	frame.command = METEOR_CAP_N_FRAMES;
	frame.lowat = 0;
	frame.hiwat = NOFRAMES-1;

   	if (ioctl(meteor_fd, METEORCAPFRM, &frame) < 0)
	{
		close(meteor_fd);
		meteor_fd = -1;
		return 11;
	}

	/* All seems to have went well */

	number_of_frames = 0;
	return 0;

} /* open_meteor_signal() */

/************************************************************************/
/* close_meteor()							*/
/*									*/
/* closes the device for the meteor frame grabber.			*/
/************************************************************************/

void close_meteor_signal(void)
{
	struct meteor_capframe frame;

	/* Is it actually open? */

	if (meteor_fd == -1) return;

	/* Now instruct the driver to stop getting frames */

	frame.command = METEOR_CAP_STOP_FRAMES;
   	ioctl(meteor_fd, METEORCAPFRM, &frame);

	/* Close it */

	frame_handler = NULL;
	close(meteor_fd);
	meteor_fd = -1;

} /* close_meteor_signal() */

/************************************************************************/
/* signal_handler()							*/
/*									*/
/* Signal service routine for frame grabber, should never be called by 	*/
/* program.								*/
/************************************************************************/

void signal_handler(int sig)
{
#ifdef JUNK
	unsigned char *frame;
#endif

/* THIS DOESN'T SEEM TO WORK RIGHT, BUT IT DOES SEEM TO WORK..... */
#ifdef JUNK
	/* Find lowest frame that is active */

	frame = mmbuf + frame_ptr * size16;
#endif

	/* Call the user routine */

	if (frame_handler != NULL) 
	{
		(*frame_handler)(mmbuf, columns, rows);
	}

	/* Let the driver know we have used this frame */

	(mm->active) &= (~(1 << frame_ptr));
	if (--(mm->num_active_bufs) < 0) mm->num_active_bufs = 0;

	/* Incriment the number of frames */

#ifdef JUNK
	frame_ptr = ((frame_ptr + 1) % NOFRAMES);
#endif
	++number_of_frames;

} /* signal_handler() */

/************************************************************************/
/* get_signal_number_of_frames()					*/
/*									*/
/* Returns the number of frames processed so far.			*/
/************************************************************************/

int get_signal_number_of_frames(void)
{
	return number_of_frames;

} /* get_signal_number_of_frames() */
