/************************************************************************/
/* scope.c								*/
/*									*/
/* Routines to emulate an osciliscope on ther screen.  			*/
/* Part of the simpleX library.						*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : August 1994   						*/
/*									*/
/* Changes								*/
/*	 7/25/96 Removed free(picture) from close_scope(), this is done */
/*               XDestroyImage() and causes a segmentation fault doing  */
/*               it twice, at least this is true for Linux.		*/
/************************************************************************/

#include <stdlib.h>
#include <math.h>
#include <values.h>
#include "../nrc/nrc.h"
#include "simpleX.h"

#define	 scope_position(Scope,x,y) (*(Scope->picture+y*Scope->xpix+x))

#define	 signal_scale(Scope,nsig,fy,y) {\
	(y)=(int)((fy-(Scope)->signals[nsig].offset)*\
	(Scope)->signals[nsig].yscale+0.5);\
	if ((y) < 0) (y) = 0;\
	else if ((y) >= (Scope)->ypix) (y) = (Scope)->ypix-1;\
	(y) = (Scope)->ypix - (y) - 1;}

/************************************************************************/
/* open_scope()								*/
/*									*/
/* Initialize a scope window.						*/
/* Returns the scope structure.						*/
/************************************************************************/

scope open_scope(char *name, int x, int y, int nx, int ny)
{
	/* Internals */

	scope	TheScope;
	int	i;

	/* Create a new window for the scope */

	TheScope.scope_window = openWindow(name,x,y,nx,ny);

	/* Set internals of the structure */

	strncpy(TheScope.name,name,80);
	TheScope.xpix = nx;
	TheScope.ypix = ny;
	TheScope.total_pix = nx*ny;
	TheScope.nsignals = 0;
	
	/* Allocate memory for the actual picture */

	TheScope.picture = (char *)malloc(nx * ny);
	TheScope.last_position = (int *)malloc(MAX_SIGNALS*sizeof(int));

	if (TheScope.picture == NULL)
	{
		fprintf(stderr,"Out of memory in init_scope().\n");
		exit(-1);
	}

	/* Reset all the sinal parameters */

	for(i=0; i<MAX_SIGNALS; i++)
	{
		TheScope.signals[i].color = 0;
		TheScope.signals[i].yscale = 1;
		TheScope.signals[i].offset = 0;
	}

	/* Set the entire picture to black */

	for(i=0;i<nx*ny;i++) *(TheScope.picture+i) = GreyScale[0];

	/* Create an image for the scope */

	TheScope.scope_image = XCreateImage(theDisplay,theVisual,theDepth,
				ZPixmap,0,
				TheScope.picture,nx,ny,8,nx);

	/* Display dumby picture */

	XPutImage(theDisplay,TheScope.scope_window,theGC,TheScope.scope_image,
			0,0,0,0,nx,ny);
	XMapWindow(theDisplay, TheScope.scope_window);

	/* That is all... */

	return TheScope;

} /* open_scope() */

/************************************************************************/
/* close_scope()							*/
/*									*/
/* Close the scope and deallocate memory for it.			*/
/************************************************************************/

void close_scope(scope *TheScope)
{
	int	i;

	/* Free up the memory */

	XDestroyWindow(theDisplay, TheScope->scope_window);
	XDestroyImage(TheScope->scope_image);

	/* Set parameters back to defaults */

	TheScope->name[0] = 0;
	TheScope->xpix = 0;
	TheScope->ypix = 0;
	TheScope->nsignals = 0;
        for(i=0; i<MAX_SIGNALS; i++)
        {
                TheScope->signals[i].color = 0;
                TheScope->signals[i].yscale = 1;
                TheScope->signals[i].offset = 0;
        }
	TheScope->picture = NULL;
	free(TheScope->last_position);
	TheScope->last_position = NULL;

} /* close_scope() */

/************************************************************************/
/* add_signal()								*/
/* 									*/
/* Add a new signal to a scope.						*/
/* Returns the position of the signal within the scope structure.	*/
/* Returns a zero if there is no more room.				*/
/************************************************************************/

int	add_signal(scope *TheScope, 
		   int red, int green, int blue,
		   float ymin,
		   float ymax)
{
	/* Internals */

	XColor	color; /* The color value */
	int	nsig;  /* The signal number */
	int	i;     /* Looping */
	char *p;

	/* Make sure we haven't got too many signals */

	if (TheScope->nsignals >= MAX_SIGNALS)
	{
		fprintf(stderr,
			"Too many signals in scope %s, new signal ignored.\n",
			TheScope->name);
		return 0;
	}

	/* Work out the new signal position */

	nsig = TheScope->nsignals;
#ifdef DEBUG
	printf("Signal number = %d\n",nsig);
#endif

	/* Try and allocate the color */

	color.red = red;
	color.green = green;
	color.blue = blue;
	color.flags = DoRed | DoGreen | DoBlue;
	if (!XAllocColor(theDisplay,theCmap,&color))
	{
		fprintf(stderr,"Could not allocate new color for scope %s\n",
			TheScope->name);
	}

	TheScope->signals[nsig].color = color.pixel;

	/* Set up the other bits of the signal */

	TheScope->signals[nsig].yscale = (float)TheScope->ypix/(ymax-ymin);

	TheScope->signals[nsig].offset = ymin;
	
	signal_scale(TheScope,nsig,0,TheScope->signals[nsig].zero_pos);

	if (TheScope->signals[nsig].zero_pos <= 0 || 
		TheScope->signals[nsig].zero_pos >= TheScope->ypix) 
			TheScope->signals[nsig].zero_pos = -1;
#ifdef DEBUG
	printf("Yscale   = %f\n",TheScope->signals[nsig].yscale);
	printf("Offset   = %f\n",TheScope->signals[nsig].offset);
	printf("Zero_pos = %d\n",TheScope->signals[nsig].zero_pos);
#endif

	/* Create X axis if there is one .... */

	if (TheScope->signals[nsig].zero_pos != -1)
	{
#ifdef DEBUG
		printf("Zero position at pixel %d\n",
			TheScope->signals[nsig].zero_pos*TheScope->xpix);
#endif
		p = TheScope->picture + 
			TheScope->signals[nsig].zero_pos * TheScope->xpix;

		for(i=0;i<TheScope->xpix;i++) 
		{
			*p++ = TheScope->signals[nsig].color;
		}
	}
		
	/* Set last position to zero */

	TheScope->last_position[nsig] = TheScope->signals[nsig].zero_pos;

	/* Update the display */

	XPutImage(theDisplay,TheScope->scope_window,theGC,TheScope->scope_image,
			0,0,0,0,TheScope->xpix,TheScope->ypix);
	XMapWindow(theDisplay, TheScope->scope_window);

	/* Incriement the number of signals */

	(TheScope->nsignals)++;

	/* Return with the number of this signal */

	return (nsig+1); /* Note this is offset by 1 for NRC reasons */

} /* add_signal() */

/************************************************************************/
/* update_scope() 							*/
/*									*/
/* DOes the updating of the scope values.....				*/
/************************************************************************/

void update_scope(scope TheScope, float *values)

{
	int	i,j;
	char *p;
	int	top;
	int	edge;
	int	pos;
	int	start_line;
	int	stop_line;

	/* First job is to shift everything to the left one pixel */

	p = TheScope.picture;
	top = TheScope.total_pix - 1;
	edge = TheScope.xpix - 1;

	for(i=0;i<top;i++)
	{
		*p = *(p+1);
		p++;
	}

	/* Now clean up the right hand edge */

	p = TheScope.picture + TheScope.xpix-1;
	for(i=0;i<TheScope.ypix;i++)
	{
		*p = GreyScale[0];
		p += TheScope.xpix;
	}
	
	/* Put the axis back where they should be and add new points */

	for(i=0;i<TheScope.nsignals;i++)
	{
		if (TheScope.signals[i].zero_pos != -1)
		{
			*(TheScope.picture + TheScope.signals[i].zero_pos *
				TheScope.xpix + edge)=TheScope.signals[i].color;
		}

		signal_scale(&TheScope,i,values[i+1],pos);

		if (pos > TheScope.last_position[i])
		{
			start_line = TheScope.last_position[i];
			stop_line  = pos;
		}
		else
		{
			start_line  = pos;
			stop_line = TheScope.last_position[i];
		}
		TheScope.last_position[i] = pos;

		for(j=start_line; j<= stop_line; j++)
		{
			*(TheScope.picture + j * TheScope.xpix + edge) = 
				TheScope.signals[i].color;
		}
	}

        /* Update the display */

	XPutImage(theDisplay,TheScope.scope_window,theGC,TheScope.scope_image,
                       0,0,0,0,TheScope.xpix,TheScope.ypix);
        XMapWindow(theDisplay, TheScope.scope_window);
}
