/************************************************************************/
/* convert.c								*/
/*									*/
/* This file contains conversion routines. Eg string to angle, angle	*/
/* to string and so on.							*/
/*									*/
/* NOTE:  The conversion routines must be able to properly handle	*/
/*	  angles such as -0 30 40.00.   Likewise, in converting		*/
/*	  decimal numbers to sexagesimal notation an angle like 0.5	*/
/*	  degrees should become -0 30 00.00.  				*/
/*									*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*									*/
/*	Author: Theo ten Brummelaar					*/
/*	Revised by: W. J. Tango						*/
/*		Telephone: +61 2 692-2544				*/
/*		ACSnet:    theo@astron.physics.su.oz			*/
/*	Date:   17 August 1990						*/
/*	Revised:   July 1991						*/
/*									*/
/************************************************************************/

/*
 * Include files
 */	

#include "chara.h"
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <stdlib.h>

/* Global types array for bands */

char *channel_types[3] = {"H","K",NULL};
char *scope_types[NUM_SCOPES+1] = {"S1", "S2", "E1", "E2", "W1", "W2", NULL};
char *ople_types[NUM_SCOPES+1] = {"S1", "S2", "E1", "E2", "W1", "W2", NULL};
char *pop_types[NUM_POPS+1] = {"POP1", "POP2", "POP3", "POP4", " END", NULL};
char *beam_types[NUM_BEAMS+1] = {"BEAM1", "BEAM2", "BEAM3","BEAM4", "BEAM5", 
				"BEAM6", NULL};


/* external variable to tell error has occured in string conv. funs */
int	susistd_err;

/* external variable set to true when converting strings to time */
int	time_not_angle = FALSE;

/************************************************************************/
/* string_to_angle()							*/
/*									*/
/* This function accepts a sexagesimal angle in string form and 	*/
/* returns a double value in radians.					*/
/* The function accepts strings of the following forms:			*/
/*	degrees[.[fraction]]						*/
/*	degrees minutes[.[fraction]]					*/
/*	degrees minutes seconds[.[fraction]]				*/
/* where [] indicates an optional decimal point and fraction. Negative	*/
/* angles are expressed in the usual way by placing a 'minus' in front	*/
/* of the degrees field (which may be zero).				*/
/*									*/
/* Returns 0.0 if any field (without a . is greater than 2 character	*/
/* long, or if invalid characters were in string.			*/
/************************************************************************/

double	string_to_angle(char *s)

{
	/* Internals */
	
	char field1[16], field2[16], field3[16];
	char *dummy;
	
	int count;
	int sign;

	double degrees;
	double minutes;
	double seconds;
	double value;
	double max;

	susistd_err = NOERROR;

	/* If this is a time conversion, max of field1 is 24 */
	if(time_not_angle)
		max = 24.0;
	else
		max = 360.0;

	/* Go to first non white space character and then
	 * check if it is a + or - and if so point dummy
	 * to the next char.
	 */
	dummy = s;
	while(isspace(*(dummy++)));
	dummy--;

	if (*dummy == '-')
		{
		dummy +=1;
		sign = TRUE;
		}
	else if (*dummy == '+')
		{
		dummy +=1;
		sign = FALSE;
		}
	else
		sign = FALSE;

	/* Now check only numbers, . or ' ' tab are in dummy */

	if(strspn(dummy,"1234567890. \t") != strlen(dummy))
		{
		susistd_err = BADSTR;
		return (0.0);
		}

	count = sscanf(dummy,"%s%s%s",field1, field2, field3);

	/*	Could not read string. Return a value of 0 */
	if (count == 0)
		{
		susistd_err = BADSTR;
		return (0.0);
		}

	if (count == 1)
		{
		/* 
		 * Only one field was present. 
		 */
		seconds = atof(field1);
		value = seconds/3600.0;
		if (sign)
			value = -value;
		if(fabs(value) > max)
			{
			susistd_err = BADSTR;
			return 0.0;
			}
		return ( D2R*value );
		}
	else if ( count == 2 )
		{
		/*  
		 *  Minutes and seconds only given. 
		 */
		minutes = atof( field1 );
		seconds = atof( field2 );
		value = (minutes + seconds/60.0)/60.0;
		if (sign) value = -value;
		return ( D2R*value );
		}
	else if (count == 3)
		{
		/*  Degrees, minutes and seconds given. */
		degrees = atof( field1 );
		minutes = atof( field2 );
		seconds = atof( field3 );
		if((degrees > max)||(minutes >= 60.0)||(seconds >= 60.0)||
			(strlen(field1)>3)||(strlen(field2)>2))
				{
				susistd_err = BADSTR;
				return 0.0;
				}
		value = degrees + (minutes + seconds/60.0)/60.0;
		if( sign )
			{
			value = -value;
			}
		return ( D2R*value );
		}
	else	{
		/*  Too many parameters! Return zero */
		susistd_err = BADSTR;
		return ( 0.0 );
		}

			
} /* string_to_angle() */

/************************************************************************/
/* angle_to_string()							*/
/*									*/
/* Convert an angle in radians to a sexagesimal display string.	The 	*/
/* should not be used for further calculations as some precision is	*/
/* lost in the the conversion!						*/
/*									*/
/* Integer arithmetic is used to avoid problems with rounding.		*/
/* 									*/
/* The string which will hold the result should be at least 14 chars	*/
/* long.  The precision of the string is 0.01 arcseconds.		*/
/*									*/
/* angle_to_string() returns a pointer to the display string.		*/
/*									*/
/************************************************************************/

char	*angle_to_string(double angle, char *display)

{
	/*	Internals	*/
	char sign;
	int degrees, minutes, seconds, frac;
	int int_val;

	/*
	 *  First get sign character and convert input
	 *  to its absolute value.
	 */
	if ( angle < 0.0 )
		{
		sign = '-';
		angle = -angle;
		}
	else	{
		sign = ' ';
		}

	
	/*
	 *  Next convert angle to integer format.
	 *  The value is the number of 1/100ths of arcseconds
	 *  in the original angle.
	 */
	angle *= (360.0E3/D2R);
	if (angle < 0)
	{
		int_val = (int)(angle - 0.5);
	}
	else
	{
		int_val = (int)(angle + 0.5);
	}

	/*  Get degrees ...	*/
	degrees = int_val/(int)(360000);

	/*  ... minutes ...	*/
	int_val -= (int)(360000)*degrees;
	minutes = int_val/(int)(6000);

	/*  ... seconds ...	*/
	int_val -= (int)(6000)*minutes;
	seconds = int_val/(int)(100);

	/*  ... and decimal fraction */
	frac = int_val - (int)(100)*seconds;

	/*
	 *  Now assemble the pieces into the string. Use
	 *  leading zeros for minutes and seconds and for
	 *  degrees < 10.
	 */
	if (degrees < 10)
		sprintf(display,
			" %c%02d %02d %02d.%02d",
			sign, degrees, minutes, seconds, frac);
	else
		sprintf(display,
			"%c%d %02d %02d.%02d",
			sign, degrees, minutes, seconds, frac);

	return (display);
	 
} /* angle_to_string */

/************************************************************************/
/* string_to_time()							*/
/*									*/
/* This function accepts a string as an argument. The string may have	*/
/* the form "hrs min sec", "hrs mins.fraction" or "hrs.fraction".	*/
/* The value so expressed is converted into radians.			*/ 
/************************************************************************/

double	string_to_time(char *s)

{
	double	value;

	time_not_angle = TRUE;
	value = ( 15.0*string_to_angle( s ) );
	time_not_angle = FALSE;
	return value;

} /* string_to_time() */



/************************************************************************/
/* time_to_string()							*/
/*									*/
/* Convert an time in radians to a sexagesimal display string.	The 	*/
/* should not be used for further calculations as some precision is	*/
/* lost in the the conversion! This routine can also be used to 	*/
/* convert time-like angles such as right ascension, hour angles, etc.  */
/* Negative values are permitted.					*/
/* Integer arithmetic is used to avoid problems with rounding.		*/
/* 									*/
/* The string which will hold the result should be at least 15 chars	*/
/* long.  The precision of the string is 1 msec. 			*/
/*									*/
/* time_to_string() returns a pointer to the display string.		*/
/*									*/
/************************************************************************/

char	*time_to_string(double time, char *display)

{
	/*	Internals	*/
	char sign;
	int hours, minutes, seconds, frac;
	int int_val;

	/*
	 *  First get sign character and convert input
	 *  to its absolute value.
	 */
	if ( time < 0.0 )
		{
		sign = '-';
		time = -time;
		}
	else	{
		sign = ' ';
		}
	
	time *= (24.0E4/D2R);
	if (time < 0)
	{
		int_val = (int)(time - 0.5);
	}
	else
	{
		int_val = (int)(time + 0.5);
	}

	/*  Get hours ...	*/
	hours = int_val/(int)(3600000);

	/*  ... minutes ...	*/
	int_val -= (int)(3600000)*hours;
	minutes = int_val/(int)(60000);

	/*  ... seconds ...	*/
	int_val -= (int)(60000)*minutes;
	seconds = int_val/(int)(1000);

	/*  ... and decimal fraction */
	frac = int_val - (int)(1000)*seconds;

	/*
	 *  Now assemble the pieces into the string. Use
	 *  leading zeros for minutes and seconds and for
	 *  hours < 10.
	 */

	if (hours < 10)
		sprintf(display,
			"%c%02d %02d %02d.%03d",
			sign, hours, minutes, seconds, frac);
	else
		sprintf(display,
			"%c%d %02d %02d.%03d",
			sign, hours, minutes, seconds, frac);

	return (display);


} /* time_to_string */

/************************************************************************/
/* chara_tm_to_tm()							*/
/*									*/
/* Moves data from struct chara_tm to struct tm.			*/
/* Only to get around 64/32 bit problems.				*/
/************************************************************************/

void    chara_tm_to_tm(struct chara_tm *from, struct tm *to)
{
        to->tm_sec = from->tm_sec;
        to->tm_min = from->tm_min;
        to->tm_hour = from->tm_hour;
        to->tm_mday = from->tm_mday;
        to->tm_mon = from->tm_mon;
        to->tm_year = from->tm_year;
        to->tm_wday = from->tm_wday;
        to->tm_yday = from->tm_yday;
        to->tm_isdst = from->tm_isdst;
#ifdef __USE_BSD
        to->tm_gmtoff = from->tm_gmtoff;
        to->tm_zone = NULL;
#else
        to->__tm_gmtoff = from->tm_gmtoff;
        to->__tm_zone = NULL;
#endif

} /* chara_tm_to_tm() */


/************************************************************************/
/* tm_to_chara_tm()							*/
/*									*/
/* Moves data from struct tm to struct chara_tm.			*/
/* Only to get around 64/32 bit problems.				*/
/************************************************************************/

void    tm_to_chara_tm(struct tm *from, struct chara_tm *to)
{
        to->tm_sec = from->tm_sec;
        to->tm_min = from->tm_min;
        to->tm_hour = from->tm_hour;
        to->tm_mday = from->tm_mday;
        to->tm_mon = from->tm_mon;
        to->tm_year = from->tm_year;
        to->tm_wday = from->tm_wday;
        to->tm_yday = from->tm_yday;
        to->tm_isdst = from->tm_isdst;
#ifdef __USE_BSD
        to->tm_gmtoff = from->tm_gmtoff;
        to->tm_zone_pointer_as_int = 0;
#else
        to->__tm_gmtoff = from->tm_gmtoff;
        to->__tm_zone_pointer_as_int = 0;
#endif

} /* tm_to_chara_tm() */

/************************************************************************/
/* get_ut_date()							*/
/*									*/
/* Work out the current UT date according to the Linux clock.		*/
/************************************************************************/

void get_ut_date(int *year, int *month, int *day, int *doy)
{
	struct  tm      *gmt_now;
        time_t current_time;

	time(&current_time);
	gmt_now = gmtime(&current_time);

	/* Note... not really Y2K cool.... */

	*year = gmt_now->tm_year + 1900;
        if (*year < 1950) *year+=100;

	*month = gmt_now->tm_mon+1;
	*day = gmt_now->tm_mday;
	*doy = gmt_now->tm_yday;

} /* get_ut_date() */
