/************************************************************************/
/* extract_system_data.c						*/
/*									*/
/* Routines which extract telescope data for various programs.		*/
/*									*/
/* CONVENTION:  Extract... functions return data from the astrometric	*/
/* data bases and require no interaction with the user.  They are	*/
/* generic functions which can be used by any program and hence form	*/
/* part of the standard library. 					*/
/*									*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*									*/
/*	Author: W. J. Tango 						*/
/*	Modified:  Dave Berger	January 1999				*/
/*	Modified:  Theo ten Brummelaar	200-2003			*/
/*									*/
/************************************************************************/

#include "chara.h"
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define	BUF_LENGTH	128
static	char	buffer[BUF_LENGTH];

/* Definition of filters */

float ir_filter_center[NUM_IR_FILTERS] = {
	K_BAND_CEN, H_BAND_CEN, CONT_K_CEN, BRAC_GAM_CEN};
float ir_filter_width[NUM_IR_FILTERS] = {
	K_BAND_WIDTH, H_BAND_WIDTH, CONT_K_WIDTH, BRAC_GAM_WIDTH};
char  *ir_filter_names[NUM_IR_FILTERS+1] = {
	"K BAND", "H BAND", "CONT K", "BRKGAM", NULL};
	
/************************************************************************/
/* extract_system_data()						*/
/*									*/
/* Reads the telescopes and system data files and extracts the data 	*/
/* there in. Uses the following two functions to do so.			*/
/* Returns 0 if all goes well -1 if there's trouble with the telescopes */
/* and -2 if there is trouble with the system.				*/
/************************************************************************/

int extract_system_data(SYSTEM *chara, TELESCOPE *Chara_scopes)
{
	int return_value = 0;

	return_value = ExtractTelescopeData(chara, Chara_scopes);

	if (ExtractSystemData(chara) != 0) return -1;

	if (return_value != 0) return -2;

	return 0;

} /* extract_system_data() */

/************************************************************************/
/*									*/
/*	ExtractTelescopeData()						*/
/*									*/
/*	This function gets the relevant data from the			*/
/*	telescope data file.  Returns -1 if something isn't right.	*/
/*	Puts the number of telescopes and which telescopes are being	*/
/*	used into the SYSTEM structure.					*/
/************************************************************************/

int ExtractTelescopeData(SYSTEM *chara, TELESCOPE *Chara_scopes)
{
	/* Internals */

	char	*label, *field;
	char	scopes_used[30], scope[5];
	int 	station_number, scope_pointer, i, j;
	FILE 	*fp;
	

	/* Try and open data file */

	if((fp = fopen(TELESCOPE_DATA_FILE,"r"))==NULL)
	{
		fprintf(stderr,
		"astromod: Could not open file : %s\n",TELESCOPE_DATA_FILE);
		return(-1);
	}

	/* Set up defaults */

	for(i=0; i<MAX_NUMBER_TELESCOPES; i++) 
	{
		Chara_scopes[i].arm = 'X';
		Chara_scopes[i].station = 0;
		Chara_scopes[i].beam = BEAM_NONE;
		Chara_scopes[i].default_beam = BEAM_1;
		Chara_scopes[i].default_pop = POP_1;
		Chara_scopes[i].longitude = LONG_DFT;
		Chara_scopes[i].latitude = LAT_DFT;
		Chara_scopes[i].ec = 0.0;
		Chara_scopes[i].sas = 0.0;
		Chara_scopes[i].hsa2 = 0.0;
		Chara_scopes[i].wlf = 0.0;
		Chara_scopes[i].airpath = 0.0;
		Chara_scopes[i].internal = 0.0;
		Chara_scopes[i].lightpipe = 0.0;
		for(j=0; j<NUM_BEAMS; j++) Chara_scopes[i].beampath[j] = 0.0;
		for(j=0; j<NUM_POPS; j++) Chara_scopes[i].poppath[j] = 0.0;
	}

	/*
	 *	Get station data from the file 
	 */
	station_number = 0;
	while ( GetDataLine(buffer, BUF_LENGTH, fp) != -1)
	{
		/*
		 *  	Get next data item.  Note 
		 * 	strcmp returns FALSE when arguments are equal!
		 */
		label = ToUpper( strtok(buffer," ;\t\n") );
		field = (char *)strtok((char *)NULL,";\n");

                /*
                 * Station letter and number
                 */

                if( !strcmp(label, "S1") || 
                    !strcmp(label, "S2") ||
                    !strcmp(label, "E1") ||
                    !strcmp(label, "E2") ||
                    !strcmp(label, "W1") ||
                    !strcmp(label, "W2")
                  )
                {
                        Chara_scopes[station_number].arm
                         = *label;
                        Chara_scopes[station_number].station
                         = atoi(++label);
                }

		/* Deal with default beam */

		else if (!strcmp(label, "DEFBEAM")) 
			Chara_scopes[station_number].default_beam = 
				atoi(field) - 1;

		/* beam selection path corrections */

		else if (!strcmp(label, "BEAM1")) 
			Chara_scopes[station_number].beampath[BEAM_1] = 
				atof(field) * 1.0e-6;

		else if (!strcmp(label, "BEAM2")) 
			Chara_scopes[station_number].beampath[BEAM_2] = 
				atof(field) * 1.0e-6;

		else if (!strcmp(label, "BEAM3")) 
			Chara_scopes[station_number].beampath[BEAM_3] = 
				atof(field) * 1.0e-6;

		else if (!strcmp(label, "BEAM4")) 
			Chara_scopes[station_number].beampath[BEAM_4] = 
				atof(field) * 1.0e-6;

		else if (!strcmp(label, "BEAM5")) 
			Chara_scopes[station_number].beampath[BEAM_5] = 
				atof(field) * 1.0e-6;

		else if (!strcmp(label, "BEAM6")) 
			Chara_scopes[station_number].beampath[BEAM_6] = 
				atof(field) * 1.0e-6;

		/* Deal with default POP */

		else if (!strcmp(label, "DEFPOP")) 
			Chara_scopes[station_number].default_pop = 
				atoi(field) - 1;

		/* beam selection path corrections */

		else if (!strcmp(label, "POP1")) 
			Chara_scopes[station_number].poppath[POP_1] = 
				atof(field) * 1.0e-6;

		else if (!strcmp(label, "POP2")) 
			Chara_scopes[station_number].poppath[POP_2] = 
				atof(field) * 1.0e-6;

		else if (!strcmp(label, "POP3")) 
			Chara_scopes[station_number].poppath[POP_3] = 
				atof(field) * 1.0e-6;

		else if (!strcmp(label, "POP4")) 
			Chara_scopes[station_number].poppath[POP_4] = 
				atof(field) * 1.0e-6;

		else if (!strcmp(label, "POP5") || !strcmp(label, "POPEND"))
			Chara_scopes[station_number].poppath[POP_5] = 
				atof(field) * 1.0e-6;

		/*
		 * Station longitude
		 */

		else if ( !strcmp(label, "LONG") )
			Chara_scopes[station_number].longitude
			 = string_to_angle(field);

		/*
		 * Station latitude
		 */
		else if ( !strcmp(label, "LAT") )
			Chara_scopes[station_number].latitude
			 = string_to_angle(field);

		/*
		 * Station x (east) offset
		 * (and converts value from microns to meters)
		 */
                else if ( !strcmp(label, "XOFFSET") )
                        Chara_scopes[station_number].baseline.x
			 = (atof(field))*1.0e-6;

                /*
                 * Station y (north) offset
                 * (and converts value from microns to meters)
                 */
                else if ( !strcmp(label, "YOFFSET") )
                        Chara_scopes[station_number].baseline.y
			 = (atof(field))*1.0e-6;

                /*
                 * Station z (height) offset
                 * (and converts value from microns to meters)
                 */
                else if ( !strcmp(label, "ZOFFSET") )
                        Chara_scopes[station_number].baseline.z
			 = (atof(field))*1.0e-6;

		/*
 		 * Cos(El) effects 
                 * (and converts value from microns to meters)
		 */
		else if ( !strcmp(label, "EC"))
                        Chara_scopes[station_number].ec = (atof(field))*1.0e-6;

		/*
 		 * Swash, Sin component.
                 * (and converts value from microns to meters)
		 */
		else if ( !strcmp(label, "SAS"))
                        Chara_scopes[station_number].sas = (atof(field))*1.0e-6;

		/*
 		 * Swash, Cos component.
                 * (and converts value from microns to meters)
		 */
		else if ( !strcmp(label, "SAC"))
                        Chara_scopes[station_number].sac = (atof(field))*1.0e-6;

		/*
 		 * Harmonic of sin(2*Az)
                 * (and converts value from microns to meters)
		 */
		else if ( !strcmp(label, "HSA2"))
                        Chara_scopes[station_number].hsa2= (atof(field))*1.0e-6;

                /*
                 * White light fringe position
                 * (and converts value from microns to meters)
                 */
                else if ( !strcmp(label, "WLF") )
                        Chara_scopes[station_number].wlf = atof(field)*1.0e-6;

		/*
                 * Amount of airpath within the telescope
                 * (and converts value from microns to meters)
                 */
                else if ( !strcmp(label, "AIRPATH") )
                        Chara_scopes[station_number].airpath
                         = atof(field)*1.0e-6;

		/*
                 * Amount of airpath for internal fringes
		 * using the default beam.
                 * (and converts value from microns to meters)
                 */
                else if ( !strcmp(label, "INTERNAL") )
                        Chara_scopes[station_number].internal
                         = atof(field)*1.0e-6;

		/*
                 * Length of the light pipe
                 * (and converts value from microns to meters)
                 */
                else if ( !strcmp(label, "LIGHT") )
                        Chara_scopes[station_number].lightpipe
                         = atof(field)*1.0e-6;

		/*
		 * End of data record found
		 */
		else if ( !strcmp(label, "END") )
			++station_number;

		else
			printf("Unknown label %s in telescope data file.\n",
				label);

	} /* end while loop */

	/*
	 * close telescope file
	 */

	fclose(fp);

	/*
	 * Get the number of telescopes in data file
	 */

	chara->num_telescopes = station_number;
	chara->num_baselines = (station_number * (station_number-1))/2;

	/*
	 * Create a string which has the telescope scopes
	 * which are being used.  It is places in the 
	 * SYSTEM structure.
	 */
        scope_pointer=0;
        for (i = 0; i < chara->num_telescopes; ++i)
        {
		sprintf (scope, "%c%d", Chara_scopes[i].arm,
                               Chara_scopes[i].station);
                scope_pointer+=sprintf(scopes_used+scope_pointer,
                                        "%s ", scope);
        }
	strcpy (chara->scopes_used, scopes_used);

	return (0);

} /* ExtractTelescopeData() */


/************************************************************************/
/*									*/
/*	ExtractSystemData()						*/
/*									*/
/*	This function gets several quasi-constant parameters which	*/
/*	are needed to properly determine the positions. 		*/ 
/************************************************************************/

int ExtractSystemData(SYSTEM *data)
{
	/*	Internals	*/
	FILE *fp;

	char *label;
	char *field;


	int done;


	/*	Set default values	*/
	data->latitude = LAT_DFT;
	data->longitude = LONG_DFT;
	strcpy(data->fixed_cart,"S1");
	data->obs_wavelength = OBS_WAVE_DFT;
	data->met_wavelength = MET_WAVE_DFT;
	data->dut    = DUT_DFT;
	data->pole.x = POLEX_DFT;
	data->pole.y = POLEY_DFT;
	
	/*	First open data file	*/
	if ( (fp = fopen(SYSTEM_DATA_FILE,"r")) == NULL)
		{
		fprintf(stderr,"Cannot open star data file.  ");
		fprintf(stderr,"Default star parameters will be used.\n");
		return (-1);
		}

	/*	Initialize 'done' to be FALSE	*/
	done = FALSE;
	
	/*
	 *	Get data from next line(s) of file. 
	 */
	while ( GetDataLine(buffer, BUF_LENGTH, fp) != -1)
	{
		label = ToUpper( strtok(buffer," ;\t\n") );
		field = (char *)strtok((char *)NULL,";\n");

		/*	get longitude	*/
		if ( !strcmp(label, "LONG") )
			data->longitude = string_to_angle(field);

		/*	get latitude	*/
		if ( !strcmp(label, "LAT") )
			data->latitude = string_to_angle(field);

		/*  get observing and metrology wavelengths	*/
		if ( !strcmp(label, "OBSWAVE") )
			data->obs_wavelength = atof(field);
		if ( !strcmp(label, "METWAVE") )
			data->met_wavelength = atof(field);
		
		/* Polar motion x coordinate*/
		else if ( !strcmp(label, "POLARX") )
			data->pole.x = AS2R*atof(field);
			
		/* Polar motion y coordinate*/
		else if ( !strcmp(label, "POLARY") )
			data->pole.y = (AS2R)*atof(field);
			 
		/*	 UT1 - UTC 	*/
		else if ( !strcmp(label, "DUT") )
			data->dut = atof(field);
			 
		/*	Height of CHARA reference plane */
		else if ( !strcmp(label, "HEIGHT") )
			data->height = atof(field);

		/*  	End of record found	*/
		else if ( !strcmp(label, "END") )
			{
			done = TRUE;	
			break;
			}
		}

	/*	Finish up.	*/

	fclose(fp);

	if (!done)
	{
		/*
		 *	If we are here the end-of-file occurred before the
	 	 *	data record finished.
	 	 */
		fprintf(stderr,"Warning: unexpected end-of-file while ");
		fprintf(stderr,"reading star data.\n");
	}
	
	return (0);
}/*ExtractSystemData()*/
