/************************************************************************/
/* socket.c								*/
/*                                                                      */
/* Routines for using a socket to accept outside commands.  		*/
/************************************************************************/
/*                                                                      */
/*                    CHARA ARRAY USER INTERFACE			*/
/*                 Based on the SUSI User Interface			*/
/*		In turn based on the CHIP User interface		*/
/*                                                                      */
/*            Center for High Angular Resolution Astronomy              */
/*              Mount Wilson Observatory, CA 91001, USA			*/
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu			                */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar 		                        */
/* Date   : July 1998							*/
/************************************************************************/

/*
 * Include files
 */

#include "charaui.h"
#include <netdb.h>
#include <resolv.h>
#include <stdarg.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>

/* Define this if you are not connected to the network. */

/* #define STANDALONE */

/* Globals */

int      active_socket;
char *active_machine;

/* 
 * Local globals
 */

static int	command_socket = -1;	/* socket used for outside commands */
static struct {
		int  fd;		/* File descriptor */     
		char *machine; 	/* Name of connecting machine */
			} sockets[MAX_CONNECTIONS]; /* Active channels */
static int	data_socket = -1;	/* socket used for data comms */
static int	data_fd = -1;		/* Dito, but the file descriptor */
static char	*data_machine;		/* The machine's name */
BUFFER		*data_buffer;		/* Data buffer (circular) */
static char	allow_socket = FALSE;	/* For blocking socket comms */
static char	myname[256];		/* This machine's name */
static int	myport = -1;		/* The port we are listening too */
static int	dataport = -1;		/* The data port we are listening too */
static char	monitor_socket_commands = FALSE; /* Should we monitor? */
static FILE	*log_file = NULL; 		/* Should we log? */

/************************************************************************/
/* open_command_socket()						*/
/* 									*/
/* Opens a command socket that can be used for accepting outside 	*/
/* commands. Tries to find an open port starting at DEFAULT_COMMAND_PORT*/
/*									*/
/* This routine is set up so that it will accept a connection from 	*/
/* anyone in our domain, so be very carefull about this.		*/
/************************************************************************/

int open_command_socket()
{
        struct sockaddr_in server; /* Server address */
	int	i;
        struct hostent *hp;     	/* Pointer to host data */
	int	buffer_size;
	char	name[256];		
	int 	port = DEFAULT_COMMAND_PORT;

	/* Has this already been done? */

	if (command_socket != -1)
	{
		error(WARNING,"Command socket already exists.");
		return NOERROR;
	}

	/* Setup a default name */

	strcpy(myname,"Unknown");

	/* Clear the current sockets */

	active_socket = -1;
	active_machine = NULL;
	for(i=0; i< MAX_CONNECTIONS; i++) sockets[i].fd = -1;

	/* Now loop through a serries of possible values for the port */

	for(port = DEFAULT_COMMAND_PORT; port<DEFAULT_COMMAND_PORT+100; port++)
	{

		/* Create a socket */

		if ((command_socket = socket(AF_INET, SOCK_STREAM, 0)) < 0)
		{
			command_socket = -1;
			continue;
		}

		/* Force buffer sizes to buffer_size */

		buffer_size = SOCKET_BUFFER;
		if (setsockopt(command_socket, SOL_SOCKET, SO_SNDBUF, 
			(char *)&buffer_size, sizeof(buffer_size)) < 0)
		{
			close(command_socket);
			command_socket = -1;
			continue;
		}

		if (setsockopt(command_socket, SOL_SOCKET, SO_RCVBUF, 
			(char *)&buffer_size, sizeof(buffer_size)) < 0)
		{
			close(command_socket);
			command_socket = -1;
			continue;
		}

		/* Set up server address */

		server.sin_family = AF_INET;
		server.sin_addr.s_addr = htonl(INADDR_ANY);
		server.sin_port = htons(port);

		/* Now bind the socket to this address */

		if (bind(command_socket,
			(struct sockaddr *)&server,sizeof(server)) < 0)
		{
			close(command_socket);
			command_socket = -1;
			continue;
		}

		/* Create a queue for client connection requests */

		if (listen(command_socket, 0) < 0)
		{
			close(command_socket);
			command_socket = -1;
			continue;
		}

		/* If we reach here it must have worked */

		break;
	}

	if (port >= DEFAULT_COMMAND_PORT+100)
	{
		error(FATAL, "Failed to get a working command port.\n");
	}

	/* Find out our own name... */

        if(gethostname(name,81) < 0 || (hp = gethostbyname(name)) == 0)
        {
                close(command_socket);
		command_socket = -1;
                return error(ERROR,"Can't identify ourselves!.");
        }
	strcpy(myname,hp->h_name);
	myport = port;

	/* It all seems to be working */

	allow_socket = TRUE;

#ifdef STANDALONE
	message(system_window,"Started standalone command sockets on port %d.",
		myport);
#else
	message(system_window,"Started command sockets on port %d.", myport);
#endif

	return NOERROR;

} /* open_command_socket() */

/************************************************************************/
/* close_command_socket()						*/
/* 									*/
/* Close the command socket if it's open. Remember this could cause     */
/* trouble if you do this while the channel is active. If the command	*/
/* socket is not open this function will do nothing.			*/
/*									*/
/* This function will normally be called by the function ui_end() so	*/
/* you probably will never need it, unless you want to close and then	*/
/* reopen the socket.							*/
/************************************************************************/

void close_command_socket(void)
{
	int i;

	/* Do nothing if it isn't open */

	if (command_socket == -1) return;

	/* Send EOF message to all listening agents */

	all_socket_print("sockQuit");

	/* Try and make sure evryone has logged off */

	for(i=0; i< 4*MAX_CONNECTIONS; i++) process_command_socket();

	/* Clear the current sockets */

	active_socket = -1;
	active_machine = NULL;
	for(i=0; i< MAX_CONNECTIONS; i++) 
	{
		if (sockets[i].fd != -1) 
		{
			close(sockets[i].fd);
			sockets[i].fd = -1;
			free(sockets[i].machine);
		}
	}

	/* Close it */

	close(command_socket);

	/* Reset things */

	command_socket = -1;
	myport = -1;

	allow_socket = FALSE;

} /* close_command_socket() */

/************************************************************************/
/* socket_print() 							*/
/*									*/
/* This is the equivalent of a printf command but sends the result	*/
/* to the given socket. 						*/
/* Returns the number of characters printed.				*/
/************************************************************************/

int	socket_print(int sock, char *fmt, ...)
{
	char	s[SOCKET_BUFFER];
	va_list args;
	int	len;
	char	err_string[256];

	/* Check that the string is not too long */

        if (strlen(fmt) > sizeof(s) - sizeof((char)0))
        {
                strcpy(err_string,"Socket message too long\n");
                strcat(err_string,"Consult software supplier");
                error(FATAL,err_string);
        }

        /* Create the string */

        va_start(args, fmt);
        len = vsprintf(s, fmt, args);

        if (len > sizeof(s) - sizeof((char)0))
        {
                strcpy(err_string,"Socket message too long\n");
                strcat(err_string,"Consult software supplier");
                error(FATAL,err_string);
        }

	/* 
	 * There is a small chance that this socket has gone away.
	 * For this reason we now check and do nothing if it has.
	 */

	if (!write_ready(sock)) 
	{
/*
		error(WARNING,"Attempted to write to non-existant socket.");
*/
		return 0;
	}

	/* Write the message to the socket */

	if (write(sock, s, strlen(s)) != strlen(s))
	{
		return 0;
	}

	/* Log if neccesarry */

	if (log_file != NULL && sock == active_socket)
	{
	    fprintf(log_file,"%s->%s",active_machine,s);
	    fflush(log_file);
	}

	return len;

} /* socket_print() */

/************************************************************************/
/* all_socket_print() 							*/
/*									*/
/* This is the equivalent of a printf command but sends the result	*/
/* to all active command sockets.					*/
/* Returns the number of characters printed.				*/
/************************************************************************/

int	all_socket_print(char *fmt, ...)
{
	char	s[SOCKET_BUFFER];
	va_list args;
	int	len;
	char	err_string[256];
	int	i;

	/* Check that the string is not too long */

        if (strlen(fmt) > sizeof(s) - sizeof((char)0))
        {
                strcpy(err_string,"Socket message too long\n");
                strcat(err_string,"Consult software supplier");
                error(FATAL,err_string);
        }

        /* Create the string */

        va_start(args, fmt);
        len = vsprintf(s, fmt, args);

        if (len > sizeof(s) - sizeof((char)0))
        {
                strcpy(err_string,"Socket message too long\n");
                strcat(err_string,"Consult software supplier");
                error(FATAL,err_string);
        }

	/* Write the message to the socket */

	
	for(i=0; i<MAX_CONNECTIONS; i++)
	{
		if (sockets[i].fd < 0) continue;

		/* 
		 * There is a small chance that a socket has gone away.
		 * For this reason we now check and close it if it has.
		 */

		if (!write_ready(sockets[i].fd)) 
		{
			error(WARNING,
				"Attempted to write to non-existant socket.");
			sockets[i].fd = -1;
			free(sockets[i].machine);
			continue;
		}

		if (sockets[i].fd != -1 &&
			write(sockets[i].fd, s, strlen(s)) != strlen(s))
		{
			return 0;
		}
	}

	return len;

} /* all_socket_print() */

/************************************************************************/
/* socket_scan() 							*/
/*									*/
/* This is the equivalent of a scanf command but reads the data		*/
/* from the given socket. 						*/
/* Returns the number of items actually read.				*/
/************************************************************************/

int	socket_scan(int sock, char *fmt, ...)
{
	char	s[SOCKET_BUFFER];
	va_list args;

	/* Get a line of text from the socket. */

	if (socket_gets(sock, s, SOCKET_BUFFER) == (char *) EOF) return 0;

        /* Create the string */

        va_start(args, fmt);
        return vsscanf(s, fmt, args);

} /* socket_scan() */

/************************************************************************/
/* test_socket_scan()							*/
/*									*/
/* A user callable routine for testing the socket_scan() function.	*/
/************************************************************************/

int test_socket_scan(int argc, char **argv)
{
	int i;
	float	f;
	char	s[80];

	if (active_socket == -1) 
		return error(WARNING,"This needs to be called from a socket.");

	socket_print(active_socket,
		"Please enter an integer, flat and string.\n");
	socket_scan(active_socket,"%d %f %s",&i,&f,s);
	socket_print(active_socket,"Integer = %d\n",i);
	socket_print(active_socket,"Float = %f\n",f);
	socket_print(active_socket,"String = %s\n",s);

	return NOERROR;

} /* test_socket_scan() */

#ifdef BODIE
/************************************************************************/
/* socket_gets()                                                        */
/*                                                                      */
/* Equivalent to the normal gets() function, this function tries        */
/* to get a string from the given socket. The return character (if      */
/* there is one) is removed. The return value is a pointer to the       */
/* buffer sent as an argument or EOF on end of file or error.           */
/* Will read up to n-1 characters leaving room for null at end.         */
/************************************************************************/

char *socket_gets(int sock, char *s, int n)
{
        int     index;

        /* Run background for keyboard user */

        while(!char_waiting(sock)) background();

        /* Get the line of text */

        for(index = 0; index < n; index++)
        {
                if(read(sock, s + index, 1) <= 0) return (char *)EOF;
                if(s[index] == '\0' || s[index] == '\n')
                        break;
        }
        if(s[index] == '\n')
                s[index] = '\0';

        return s;

} /* socket_gets() */

#else
/************************************************************************/
/* socket_gets() 							*/
/* 									*/
/* Equivalent to the normal gets() function, this function tries	*/
/* to get a string from the given socket. The return character (if	*/
/* there is one) is removed. The return value is a pointer to the 	*/
/* sent as an argument or EOF on end of file or error.			*/
/* Will read up to n-1 characters leaving room for null at end.		*/
/* Will stop reading at new line character or NULL.			*/
/************************************************************************/

char *socket_gets(int sock, char *s, int n)
{
	int	len;

	/* Run background for keyboard user */

	while(!char_waiting(sock)) background();

        /* Get the line of text */

        if ((len = read(sock, s, n-1)) <= 0) return (char *)EOF;

        /* Terminate the string */

        s[len] = 0;

        /* Remove the final return character */

        if (s[len-1] == '\n') s[len-1] = 0;

	return s;

} /* socket_gets() */
#endif

/************************************************************************/
/* process_command_socket() 						*/
/*									*/
/* Checks through the array of sockets and see's if any have become	*/
/* domant. If any have it releases them. Checks the command_socket	*/
/* to see if there are any new connects. Adds them to the array of	*/
/* sockets if there are. Checks through the sockets array to see	*/
/* if any new commands have arrived. If so, set's active_socket to	*/
/* that socket, runs the command and returns a message to the socket	*/
/* concerning success or failure.					*/
/*									*/
/* This should be run all the time, most probably from get_command().	*/
/* For testing it can be run as a background job.			*/
/* Returns the error level.						*/
/************************************************************************/

int	process_command_socket(void)
{
	int	i;			/* Looping */
	char	line[SOCKET_BUFFER];	/* Line buffer */
	char	s[81];			/* A string buffer */
        int     function_pt;            /* index to function array */
        char    *first_word;            /* in the command line */
        char    *args;                  /* of the command line */
        struct sockaddr_in connector; 	/* Connector's address */
	unsigned int	len;			/* Lenght of address */
	int	error_level;		/* Of command called */
#ifndef STANDALONE
        struct hostent *hp;     	/* Pointer to host data */
	char	*domain;		/* Pointer to domain in name */
#endif
        struct sscripts *script_pt;     /* A pointer to a script */

	/* 
	 * If there is no command_socket or if there is an active_socket
	 * do nothing.
	 */

	if (command_socket == -1 || active_socket != -1) return NOERROR;

	/* 
	 * OK, let's loop through the current sockets 
	 * and see if any are dormant or have any data in them to read.
	 */

	for(i=0; i<MAX_CONNECTIONS; i++)
	{
		/* If it's not open move on */

		if (sockets[i].fd == -1) continue;

		/* If there is a character waiting do something about it */

		if (char_waiting(sockets[i].fd))
		{
			if (socket_gets(sockets[i].fd, line, SOCKET_BUFFER) == 
				(char *)EOF)
			{
				/* Read failed, connection has died */
	
				sockets[i].fd = -1;
				message(system_window,
				   "%s just logged off command socket.\n",
					sockets[i].machine);
				free(sockets[i].machine);
				continue;
			}

			/* Are we allowed to do anything ? */

			if (!allow_socket)
			{
				socket_print(sockets[i].fd,
					"Socket commands have been blocked.\n");
				continue;
			}

			/* If we are monitoring things then check */

			
			sprintf(s,"Can %s run the following command?",
				sockets[i].machine);
		 	if (monitor_socket_commands && !ask_yes_no(s,line))
			{
				socket_print(sockets[i].fd,
					"Stopped by keyboard user.\n");
				continue;
			}

			/* Tell keyboard user what's happening */

#ifdef VERBOSE
			if (strcmpi("ping",line))
			{
				message(system_window,
					"Processing socket %d command:\n%s",
					sockets[i].fd,line);
			}
#endif

			/* If we're logging... log */

			if (log_file != NULL)
			{
			    fprintf(log_file,"%s<-%s\n",
					sockets[i].machine,line);
			    fflush(log_file);
			}
					
			/* This becomes the active socket */

			active_socket = sockets[i].fd;
			active_machine = sockets[i].machine;

			/* split off first word from the argument list */

			first_word = split_string(line,&args);

			if (*first_word != (char)NULL) 
			{
				/* Is it a function or script name? */

				if ((function_pt = find_function(first_word)) 
					!= (int)NULL)
				{
					error_level = 
						call_function(function_pt,args);
				}
				else if ((script_pt=find_script(first_word)) != 
					(struct sscripts *)NULL)
				{
					error_level = call_script(script_pt);
				}
				else
				{
					error_level = error(ERROR,
						"%s is not a command.\n",
						first_word);
				}
				if (write_ready(active_socket))
				{
					socket_print(active_socket,
					SOCKET_COMMAND_STR,
					error_level);
				}
			}

			/* OK, we're finished with this socket for now */
		
			active_socket = -1;
			active_machine = NULL;
			if (strcmpi("ping",line))
			{
				werase(system_window);
				wrefresh(system_window);
			}
		}

		background();
	}

	/* See if someone wants a new connection */

	if (char_waiting(command_socket))
	{
		/* Find an empty slot for this new socket */

		for(i=0; i<MAX_CONNECTIONS && sockets[i].fd != -1; i++);

		if (i < MAX_CONNECTIONS)  /* We found one */
		{
			len = sizeof(connector);
                        if ((sockets[i].fd = accept(command_socket,
				(struct sockaddr *)&connector,&len)) < 0)
                        {
                                error(WARNING,
				      "Failed to accept socket connection.\n");
                        }

#ifndef STANDALONE
			/* OK, extract the name */

			if ((hp = gethostbyaddr((char *)&connector.sin_addr,
                                        len,connector.sin_family)) == NULL)
			{
				error(WARNING,
				"Failed to identify host. Blocking socket.");
				socket_print(sockets[i].fd,
				"Failed to identify host. Blocking socket.\n");
				close(sockets[i].fd);
				sockets[i].fd = -1;
			}
			else
			{
				/* Put the name of the machine in structure */

				if ((sockets[i].machine = 
					malloc(strlen(hp->h_name)+1)) == NULL)
				{
					error(FATAL,FATAL_1,
						"socket connection");
				}
				strcpy(sockets[i].machine,hp->h_name);

				/* Separrate out it's domain part. */

				for(domain=hp->h_name; *domain != 0 &&
					*domain != '.'; domain++);
				if (*domain == '.') ++domain;

				/* Is this OK ? */

				if (strcmp(myname,hp->h_name) && 
				    strcmp(domain,ALLOWED_DOMAIN))
				{
					socket_print(sockets[i].fd,
			"Not in correct domain. Blocking socket.\n\n");
					close(sockets[i].fd);
					sockets[i].fd = -1;
					free(sockets[i].machine);
					error(WARNING,
			"Host %s not in correct domain. Blocking socket.", 
					hp->h_name);
				}
				else message(system_window,
				    "%s just connected to the command socket.",
					hp->h_name);
			}

#else /* We are standalone, assummed NOT conencted to network */

			if ((sockets[i].machine = 
				malloc(strlen(myname)+1)) == NULL)
			{
				error(FATAL,FATAL_1,
					"socket connection");
			}
			strcpy(sockets[i].machine,myname);
			message(system_window,
			    "%s just connected to the command socket.",myname);
#endif
		}
	}

	return NOERROR;

} /* process_command_socket() */

/************************************************************************/
/* get_machine_name()							*/
/*									*/
/* Get the name of this machine. Expects the string sent to have memory */
/* already allocated. Returns the string sent as an argument.		*/
/* Note that if the socket command channel is not open yet you'll	*/
/* get garbage in the string.						*/
/************************************************************************/

char *get_machine_name(char *name)
{

	strcpy(name, myname);
	return name;

} /* get_machine_name() */

/************************************************************************/
/* get_command_port()							*/
/*									*/
/* Returns the command port.						*/
/************************************************************************/

int get_command_port()
{
	return myport;

} /* get_command_port() */

/************************************************************************/
/* get_data_port()							*/
/*									*/
/* Returns the data port.						*/
/************************************************************************/

int get_data_port()
{
	return dataport;

} /* get_data_port() */

/************************************************************************/
/* block_sockets() 							*/
/*									*/
/* Sets the allow_socket variable to FLASE and stops any socket		*/
/* commands from working. 						*/
/************************************************************************/

void block_sockets(void)
{
	/* Doit */

	allow_socket = FALSE;

	/* And let any open sockets know about it */

	all_socket_print("Socket commands have been blocked.\n");

} /* block_sockets() */

/************************************************************************/
/* unblock_sockets() 							*/
/*									*/
/* Sets the allow_socket variable to TRUE and starts socket		*/
/* commands working. 							*/
/************************************************************************/

void unblock_sockets(void)
{
	/* Doit */

	allow_socket = TRUE;

	/* Flush all previous commands */

	flush_sockets();

	/* And let any open sockets know about it */

	all_socket_print("Socket commands have been unblocked.\n");

} /* unblock_sockets() */

/************************************************************************/
/* call_block_sockets() 						*/
/*									*/
/* User callable routine that calls block_sockets().			*/
/************************************************************************/

int call_block_sockets(int argc, char **argv)
{
	/* We can not do this from a socket! */

	no_socket();

	/* OK, do it */

	block_sockets();

	return NOERROR;

} /* call_block_sockets() */

/************************************************************************/
/* call_unblock_sockets() 						*/
/*									*/
/* User callable routine that calls unblock_sockets().			*/
/************************************************************************/

int call_unblock_sockets(int argc, char **argv)
{
	/* We can not do this from a socket! */

	no_socket();

	/* OK, do it */

	unblock_sockets();

	return NOERROR;

} /* call_unblock_sockets() */

/************************************************************************/
/* open_connect_socket()						*/
/*									*/
/* Attempts to open up a socket port for connection. Requires a 	*/
/* machine name and a port number. Returns the socket file descriptor	*/
/* if successful or a -1 if it fails.					*/
/* The corresponding function close_connect_socket() is a simple	*/
/* macro for close().							*/
/* This may not work so well for STANDALONE machines...  		*/
/************************************************************************/

int open_connect_socket(char *machine, int port)
{
        struct sockaddr_in server; 	/* Server address */
        struct hostent *hp;     	/* Pointer to host data */
        int     sock;       		/* Socket number */
	int	buf_size;		/* Size of buffer */

        /* First see if we can identify the host. */

        if((hp = gethostbyname(machine)) == 0)
        {
                error(ERROR,"Host %s unknown.",machine);
                return -1;
        }

	/* We can not connect to ourselves. */

	if (strcmp(hp->h_name,myname) == 0 && port == myport)
	{
		return error(WARNING,"We can not connect to ourselves.\n");
	}

        /* Create a socket */

        if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0)
        {
                error(ERROR,"Failed to create socket.");
                return -1;
        }

        /* Force buffer sizes to SOCKET_BUFFER */

        buf_size = SOCKET_BUFFER;

        if (setsockopt(sock, SOL_SOCKET, SO_SNDBUF,
                (char *)&buf_size, sizeof(buf_size)) < 0)
        {
                close(sock);
                error(ERROR,"Failed to change send buffer size to %d\n",
                        buf_size);
                return -1;
        }

        if (setsockopt(sock, SOL_SOCKET, SO_RCVBUF,
                (char *)&buf_size, sizeof(buf_size)) < 0)
        {
                close(sock);
                error(ERROR,"Failed to change receive buffer size to %d\n",
                        buf_size);
                return -1;
        }

        /* Create server address */

        server.sin_family = AF_INET;
        strncpy((char *)&server.sin_addr, (char *)hp->h_addr, hp->h_length);
        server.sin_port = htons(port);

        /* Let's try and connect now.... */

        if (connect(sock, (struct sockaddr *)&server, sizeof(server)) < 0)
        {
                close(sock);
                message(system_window,
			"Failed to connect to %s port %d.",machine,port);
                return -1;
        }

	/* All is well it seems */

	return sock;

} /* open_connect_socket() */

/************************************************************************/
/* soccom()								*/
/*									*/
/* Simple ascii comms with a socket port. Really just an example	*/
/* of the use of the socket port routines.				*/
/************************************************************************/

int	soccom(int argc, char **argv)
{
	int	port; 	/* The port */
	char	s[81];	/* Edit string */
	int	sock;
	char	machine[81];
	char	line[SOCKET_BUFFER];

	no_socket();

	/* Check command line */

	if (argc < 2) 	/* No name given */
	{
		strcpy(s,"       ");
		strcat(s,myname);
		if (quick_edit("Machine",s, s,NULL,STRING)==KEY_ESC) 
		{
			return NOERROR;
		}
		sscanf(s,"%s",machine);
	}
	else
	{
                sscanf(argv[1],"%s",machine);
	}

	if (argc < 3) 	/* No port given */
	{
		sprintf(s,"1024");
		if (quick_edit("Port",s, s,NULL,INTEGER)==KEY_ESC) 
		{
			return NOERROR;
		}
		sscanf(s,"%d",&port);
	}
	else
	{
		sscanf(argv[2],"%d",&port);
	}

	/* open the port */

	if ((sock = open_connect_socket(machine, port)) == -1) 
	{
		return ERROR;
	}

	/* Alright, here we are and off we go */

	background_off(0,NULL);
	plain_screen_on();
	printf("Type <ESC> and enter to return to program.\n\r");
	fflush(stdout);
	system("stty sane");

        while(1)
        {
		if (char_waiting(0))
		{
			fgets(line, SOCKET_BUFFER, stdin);
			if (*line == KEY_ESC) break;
			socket_print(sock,"%s",line);
		}

                if (char_waiting(sock))
                {
			if (socket_gets(sock, line, SOCKET_BUFFER)==(char *)EOF)
			{
				printf("Socket closed by %s\n",machine);
				fflush(stdout);
                                break;
                        }

			printf("%s\n",line);
			fflush(stdout);
                }
	}

	/* That is all */

	close_connect_socket(sock);
	background_on(0,NULL);
	plain_screen_off();

	return NOERROR;

} /* soccom() */

/************************************************************************/
/* ping()								*/
/*									*/
/* Lets you know if this program is responding or not. Write message	*/
/* either to the screen or to a socket, depending on the value of	*/
/* the active_socket global.						*/
/************************************************************************/

int	ping(int argc, char **argv)
{
	if (active_socket == -1)
	{
		message(system_window,"I'm alive");
	}
	else
	{
		socket_print(active_socket,"%s on %s port %d is alive.\n",
			TITLE,myname,myport);
	}

	return NOERROR;

} /* ping() */

/************************************************************************/
/* start_socket_command_monitor()					*/
/*									*/
/* User callable function to initiate socket command monitoring.	*/
/* When switch on, the keyboard user get's to see what any socket 	*/
/* command user is up to. Each time a socket asks for a command		*/
/* the user is shown this command and asked if it is alright to run it. */
/* This monitoring is stopped by the command 				*/
/* stop_socket_command_monitor().					*/
/************************************************************************/

int	start_socket_command_monitor(int argc, char **argv)
{
	no_socket();

	monitor_socket_commands = TRUE;

	all_socket_print("The keyboard user is monitoring socket commands.\n");

	return NOERROR;

} /* start_socket_command_monitor() */

/************************************************************************/
/* stop_socket_command_monitor()					*/
/* 									*/
/* Stops the socket command monitor started by the function		*/
/* start_socket_command_monitor() 					*/
/************************************************************************/

int	stop_socket_command_monitor(int argc, char **argv)
{
	no_socket();

	monitor_socket_commands = FALSE;

	all_socket_print(
		"The keyboard user has stopped monitoring socket commands.\n");

	return NOERROR;

} /* stop_socket_command_monitor() */

/************************************************************************/
/* start_socket_command_log()						*/
/*									*/
/* User callable function to initiate socket command logging.		*/
/* All commands lsent in via sockets will be logged to the specified	*/
/* file.								*/
/*									*/
/* Turned off by function start_socket_command_log()			*/
/************************************************************************/

int	start_socket_command_log(int argc, char **argv)
{
	char	filename[81];
	char	s[81];

	no_socket();

	if (log_file != NULL)
	{
		return error(MESSAGE,
			"We're already logging socket commands.\n");
	}

	if (argc < 2) 	/* No name given */
	{
		strcpy(s,"socket_command.log");
		if (quick_edit("Log filename",s, s,NULL,STRING)==KEY_ESC) 
		{
			return NOERROR;
		}
		sscanf(s,"%s",filename);
	}
	else
	{
                sscanf(argv[1],"%s",filename);
	}

	if ((log_file = fopen(filename,"r")) != NULL)
	{
		fclose(log_file);
		log_file = NULL;
		return error(WARNING,"File %s already exists.",filename);
	}

	if ((log_file = fopen(filename,"w")) == NULL)
	{
		return error(WARNING,"Failed to open file %s.",filename);
	}

	all_socket_print("The keyboard user is logging socket commands.\n");

	return NOERROR;

} /* start_socket_command_log() */

/************************************************************************/
/* stop_socket_command_log()						*/
/* 									*/
/* Stops the socket command logging started by the function		*/
/* start_socket_command_log()	 					*/
/************************************************************************/

int	stop_socket_command_log(int argc, char **argv)
{
	no_socket();

	if (log_file != NULL)
	{
		fclose(log_file);
		log_file = NULL;
	}

	all_socket_print(
		"The keyboard user has stopped logging socket commands.\n");

	return NOERROR;

} /* stop_socket_command_log() */

/************************************************************************/
/* machines_connected()							*/
/* 									*/
/* User callable function that lists all the machines currently 	*/
/* connected to the command socket.					*/
/************************************************************************/

int	machines_connected(int argc, char **argv)
{
	int	i,line;

	clear_display();

	for(line = 0, i=0; i<MAX_CONNECTIONS; i++)
	{
		if (sockets[i].fd == -1) continue;

		if (++line >= NUM_TEXT_LINES)
		{
			--line;
			error(WARNING,"Too many lines for display.");
			break;
		}

		if((display->string[line-1] = (char *)malloc(LINE_LENGTH)) 
                                                        == NULL)
		{
			error(WARNING,
				"Cannot allocate any more lines in display");
			break;
		}

		sprintf(display->string[line-1],"%2d       %s",
			sockets[i].fd,sockets[i].machine);
	}
	display->number_items = line;

	/* Now display it */

        if (active_socket == -1)
        {
                /* Called by keyboard user, scroll the text on screen */

		if (line > 0)
		{
			active_window = main_window;
			heading(heading_window,
				"The following machines are connected :","");
			scroll_text(sub_main_window);
		}
		else
		{
			message(system_window,"Noone is connected");
		}
        }
        else
        {
                /* Called by a socket, spit it out the socket */

		if (line > 0)
		{
			socket_print(active_socket,
				"The following machines are connected:\n");
			for(i=0; i<display->number_items; i++)
			{
				socket_print(active_socket,"%s\n",
					display->string[i]);
			}
		}
		else
		{
			socket_print(active_socket,"Noone is connected.\n");
		}
        }

	clear_display();

	return NOERROR;

} /* machines_connected() */

/************************************************************************/
/* open_data_socket()							*/
/* 									*/
/* Opens a data socket that can be used for reading outside 		*/
/* data or sending data to the outside world. 				*/
/* Tries to find the first open port.					*/
/*									*/
/* This routine is set up so that it will accept a connection from 	*/
/* in our domain, so be very carefull about this.			*/
/************************************************************************/

int open_data_socket(void)
{
        struct sockaddr_in server; /* Server address */
	int	buffer_size;
	int	port;

	/* Has this already been done? */

	if (data_socket != -1)
	{
		error(WARNING,"Data socket already exists.");
		return NOERROR;
	}

	/* Cycle through a list of ports */

	for(port = DEFAULT_DATA_PORT; port <DEFAULT_DATA_PORT+100; port++)
	{
		/* Create a socket */

		if ((data_socket = socket(AF_INET, SOCK_STREAM, 0)) < 0)
		{
			data_socket = -1;
			continue;
		}

		/* Force buffer sizes to buffer_size */

		buffer_size = SOCKET_BUFFER;
		if (setsockopt(data_socket, SOL_SOCKET, SO_SNDBUF, 
			(char *)&buffer_size, sizeof(buffer_size)) < 0)
		{
			close(data_socket);
			data_socket = -1;
			continue;
		}

		if (setsockopt(data_socket, SOL_SOCKET, SO_RCVBUF, 
			(char *)&buffer_size, sizeof(buffer_size)) < 0)
		{
			close(data_socket);
			data_socket = -1;
			continue;
		}

		/* We make this a non-blocking port */

		if (fcntl(data_socket, F_SETFL, O_NONBLOCK) < 0)
		{
			close(data_socket);
			data_socket = -1;
			continue;
		}

		/* Set up server address */

		server.sin_family = AF_INET;
		server.sin_addr.s_addr = htonl(INADDR_ANY);
		server.sin_port = htons(port);

		/* Now bind the socket to this address */

		if (bind(data_socket,
			(struct sockaddr *)&server,sizeof(server)) < 0)
		{
			close(data_socket);
			data_socket = -1;
			continue;
		}

		/* Create a queue for client connection requests */

		if (listen(data_socket, 0) < 0)
		{
			close(data_socket);
			data_socket = -1;
			return error(WARNING,"Failed to listen to socket");
		}

		/* Create some buffer space */

		if ((data_buffer = set_up_buffer(BUFFER_SIZE)) == 0)
		{
			close(data_socket);
			data_socket = -1;
			continue;
		}

		/* If we get here it all must have worked */

		break;
	}

	if (port >= DEFAULT_DATA_PORT+100)
	{
		error(FATAL,"Failed to open data port.\n");
	}

	/* That should be all */

	dataport = port;
	return NOERROR;

} /* open_data_socket() */

/************************************************************************/
/* close_data_socket()							*/
/* 									*/
/* Close the data socket if it's open. Remember this could cause	*/
/* trouble if you do this while the channel is active. If the data	*/
/* socket is not open this function will do nothing.			*/
/*									*/
/* This function will normally be called by the function ui_end() so	*/
/* you probably will never need it, unless you want to close and then	*/
/* reopen the socket.							*/
/************************************************************************/

void close_data_socket(void)
{
	/* Do nothing if it isn't open */

	if (data_socket == -1) return;

	/* Close it */

	if (data_fd != -1) close(data_fd);
	close(data_socket);

	/* Reset things */

	data_socket = -1;

	/* Delete the buffer */

	remove_buffer(data_buffer);

} /* close_data_socket() */

/************************************************************************/
/* read_data()	 							*/
/* 									*/
/* Attempts to read n bytes of data from the data socket and puts it 	*/
/* into the buffer area. No checking is done to see if there is enough  */
/* space. If data socket is not open it does nothing. Returns number 	*/
/* of bytes actually read or negative on error.				*/
/* If the socket is open but noone is connected, it tries to connect.   */
/* This is a blocking read but will run the background.  		*/
/************************************************************************/

int read_data(void *buf, int n)
{
	int	len;			/* Length in bytes */
	int	ibuf;
	unsigned char *pt;

	if (!data_connected()) return 0;

	/* Is there something in the buffer already ? */

	for(pt=buf, ibuf=0; n!=0 && buffer_not_empty(data_buffer); n--, ibuf++)
	{
		buffer_out(data_buffer,*pt++);
	}

	/* Run background for keyboard user */

	while(!char_waiting(data_fd)) background();

	/* Get the data */

	len = read(data_fd, pt, n);

	/* Is there still someone there? */

	if (len <= 0)
	{
		/* They buggered off ... */

		data_fd = -1;
		message(system_window,
			"%s just disconnected from the data socket.",
			data_machine);
		free(data_machine);
		return len;
	}

	/* It seems to have worked */

	return len+ibuf;
		
} /* read_data() */

/************************************************************************/
/* read_data_fast() 							*/
/* 									*/
/* Simmilar to read_data except it does not polling of the socket 	*/
/* and no other tests. It could cause problems but will be much faster. */
/* Attempts to read n bytes of data from the data socket and puts it 	*/
/* into the buffer area. No checking is done to see if there is enough  */
/* space. If data socket is not open it does nothing. 			*/
/* Returns number of bytes actually read or negative if error.		*/
/* If an error is created the socket is assumed closed.			*/
/* It will not try and create a new connection, the connection		*/
/* must first be created using data_connected().			*/
/* This is a blocking read and will not run the background.  		*/
/************************************************************************/

int read_data_fast(void *buf, int n)
{
	int	len;			/* Length in bytes */
	int	ibuf;
	unsigned char *pt;

	if (data_socket < 0 || data_fd < 0) return 0;

	/* Is there something in the buffer already ? */

	for(pt=buf, ibuf=0; n!=0 && buffer_not_empty(data_buffer); n--, ibuf++)
	{
		buffer_out(data_buffer,*pt++);
	}

	/* Get the data */

	len = read(data_fd, pt, n);

	/* Is there still someone there? */

	if (len <= 0)
	{
		/* They buggered off ... */

		data_fd = -1;
		message(system_window,
			"%s just disconnected from the data socket.",
			data_machine);
		free(data_machine);
		return len;
	}

	/* It seems to have worked */

	return len+ibuf;
		
} /* read_data_fast() */

/************************************************************************/
/* data_ready() 							*/
/* 									*/
/* Checks to see if there is anything waiting in the data socket 	*/
/* for reading. If the socket is not open or noone is connected it'll	*/
/* return FALSE. If there is someone there but no data is there it'll	*/
/* also return FALSE. If there is data there it'll return, you guessed 	*/
/* it, TRUE.								*/
/* If someone is trying to connect it'll allow the connection.  	*/
/************************************************************************/

int data_ready(void)
{
	if (!data_connected()) return FALSE;

	if (buffer_not_empty(data_buffer)) 
		return TRUE;
	else
		return char_waiting(data_fd);

} /* data_ready() */

/************************************************************************/
/* write_data()	 							*/
/* 									*/
/* Attempts to write n bytes of data to the data socket			*/
/* from the buffer area.  If data socket is not open it does nothing.   */
/* Returns number of bytes actually written or noegative on error.	*/
/* If the socket is open but noone is connected, it tries to connect.   */
/************************************************************************/

int write_data(void *buf, int n)
{
	int	len;			/* Length in bytes */

	if (!data_connected()) return 0;

	/* 
	 * There is a small chance that a socket has gone away.
	 * For this reason we now check and do nothing if it has.
	 */

	if (!write_ready(data_fd)) 
	{
		error(WARNING,
			"Attempted to write to non-existant socket.");
		data_fd = -1;
		message(system_window,
			"%s just disconnected from the data socket.",
			data_machine);
		free(data_machine);
		return 0;
	}

	/* Send the data */

	len = write(data_fd, buf, n);

	/* Is there still someone there? */

	if (len <= 0)
	{
		/* They buggered off ... */

		data_fd = -1;
		message(system_window,
			"%s just disconnected from the data socket.",
			data_machine);
		free(data_machine);
		return len;
	}

	/* It seems to have worked */

	return len;
		
} /* write_data() */

/************************************************************************/
/* write_data_fast() 							*/
/* 									*/
/* Simmilar to write_data() but does no polling of the socket so	*/
/* it should be much faster (and more likely to cause an error).	*/
/* Attempts to write n bytes of data to the data socket			*/
/* from the buffer area.  If data socket is not open it does nothing.   */
/* Returns number of bytes actually written or negative on error.	*/
/* Doesn't create a connection so you should call data_connected() 	*/
/* before using this function.						*/
/* DOES NOT CHECKING.... BEWARE, IT COULD CRASH THE SYSTEM.		*/
/************************************************************************/

int write_data_fast(void *buf, int n)
{
	int	len;			/* Length in bytes */

	if (data_socket < 0 || data_fd < 0) return 0;

	/* Send the data */

	len = write(data_fd, buf, n);

	/* Is there still someone there? */

	if (len <= 0)
	{
		/* They buggered off ... */

		data_fd = -1;
		message(system_window,
			"%s just disconnected from the data socket.",
			data_machine);
		free(data_machine);
		return len;
	}

	/* It seems to have worked */

	return len;
		
} /* write_data_fast() */

/************************************************************************/
/* data_connected()							*/
/*									*/
/* Checks if someone is trying to connect to the data socket.		*/
/* If they are it trys to let them. If it does connect it returns	*/
/* TRUE, returns FALSE otherwise.					*/
/* If someone is already connected returns TRUE. 			*/
/************************************************************************/

int data_connected(void)
{
	unsigned int	len;			/* Length in bytes */
        struct sockaddr_in connector; 	/* Connector's address */
	unsigned char	c;		/* Single character read */
#ifndef STANDALONE
        struct hostent *hp;     	/* Pointer to host data */
	char	*domain;		/* Pointer to domain in name */
#endif

	if (data_socket == -1) return FALSE;

	/* Is someone already connected ? */

	if (data_fd != -1 && char_waiting(data_fd)) 
	{
		/* Try and do a read to see if they have gone */

		len = read(data_fd, &c, 1);
		
		/* Is there still someone there? */

		if (len <= 0)
		{
			/* They buggered off ... */

			message(system_window,
				"%s just disconnected from the data socket.",
				data_machine);
			data_fd = -1;
			free(data_machine);
			return FALSE;
		}

		/* They didn't bugger off they stayed */
	
		if (buffer_full(data_buffer))
		{
			error(WARNING,
				"Data buffer is full. We're loosing data.");
			return TRUE;
		}

		buffer_in(data_buffer,c);
		return TRUE;
	}

	/* Is anyone trying to connect? */

	if (char_waiting(data_socket))
	{
		/* Yes they are, so connect */

		len = sizeof(connector);
		if ((data_fd = accept(data_socket,
			(struct sockaddr *)&connector,&len)) < 0)
		{
			error(WARNING,
		      "Failed to accept data socket connection.\n");
			data_fd = -1;
			return FALSE;
		}

#ifndef STANDALONE
		/* Find out who this is */

		if ((hp = gethostbyaddr((char *)&connector.sin_addr,
				len,connector.sin_family)) == NULL)
		{
			error(WARNING,
			"Failed to identify host. Refusing data socket.");
			close(data_fd);
                        data_fd = -1;
			return FALSE;
		}
	
		/* Separrate out it's domain part. */

		for(domain=hp->h_name; *domain != 0 &&
				*domain != '.'; domain++);
		if (*domain == '.') ++domain;

		/* Is this OK ? */

		if (strcmp(myname,hp->h_name) && strcmp(domain,ALLOWED_DOMAIN))
		{
			close(data_fd);
			data_fd = -1;
			error(WARNING,
                        "Host %s not in correct domain. Blocking data socket.",
                                        hp->h_name);
			return FALSE;
		}
		else 
		{
			if ((data_machine = malloc(strlen(hp->h_name)+1))==NULL)
			{
				error(FATAL,FATAL_1,
					"socket connection");
			}
			strcpy(data_machine,hp->h_name);
			message(system_window,
				    "%s just connected to the data socket.",
					data_machine);
		}

#else /* We are standalone, assummed NOT conencted to network */

		if ((data_machine = malloc(strlen(myname)+1))==NULL)
		{
			error(FATAL,FATAL_1,
				"socket connection");
		}
		strcpy(data_machine,myname);
		message(system_window,"%s just connected to the data socket.",
				data_machine);
#endif

		return TRUE;
	}

	if (data_fd != -1)
		return write_ready(data_fd);
	else
		return FALSE;

} /* data_connected() */

/************************************************************************/
/* data_open()								*/
/*									*/
/* Returns TRUE of the data socket has been opened and FALSE otherwise. */
/************************************************************************/

int	data_open(void)
{
	return (data_socket != -1);

} /* data_open() */

/************************************************************************/
/* send_socket_command()						*/
/*									*/
/* Function to send a command to a socket connection.  			*/
/* The first argument is assumed to be the name of the machine to send  */
/* the command to, the second is the port number to use,		*/
/* while the third is used as the command line to			*/
/* send. It's very simple minded and any return text is completely	*/
/* ignored. The return value is the returned ERROR level of the		*/
/* remote called function. If this is not NOERROR, an error message	*/
/* is sent to the screen.						*/ 
/* Any text is displayed in the main window, one line at a time, 	*/
/* anything typed by the user is sent to the socket.			*/
/************************************************************************/

int	send_socket_command(char *machine, int port, char *command)
{
	int	fd;
	int	error_level;
	time_t  start_time;
	char	c,*p;
	char	s[BUFFER_SIZE];
	int	line;

	/* Try and open the connection to the port */

	if ((fd = open_connect_socket(machine, port)) == -1)
	{
		return error(ERROR,"Failed to open port %d on machine %s.\n",
				port, machine);
	}

	/* Send the command to the port */

	socket_print(fd,"%s\n",command);

	/* Set up for displaying things */

	line = -1;
	if (active_socket == -1)
	{
		active_window = main_window;
		werase(main_window);
		scrollok(main_window,TRUE);
	}

	/* Now cycle through returned strings until the end */

	start_time = time(NULL);
	while(TRUE)
	{
		if (char_waiting(fd)) 
		{
                        for(p = s, c = 0; 
                            c != '\n' && c != '\r' && p-s < BUFFER_SIZE; 
                            p++)
                        {
                                if (read(fd, &c, 1) != 1)
                                {
                                        close(fd);
                                        return error(NOERROR,"Socket closed.\n");
                                }
                                *p = c;
                        }
                        *p = 0;

			/* Display this message */

			if (active_socket == -1)
			{
				if (++line > MAIN_LINES)
				{
					scroll(main_window);
					line--;
				}
				mvwaddstr(main_window,line,0,s);
				wrefresh(main_window);
			}
			else
			{
				socket_print(active_socket,"%s",s);
			}

			/* Is this the final line ? */

			if (sscanf(s,SOCKET_COMMAND_STR,&error_level) == 1)
			{
				break;
			}
		}

		/* Did the user type something ? */

		if (active_socket == -1)
		{
			if (kbhit())
			{
				s[0] = get_command();
				s[1] = 0;
				if (++line > MAIN_LINES)
				{
					scroll(main_window);
					line--;
				}
				line_edit(main_window,line,0,s,
					MAIN_COLS-1,STRING,
					check_command,TRUE);
				socket_print(fd,"%s\n",s);
			}
		}
		else
		{
			if (char_waiting(active_socket))
			{
				socket_gets(active_socket,s,BUFFER_SIZE);
				socket_print(fd,"%s",s);
			}
		}

		if (start_time+TIME_OUT < time(NULL))
		{
		    error_level = error(ERROR,
			"Timed out waiting for command to complete.");
		    break;
		}
	}

	/* Was there an error ? */

	if (error_level != 0)
	{
		error(error_level,"Call to %s on port %d failed:\n%s",
				machine, port, command);
	}

	/* That is all */

	close_connect_socket(fd);

	return error_level;

} /* send_socket_command() */

/************************************************************************/
/* call_send_socket_command()						*/
/*									*/
/* User callable function to reach the send_socket_command() routine.	*/
/* First argument is the machine to call, second the port and the	*/
/* rest is the command itself.						*/
/************************************************************************/

int	call_send_socket_command(int argc, char **argv)
{
	char	machine[LENGTH_CL+1];
	int	port;
	char	command[LENGTH_CL+1];
	char	s[81],*ps;
	int	i;

	socket_test_args(3,"machine port command");

	/* Check out the command line */

        if (argc > 1)
        {
                strcpy(machine,argv[1]);
        }
        else
        {
		clean_command_line();
		werase(system_window);
		wrefresh(system_window);
                strcpy(machine,"decatur.mtwilson.edu");
                if (line_edit(command_window,0,2,machine,LENGTH_CL,
			STRING,return_true,TRUE) == KEY_ESC) return NOERROR;
        }

        if (argc > 2)
        {
		ps = argv[2];
        }
        else
        {
		clean_command_line();
		werase(system_window);
		wrefresh(system_window);
                sprintf(s,"1024");
                if (quick_edit("Port",s,s,NULL,INTEGER) ==
                        KEY_ESC) return NOERROR;
		ps = s;
        }
	sscanf(ps,"%d",&port);

	if (argc > 3)
	{
		strcpy(command,argv[3]);
		for (i=4;i<argc;i++)
		{
			strcat(command," ");
			strcat(command, argv[i]);
		}
	}
	else
        {
		strcpy(command,"ls");
		clean_command_line();
		message(system_window,"Enter the command you want to send.");
                if (line_edit(command_window,0,2,command,LENGTH_CL,
			STRING,return_true,TRUE) == KEY_ESC) return NOERROR;
        }


	switch(send_socket_command(machine, port, command))
	{
		case MESSAGE: message(system_window,
					"\"%s\" returned MESSAGE.",command);
			      break;
	
		case NOERROR: message(system_window,
					"\"%s\" returned NOERROR.",command);
			      break;

		case WARNING: message(system_window,
					"\"%s\" returned WARNING.",command);
			      break;

		case ERROR: message(system_window,
					"\"%s\" returned ERROR.",command);
			      break;

		case FATAL: message(system_window,
					"\"%s\" returned FATAL.",command);
			      break;
		
		
		default: message(system_window,
					"\"%s\" returned UNKNOWN.",command);
			      break;
	}

	return NOERROR;

} /* call_send_socket_command() */

/************************************************************************/
/* flush_sockets()							*/
/*									*/
/* Goes through all command sockets and empties them of any text.	*/
/* Will also close them out if required. This function should be called */
/* whenever you stop listening to the sockets.				*/
/* This will happen when you are not calling get_command() for a long	*/
/* time.								*/
/************************************************************************/

void flush_sockets(void)
{
	int	i;
	int	count,closed;
	char	c;

	for(closed = 0, count=0, i=0; i<MAX_CONNECTIONS; i++)
	{
		if (sockets[i].fd == -1) continue;

		while (char_waiting(sockets[i].fd))
		{
			if (read(sockets[i].fd, &c, 1) <= 0)
			{
				/* Read failed, connection has died */

				closed++;
				sockets[i].fd = -1;
				free(sockets[i].machine);
				break;
			}
			count++;
		}
	}

	if (count + closed > 0)
	{
		message(system_window,"Flushed %d bytes and closed %d sockets.",
			count, closed);
	}

} /* flush_sockets() */

/************************************************************************/
/* sig_pipe_text_sockets()						*/
/*									*/
/* This is called if there has been a broken pipe and the server.c	*/
/* signal catcher does not think it belongs to it.			*/
/* Works out which socket caused the fault and closes things down 	*/
/* on that socket.							*/
/************************************************************************/

void sig_pipe_text_sockets(int sig)
{
	int	i;

	/* Is there a current active socket? */

	if (active_socket == -1)
	{
		message(system_window,
			"There has been a broken pipe I can not identify!");
		return;
	}

	/* OK, which one is it? */

	for(i=0; i<MAX_CONNECTIONS; i++)
	{
		if (sockets[i].fd == -1) continue;

		if (sockets[i].fd == active_socket)
		{
			/* Read failed, connection has died */

			active_socket = -1;
			message(system_window,
			   "%s just caused a broken pipe. Logged off.\n",
				sockets[i].machine);
			active_socket = sockets[i].fd;
			sockets[i].fd = -1;
			free(sockets[i].machine);
			continue;

		}
	}

	/* There can no longer be an actice socket! */

	active_socket = -1;

} /* sig_pipe_text_sockets() */
