/************************************************************************/
/* bandpass.c								*/
/*									*/
/* Calculates the response of a bandpass filter.			*/
/************************************************************************/
/*                                        	                      	*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.						*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : Jan 1995     						*/ 
/*									*/
/* Changes:								*/
/*									*/
/************************************************************************/

#include <values.h>
#include <math.h>
#include "../nrc/nrc.h"
#include "filter.h"

float	process_bandpass(float *params, float input);

/************************************************************************/
/* bandpass()								*/
/*									*/
/* Set's up the quadratic lag filter setup.				*/
/* Returns centre frequency (eventually, 0 for now.			*/
/* See Numerical Recipes in C section 13.5 for explanantion.		*/
/************************************************************************/

float	bandpass(digital_filter *filter, float dt, float f0, float e)
{
	float	w0,t;

	w0 = tan(M_PI*f0*dt);
	t = 1.0 + e * w0;
	t = t*t + w0*w0;

	/*
	 * Set up the parameters to be used.
	 */

	filter->dt = dt;
	filter->n_params = 9;
	filter->params = vector(1,9);
	
	filter->params[1] = (2*e*w0 + e*e*w0*w0)/t;     	/* C0  */
	filter->params[2] = (2*e*e*w0*w0)/t;     		/* C1  */
	filter->params[3] = (e*e*w0*w0 - 2*e*w0)/t;     	/* C2  */
	filter->params[4] = 2*(1-e*e*w0*w0-w0*w0)/t;		/* D1  */
	filter->params[5] = -1*((1-e*w0)*(1-e*w0)+w0*w0)/t; 	/* D2  */
#ifdef DEBUG
	printf("C0 = %f\n",filter->params[1]);
	printf("C1 = %f\n",filter->params[2]);
	printf("C2 = %f\n",filter->params[3]);
	printf("D0 = %f\n",filter->params[4]);
	printf("D1 = %f\n",filter->params[5]);
#endif

	filter->params[6] = 0.0;				/* I_(n-1) */
	filter->params[7] = 0.0;				/* I_(n-2) */
	filter->params[8] = 0.0;				/* O_(n-1) */
	filter->params[9] = 0.0;				/* O_(n-1) */

	/* Set the processing function */

	filter->process = process_bandpass;

	return 0.0;

} /* bandpass() */

/************************************************************************/
/* process_bandpass(float *params, float input)				*/
/*									*/
/* Does the processing for the bandpass filter.				*/
/************************************************************************/

float	process_bandpass(float *params, float input)
{
	float	output;

	output = params[1] * input +
		 params[2] * params[6] +
		 params[3] * params[7] +
		 params[4] * params[8] +
		 params[5] * params[9];

	params[9] = params[8];
	params[8] = output;
	params[7] = params[6];
	params[6] = input;

	return output;

} /* process_bandpass() */

/************************************************************************/
/* bandpass_response()							*/
/*									*/
/* Calculates the response of a bandpass filter.			*/
/* Not done analytically yet, always returns zeros for now.		*/
/************************************************************************/

fcomplex bandpass_response(float freq, float f0, float e, float dt)
{
        fcomplex response;
	fcomplex t,enumerator,denominator;
	float	w,w0;

	w0 = tan(M_PI*f0*dt);
	w = tan(M_PI*freq*dt);

        t.r = w;
        t.i = -1.0*e*w0;
	enumerator = denominator = Cmul(t,t);
	denominator.r -= w0*w0;
	enumerator.r -= w*w;

	response = Cdiv(enumerator,denominator);

	return response;

} /* bandpass_response() */
