/************************************************************************/
/* photon_models.c							*/
/*									*/
/* Routines for modeling things with real photons.			*/
/*									*/
/************************************************************************/
/*                                        	                      	*/
/* (C) This source code and its associated executable           	*/
/* program(s) are copyright.                                    	*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : April 1995							*/
/************************************************************************/

#include <math.h>
#include <nrc.h>
#include "interferometry.h"
#include <values.h>

/************************************************************************/
/* expected_photon_count()						*/
/*									*/
/* Given aperture size, bandwidth, sample time etc this routine returns */
/* the expected average photon count from a star of given magnitude.	*/
/************************************************************************/

float	expected_photon_count(
	float	mag,		/* Magnitude of star */
	float	R,		/* aperture radius in meters */
	float	tau,		/* Sample time in seconds */
	float	bandwidth,	/* optical bandwidth in nanometers */
	float	dqe_sys,	/* DQE of optical system */
	float	dqe_det)	/* DQE of detector */
	
{
	float	result;

	result = R * R * M_PI * tau * 10.0 * bandwidth * dqe_sys * dqe_det *
		 pow(10.0,(7.0-mag/2.5));

	return result;

} /* expected_photon_count() */

/************************************************************************/
/* photon_count_2_mag()                                                 */
/*                                                                      */
/* Given aperture size, bandwidth, sample time etc this routine returns */
/* the expected magnitude of a star at a curtain count rate.    	*/
/************************************************************************/

float   photon_count_2_mag(
        float   n,              /* Photon Count */
        float   R,              /* aperture radius in meters */
        float   tau,            /* Sample time in seconds */
        float   bandwidth,      /* optical bandwidth in nanometers */
        float   dqe_sys,        /* DQE of optical system */
        float   dqe_det)        /* DQE of detector */

{
        float   result;

        result = 2.5*(7.0-log10(n/(R * R * M_PI * tau * 10.0 * 
			bandwidth * dqe_sys * dqe_det)));

        return result;

} /* photon_count_2_mag() */


/************************************************************************/
/* detect_counts()							*/
/* 									*/
/* Simulates the counts recieved in the two detectors either side of a	*/
/* beam splitter. Returns total count.					*/
/************************************************************************/

float detect_counts(float phase,float v,float n,float s,float *n1,float *n2)
{
	float k1,k2,r1,r2,I1,I2;
	float vcos;

	vcos = v * cos(phase);

	I1 = n * (1.0 + vcos);
	I2 = n * (1.0 - vcos);

	k1 = poidev(I1, &random_seed);
	k2 = poidev(I2, &random_seed);

	if (s != 0.0)
	{
		r1 = gasdev(&random_seed) * s;
		r2 = gasdev(&random_seed) * s;
	}
	else
	{
		r1 = 0.0;
		r2 = 0.0;
	}

	*n1 = k1 + r1;
	*n2 = k2 + r2;

#ifdef DEBUG
	printf("detect_counts(%f,%f,%f,%f,%f,%f)\n",phase,v,n,s,*n1,*n2);
	printf("k1 = %f k2 = %f r1 = %f r2 = %f\n",k1,k2,r1,r2);
#endif

	return (*n1+*n2);

} /* detect_counts() */

/************************************************************************/
/* fringe_value()							*/
/*									*/
/* Calculates the fringe value given a set of parameters.		*/
/* Bandpass assumed square.						*/
/************************************************************************/

float fringe_value(float pos, /* micron */
		   float v,
		   float lambda, /* micron */
		   float bw	 /* micron */,
		   float phase)  /* Rad */
{
	float	frg, envel;
	float	wn,wn1,wn2,dwn,x;

	wn = 1.0/lambda;
	wn1 = 1.0/(lambda - bw/2.0);
	wn2 = 1.0/(lambda + bw/2.0);
	dwn = fabs(wn1-wn2);

	if (pos == 0.0)
	{
		envel = 1.0;
	}
	else
	{
		x = M_PI * pos * dwn;	
		envel = sin(x)/x;
	}

	frg = 1.0 + v * envel * cos(2.0*M_PI*pos*wn+phase);

#ifdef DEBUG
	printf("%f\t%f\t%f\n", pos, envel, frg);
#endif

	return frg;

} /* fringe_value() */

/************************************************************************/
/* detect_fringe_value()						*/
/*									*/
/* Uses fringe_value to calculate a number of photons detected. 	*/
/* Includes photon and read noise.					*/
/************************************************************************/

float detect_fringe_value(float	n, /* Mean photon count */
		   float gain, /* Gain of camera */
		   float sig, /* Read noise */
		   float pos, /* micron */
		   float v,
		   float lambda, /* micron */
		   float bw,	 /* micron */
		   float phase) /* Rad */
{
	n = n * fringe_value(pos, v, lambda, bw, phase);

	/* Calculate the count */

	return (gain * poidev(n, &random_seed) + gasdev(&random_seed) * sig);

} /* detect_fringe_value() */
