/************************************************************************/
/* visibility_functions.c 						*/
/*									*/
/* Routines that calculate visibility functions, like uniform disk etc.	*/
/* THis is the most recent version and uses only do_simpson integration */
/************************************************************************/
/*                                        	                      	*/
/* Center for High Angular Resolution Astronomy				*/
/*									*/
/* ZERNIKE CODE LIBRARY.						*/
/*									*/
/* Telephone: 404-651-2932						*/
/* Fax      : 404-651-1384						*/
/* Internet : theo@@chara.gsu.edu					*/
/*									*/
/* (C) This source code and its associated executable           	*/
/* program(s) are copyright.                                    	*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : January 2001    						*/
/************************************************************************/

#include <math.h>
#include <float.h>
#include <values.h>
#include <nrc.h>
#include "interferometry.h"
#include <chara.h>

/* Function and globals for v_broadband() */

static float	(*vis_function)(float, float, float);
static float v_broadband_baseline;
static float v_broadband_theta;

/* Number of parts used in simpson integrations */

#define NUM_LIMB_DARK	1000	
#define NUM_BROADBAND	100	

/* Globals used for v_limb_dark_mp() */

static float limb_dark_mp_theta0 = 0.0;
static float limb_dark_mp_p = 0.0;

/* Globals used for v_limb_dark_lin() */

static float limb_dark_lin_theta0 = 0.0;
static float limb_dark_lin_p = 0.0;

/* Globals used for v_limb_dark_pow() */

static float limb_dark_pow_p = 0.0;

/* Globals used for axisymmetric_direct_calculation() */

static	float (*last_intensity)(float) = NULL;
static  float int_limit = (0.1*(AS2R/1000.0));
static  float last_u = 0.0;

/* Globals used for limb darkened calculation. */

static float limb_dark_mu = 0.0;

/************************************************************************/
/* v_uniform_disk()							*/
/*									*/
/* Calculates visibiltiy of a uniform disk given baseline in meters,	*/
/* wavelength in meters and angular size in radians.			*/
/************************************************************************/

float v_uniform_disk(float baseline, float wavelength, float theta)
{
	float	x;

	/* If theta or baseline is zero the answer is easy */

	if (theta == 0.0 || baseline == 0.0 ) return 1.0;

	x = M_PI * theta * baseline / wavelength;
#ifdef DEBUG
	printf("theta = %g b=%g wl=%g x=%g\n",theta,baseline,wavelength,x);
#endif

	if (x == 0.0)
		return 1.0;
	else
		return 2.0*bessj1(x)/x;

} /* v_uniform_disk() */

/************************************************************************/
/* v_binary()								*/
/* 									*/
/* Calculates the visiblity of a binary system with two unform disks	*/
/* given baseline in meters, sizes and separation of the stars in 	*/
/* raians, wavelength in meters, intensity ratio, and wavelength in	*/
/* meters. Returns -1.0 of there is an error.				*/
/************************************************************************/

float v2_binary(float baseline, float wavelength, float theta1,
		float theta2, float sep, float r)
{
	float	v1, v2;

	if (r <= 0.0 || r > 1.0) return -1.0;

	v1 = fabs(v_uniform_disk(baseline, wavelength, theta1));
	v2 = fabs(v_uniform_disk(baseline, wavelength, theta2));

	return (v1*v1 + r*r*v2*v2 + 
			2.0*r*v1*v2*cos(2.0*M_PI*baseline*sep/wavelength))
			/ ((1.0+r)*(1.0+r));

} /* v_binary() */

/************************************************************************/
/* v_gaussian()								*/
/*									*/
/* Calculates the visbility of a gaussian profile given baseline in 	*/
/* meters, wavelength in meters and angular size in radians.            */
/* Angular size is the FWHM of the gaussian.				*/
/************************************************************************/

float v_gaussian(float baseline, float wavelength, float theta)
{
        float   x,c;

	/* If theta or baseline is zero the answer is easy */

	if (theta == 0.0 || baseline == 0.0 ) return 1.0;

        x = baseline / wavelength;
#warning REALLY UNSURE ABOUT THIS... SHOULD IT BE DIVIDE????
	c = theta * 2.35482;

	fflush(stdout);
	return exp(c*c*x*x/-2.0);

} /* v_gaussian() */

/************************************************************************/
/* v_broadband()							*/
/*									*/
/* Caculates the visibility of a given type but integrated over a 	*/
/* specified badnwidth. Assums a square bandpass.			*/
/* Also assumes that the detector is a photon detector and so the mean	*/
/* is weighted by wavelength.						*/
/************************************************************************/

float v_broadband(
	int profile_type,
	float baseline,float wavelength,float bandwidth,float theta)
{
	float result;

	/* If theta or baseline is zero the answer is easy */

	if (theta == 0.0 || baseline == 0.0 ) return 1.0;

	/* Set a function pointer to the right function */

	switch(profile_type)
	{
		case UNIFORM_DISK: vis_function = v_uniform_disk;
				   break;

		case GAUSSIAN: vis_function = v_gaussian;
			       break;

		case LIMB_DARK_MP: vis_function = v_limb_dark_mp;
				break;

		case LIMB_DARK_LIN: vis_function = v_limb_dark_lin;
				break;

		case LIMB_DARK_POW: vis_function = v_limb_dark_pow;
				break;

		case LIMB_DARK: vis_function = v_limb_dark;
				break;

		default: fprintf(stderr,"Unknown stellar profile type.\n");
			 return 0.0;

	}

	/* What's the zero bandwidth result ? */

	result = (*vis_function)(baseline, wavelength, theta);
#ifdef DEBUG
	printf("B=%f L=%f dL=%f T=%f V=%f\n",
		baseline, wavelength*1e6, 
		bandwidth*1e6, theta*(1000.0/AS2R), result);
#endif

	/* If there is no bandwidth just call ordinary function */

	if (bandwidth <= 0.0) return result;

	/* Set some globals up for use */

	v_broadband_baseline = baseline;
	v_broadband_theta = theta;

	/* Now do the integration using simpson's rule */

	result =  do_simpson(v_broadband_int_funct, wavelength-bandwidth/2.0,
			     wavelength+bandwidth/2.0, NUM_BROADBAND)/
				(wavelength*bandwidth);

	return result;

} /* v_broadband() */

/************************************************************************/
/* v_broadband_int_funct()						*/
/*									*/
/* For caculating the function to be integrated for v_broadband().	*/
/* Note that we are multiplying by wavelength here since the detector   */
/* is a photon detector.						*/
/************************************************************************/

float v_broadband_int_funct(float wavelength)
{
	return wavelength*(*vis_function)(v_broadband_baseline, wavelength, 
			v_broadband_theta);

} /* v_broadband_int_funct() */

/************************************************************************/
/* axisymmetric_direct_calculation()					*/
/*									*/
/* This routine does a direct integration of the Hankel transform	*/
/* given a function that defines the intensity of an axisymmetric	*/
/* disk.								*/
/*									*/
/* It first tries to see if it has been called before with the same	*/
/* function. If so it uses the last denominator, if not it first	*/
/* calculates the denominator.						*/
/************************************************************************/

float axisymmetric_direct_calculation(float (*intensity)(float), float u)
{
	static  float total_intensity;
	float	result;
	float	temp;

#ifdef DEBUG
	printf("axisymmetric_direct_calculation(function,%e)\n",u);
#endif
	/* Have we been called with this intensity distribution before ? */

	if (last_intensity != intensity)
	{
		/* 
   		 * No we have not... calculate the denominator 
		 * We're assuming here that it will not extend beyond
		 * limb_dark_int_limit arcseconds.
		 */

		last_intensity = intensity;
		total_intensity = do_simpson(axisymmetric_direct_tot_int_funct, 
			0.0, int_limit, NUM_LIMB_DARK);
#ifdef DEBUG
		printf("Final total_intensity = %e\n",total_intensity);
#endif
	}

	/* 
	 * Now do the real calculation.
	 * We're assuming here that it will not extend beyond
	 * int_limit arcsecond.
	 */

	last_u = u;
	temp = do_simpson(axisymmetric_direct_int_funct,0,
			int_limit,NUM_LIMB_DARK); 
#ifdef DEBUG
	printf("Enumerator = %g\n",temp);
#endif

	result = temp/total_intensity;

	return result;

} /* axisymmetric_direct_calculation() */

/************************************************************************/
/* axisymmetric_direct_int_funct()					*/
/*									*/
/* Calculates the integral function of the Hankel transform.		*/
/************************************************************************/

float axisymmetric_direct_int_funct(float theta)
{
	return theta * (*last_intensity)(theta) * bessj0(2*M_PI*theta*last_u);
}

/************************************************************************/
/* axisymmetric_direct_tot_int_funct()					*/
/*									*/
/* Calculates the integral function of the total power.			*/
/************************************************************************/

float axisymmetric_direct_tot_int_funct(float theta)
{
	return theta * (*last_intensity)(theta);

} /* axisymmetric_direct_tot_int_funct() */

/************************************************************************/
/* limb_dark_mp()							*/
/*									*/
/* Function that calculates a limb darkened disk function as per	*/
/* Michelson and Pease ApJ 53 (1921) 256.				*/
/************************************************************************/

float	limb_dark_mp(float b0, float b, float p)
{
	float	center,b02;

	/* If we're outside the disk the result is always zero */

	if (b > b0) return 0.0;

	/* What is the value in the center ? */

	b02 = b0 * b0;
	center = pow(b02,p);

	/* Return the value at position asked for with maximum of 1.0  */

	return pow((b02 - b*b), p)/center;

} /* limb_dark_mp() */

/************************************************************************/
/* setup_v_limb_dark_mp()						*/
/*									*/
/* Set's up some global variables for v_limb_dark_mp() to use. This	*/
/* is done so that v_limb_dark_mp() looks like other visbility calls.	*/
/************************************************************************/

void setup_v_limb_dark_mp(float p)
{
	if (p > 2.9)
		fprintf(stderr,"Warning p values > 2.9 do odd things.\n");

	limb_dark_mp_p = p;

	/* We'll need to force a recalculation of the total intensity */

	last_intensity = NULL;

} /* setup_v_limb_dark_mp() */

/************************************************************************/
/* v_limb_dark_mp()							*/
/*									*/
/* Uses axisymmetric_direct_calculation() and limb_dark_mp() to 	*/
/* calculate the visibility at a given baseline and wavelength for	*/
/* a limb darkened disk of the Michelson/Pease variety.			*/
/* You must call setup_v_limb_dark_mp() first!				*/
/************************************************************************/

float v_limb_dark_mp(float baseline, float wavelength, float theta)
{
	float	u;
	float result;

	/* If theta or baseline is zero the answer is easy */

	if (theta == 0.0 || baseline == 0.0 ) return 1.0;

	/* Do the calculation */

	limb_dark_mp_theta0 = theta/2.0;
	int_limit = theta * 0.51;
	u = baseline / wavelength;

	result = axisymmetric_direct_calculation(v_limb_dark_mp_int_funct, u);

#ifdef DEBUG
	printf("b=%g w=%g t=%g v=%f\n",
		baseline,wavelength*1e6,theta/(AS2R/1000.0),result);
#endif

	return result;

} /* v_limb_dark_mp() */

/************************************************************************/
/* v_limb_dark_mp_int_funct()						*/
/*									*/
/* Function to be called by axisymmetric_direct_calculation() 		*/
/* as used by v_limb_dark_mp() 						*/
/************************************************************************/

float v_limb_dark_mp_int_funct(float theta)
{
	float	result;

	result = limb_dark_mp(limb_dark_mp_theta0 , theta, limb_dark_mp_p);

#ifdef DEBUG_2
	printf("v_limb_dark_mp_int_funct(%f) = %f\n",theta/(AS2R/1000.0),
			result);
#endif

	return result;

} /* v_limb_dark_mp_int_funct() */

/************************************************************************/
/* limb_dark_lin()							*/
/*									*/
/* Function that calculates a limb darkened disk function as per	*/
/* linear limb darkening law.						*/
/************************************************************************/

float	limb_dark_lin(float b0, float b, float p)
{
	/* If we're outside the disk the result is always zero */

	if (b > b0) return 0.0;

	/* Return the value at position asked for with maximum of 1.0  */

	return 1.0 - p * (1.0 - sqrt(1.0 - (b*b)/(b0*b0)));

} /* limb_dark_lin() */

/************************************************************************/
/* setup_v_limb_dark_lin()						*/
/*									*/
/* Set's up some global variables for v_limb_dark_lin() to use. This	*/
/* is done so that v_limb_dark_lin() looks like other visbility calls.	*/
/************************************************************************/

void setup_v_limb_dark_lin(float p)
{
	limb_dark_lin_p = p;

	/* We'll need to force a recalculation of the total intensity */

	last_intensity = NULL;

} /* setup_v_limb_dark_lin() */

/************************************************************************/
/* v_limb_dark_lin()							*/
/*									*/
/* Uses axisymmetric_direct_calculation() and limb_dark_lin() to 	*/
/* calculate the visibility at a given baseline and wavelength for	*/
/* a limb darkened disk of the linear variety.				*/
/* You must call setup_v_limb_dark_lin() first!				*/
/************************************************************************/

float v_limb_dark_lin(float baseline, float wavelength, float theta)
{
	float	u;
	float result;

	/* If theta or baseline is zero the answer is easy */

	if (theta == 0.0 || baseline == 0.0 ) return 1.0;

	/* Do the calculation */

	limb_dark_lin_theta0 = theta/2.0;
	int_limit = theta * 0.51;
	u = baseline / wavelength;

	result = axisymmetric_direct_calculation(v_limb_dark_lin_int_funct, u);

#ifdef DEBUG
	printf("b=%g w=%g t=%g v=%f\n",
		baseline,wavelength*1e6,theta/(AS2R/1000.0),result);
#endif

	return result;

} /* v_limb_dark_lin() */

/************************************************************************/
/* v_limb_dark_lin_int_funct()						*/
/*									*/
/* Function to be called by axisymmetric_direct_calculation() 		*/
/* as used by v_limb_dark_lin() 					*/
/************************************************************************/

float v_limb_dark_lin_int_funct(float theta)
{
	float	result;

	result = limb_dark_lin(limb_dark_lin_theta0 , theta, limb_dark_lin_p);

#ifdef DEBUG_2
	printf("v_limb_dark_lin_int_funct(%f) = %f\n",theta/(AS2R/1000.0),
			result);
#endif

	return result;

} /* v_limb_dark_lin_int_funct() */

/************************************************************************/
/* limb_dark_pow()							*/
/*									*/
/* Function that calculates a limb darkened disk function as per	*/
/* powear limb darkening law.						*/
/* Note that this is essentially the same as the michelson/pease law	*/
/* See Hestroffer AA 327, 199 (1997)					*/
/************************************************************************/

float	limb_dark_pow(float b0, float b, float p)
{
	float	cos_theta;

	/* If we're outside the disk the result is always zero */

	if (b > b0) return 0.0;

	/* Return the value at position asked for with maximum of 1.0  */

	cos_theta = sqrt(1.0 - (b*b)/(b0*b0));
	return pow(cos_theta, p);

} /* limb_dark_pow() */

/************************************************************************/
/* setup_v_limb_dark_pow()						*/
/*									*/
/* Set's up some global variables for v_limb_dark_pow() to use. This	*/
/* is done so that v_limb_dark_pow() looks like other visbility calls.	*/
/************************************************************************/

void setup_v_limb_dark_pow(float p)
{
	limb_dark_pow_p = p;

} /* setup_v_limb_dark_pow() */

/************************************************************************/
/* v_limb_dark_pow()							*/
/*									*/
/* Calculates the visibility at a given baseline and wavelength for	*/
/* a limb darkened disk of the power law variety.			*/
/* You must call setup_v_limb_dark_pow() first!				*/
/* See Hestroffer AA 327, 199 (1997)					*/
/************************************************************************/

float v_limb_dark_pow(float baseline, float wavelength, float theta)
{
	float	xnu;
	float	x;
	float	rj, ry, rjp, ryp;

	x = M_PI * theta * baseline / wavelength;

	if (x == 0.0) return 1.0;

	xnu = limb_dark_pow_p/2.0 + 1.0;

	bessjy(x, xnu, &rj, &ry, &rjp, &ryp);

	return exp(gammln(xnu+1.0)) *  rj / pow(x/2.0, xnu);
}

/************************************************************************/
/* setup_v_limb_dark()							*/
/*									*/
/* Set's up some global variables for v_limb_dark() to use. This	*/
/* is done so that v_limb_dark() looks like other visbility calls.	*/
/* Look into Hanbury Brown 1974 MNRAS 167,121 or Lane 2001 ApJ 551,81.	*/
/************************************************************************/

void setup_v_limb_dark(float mu)
{
	limb_dark_mu = mu;

} /* setup_v_limb_dark() */

/************************************************************************/
/* limb_dark()								*/
/*									*/
/* Function that calculates a limb darkened disk function as per	*/
/* law used on HD189733.						*/
/* We need to check that this is correct!!!!				*/
/* Look into Hanbury Brown 1974 MNRAS 167,121 or Lane 2001 ApJ 551,81.	*/
/************************************************************************/

#warning Not at all sure about this one
float	limb_dark(float b0, float b, float mu)
{
	/* If we're outside the disk the result is always zero */

	if (b > b0) return 0.0;

	/* Return the value at position asked for with maximum of 1.0  */

	return 1.0 - mu*(1.0-cos(b/b0));

} /* limb_dark_lin() */

/************************************************************************/
/* v_limb_dark()							*/
/*									*/
/* Calculates the visibility at a given baseline and wavelength for	*/
/* a limb darkened disk of the variety used for HD189733.		*/
/* You must call setup_v_limb_dark() first!				*/
/* We really need a citation for this!					*/
/* Look into Hanbury Brown 1974 MNRAS 167,121 or Lane 2001 ApJ 551,81.	*/
/************************************************************************/

float v_limb_dark(float baseline, float wavelength, float theta)
{
	float	j;
	float	x;
	float   mu;

	x = M_PI * theta * baseline / wavelength;

	if (x == 0.0) return 1.0;

	j = bessj1(x)/x;
	mu = limb_dark_mu;

	return ((1.0-mu)*bessj1(x)/x + (mu*(sin(x)/x - cos(x)))/(x*x))/
		((1.0-mu)/2.0 + mu/3.0);

} /* v_limb_dark() */
