/************************************************************************/
/* matrox_continuous.c							*/
/*									*/
/* Routines to work with the meteor matrox frame grabber.		*/
/* This is for the continuous frame method. The frame grabber keeps     */
/* grabbing frames, the user proces can just read them anytime it is	*/
/* ready to, which may not be synchronis with the frames at all.	*/
/************************************************************************/
/*                                        	                      	*/
/*     Center for High Angular Resolution Astronomy			*/
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.		*/
/*									*/
/*									*/
/* Telephone: 1-404-651-1882						*/
/* Fax      : 1-404-651-1389						*/
/* email    : theo@chara.gsu.edu					*/
/* WWW	    : http://www.chara.gsu.edu/~theo/theo.html			*/
/*									*/
/* (C) This source code and its associated executable           	*/
/* program(s) are copyright.                                    	*/
/*                                                              	*/
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar						*/
/* Date   : April 1999   						*/
/************************************************************************/

#include "matrox_meteor.h"
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/fcntl.h>
#include <ioctl_meteor.h>
#include <stdlib.h>

static unsigned char	*mmbuf = NULL;
static int size8 = 0;
static int size16 = 0;
static int columns = 0;
static int rows = 0;

/************************************************************************/
/* open_meteor_continuous()						*/
/*									*/
/* Tries to open the matrox meteor board. If a filename is given it	*/
/* tries to use it, otherwise it uses the default name. Returns 0	*/
/* if it works or the following errors:					*/
/* 1 - Device already open or failed to open it.			*/
/* 2 - ioctl call for FORMAT failed.					*/
/* 3 - ioctl call for geom failed.					*/
/* 4 - ioctl call for contrast failed.					*/
/* 5 - ioctl call for brightness failed.				*/
/* 6 - ioctl call for FPS failed.					*/
/* 8 - memory allocation failed						*/
/* 9 - continuous allocation failed					*/
/* 10 - continuous setting in driver failed				*/
/* 11- Failed to tell driver to start getting frames.			*/
/*									*/
/* If there is an error the errorno global will be set appropriately.   */
/************************************************************************/

int open_meteor_continuous(char *devname, int col, int row) 
{
	char	*s;
	struct meteor_geomet geo;
	int		c;

	/* Is it already open? */

	if (meteor_fd != -1) return 1;

	/* Otherwise. let's open it */
	
	if (devname == NULL)
		s = DEFAULT_DEVICE;
	else
		s = devname;

	if ((meteor_fd = open(s,O_RDONLY)) < 0) return 1;

	/* It will always be NTCS as far as we are concerned */

        c = METEOR_FMT_NTSC;

        if (ioctl(meteor_fd, METEORSFMT, &c) < 0) 
	{
		close(meteor_fd);
		meteor_fd = -1;
		return 2;
	}

	if (col < 1) col = DEFAULT_COLS;
	if (row < 1) row = DEFAULT_ROWS;

	geo.rows = row;
	geo.columns = col;
	geo.frames = NOFRAMES;
	geo.oformat = METEOR_GEO_YUV_PLANER; /* Always B&W */

        if (ioctl(meteor_fd, METEORSETGEO, &geo) < 0) 
	{
		close(meteor_fd);
		meteor_fd = -1;
		return 3;
	}

	/* Set defaut Contrast and Brightness */

	if (last_brightness < 0)
	{
		if (set_meteor_brightness(DEFAULT_BRIGHTNESS)) 
		{
			close(meteor_fd);
			meteor_fd = -1;
			return 4;
		}
	}
	else
	{
		if (set_meteor_brightness(last_brightness)) 
		{
			close(meteor_fd);
			meteor_fd = -1;
			return 4;
		}
	}

	if (last_contrast < 0)
	{
		if (set_meteor_contrast(DEFAULT_CONTRAST)) 
		{
			close(meteor_fd);
			meteor_fd = -1;
			return 5;
		}
	}
	else
	{
		if (set_meteor_contrast(last_contrast)) 
		{
			close(meteor_fd);
			meteor_fd = -1;
			return 5;
		}
	}

	/* Set maximum frames per second */

	c = FPS;
	if (ioctl(meteor_fd,METEORSFPS,&c) < 0) 
	{
		close(meteor_fd);
		meteor_fd = -1;
		return 6;
	}

	/* Setup the sizes and so on */

	size8 = col * row;
	size16 = size8*2;
	columns = col;
	rows = row;

	/* Allocate the memory */

	if ((mmbuf=(unsigned char *) mmap((caddr_t)0, 
				          size16,
					  PROT_READ | PROT_WRITE, 
				 	  MAP_FILE|MAP_PRIVATE, 
					  meteor_fd, 
					  (off_t)0)) == (unsigned char *)(-1)) 
	{
		close(meteor_fd);
		meteor_fd = -1;
		return 8;
	}

	c = METEOR_CAP_CONTINOUS;
   	if (ioctl(meteor_fd, METEORCAPTUR, &c) < 0)
	{
		close(meteor_fd);
		meteor_fd = -1;
		return 11;
	}

	/* All seems to have went well */

	return 0;

} /* open_meteor_continuous() */

/************************************************************************/
/* close_meteor()							*/
/*									*/
/* closes the device for the meteor frame grabber.			*/
/************************************************************************/

void close_meteor_continuous(void)
{
	int	c;

	/* Is it actually open? */

	if (meteor_fd == -1) return;

	/* Now instruct the driver to stop getting frames */

	c = METEOR_CAP_STOP_CONT;
   	ioctl(meteor_fd, METEORCAPTUR, &c);

	/* Close it */

	close(meteor_fd);
	meteor_fd = -1;

} /* close_meteor_continuous() */

/************************************************************************/
/* meteor_buffer_location() 						*/
/*									*/
/* Returns the position in memory of the buffer containing the image.   */
/* Retunrs NULL if it is not going yet.					*/
/************************************************************************/

unsigned char *meteor_buffer_location(void)
{

	/* Is it actually open? */

	if (meteor_fd == -1) return NULL;

	return mmbuf;

} /* meteor_buffer_location() */

/************************************************************************/
/* meteor_continuous_get_frame()					*/
/*									*/
/* Coppies whatever is in the frame buffer into a user specified	*/
/* buffer. Assumes the calling function has allocated enough memory.	*/
/* Also returns the maximum intensity and sets the position of that	*/
/* maximum intensity.							*/
/************************************************************************/

int meteor_continuous_get_frame(unsigned char *buffer, int *xpos, int *ypos)
{
	int i,j;
	int max;
	unsigned char *p;
	
	/* Is it actually open? */

	if (meteor_fd == -1) return 0;

	max = -1;
	*xpos = *ypos = 0;

	p = mmbuf;
	for(j=0; j < rows; j++)
	for(i=0; i < columns; i++)
	{
		if ((*buffer = *p++) > max)
		{
			*xpos = i;
			*ypos = j;
			max = *buffer;
		}
		buffer++;
	}

	return max;

} /* meteor_continuous_get_frame() */
