/************************************************************************/
/* nrc_lsq_fit.c							*/
/*                                                                      */
/* Non linear least squares fit routines that use the NRC library.	*/
/*									*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : 31st July 1996						*/
/************************************************************************/

#include <stdio.h>
#include <math.h>
#include <values.h>
#include "nrc.h"

/************************************************************************/
/* non_linear_fit()							*/
/*									*/
/* Non-linear least squares fit routine.  				*/
/************************************************************************/

int	non_linear_fit(int 	nvar,
	               float 	*var,	/* 1..nvar */
	               int 	ndata,
	               float	*data,	/* 1..ndata */
		       int	nparam,
		       float	**param, /* 1..ndata 1..nparam */
		       int	deriv_prov, /* TRUE/FALSE */
	               void	(*calfunc)(),
	               float	*rmsres,
	               float	*maxres,
                       float per_rms_err,
                       int     max_nit,
                       int     show)
{
	/* Internals */

	float	**FTF;		/* Matrix F transform times matrix F */
	float	*FTR;		/* Matrix F transform times matrix R */
	float	*ftn;		/* Returned from */
	int	*indx;
	int	iteration;
	float     d;
	register int i,j,k;
	float	lastrms;
	float	dy,dx,x,y;

	/* Set up arrays and matrices */

	FTF = matrix(1,nvar,1,nvar);
	FTR = vector(1,nvar);
	ftn = vector(0,nvar);
	indx = ivector(1,nvar);

	/* If the first guesses are zero things will break ..... */

	for(i=1; i<=nvar; i++)
	{
		if (var[i] == 0.0) var[i] = 1e-20;
	}

	/* Put up header info if required */

	if (show)
	{
		printf("\n it   ");
		if (show == SHOW_ALL)
		{
			for(i=1;i<=nvar;i++) printf("var[%2d]    ",i);
		}
		printf("rmsres\n");
	}

	/* Find initial RMS residue */

	*rmsres = 0;
	for(i=1;i<=ndata;i++)
	{
		(*calfunc)(ftn, var, param[i]);
		d = data[i] - ftn[0];
		*rmsres += (d * d);
	}

	*rmsres = sqrt(*rmsres/(float)ndata);

	/* OK go for it */

        per_rms_err /= 100.0;
	for(iteration=1; iteration <= max_nit; iteration++)
	{
		if (show)
		{
			printf("%3d ",iteration);
			if (show == SHOW_ALL)
			{
				for(i=1;i<=nvar;i++) printf("%10.3e ",var[i]);
			}
			printf("%10.3e\n",*rmsres);
			fflush(stdout);
		}

		/* Build the matrices */

		for(i=1;i<=nvar;i++)
		{
			FTR[i] = 0; 
			for(j=1;j<=nvar;j++) FTF[i][j] = 0;
		}

		for(i=1;i<=ndata;i++)
		{
			if (deriv_prov)
			{
			    /* calfunc provides derivatives */

			    (*calfunc)(ftn, var, param[i]);
			}
			else
			{
			    /* Approximate derivatives */
#ifdef DEBUG
			   printf("\nndata = %d\n",i);
#endif

			   (*calfunc)(ftn, var, param[i]);
			    y = ftn[0];
			    for(j=1;j<=nvar;j++)
			    {
				x = var[j];
#ifdef DEBUG
				    printf("var[%d]: y = %e x = %e\n",j,y,x);
#endif

/* 
 * This used to be 
 * for(dx = 1.0; dx>MINFLOAT; dx /= 10.0)
 * but it turned out unstable for fitting some functions.
 * We need a much more robust method. 
 */

				for(dx = 0.01; dx>MINFLOAT; dx /= 10.0)
				{
				    var[j] = (1.0 - dx) * x;
                                    (*calfunc)(ftn, var, param[i]);
				    dy = (y - ftn[0])/y;
#ifdef DEBUG
				    printf("x = %e y = %e dy = %e dx = %e\n",
					var[j], ftn[0], y*dy,x*dx);
#endif
				    if (dy < 0.0001) break;
				}

				var[j] = x;
#ifdef DEBUG
			printf("y = %f dy = %f x = %f dx = %f\n",y,dy,x,dx);
#endif
				ftn[j] = (y * dy)/(x * dx);
			    }
			    ftn[0] = y;
			}
#ifdef DEBUG
			printf("data = %f ftn[0] = %f ",data[i], ftn[0]);
			for(j=1;j<=nvar;j++) 
				printf("ftn[%d] = %f ",j,ftn[j]);
			putchar('\n');
#endif

			for(j=1;j<=nvar;j++)
			{
				FTR[j] += ((data[i] - ftn[0]) * ftn[j]);

				for(k=1;k<=nvar;k++)
					FTF[j][k] += (ftn[j] * ftn[k]);
			}
		}
#ifdef DEBUG
		printf("\nFTF:\n");
		for(i=1;i<=nvar;i++)
		{
			for(j=1;j<=nvar;j++) printf("%6f ",FTF[i][j]);
			putchar('\n');
		}

		printf("\nFTR:\n");
		for(j=1;j<=nvar;j++) printf("%6f ",FTR[j]);
		putchar('\n');
#endif
			
		/* Solve the equation */

		ludcmp(FTF,nvar,indx,&d);
		lubksb(FTF,nvar,indx,FTR);

		/* Adjust estimates by this correction */

		for(i=1;i<=nvar;i++) var[i] += FTR[i];

		/* Find RMS residue */

		lastrms = *rmsres;
		*rmsres = 0;
		*maxres = 0;
		for(i=1;i<=ndata;i++) 
		{
			(*calfunc)(ftn, var, param[i]);
			d = data[i] - ftn[0];
			*rmsres += (d * d);
			if (fabs((double)d) > fabs((double)(*maxres))) 
				*maxres= d;
		}

		*rmsres = sqrt(*rmsres/(float)ndata);

		/* If the residue has not change enough that's it */

		if (fabs(lastrms - *rmsres) <= per_rms_err* *rmsres) break;
	}

	if (show)
	{
		printf("FIN ");
		if (show == SHOW_ALL)
		{
			for(i=1;i<=nvar;i++) printf("%10.3e ",var[i]);
		}
		printf("%10.3e\n\n",*rmsres);
		fflush(stdout);
	}

	/* Free memory */

	free_matrix(FTF,1,nvar,1,nvar);
	free_vector(FTR,1,nvar);
	free_vector(ftn,0,nvar);
	free_ivector(indx,1,nvar);

	return iteration;

} /* non_linear_fit() */

/************************************************************************/
/* var_non_linear_fit()                                                 */
/*                                                                      */
/* Calculates variance of variables calculated by non_linear_fit using  */
/* a numerical estimate of partital derivatives and varriances of the	*/
/* parameters.								*/
/************************************************************************/

float *var_non_linear_fit(int      nvar,
                       float    *var,   /* 1..nvar */
                       int      ndata,
                       float    *data,  /* 1..ndata */
                       int      nparam,
                       float    **param, /* 1..ndata 1..nparam */
                       float    **var_param, /* 1..ndata 1..nparam */
                       int      deriv_prov, /* TRUE/FALSE */
                       void     (*calfunc)(),
                       float per_rms_err,
                       int     max_nit,
                       int     show)
{
	float	*var_var;	/* Resulting variable variances 1..nvar */
	float	*temp_var;	/* Tempoaray storage for variables */
	float	delta_var;	/* change in variables 1..nparam*/
	float	delta_param;	/* change in variables 1..nparam*/
	float	*temp_param;	/* change in variables 1..nparam*/
	float	old_param;
	float	deriv;
	int	i,j,k;		/* Looping variables */
        float   rmsres;
        float   maxres;

	/* Allocate memory */

	var_var = vector(1,nvar);
	temp_var = vector(1,nvar);
	temp_param = vector(1,nparam);

	/* Initialize */

	for(i=1; i<= nvar; i++) var_var[i] = 0.0;
		
#ifdef DEBUG
	printf(
"   i    j    k  D_param    param      D_var     var      Deriv    Var_var\n");
#endif

	for(i = 1; i <= ndata; i++) /* Ok loop through the data */
	{
		for(j=1; j<=nparam; j++) /* Loop through the parameters */
		{
			/* Save the current parameter value */

			old_param = param[i][j];

			/* Creat a change in parameter */

			delta_param = DELTA_PERCENT * param[i][j];
			param[i][j] += delta_param;

			/* Refit things */

			for(k=1; k<= nvar; k++) temp_var[k] = var[k];

			non_linear_fit(nvar,temp_var,ndata,data,nparam,param,
			       deriv_prov, calfunc,&rmsres,&maxres, per_rms_err,
			       max_nit, show);

			/* Now go through variables for changes */

			for(k=1; k<= nvar; k++)
			{
				/* calculate the change in the variables */

				delta_var = temp_var[k] - var[k];

				/* Estimate the partial derivative */

				deriv = delta_var/delta_param;

				/* Add the variance power to total */

				var_var[k] += (deriv*deriv*var_param[i][j]);
#ifdef DEBUG
		printf("%4d %4d %4d %9.2e %9.2e %9.2e %9.2e %9.2e %9.2e\n",
					i,j,k,delta_param,param[i][j],
					delta_var,var[k],deriv,var_var[k]);
#endif
			}

			/* Change parameter value back to what it was */
			
			param[i][j] = old_param;
		}
	}

	/* Cleanup and go */

	free_vector(temp_param,1,nparam);
	free_vector(temp_var,1,nvar);

	return var_var;

} /* var_non_linear_fit() */

/************************************************************************/
/* error_non_linear_fit()						*/
/* 									*/
/* Attempts to find the errors in a fit as produced by non_linear_fit.	*/
/* Plays with each variable and trys to find change require to		*/
/* result in an RMS error multiplied by a user supplied factor.		*/
/* Returned value is a vector[1..nvar] of the resulting error values.	*/
/* It's up to the calling function to free this memory.			*/
/************************************************************************/

float	*error_non_linear_fit(int nvar, 
                       float    *var,   /* 1..nvar */
                       int      ndata,
                       float    *data,  /* 1..ndata */
                       int      nparam,
                       float    **param, /* 1..ndata 1..nparam */
                       void     (*calfunc)(),
                       float    rms_change)
{
	float	rmsres;
	float	orig_rmsres;
	float 	new_rmsres;
	int	i,j,k;
	float	d;
	float	var_orig,var_high,var_low;
	float	*result;
	float	*ftn;

	/* Allocated the memory */

	result = vector(1,nvar);
	ftn = vector(0,nvar);

        /* Find initial RMS residue */

        rmsres = 0;
        for(i=1;i<=ndata;i++)
        {
                (*calfunc)(ftn, var, param[i]);
                d = data[i] - ftn[0];
                rmsres += (d * d);
        }
        rmsres = sqrt(rmsres/(float)ndata);

	/* What is the value of the rms residue we need to find? */

	orig_rmsres = rmsres;
	new_rmsres = rms_change * rmsres;

	/* OK, go through all of the variables */

	for(j=1; j<= nvar; j++)
	{
		/* Save the  starting value */

		var_orig = var[j];

		/* Now move up until we find the right value */

		for(k=0, var_orig = var[j]; 
			(k < MAX_IT) && (rmsres < new_rmsres); 
			var[j] *= 1.001)
		{
			rmsres = 0;
			for(i=1;i<=ndata;i++)
			{
				(*calfunc)(ftn, var, param[i]);
				d = data[i] - ftn[0];
				rmsres += (d * d);
			}
			rmsres = sqrt(rmsres/(float)ndata);
#ifdef DEBUG
			printf("var[%d] = %g rmsres = %g\n",j,var[j],rmsres);
#endif
		}
		if (k >= MAX_IT)
		{
		  fprintf(stderr,
			"Iteration limit reached gogin up variable %d.\n",
			k);
		}
		var_high = var[j];
		var[j] = var_orig;
		rmsres = orig_rmsres;
		
		/* Now move down and do the same thing */

		for(k=0, var_orig = var[j]; 
			(k< MAX_IT) && (rmsres < new_rmsres); 
			var[j] *= 0.999)
		{
			rmsres = 0;
			for(i=1;i<=ndata;i++)
			{
				(*calfunc)(ftn, var, param[i]);
				d = data[i] - ftn[0];
				rmsres += (d * d);
			}
			rmsres = sqrt(rmsres/(float)ndata);
#ifdef DEBUG
			printf("var[%d] = %g rmsres = %g\n",j,var[j],rmsres);
#endif
		}
		if (k >= MAX_IT)
		{
		  fprintf(stderr,
			"Iteration limit reached going down variable %d.\n",
			k);
		}
		var_low = var[j];
		var[j] = var_orig;
		rmsres = orig_rmsres;
		
		/* So we have found the two results */

		result[j] = fabs(var_high - var_low)/2.0;		
		printf("Var[%d] = %g. Range %g to %g. Error = %g\n",
			j,var[j],var_low,var_high,result[j]);
	}

	/* That is all */

	free_vector(ftn,0,nvar);
	return result;

} /* error_non_linear_fit() */
