/************************************************************************/
/* wobble.c								*/
/*									*/
/* Routines to simulate a tip tilt servo.  				*/
/* These functions use the FILTER library, which we assume is here.	*/
/************************************************************************/
/*                                                                      */
/*              THE ZERNIKE LIBRARY CODE.				*/
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-404-651-1882                                            */
/* Fax      : 1-404-651-1389                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : 1st August 1996                                             */
/************************************************************************/

#include <stdlib.h>
#include <math.h>
#include "zernike.h"
#include <filter.h>

/************************************************************************/
/* set_up_quad_detect() 						*/
/*									*/
/* Sets up and returns quadrant_detector structure.			*/
/* Requires the maximum number of modes to be used, central wavelength	*/
/* in microns of band fo detector and the angle at which the tilt	*/
/* is to be measured in radians.					*/
/* Usually called by set_up_wobbler_servo().				*/
/************************************************************************/

quadrant_detector set_up_quad_detect(int        maxJ,
                                     float      lambda,
                                     float      theta)
{
	quadrant_detector quad;

	/* Check that the values are reasonable */

	if (maxJ < 1 || maxJ > zernike_max_j)
	{
		fprintf(stderr,"set_up_quad_detect: must use 1<=maxJ<%d.\n",
			zernike_max_j);
		exit(-1);
	}

	if (lambda <= 0.0)
	{
		fprintf(stderr,"set_up_quad_detect: lambda must be > 0.\n");
		exit(-1);
	}

	if (theta < 0.0 || theta > 2.0*M_PI)
	{
		fprintf(stderr,
			"set_up_quad_detect: theta must be <=2PI, >=0.\n");
		exit(-1);
	}

	/* OK, put the appropriate values into the structure */

	quad.maxJ = maxJ;
	quad.lambda = lambda;
	quad.theta = theta;
	quad.maximum = 0.6 * lambda;
	quad.err_norm = 0.147262156 * lambda;

#ifdef DEBUG
	printf("Setting up a quadrant detector:\n");
	printf("maxJ    = %d\n",quad.maxJ);
	printf("lambda    = %f\n",quad.lambda);
	printf("theta    = %f\n",quad.theta);
	printf("maximum    = %f\n",quad.maximum);
	printf("err_norm    = %f\n\n",quad.err_norm);
#endif

	/* That is all .... */

	return quad;

} /* set_up_quad_detect() */

/************************************************************************/
/* quad_detect() 							*/
/*									*/
/* Simulates a quadrant detector including photon noise.		*/
/* Requires quadranmt detector structure as created by 			*/
/* set_up_quad_detect() but more usually done by set_up_wobbler_servo(),*/
/* a vector of zernike coefficients normalized for OPD in microns and   */
/* the photon count this sample n.							*/
/************************************************************************/

float	quad_detect(quadrant_detector quad, float *a, float n) 
{
	float	result;
	float	measurement;
	float	error;
#ifdef DEBUG
	int	j;
#endif

	measurement = 0.5 * phase_gradient(quad.maxJ, quad.theta, a);

	error = quad.err_norm * gasdev(&random_seed) / sqrt(n);
	
	if ((result=measurement+error) > quad.maximum) result = quad.maximum;

	if (result < -quad.maximum) result = -quad.maximum;

#ifdef DEBUG
	printf("quadrant_detector: n = %.0f meas = %f err = %f result = %f\n",
		n,measurement,error,result);
	for(j=1;j<=quad.maxJ;j++) printf("a[%d] = %f\n",j,a[j]);
#endif
	return result;

} /* quad_detect() */

/************************************************************************/
/* set_up_tip_tilt_mirror()						*/
/*									*/
/* Set's up tip_tilt_mirror structure.					*/
/* Requires the time step to be used in seconds, the natural resonance  */
/* frequency of the mirror, the damping factor (as in a quadratic lag), */
/* the maximum throw in radius and the beam radius in meters.		*/
/* Usually called by set_up_wobbler_servo().				*/
/* If the parameters are rediculous, defaults are used.			*/
/************************************************************************/

tip_tilt_mirror set_up_tip_tilt_mirror(

		float	dt,		/* Time step used */
		float	resonance,	/* Natural resonance frequency */
		float	damp,		/* Damping factor */
		float	maximum,	/* Maximum throw in radians */
		float	R		/* Beam radius */
				       )
{
	tip_tilt_mirror tip_tilt;

	/* Check that everything is reasonable */

	if (dt <= 0)
	{
		fprintf(stderr,
	"set_up_tip_tilt_mirror: time step forced to be positive (%f)\n",
			DEFAULT_DT);
		dt = DEFAULT_DT;
	}

	if (resonance <= 0)
	{
		fprintf(stderr,
	"set_up_tip_tilt_mirror: resonance forced to be positive (%f)\n",
			DEFAULT_RESONANCE);
		resonance = DEFAULT_RESONANCE;
	}

	if (damp <= 0)
	{
		fprintf(stderr,
		"set_up_tip_tilt_mirror: damp forced to be positive (%f)\n",
			DEFAULT_DAMP);
		damp = DEFAULT_DAMP;
	}

	if (maximum <= 0)
	{
		fprintf(stderr,
		"set_up_tip_tilt_mirror: maximum forced to be positive (%f)\n",
			DEFAULT_MAXIMUM);
		maximum = DEFAULT_MAXIMUM;
	}

	if (R <= 0)
	{
		fprintf(stderr,
	"set_up_woobler: R forced to be positive (%f)\n",
			DEFAULT_R);
		R = DEFAULT_R;
	}

	/* Set up internal parameters */

	tip_tilt.resonance = resonance;
	tip_tilt.damp = damp;
	tip_tilt.maximum = maximum * R * 1e6; /* Convert to um per m */
	
	/* Allocate memory */

	tip_tilt.response = (struct DIGITAL_FILTER *)
				malloc(sizeof(struct DIGITAL_FILTER));

	if (tip_tilt.response == (struct DIGITAL_FILTER *)NULL)
	{
		fprintf(stderr,"set_up_tip_tilt_mirror: not enough memory.\n");
		exit(-1);
	}

	/* Create the quadratic lag response required */

	tip_tilt.f6db = quadratic_lag(tip_tilt.response, dt, resonance, damp);

#ifdef DEBUG
	printf("6dB point = %f\n",tip_tilt.f6db);
#endif

	/* That is all */

	return tip_tilt;

} /* set_up_tip_tilt_mirror() */

/************************************************************************/
/* clean_up_tip_tilt_mirror() 						*/
/*									*/
/* Deallocates memory for a tip_tilt_mirror structure.			*/
/************************************************************************/

void clean_up_tip_tilt_mirror(tip_tilt_mirror tip_tilt)
{
	clean_up_filter(*tip_tilt.response);
	free(tip_tilt.response);

} /* clean_up_tip_tilt_mirror() */

/************************************************************************/
/* move_mirror()							*/
/* 									*/
/* Process new position for a tip tilt mirror.				*/
/* Requires a tip_tilt_mirror structure as created by 			*/
/* set_up_tip_tilt_mirror() and a new position in radians to go to.	*/
/************************************************************************/

float move_mirror(tip_tilt_mirror tip_tilt, float new_pos)
{
	float	pos;

	if (new_pos > tip_tilt.maximum) new_pos = tip_tilt.maximum;
	if (new_pos < -tip_tilt.maximum) new_pos = -tip_tilt.maximum;

	pos = process_filter((*(tip_tilt.response)), new_pos);

	return pos;

} /* move_mirror() */

/************************************************************************/
/* set_up_wobbler_servo() 						*/
/*									*/
/* Set's up a wobbler servo structure.					*/
/* The servo has proportional (C1) and differential (C2) terms.		*/
/* Function sets up two lots of quadrant detectors and tip/tilt mirrors */
/* using the functions set_up_quad_detect() and set_up_quad_detect().   */
/* It will therefore need all the parameters these functions need.	*/
/* The parameter history specifies the number of samples over which	*/
/* to take a running mean for the quadrant detectors, normally used	*/
/* for extreemly low light levels.					*/
/************************************************************************/

wobbler_servo   set_up_wobbler_servo(
		float	C1, float C2,	/* Servo parameters */
                int     maxJ,           /* Maximum Zernike mode */
                float   lambda,         /* Wavelength in microns */
                float   dt,             /* Time step used */
                float   resonance,      /* Natural resonance frequency mirror */
                float   damp,           /* Damping factor mirror */
                float   maximum,        /* Maximum throw in radians mirror */
                float   R,              /* Beam radius */
                int     history)        /* history length detector */
{
	wobbler_servo wobbler;
	int i;

	/* Allocate memory */

	if ((wobbler.detector2 = 
		(struct QUADRANT_DETECTOR *) 
		malloc(sizeof(struct QUADRANT_DETECTOR))) == NULL)
	{
		fprintf(stderr,"set_up_wobbler_servo: Not enough memory.\n");
		exit(-3);
	}

	if ((wobbler.detector3 = 
		(struct QUADRANT_DETECTOR *) 
		malloc(sizeof(struct QUADRANT_DETECTOR))) == NULL)
	{
		fprintf(stderr,"set_up_wobbler_servo: Not enough memory.\n");
		exit(-3);
	}

	if ((wobbler.mirror2 = 
		(struct TIP_TILT_MIRROR *) 
		malloc(sizeof(struct TIP_TILT_MIRROR))) == NULL)
	{
		fprintf(stderr,"set_up_wobbler_servo: Not enough memory.\n");
		exit(-3);
	}

	if ((wobbler.mirror3 = 
		(struct TIP_TILT_MIRROR *) 
		malloc(sizeof(struct TIP_TILT_MIRROR))) == NULL)
	{
		fprintf(stderr,"set_up_wobbler_servo: Not enough memory.\n");
		exit(-3);
	}

	wobbler.detector_history2 = vector(1,history);
	wobbler.detector_history3 = vector(1,history);

	/* Initialize the detectors */

	*(wobbler.detector2) = set_up_quad_detect( maxJ, lambda, 0.0);

	*(wobbler.detector3) = set_up_quad_detect( maxJ, lambda, M_PI/2.0);

	/* Initilize the mirrors */

	*(wobbler.mirror2) = set_up_tip_tilt_mirror(dt,
				resonance, damp, maximum, R);

	*(wobbler.mirror3) = set_up_tip_tilt_mirror(dt,
				resonance, damp, maximum, R);

	/* Initialze detector history */

	wobbler.history_length = history;

	for(i=1;i<=wobbler.history_length; i++)
	{
		wobbler.detector_history2[i] = 0.0;
		wobbler.detector_history3[i] = 0.0;
	}

	wobbler.detector_output2 = 0.0;
	wobbler.detector_output3 = 0.0;
	wobbler.current_history_pos = 1;

	/* Set up servo parameters */

	wobbler.C1 = C1;
	wobbler.C2 = C2;

	/* Finally the next demand positions of the mirror */

	wobbler.next_demand_position2 = 0.0;
	wobbler.next_demand_position3 = 0.0;
	wobbler.last_move_2 = 0.0;
	wobbler.last_move_3 = 0.0;

	/* That is all */

	return wobbler;

} /* set_up_wobbler_servo() */

/************************************************************************/
/* clean_up_wobbler_servo()						*/
/*									*/
/* House keeping for wobbler servos.					*/
/* Deallocates memory for a wobbler_servo structure.			*/
/************************************************************************/

void    clean_up_wobbler_servo(wobbler_servo wobbler)
{
	/* Clean up the mirror structures */

	clean_up_tip_tilt_mirror(*(wobbler.mirror2));
	clean_up_tip_tilt_mirror(*(wobbler.mirror3));

	/* Deallocate the memory */

	free(wobbler.detector2);
	free(wobbler.detector3);
	free(wobbler.mirror2);
	free(wobbler.mirror3);

	free_vector(wobbler.detector_history2,1,wobbler.history_length);
	free_vector(wobbler.detector_history3,1,wobbler.history_length);

	/* That is all */

} /* clean_up_wobbler_servo */

/************************************************************************/
/* wobble() 								*/
/*									*/
/* Actually performs wobbler work.					*/
/* Requires a wobbler_servo structure as setup by 			*/
/* set_up_wobbler_servo(), a serries of Zernike coefficients normalized */
/* for OPD in microns and the number of photons in the sample n.	*/
/* The actual coefficients are not changed but the tip_tilt_x and	*/
/* tip_tilt_y parameters are.						*/
/************************************************************************/

void    wobble(wobbler_servo *wobbler, zernike_sample *sample, float n)
{
	/* First of all, we impliment the last demand positions */

	sample->tip_tilt_x = move_mirror(*(wobbler->mirror2), 
				wobbler->next_demand_position2);

	sample->tip_tilt_y = move_mirror(*(wobbler->mirror3), 
				wobbler->next_demand_position3);

	sample->a[2] -= sample->tip_tilt_x;
	sample->a[3] -= sample->tip_tilt_y;

#ifdef DEBUG
	printf("Wobbler channel 2:\n");
	printf("New mirror pos 2 = %f\n",sample->tip_tilt_x);
	printf("New mirror pos 3 = %f\n",sample->tip_tilt_y);
#endif

	/* Do running mean calculations on detector */

	wobbler->detector_output2 -= 
		wobbler->detector_history2[wobbler->current_history_pos];

	wobbler->detector_history2[wobbler->current_history_pos] = 
		quad_detect(*(wobbler->detector2), sample->a, n);

	wobbler->detector_output2 += 
		wobbler->detector_history2[wobbler->current_history_pos];

	wobbler->detector_output3 -= 
		wobbler->detector_history3[wobbler->current_history_pos];

	wobbler->detector_history3[wobbler->current_history_pos] = 
		quad_detect(*(wobbler->detector3), sample->a, n);

	wobbler->detector_output3 += 
		wobbler->detector_history3[wobbler->current_history_pos];

	if (++wobbler->current_history_pos > wobbler->history_length)
	{
		wobbler->current_history_pos = 1;
	}
	
#ifdef DEBUG
	printf("New detected position error 2 %f\n",wobbler->detector_output2);
	printf("New detected position error 3 %f\n",wobbler->detector_output3);
#endif

	/* Workout new movement demands */

	wobbler->last_move_2 = wobbler->C1 * wobbler->detector_output2/
				wobbler->history_length -
			       wobbler->C2 * wobbler->last_move_2;

	wobbler->last_move_3 = wobbler->C1 * wobbler->detector_output3/
				wobbler->history_length -
			       wobbler->C2 * wobbler->last_move_3;

#ifdef DEBUG
	printf("New movement command 2 = %f\n",wobbler->last_move_2);
	printf("New movement command 3 = %f\n",wobbler->last_move_3);
#endif

	/* Now work out new positons */

	wobbler->next_demand_position2 += wobbler->last_move_2;
	wobbler->next_demand_position3 += wobbler->last_move_3;

#ifdef DEBUG
	printf("New demand position 2 = %f\n",wobbler->next_demand_position2);
	printf("New demand position 3 = %f\n",wobbler->next_demand_position3);
#endif

	/* That is all */

} /* wobble() */

/************************************************************************/
/* wobbler_response() 							*/
/*									*/
/* Calculates theoretical response of wobbler system at a specified	*/
/* frequency.								*/
/* All based on the tip/tilt paper by ten Brummelaar.			*/
/************************************************************************/

fcomplex wobbler_response(wobbler_servo wobbler, float freq)
{
	fcomplex H,F,result;
	fcomplex denominator;
	fcomplex enumerator;

	H = quadratic_lag_response(freq, wobbler.mirror2->resonance,
				     wobbler.mirror2->damp);

	F = computer_response(freq, wobbler.mirror2->response->dt,
				    wobbler.history_length,
				    wobbler.C1, wobbler.C2);

	denominator = Cmul(H,F);
	enumerator.r = 1.0 + denominator.r;
	enumerator.i = denominator.i;

	result = Cdiv(denominator,enumerator);

	return result;

} /* wobbler_response() */

/************************************************************************/
/* computer_response()                                                  */      
/*                                                                      */
/* Calculates theoretical response of Computer system for a given 	*/
/* freqency. 								*/
/* All based on the tip/tilt paper by ten Brummelaar.			*/
/* The servo has proportional (C1) and differential (C2) terms.		*/
/* The parameter nhist specifies the number of samples over which	*/
/* to take a running mean for the quadrant detectors, normally used	*/
/* for extreemly low light levels.					*/
/* The sample time dt is in seconds.					*/
/************************************************************************/

fcomplex computer_response(float freq, float dt, int nhist, float C1, float C2)
{
	int	j;
	float	 wT;
	fcomplex expjsT;
	fcomplex expsT;
	fcomplex denominator;
	fcomplex enumerator;
	fcomplex result;

	/* Calculate constants */

	wT = 2.0 * M_PI * freq * dt;

	expsT.r = cos(wT);
	expsT.i = -1.0 * sin(wT);

	/* Calculate denominator */

	expjsT.r = 1.0;
	expjsT.i = 0.0;

	denominator.r = denominator.i = 0.0;

	for(j=1; j<= nhist; j++)
	{
		expjsT = Cmul(expjsT,expsT);
		denominator = Cadd(denominator,expjsT);
	}

	denominator = RCmul(C1/(float)nhist,denominator);

	/* And the enumerator */

	enumerator.r = 1.0;
	enumerator.i = 0.0;

	enumerator = Cadd(enumerator,RCmul((C2-1.0),expsT));

	enumerator = Cadd(enumerator,RCmul(-1.0*C2,Cmul(expsT,expsT)));

	/* That's about it... */

	result = Cdiv(denominator,enumerator);

	return result;

} /* computer_response() */
